@@ -12,6 +12,8 @@
 #   include	<ctype.h>
 
 #   include	"tedApp.h"
+#   include	"tedDraw.h"
+#   include	"tedLayout.h"
 #   include	"tedRuler.h"
 
 #   include	<appDebugon.h>
@@ -35,20 +37,25 @@ static int tedShowIBar(	void *		voided )
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    AppDrawingData *		add= &(ed->edDrawingData);
-
-    int				ox= ed->edVisibleRect.drX0;
-    int				oy= ed->edVisibleRect.drY0;
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
+    DocumentRectangle		drPixels;
+
+    LayoutContext		lc;
+
+    layoutInitContext( &lc );
+    tedSetLayoutContext( &lc, ed );
 
     td->tdShowIBarId= (guint)0;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd,
+			(DocumentTree **)0, (const BufferItem **)0, td ) )
 	{ LDEB(1); return 0;	}
 
-    tedDrawIBar( &(sg.sgBegin), ox, oy, add );
+    tedGetIBarRect( &drPixels, &(sg.sgBegin), &lc );
+    tedDrawIBar( &drPixels, &lc );
 
     td->tdHideIBarId= gtk_timeout_add( TED_BLINK_VISIBLE,
 						tedHideIBar, (void *)ed );
@@ -88,20 +95,24 @@ void tedStopCursorBlink(	EditDocument *	ed )
 
     if  ( td->tdShowIBarId )
 	{
-	AppDrawingData *	add= &(ed->edDrawingData);
+	DocumentSelection		ds;
+	SelectionGeometry		sg;
+	SelectionDescription		sd;
+	DocumentRectangle		drPixels;
 
-	int			ox= ed->edVisibleRect.drX0;
-	int			oy= ed->edVisibleRect.drY0;
+	LayoutContext			lc;
 
-	DocumentSelection	ds;
-	SelectionGeometry	sg;
+	layoutInitContext( &lc );
+	tedSetLayoutContext( &lc, ed );
 
 	gtk_timeout_remove( td->tdShowIBarId );
 
-	if  ( tedGetSelection( &ds, &sg, td ) )
+	if  ( tedGetSelection( &ds, &sg, &sd,
+			   (DocumentTree **)0, (const BufferItem **)0, td ) )
 	    { LDEB(1); return;	}
 
-	tedDrawIBar( &(sg.sgBegin), ox, oy, add );
+	tedGetIBarRect( &drPixels, &(sg.sgBegin), &lc );
+	tedDrawIBar( &drPixels, &lc );
 	}
 
     td->tdHideIBarId= (guint)0;
