@@ -10,7 +10,7 @@
 #   include	<stdio.h>
 #   include	<stddef.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
@@ -30,23 +30,17 @@ static void tedFormatToolRefreshCellPage(	CellTool *	ct )
     const TableRectangle *	tr= &(ct->ctTableRectangle);
     const CellProperties *	cp= &(rp->rpCells[tr->trCol0]);
 
-    EditApplication *		ea= ct->ctApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, ct->ctApplication );
 
-    TedDocument *		td;
-    BufferDocument *		bd;
-    DocumentProperties *	dp;
-
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
-    dp= &(bd->bdProperties);
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
 
     appIntegerToTextWidget( ct->ctRowText, tr->trRow0- tr->trRow00+ 1 );
     appIntegerToTextWidget( ct->ctColumnText, tr->trCol0+ 1 );
 
+    appSetOptionmenu( &(ct->ctValignMenu), cp->cpValign );
+
     if  ( tr->trCol0 == tr->trCol1	&&
 	  tr->trRow0 == tr->trRow1	&&
 	  ! rp->rpIsTableHeader		)
@@ -65,23 +59,26 @@ static void tedFormatToolRefreshCellPage(	CellTool *	ct )
 	appStringToTextWidget( ct->ctRowspanText, "" );
 	}
 
-    tedBorderToolSetProperties( &(ct->ctTopBorderTool), dp,
-						&(cp->cpTopBorder) );
-    tedBorderToolSetProperties( &(ct->ctBottomBorderTool), dp,
-						&(cp->cpBottomBorder) );
-    tedBorderToolSetProperties( &(ct->ctLeftBorderTool), dp,
-						&(cp->cpLeftBorder) );
-    tedBorderToolSetProperties( &(ct->ctRightBorderTool), dp,
-						&(cp->cpRightBorder) );
+    tedBorderToolSetPropertiesByNumber( &(ct->ctTopBorderTool), bd,
+						cp->cpTopBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(ct->ctBottomBorderTool), bd,
+						cp->cpBottomBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(ct->ctLeftBorderTool), bd,
+						cp->cpLeftBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(ct->ctRightBorderTool), bd,
+						cp->cpRightBorderNumber );
 
-    tedSetShadingTool( &(ct->ctShadingTool), dp, &(cp->cpShading) );
+    tedSetShadingToolByNumber( &(ct->ctShadingTool), bd, cp->cpShadingNumber );
     }
 
-void tedFormatToolRefreshCellTool(
+void tedRefreshCellTool(
 				CellTool *			ct,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	ds )
+				const DocumentSelection *	ds,
+				const SelectionDescription *	sd,
+				const BufferDocument *		bd )
     {
     TableRectangle *		tr= &(ct->ctTableRectangle);
 
@@ -90,22 +87,20 @@ void tedFormatToolRefreshCellTool(
     const BufferItem *		rowBi;
     const BufferItem *		cellBi;
 
-    if  ( docGetTableRectangle( tr, ds )	||
-	  tr->trRow1 != tr->trRow0		||
-	  tr->trCol1 != tr->trCol0		)
+    if  ( docGetTableRectangle( tr, ds ) )
 	{ docInitTableRectangle( tr ); *pEnabled= 0; return; }
 
-    cellBi= ds->dsBegin.dpBi;
+    cellBi= ds->dsHead.dpBi;
     cellBi= cellBi->biParent;
     rowBi= cellBi->biParent;
 
     rp= &(rowBi->biRowProperties);
 
-    if  ( docCopyRowProperties( &(ct->ctRowPropertiesChosen),
-						    rp, (const int *)0 ) )
+    if  ( docCopyRowProperties( &(ct->ctRowPropertiesChosen), rp,
+				(const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return;	}
-    if  ( docCopyRowProperties( &(ct->ctRowPropertiesSet),
-						    rp, (const int *)0 ) )
+    if  ( docCopyRowProperties( &(ct->ctRowPropertiesSet), rp,
+				(const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return;	}
 
     appGuiEnableWidget( is->isPrevButton, tr->trCol0 > 0 );
@@ -113,7 +108,7 @@ void tedFormatToolRefreshCellTool(
 
     tedFormatToolRefreshCellPage( ct );
 
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
@@ -138,9 +133,9 @@ static APP_TXACTIVATE_CALLBACK_H( tedCellToolRowspanChanged, w, voidct )
 
     maxValue= tr->trRow11- tr->trRow0+ 1;
 
-    if  ( ! appGetIntegerFromTextWidget( w, &rowspan,
+    if  ( ! appGetIntegerFromTextWidget( ct->ctRowspanText, &rowspan,
 			    minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ appIntegerToTextWidget( w, rowspan );	}
+	{ appIntegerToTextWidget( ct->ctRowspanText, rowspan );	}
 
     return;
     }
@@ -159,9 +154,9 @@ static APP_TXACTIVATE_CALLBACK_H( tedCellToolColspanChanged, w, voidct )
 
     maxValue= tr->trCol11- tr->trCol0+ 1;
 
-    if  ( ! appGetIntegerFromTextWidget( w, &colspan,
+    if  ( ! appGetIntegerFromTextWidget( ct->ctColspanText, &colspan,
 			    minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ appIntegerToTextWidget( w, colspan );	}
+	{ appIntegerToTextWidget( ct->ctColspanText, colspan );	}
 
     return;
     }
@@ -184,11 +179,8 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeCellPushed, w, voidct )
     BorderProperties		bp;
 
     /**/
-    EditApplication *		ea= ct->ctApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
-
-    TedDocument *		td;
-    BufferDocument *		bd;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, ct->ctApplication );
     DocumentProperties *	dp;
 
     int				minValue= 1;
@@ -199,21 +191,20 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeCellPushed, w, voidct )
     int				rowspan= 0;
     int				colspan= 0;
 
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
     dp= &(bd->bdProperties);
 
     /**/
     docInitBorderProperties( &bp );
 
-    PROPmaskCLEAR( &rpSetMask );
-    PROPmaskCLEAR( &cpSetMask );
+    utilPropMaskClear( &rpSetMask );
+    utilPropMaskClear( &cpSetMask );
 
     /****/
 
+    PROPmaskADD( &cpSetMask, CLpropVALIGN );
+
     if  ( tr->trCol0 == tr->trCol1	&&
 	  tr->trRow0 == tr->trRow1	&&
 	  ! rp->rpIsTableHeader		)
@@ -244,58 +235,63 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeCellPushed, w, voidct )
 	}
 
     /****/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 						&(ct->ctTopBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
-	docRowSetTopBorderInCols( rp, tr->trCol0, tr->trCol1, &bpSetMask, &bp );
+	docRowSetTopBorderInCols( rp, tr->trCol0, tr->trCol1,
+					    &(bd->bdBorderPropertyList),
+					    &bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropTOP_BORDER );
 	}
 
     /****/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 					    &(ct->ctBottomBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
 	docRowSetBottomBorderInCols( rp, tr->trCol0, tr->trCol1,
-							    &bpSetMask, &bp );
+					    &(bd->bdBorderPropertyList),
+					    &bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropBOTTOM_BORDER );
 	}
 
     /****/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 						&(ct->ctLeftBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
 	docRowSetLeftBorderInCols( rp, tr->trCol0, tr->trCol1,
-							    &bpSetMask, &bp );
+					    &(bd->bdBorderPropertyList),
+					    &bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropLEFT_BORDER );
 	}
 
     /****/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 					    &(ct->ctRightBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
 	docRowSetRightBorderInCols( rp, tr->trCol0, tr->trCol1,
-							    &bpSetMask, &bp );
+					    &(bd->bdBorderPropertyList),
+					    &bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropRIGHT_BORDER );
 	}
 
     if  ( tedFormatToolGetShading( &cpSetMask, rp, tr->trCol0, tr->trCol1,
-						dp, &(ct->ctShadingTool) ) )
+						bd, &(ct->ctShadingTool) ) )
 	{ LDEB(1); goto ready;	}
 
     tedDocSetTableProperties( ed, &(ct->ctTableRectangle),
@@ -307,12 +303,36 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeCellPushed, w, voidct )
     return;
     }
 
+static APP_OITEM_CALLBACK_H( tedCellValignChosen, w, voidct )
+    {
+    CellTool *			ct= (CellTool *)voidct;
+    const TableRectangle *	tr= &(ct->ctTableRectangle);
+    RowProperties *		rp= &(ct->ctRowPropertiesChosen);
+
+    int				col;
+    CellProperties *		cp;
+
+    int				valign;
+
+    valign= appGuiGetOptionmenuItemIndex( &(ct->ctValignMenu), w );
+    if  ( valign < 0 || valign >= DOCtva_COUNT )
+	{ LLDEB(valign,DOCtva_COUNT); return;	}
+
+    cp= &(rp->rpCells[tr->trCol0]);
+    for ( col= tr->trCol0; col <= tr->trCol1; cp++, col++ )
+	{ cp->cpValign= valign;	}
+
+    return;
+    }
+
+
 static APP_BUTTON_CALLBACK_H( tedFormatRevertCellPushed, w, voidct )
     {
-    CellTool *	ct= (CellTool *)voidct;
+    CellTool *			ct= (CellTool *)voidct;
 
     docCopyRowProperties( &(ct->ctRowPropertiesChosen),
-				&(ct->ctRowPropertiesSet), (const int *)0 );
+				&(ct->ctRowPropertiesSet),
+				(const DocumentAttributeMap *)0 );
 
     tedFormatToolRefreshCellPage( ct );
 
@@ -330,7 +350,6 @@ static APP_BUTTON_CALLBACK_H( tedCellPreviousColumn, w, voidct )
     CellTool *		ct= (CellTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -338,7 +357,6 @@ static APP_BUTTON_CALLBACK_H( tedCellPreviousColumn, w, voidct )
 	{ XDEB(ed); return;	}
 
     tr= ct->ctTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     if  ( docShiftTableRectangleByColumns( &tr, -1 ) )
 	{ return;	}
@@ -352,7 +370,6 @@ static APP_BUTTON_CALLBACK_H( tedTableSelectCell, w, voidct )
     CellTool *		ct= (CellTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -360,7 +377,6 @@ static APP_BUTTON_CALLBACK_H( tedTableSelectCell, w, voidct )
 	{ XDEB(ed); return;	}
 
     tr= ct->ctTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     tedAppSetTableSelection( ed, &tr );
     }
@@ -371,7 +387,6 @@ static APP_BUTTON_CALLBACK_H( tedCellNextColumn, w, voidct )
     CellTool *		ct= (CellTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -379,7 +394,6 @@ static APP_BUTTON_CALLBACK_H( tedCellNextColumn, w, voidct )
 	{ XDEB(ed); return;	}
 
     tr= ct->ctTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     if  ( docShiftTableRectangleByColumns( &tr, +1 ) )
 	{ return;	}
@@ -393,6 +407,10 @@ static APP_BUTTON_CALLBACK_H( tedCellNextColumn, w, voidct )
 /*									*/
 /************************************************************************/
 
+/* values are arbitrary but cannot be equal to real CLpropSOMETHING values */
+# define CLprop_SHADE_BACK_COLOR CLprop_COUNT+ 1
+# define CLprop_SHADE_FORE_COLOR CLprop_COUNT+ 2
+
 static void tedCellToolGotColor(	void *			voidct,
 					int 			which,
 					const RGB8Color *	rgb8 )
@@ -417,8 +435,8 @@ static void tedCellToolGotColor(	void *			voidct,
 	    tedBorderSetExplicitColorChoice( &(ct->ctRightBorderTool), rgb8 );
 	    break;
 
-	case CLpropSHADE_BACK_COLOR:
-	case CLpropSHADE_FORE_COLOR:
+	case CLprop_SHADE_BACK_COLOR:
+	case CLprop_SHADE_FORE_COLOR:
 	    tedShadeSetExplicitColorChoice( &(ct->ctShadingTool),
 							    which, rgb8 );
 	    break;
@@ -468,6 +486,11 @@ void tedFormatFillCellPage(	CellTool *			ct,
     appMakeLabelAndTextRow( &row, &label, &(ct->ctColumnText),
 			    pageWidget, cpr->cprColumn, textColumns, 0 );
 
+    /**/
+    appInspectorMakeMenuRow( &row, &(ct->ctValignMenu),
+				&label, pageWidget,
+				cpr->cprValign );
+
     /**/
 
     appMakeLabelAndTextRow( &row, &label, &(ct->ctRowspanText),
@@ -501,7 +524,7 @@ void tedFormatFillCellPage(	CellTool *			ct,
     /**/
     tedFormatMakeShadingTool( &(ct->ctShadingTool), ai, pageWidget,
 		cpr->cprShading, &(cpr->cprShadingResources),
-		subjectPage, CLpropSHADE_FORE_COLOR, CLpropSHADE_BACK_COLOR,
+		subjectPage, CLprop_SHADE_FORE_COLOR, CLprop_SHADE_BACK_COLOR,
 		(TedShadingToolCallback)0, (void *)ct );
 
     /**/
@@ -526,9 +549,14 @@ void tedFormatFillCellPage(	CellTool *			ct,
     return;
     }
 
-void tedFormatFillCellChoosers(		CellTool *			ct,
+void tedCellToolFillChoosers(		CellTool *			ct,
 					const CellPageResources *	cpr )
     {
+    appFillInspectorMenu( DOCtva_COUNT, DOCtvaTOP,
+					    ct->ctValignItems,
+					    cpr->cprValignItemTexts,
+					    &(ct->ctValignMenu),
+					    tedCellValignChosen, (void *)ct );
     return;
     }
 
@@ -536,6 +564,8 @@ void tedFormatFinishCellPage(		CellTool *			ct,
 					TedFormatTool *			tft,
 					const CellPageResources *	cpr )
     {
+    appOptionmenuRefreshWidth( &(ct->ctValignMenu) );
+
     tedFinishBorderTool( &(ct->ctTopBorderTool) );
     tedFinishBorderTool( &(ct->ctBottomBorderTool) );
     tedFinishBorderTool( &(ct->ctLeftBorderTool) );
@@ -561,6 +591,8 @@ void tedInitCellTool(	CellTool *	ct )
     ct->ctRowText= (APP_WIDGET)0;
     ct->ctColumnText= (APP_WIDGET)0;
 
+    appInitOptionmenu( &(ct->ctValignMenu) );
+
     docInitRowProperties( &(ct->ctRowPropertiesSet) );
     docInitRowProperties( &(ct->ctRowPropertiesChosen) );
 
@@ -705,23 +737,41 @@ static AppConfigurableResource TED_TedCellToolResourceTable[]=
 
     /**/
     APP_RESOURCE( "tableToolCellVerticalAlignment",
-	    offsetof(CellPageResources,cprVerticalAlignment),
+	    offsetof(CellPageResources,cprValign),
 	    "Vertical Alignment" ),
+    APP_RESOURCE( "tableToolCellValignTop",
+		offsetof(CellPageResources,cprValignItemTexts[DOCtvaTOP]),
+		"Top" ),
+    APP_RESOURCE( "tableToolCellValignCenter",
+		offsetof(CellPageResources,cprValignItemTexts[DOCtvaCENTERED]),
+		"Center" ),
+    APP_RESOURCE( "tableToolCellValignBottom",
+		offsetof(CellPageResources,cprValignItemTexts[DOCtvaBOTTOM]),
+		"Bottom" ),
     };
 
 void tedFormatToolGetCellResourceTable(	EditApplication *		ea,
 					CellPageResources *		cpr,
 					InspectorSubjectResources *	isr )
     {
-    appGuiGetResourceValues( ea, (void *)cpr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)cpr,
 				TED_TedCellToolResourceTable,
 				sizeof(TED_TedCellToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedCellSubjectResourceTable,
 				sizeof(TED_TedCellSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
