@@ -24,6 +24,8 @@ typedef struct CellPageResources
     char *			cprColspan;
     char *			cprRowspan;
 
+    const char *		cprValignItemTexts[DOCtva_COUNT];
+
     char *			cprWidth;
     char *			cprLeftBorder;
     char *			cprRightBorder;
@@ -31,7 +33,7 @@ typedef struct CellPageResources
     char *			cprTopBorder;
     char *			cprBottomBorder;
     char *			cprShadingPattern;
-    char *			cprVerticalAlignment;
+    char *			cprValign;
 
     char *			cprShading;
     ShadingToolResources	cprShadingResources;
@@ -54,6 +56,10 @@ typedef struct CellTool
 
     APP_WIDGET			ctRowText;
     APP_WIDGET			ctColumnText;
+
+    AppOptionmenu		ctValignMenu;
+    APP_WIDGET			ctValignItems[DOCtva_COUNT];
+
     APP_WIDGET			ctColspanText;
     APP_WIDGET			ctRowspanText;
 
@@ -74,13 +80,16 @@ typedef struct CellTool
 /*									*/
 /************************************************************************/
 
-extern void tedFormatToolRefreshCellTool(
+extern void tedRefreshCellTool(
 				CellTool *			ct,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	bs );
+				const DocumentSelection *	ds,
+				const SelectionDescription *	sd,
+				const BufferDocument *		bd );
 
-extern void tedFormatFillCellChoosers(	CellTool *			ct,
+extern void tedCellToolFillChoosers(	CellTool *			ct,
 					const CellPageResources *	cpr );
 
 extern void tedFormatFillCellPage( CellTool *			ct,
