@@ -11,7 +11,7 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
@@ -32,84 +32,76 @@ static void tedFormatToolRefreshColumnPage(	ColumnTool *	ct )
     const TableRectangle *	tr= &(ct->ctTableRectangle);
 
     int				width;
-    char			scratch[50];
 
-    EditApplication *		ea= ct->ctApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, ct->ctApplication );
 
-    TedDocument *		td;
-    BufferDocument *		bd;
-    DocumentProperties *	dp;
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
 
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
-    dp= &(bd->bdProperties);
-
-    appIntegerToTextWidget( ct->ctNumberText, tr->trCol0+ 1 );
-
-    if  ( tr->trCol0 == 0 )
+    if  ( tr->trCol0 == tr->trCol1 )
 	{
-	width= rp->rpCells[tr->trCol0  ].cpRightBoundaryTwips-
-						    rp->rpLeftIndentTwips;
+	appIntegerToTextWidget( ct->ctNumberText, tr->trCol0+ 1 );
 	}
     else{
-	width= rp->rpCells[tr->trCol0  ].cpRightBoundaryTwips-
-			    rp->rpCells[tr->trCol0-1].cpRightBoundaryTwips;
+	char	scratch[30];
+	sprintf( scratch, "%d - %d", tr->trCol0+ 1, tr->trCol1+ 1 );
+	appStringToTextWidget( ct->ctNumberText, scratch );
 	}
 
-    appGeoLengthToString( scratch, width, UNITtyPOINTS );
-    appStringToTextWidget( ct->ctWidthText, scratch );
-
     cp= rp->rpCells;
 
-    tedBorderToolSetProperties( &(ct->ctLeftBorderTool), dp,
-					&(cp[tr->trCol0].cpLeftBorder) );
-    tedBorderToolSetProperties( &(ct->ctRightBorderTool), dp,
-					&(cp[tr->trCol0].cpRightBorder) );
+    if  ( tr->trCol0 == 0 )
+	{
+	width= cp[tr->trCol0].cpRightBoundaryTwips- rp->rpLeftIndentTwips;
+	}
+    else{
+	width= cp[tr->trCol0].cpRightBoundaryTwips-
+				    cp[tr->trCol0- 1].cpRightBoundaryTwips;
+	}
+
+    appLengthToTextWidget( ct->ctWidthText, width, UNITtyPOINTS );
+
+    tedBorderToolSetPropertiesByNumber( &(ct->ctLeftBorderTool), bd,
+					cp[tr->trCol0].cpLeftBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(ct->ctRightBorderTool), bd,
+					cp[tr->trCol1].cpRightBorderNumber );
     }
 
-void tedFormatToolRefreshColumnTool(
+void tedRefreshColumnTool(
 				ColumnTool *			ct,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	ds )
+				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
+				const SelectionDescription *	sd )
     {
     const RowProperties *	rp;
-    const DocumentGeometry *	dg;
 
     const BufferItem *		rowBi;
-    const BufferItem *		sectBi;
-
     TableRectangle *		tr= &(ct->ctTableRectangle);
 
-    if  ( docGetTableRectangle( tr, ds )	||
-	  tr->trCol1 != tr->trCol0		)
+    if  ( docGetTableRectangle( tr, ds ) )
 	{
 	docInitTableRectangle( tr );
 	*pEnabled= 0; return;
 	}
 
-    rowBi= ds->dsBegin.dpBi;
-    rowBi= rowBi->biParent;
-    rowBi= rowBi->biParent;
-    sectBi= rowBi->biParent;
+    rowBi= docGetRowItem( ds->dsHead.dpBi );
+    if  ( ! rowBi )
+	{ XDEB(rowBi); return;	}
 
-    dg= &(sectBi->biSectDocumentGeometry);
-
-    ct->ctPageRight= dg->dgPageWideTwips-
-			    dg->dgLeftMarginTwips- dg->dgRightMarginTwips;
-    ct->ctPageLeftMargin= dg->dgLeftMarginTwips;
+    ct->ctBlockFrameWide= sg->sgBegin.pgBlockFrameX1-
+						sg->sgBegin.pgBlockFrameX0;
 
     rp= &(rowBi->biRowProperties);
 
-    if  ( docCopyRowProperties( &(ct->ctPropertiesChosen),
-						    rp, (const int *)0 ) )
+    if  ( docCopyRowProperties( &(ct->ctPropertiesChosen), rp,
+				(const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return;	}
-    if  ( docCopyRowProperties( &(ct->ctPropertiesSet),
-						    rp, (const int *)0 ) )
+    if  ( docCopyRowProperties( &(ct->ctPropertiesSet), rp,
+				(const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return;	}
 
     appGuiEnableWidget( is->isPrevButton,
@@ -122,72 +114,134 @@ void tedFormatToolRefreshColumnTool(
 
     appGuiEnableWidget( is->isDeleteButton, rp->rpCellCount > 1 );
 
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
-static int tedFormatToolGetColumnWidth(	const RowProperties *	rp,
-					int			col0,
-					int			pageRight,
-					int *			pValue,
-					int *			pShift,
-					APP_WIDGET		w )
+/************************************************************************/
+/*									*/
+/*  Retrieve the width of the column(s) from the width widget.		*/
+/*									*/
+/*  1)  Determine original width. This is not really necessary, but the	*/
+/*	widget routine expects a value.					*/
+/*  2)  Retrieve width from GUI.					*/
+/*  3)  Decide from what column we can steal space.			*/
+/*  4)  Determine what column width is used by other columns.		*/
+/*  5)  Determine how much we, including our victim need.		*/
+/*  6)  Determine how much column width is availanable.			*/
+/*  7)  If no space is avialable, not even if we rob a viction: fail.	*/
+/*  8)  Rob victim if necessary.					*/
+/*									*/
+/************************************************************************/
+
+static int tedFormatToolGetColumnWidth(	const ColumnTool *	ct,
+					int *			pVictim,
+					int *			pLeftForVictim,
+					int *			pWidth )
     {
+    const RowProperties *	rp= &(ct->ctPropertiesChosen);
+    const TableRectangle *	tr= &(ct->ctTableRectangle);
+    const CellProperties *	cp0= rp->rpCells+ tr->trCol0;
+
     int				width;
     int				changed;
 
-    const CellProperties *	cp= &(rp->rpCells[col0]);
-    int				rightRight;
-
     int				right;
-    int				d;
 
-    const int			minValue= 1;
+    const int			minValue= 20* 5+ 2* rp->rpHalfGapWidthTwips;
     const int			adaptToMin= 0;
     const int			maxValue= INT_MAX;
     const int			adaptToMax= 0;
-    
-    right= cp->cpRightBoundaryTwips;
-    if  ( col0 == 0 )
-	{ width= right- rp->rpLeftIndentTwips;		}
-    else{ width= right- cp[-1].cpRightBoundaryTwips;	}
 
-    if  ( appGetLengthFromTextWidget( w, &width, &changed, UNITtyPOINTS,
+    int				victim= -1;
+    int				usedByOthers;
+    int				neededForUs;
+    int				available;
+    int				leftForVictim= INT_MAX;
+    int				c0= tr->trCol0;
+    int				c1= tr->trCol1;
+
+    /*  1  */
+    right= cp0->cpRightBoundaryTwips;
+    if  ( tr->trCol0 == 0 )
+	{ width= right- rp->rpLeftIndentTwips;		}
+    else{ width= right- cp0[-1].cpRightBoundaryTwips;	}
+
+    /*  2  */
+    if  ( appGetLengthFromTextWidget( ct->ctWidthText,
+				&width, &changed, UNITtyPOINTS,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
 	{ return -1; }
 
-    if  ( col0 == 0 )
-	{ right= rp->rpLeftIndentTwips+ width;		}
-    else{ right= cp[-1].cpRightBoundaryTwips+ width;	}
-
-    d= right- cp->cpRightBoundaryTwips;
-
-    rightRight= rp->rpCells[rp->rpCellCount-1].cpRightBoundaryTwips;
-    rightRight -= rp->rpHalfGapWidthTwips;
-
-    if  ( rightRight+ d >= pageRight )
+    /*  3  */
+    if  ( tr->trCol1 >= rp->rpCellCount- 1 )
 	{
-	char		scratch[50];
+	if  ( tr->trCol0 > 0 )
+	    { c0= victim= tr->trCol0- 1;	}
+	}
+    else{ c1= victim= tr->trCol1+ 1;		}
 
-	d= pageRight- rightRight;
-	right= cp->cpRightBoundaryTwips+ d;
+    /*  4  */
+    usedByOthers= 0;
+    if  ( c0 > 0 )
+	{
+	usedByOthers += rp->rpCells[c0-1].cpRightBoundaryTwips-
+							rp->rpLeftIndentTwips;
+	}
+    if  ( c1 < rp->rpCellCount- 1 )
+	{
+	usedByOthers += rp->rpCells[rp->rpCellCount- 1].cpRightBoundaryTwips-
+					rp->rpCells[c1].cpRightBoundaryTwips;
+	}
 
-	if  ( col0 == 0 )
-	    { width= right- rp->rpLeftIndentTwips;		}
-	else{ width= right- cp[-1].cpRightBoundaryTwips;	}
+    /*  5  */
+    neededForUs= ( tr->trCol1- tr->trCol0+ 1 )* width;
+    if  ( victim >= 0 )
+	{ neededForUs += 20* 5+ 2* rp->rpHalfGapWidthTwips;	}
 
-	width -= 10;
+    if  ( tr->trCol0 == 0 )
+	{ right= rp->rpLeftIndentTwips+ width;		}
+    else{ right= cp0[-1].cpRightBoundaryTwips+ width;	}
 
-	if  ( width <= 0 )
-	    { appRefuseTextValue( w ); return -1;	}
+    /*  6  */
+    available= ct->ctBlockFrameWide-
+			    rp->rpLeftIndentTwips+ rp->rpHalfGapWidthTwips;
+    available -= usedByOthers;
 
-	appGeoLengthToString( scratch, width, UNITtyPOINTS );
-	appStringToTextWidget( w, scratch );
-	appRefuseTextValue( w );
+    /*  7  */
+    if  ( neededForUs > available )
+	{
+	int		availableWidth= available;
+
+	if  ( victim >= 0 )
+	    { availableWidth -= 20* 5+ 2* rp->rpHalfGapWidthTwips;	}
+
+	availableWidth /= ( tr->trCol1- tr->trCol0+ 1 );
+
+	appLengthToTextWidget( ct->ctWidthText, availableWidth, UNITtyPOINTS );
+	appRefuseTextValue( ct->ctWidthText );
 	return 1;
 	}
 
-    *pValue= width; *pShift= d; return 0;
+    /*  8  */
+    if  ( victim >= 0 )
+	{
+	int	lv;
+
+	leftForVictim= available- ( tr->trCol1- tr->trCol0+ 1 )* width;
+
+	if  ( victim == 0 )
+	    { lv= rp->rpLeftIndentTwips;			}
+	else{ lv= rp->rpCells[victim- 1].cpRightBoundaryTwips;	}
+
+	if  ( rp->rpCells[victim].cpRightBoundaryTwips- lv <= leftForVictim )
+	    { victim= -1; leftForVictim= INT_MAX;	}
+	}
+
+    *pWidth= width;
+    *pVictim= victim;
+    *pLeftForVictim= leftForVictim;
+    return 0;
     }
 
 /************************************************************************/
@@ -202,9 +256,8 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeColumnPushed, w, voidct )
     RowProperties *		rp= &(ct->ctPropertiesChosen);
     const TableRectangle *	tr= &(ct->ctTableRectangle);
 
-    int				col;
-
-    int				d;
+    int				victim;
+    int				leftForVictim;
     int				width;
     int				res;
 
@@ -217,71 +270,68 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeColumnPushed, w, voidct )
     PropertyMask		bpSetMask;
 
     /**/
-    EditApplication *		ea= ct->ctApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
-
-    TedDocument *		td;
-    BufferDocument *		bd;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, ct->ctApplication );
     DocumentProperties *	dp;
 
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
     dp= &(bd->bdProperties);
 
     /**/
 
-    PROPmaskCLEAR( &rpSetMask );
-    PROPmaskCLEAR( &cpSetMask );
+    utilPropMaskClear( &rpSetMask );
+    utilPropMaskClear( &cpSetMask );
 
     docInitBorderProperties( &bp );
 
     /****/
-    res= tedFormatToolGetColumnWidth( rp, tr->trCol0,
-			    ct->ctPageRight, &width, &d, ct->ctWidthText );
+    res= tedFormatToolGetColumnWidth( ct, &victim, &leftForVictim, &width );
     if  ( res != 0 )
 	{ goto ready;	}
-    PROPmaskADD( &cpSetMask, CLpropRIGHT_BOUNDARY );
+    PROPmaskADD( &cpSetMask, CLpropCELLX );
 
     /****/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 					    &(ct->ctLeftBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
-	docRowSetLeftBorderInCols( rp, tr->trCol0, tr->trCol1,
-							    &bpSetMask, &bp );
+	docRowSetLeftBorderInCols( rp, tr->trCol0, tr->trCol0,
+						&(bd->bdBorderPropertyList),
+						&bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropLEFT_BORDER );
 	}
 
     /****/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 					    &(ct->ctRightBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
-	docRowSetRightBorderInCols( rp, tr->trCol0, tr->trCol1,
-							    &bpSetMask, &bp );
+	docRowSetRightBorderInCols( rp, tr->trCol1, tr->trCol1,
+						&(bd->bdBorderPropertyList),
+						&bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropRIGHT_BORDER );
 	}
 
-    {
-    CellProperties *		cp;
-
-    cp= &(rp->rpCells[tr->trCol0]);
-    for ( col= tr->trCol0; col < rp->rpCellCount; cp++, col++ )
-	{ cp->cpRightBoundaryTwips += d; }
-    }
+    /****/
+    docRowPropertiesSetWidth( rp, tr->trCol0, tr->trCol1, width,
+						    victim, leftForVictim );
 
+    /****/
     trChange= *tr;
     docExpandTableRectangleToWholeColumns( &trChange );
 
+    if  ( victim >= 0 && trChange.trCol0 > victim )
+	{ trChange.trCol0=  victim;	}
+    if  ( victim >= 0 && trChange.trCol1 < victim )
+	{ trChange.trCol1=  victim;	}
+
     tedAppSetTableProperties( ct->ctApplication, &trChange,
 						&rpSetMask, &cpSetMask, rp );
 
@@ -293,10 +343,10 @@ static APP_BUTTON_CALLBACK_H( tedTableChangeColumnPushed, w, voidct )
 
 static APP_BUTTON_CALLBACK_H( tedFormatRevertColumnPushed, w, voidct )
     {
-    ColumnTool *	ct= (ColumnTool *)voidct;
+    ColumnTool *		ct= (ColumnTool *)voidct;
 
-    docCopyRowProperties( &(ct->ctPropertiesChosen),
-				    &(ct->ctPropertiesSet), (const int *)0 );
+    docCopyRowProperties( &(ct->ctPropertiesChosen), &(ct->ctPropertiesSet),
+					    (const DocumentAttributeMap *)0 );
 
     tedFormatToolRefreshColumnPage( ct );
 
@@ -312,21 +362,12 @@ static APP_BUTTON_CALLBACK_H( tedFormatRevertColumnPushed, w, voidct )
 static APP_TXACTIVATE_CALLBACK_H( tedColumnWidthChanged, w, voidct )
     {
     ColumnTool *		ct= (ColumnTool *)voidct;
-    RowProperties *		rp= &(ct->ctPropertiesChosen);
-    const TableRectangle *	tr= &(ct->ctTableRectangle);
-
-    int				d;
+    int				victim;
+    int				leftForVictim;
     int				width;
 
-    if  ( ! tedFormatToolGetColumnWidth( rp, tr->trCol0,
-			    ct->ctPageRight, &width, &d, ct->ctWidthText ) )
-	{
-	char	scratch[50];
-
-	appGeoLengthToString( scratch, width, UNITtyPOINTS );
-
-	appStringToTextWidget( ct->ctWidthText, scratch );
-	}
+    if  ( ! tedFormatToolGetColumnWidth( ct, &victim, &leftForVictim, &width ) )
+	{ appLengthToTextWidget( ct->ctWidthText, width, UNITtyPOINTS ); }
     }
 
 /************************************************************************/
@@ -340,7 +381,6 @@ static APP_BUTTON_CALLBACK_H( tedColumnPreviousColumn, w, voidct )
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -348,7 +388,6 @@ static APP_BUTTON_CALLBACK_H( tedColumnPreviousColumn, w, voidct )
 	{ XDEB(ed); return;	}
 
     tr= ct->ctTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     docExpandTableRectangleToWholeColumns( &tr );
 
@@ -365,7 +404,6 @@ static APP_BUTTON_CALLBACK_H( tedTableSelectColumn, w, voidct )
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -373,7 +411,6 @@ static APP_BUTTON_CALLBACK_H( tedTableSelectColumn, w, voidct )
 	{ XDEB(ed); return;	}
 
     tr= ct->ctTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     docExpandTableRectangleToWholeColumns( &tr );
 
@@ -387,7 +424,6 @@ static APP_BUTTON_CALLBACK_H( tedColumnNextColumn, w, voidct )
     ColumnTool *	ct= (ColumnTool *)voidct;
     EditApplication *	ea= ct->ctApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -395,7 +431,6 @@ static APP_BUTTON_CALLBACK_H( tedColumnNextColumn, w, voidct )
 	{ XDEB(ed); return;	}
 
     tr= ct->ctTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     docExpandTableRectangleToWholeColumns( &tr );
 
@@ -427,8 +462,6 @@ static APP_BUTTON_CALLBACK_H( tedTableDeleteColumn, w, voidct )
 		tr->trRow00, tr->trRow11, tr->trCol0, tr->trCol1 ) )
 	{ LLDEB(tr->trRow00,tr->trRow11); return;	}
 
-    appDocumentChanged( ed, 1 );
-
     return;
     }
 
@@ -564,14 +597,13 @@ void tedFormatFillColumnPage(	ColumnTool *			ct,
     return;
     }
 
-void tedFormatFillColumnChoosers(	ColumnTool *			ct,
+void tedColumnToolFillChoosers(	ColumnTool *			ct,
 					const ColumnPageResources *	cpr )
     {
     return;
     }
 
 void tedFormatFinishColumnPage(		ColumnTool *			ct,
-					TedFormatTool *			tft,
 					const ColumnPageResources *	cpr )
     {
     tedFinishBorderTool( &(ct->ctLeftBorderTool) );
@@ -592,8 +624,7 @@ void tedInitColumnTool(	ColumnTool *	ct )
 
     docInitTableRectangle( &(ct->ctTableRectangle) );
 
-    ct->ctPageRight= 0;
-    ct->ctPageLeftMargin= 0;
+    ct->ctBlockFrameWide= 0;
 
     docInitRowProperties( &(ct->ctPropertiesSet) );
     docInitRowProperties( &(ct->ctPropertiesChosen) );
@@ -693,15 +724,24 @@ void tedFormatToolGetColumnResourceTable(EditApplication *		ea,
 					ColumnPageResources *		cpr,
 					InspectorSubjectResources *	isr )
     {
-    appGuiGetResourceValues( ea, (void *)cpr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)cpr,
 				TED_TedColumnToolResourceTable,
 				sizeof(TED_TedColumnToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedColumnSubjectResourceTable,
 				sizeof(TED_TedColumnSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
