@@ -12,181 +12,165 @@
 #   include	<string.h>
 #   include	<ctype.h>
 
+#   include	<utilOfficeCharset.h>
 #   include	"tedApp.h"
+#   include	"tedEdit.h"
+#   include	"tedLayout.h"
+#   include	<appMatchFont.h>
+#   include	"docRtfReadWrite.h"
 
 #   include	<sioMemory.h>
 #   include	<sioXprop.h>
+#   include	<sioStdio.h>
 
 #   include	<appDebugon.h>
 
+/************************************************************************/
+/*									*/
+/*  Write copy/paste to file for debugging purposes.			*/
+/*									*/
+/************************************************************************/
+
+static void tedSaveSelectionToFile(
+				BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				int				rtfFlags,
+				const char *			filename )
+    {
+    SimpleOutputStream *	sos;
+
+    sos= sioOutStdioOpen( filename );
+    if  ( ! sos )
+	{ SPDEB( filename, sos );	}
+    else{
+	if  ( docRtfSaveDocument( sos, bd, ds, rtfFlags ) )
+	    { LDEB(1); 	}
+	if  ( sioOutClose( sos ) )
+	    { LDEB(1);	}
+	}
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Edit menu actions.							*/
 /*									*/
-/*  2)  If the beginning of the selection is the beginning of a		*/
-/*	kyperlink, try to include the link.				*/
 /*  8)  Also put a copied picture on the clipboard that 'xv' implements	*/
 /*	on the root window.						*/
 /*									*/
 /************************************************************************/
 
-static void tedFinishTextCopy(	TedDocument *			td,
-				AppDrawingData *		add,
-				const DocumentSelection *	ds )
+static int tedTextCopyClosed(	TedDocument *			td,
+				const LayoutContext *		lc,
+				const DocumentSelection *	ds,
+				const BufferItem *		bodySectBi )
     {
-    BufferDocument *		bd= td->tdDocument;
-    SelectionGeometry		sg;
+    PositionGeometry		pgBegin;
+    PositionGeometry		pgEnd;
+    const int			lastLine= 1;
 
-    tedSelectionGeometry( &sg, ds, bd, add );
+    int				copyClosed= 1;
 
-    td->tdCopiedSelectionClosed= 1;
+    tedPositionGeometry( &pgBegin, &(ds->dsHead), bodySectBi,
+					ds->dsDirection > 0 || lastLine,
+					lc );
+    tedPositionGeometry( &pgEnd, &(ds->dsTail), bodySectBi,
+					ds->dsDirection > 0 || lastLine,
+					lc );
 
-    if  ( (unsigned)ds->dsEnd.dpStroff < ds->dsEnd.dpBi->biParaStrlen )
-	{ td->tdCopiedSelectionClosed= 0;	}
+    if  ( ds->dsTail.dpStroff < docParaStrlen( ds->dsTail.dpBi ) )
+	{ copyClosed= 0;	}
 
     if  ( docIsParaSelection( ds )		&&
-	  sg.sgBegin.pgLine == sg.sgEnd.pgLine	)
-	{ td->tdCopiedSelectionClosed= 0;	}
+	  pgBegin.pgLine == pgEnd.pgLine	)
+	{ copyClosed= 0;	}
 
-    appCleanBitmapImage( &(td->tdCopiedImage) );
-    appInitBitmapImage( &(td->tdCopiedImage) );
-
-    return;
+    return copyClosed;
     }
 
-static int tedDocCopySelection(	EditDocument *	ed )
+static int tedDocSaveSelection(	MemoryBuffer *		mb,
+				int *			pClosed,
+				DocumentSelection *	ds,
+				SelectionDescription *	sd,
+				const LayoutContext *	lc,
+				TedDocument *		td )
     {
-    EditApplication *		ea= ed->edApplication;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
-    AppDrawingData *		add= &(ed->edDrawingData);
-
-    BufferItem *		bi;
-    TextParticule *		tp;
-    DocumentPosition		dpObject;
-    InsertedObject *		io;
-
     SimpleOutputStream *	sos;
 
-    DocumentSelection		dsNew;
-    const int			saveBookmarks= 0;
-
-    int				startPart;
-    int				endPart;
-
-    DocumentSelection		ds;
     SelectionGeometry		sg;
+    const BufferItem *		bodySectBi;
 
-    const char *		fileName= (const char *)0;
-    int				fileSize= 0;
-    const char *		markName= (const char *)0;
-    int				markSize= 0;
+    const int			rtfFlags= RTFflagNO_BOOKMARKS|RTFflagCLOSE_LAST;
 
     if  ( ! tedHasSelection( td ) || tedHasIBarSelection( td ) )
 	{ return -1; }
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( ds, &sg, sd, (DocumentTree **)0, &bodySectBi, td ) )
 	{ return -1;	}
 
-    dsNew= ds;
-
-    /*  2  */
-    if  ( ! docGetHyperlinkForPosition( bd, &(dsNew.dsBegin),
-				&startPart, &endPart,
-				&fileName, &fileSize, &markName, &markSize ) )
-	{
-	bi= dsNew.dsBegin.dpBi;
-	tp= bi->biParaParticules+ dsNew.dsBegin.dpParticule;
-
-	if  ( startPart < dsNew.dsBegin.dpParticule			&&
-	      bi->biParaParticules[startPart].tpStroff == tp->tpStroff	)
-	    { dsNew.dsBegin.dpParticule= startPart;	}
-
-	if  ( dsNew.dsEnd.dpBi == bi )
-	    {
-	    tp= bi->biParaParticules+ dsNew.dsEnd.dpParticule;
-
-	    if  ( endPart > dsNew.dsEnd.dpParticule			&&
-		  bi->biParaParticules[endPart].tpStroff == tp->tpStroff )
-		{ dsNew.dsEnd.dpParticule= endPart;	}
-	    }
-	}
-
-    sos= sioOutMemoryOpen( &(td->tdCopiedSelection) );
+    sos= sioOutMemoryOpen( mb );
     if  ( ! sos )
 	{ XDEB(sos); return -1;    }
 
-    if  ( docRtfSaveDocument( sos, td->tdDocument, &dsNew, saveBookmarks ) )
+    if  ( docRtfSaveDocument( sos, td->tdDocument, ds, rtfFlags ) )
 	{ LDEB(1); sioOutClose( sos ); return -1;	}
 
     if  ( sioOutClose( sos ) )
 	{ LDEB(1); return -1;	}
 
-    tedFinishTextCopy( td, add, &dsNew );
+    if  ( getenv( "TED_SAVE_COPIES" ) )
+	{
+	tedSaveSelectionToFile( td->tdDocument, ds, rtfFlags,
+							"/tmp/copied.rtf" );
+	}
+
+    *pClosed= tedTextCopyClosed( td, lc, ds, bodySectBi );
+    return 0;
+    }
+
+static int tedDocCopySelection(	EditDocument *	ed )
+    {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    DocumentPosition		dpObject;
+    int				partObject;
+    InsertedObject *		io;
+
+    DocumentSelection		ds;
+    SelectionDescription	sd;
+
+    LayoutContext		lc;
+
+    layoutInitContext( &lc );
+    tedSetLayoutContext( &lc, ed );
+
+    if  ( tedDocSaveSelection( &(td->tdCopiedSelection),
+				&(td->tdCopiedSelectionClosed),
+				&ds, &sd, &lc, td ) )
+	{ LDEB(1); return -1;	}
+
+    bmCleanBitmapImage( &(td->tdCopiedImage) );
+    bmInitBitmapImage( &(td->tdCopiedImage) );
 
     docInitDocumentPosition( &dpObject );
 
-    if  ( ea->eaSupportXvCopyPaste				&&
-	  ! tedGetObjectSelection( td, &dpObject, &io )		&&
-	  ! tedSaveObjectPicture( &(td->tdCopiedImage), io )	)
+    if  ( sd.sdIsObjectSelection					&&
+	  ! docGetObjectSelection( &ds, td->tdDocument,
+				      &partObject, &dpObject, &io )	)
 	{
-	/*  8  */
-	if  ( appImgPutXvClipboard( &(td->tdCopiedImage.abiBitmap),
-		td->tdCopiedImage.abiBuffer, ea->eaToplevel.atTopWidget ) )
+	if  ( tedSaveObjectPicture( &(td->tdCopiedImage), io )	)
 	    { LDEB(1);	}
 	}
 
-    if  ( appDocOwnSelection( ed, "PRIMARY",
-		TedPrimaryTextTargets, TedPrimaryTextTargetCount ) )
+    if  ( appDocOwnSelection( ed, "CLIPBOARD",
+					TedClipboardTextTargets,
+					TedClipboardTextTargetCount ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Copy a part of the document, in practice, a bookmark as a link	*/
-/*									*/
-/************************************************************************/
-
-int tedCopyAsLink(	EditDocument *			ed,
-			const DocumentSelection *	ds,
-			int				asRef,
-			int				asPageref,
-			const char *			fileName,
-			const char *			markName,
-			int				markSize )
-    {
-    SimpleOutputStream *	sos;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    AppDrawingData *		add= &(ed->edDrawingData);
-
-    int				fileSize= 0;
-
-    if  ( fileName )
-	{ fileSize= strlen( fileName );	}
-
-    sos= sioOutMemoryOpen( &(td->tdCopiedSelection) );
-    if  ( ! sos )
-	{ XDEB(sos); return -1;    }
-
-    if  ( docRtfSaveSelectionAsLink( sos, td->tdDocument, ds,
-				asRef, asPageref,
-				(const unsigned char *)fileName, fileSize,
-				(const unsigned char *)markName, markSize ) )
-	{ LDEB(1); sioOutClose( sos ); return -1;	}
-
-    if  ( sioOutClose( sos ) )
-	{ LDEB(1); return -1;	}
-
-    tedFinishTextCopy( td, add, ds );
-
-    if  ( appDocOwnSelection( ed, "PRIMARY",
-		    TedPrimaryTextTargets, TedPrimaryTextTargetCount ) )
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
 
 void tedDocCut(			EditDocument *	ed )
     {
@@ -199,8 +183,10 @@ void tedDocCut(			EditDocument *	ed )
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td ) )
 	{ return;	}
 
     if  ( ! docGetTableSliceSelection( &isRowSlice, &isColSlice, &tr, &ds ) )
@@ -213,8 +199,8 @@ void tedDocCut(			EditDocument *	ed )
 	{ LDEB(1); return;	}
 
     if  ( isTableSlice )
-	{ tedDeleteTableSliceSelection( ed );				}
-    else{ tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );	}
+	{ tedDeleteTableSliceSelection( ed );			}
+    else{ tedDocReplaceSelection( ed, (const char *)0, 0 );	}
 
     return;
     }
@@ -225,11 +211,13 @@ void tedDocCopy(		EditDocument *	ed )
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     const int			scrolledX= 0;
     const int			scrolledY= 0;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td ) )
 	{ return;	}
 
     if  ( tedDocCopySelection( ed ) )
@@ -248,7 +236,7 @@ void tedDocCopy(		EditDocument *	ed )
 /*									*/
 /************************************************************************/
 
-void tedPrimaryLost(	APP_WIDGET			w,
+void tedClipboardLost(	APP_WIDGET			w,
 			void *				voided,
 			APP_EVENT *			event )
     {
@@ -257,13 +245,15 @@ void tedPrimaryLost(	APP_WIDGET			w,
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td ) )
 	{ return;	}
 
     td->tdVisibleSelectionCopied= 0;
 
-    if  ( tedHasSelection( td ) && ! tedHasIBarSelection( td ) )
+    if  ( tedHasSelection( td ) && ! sd.sdIsIBarSelection )
 	{
 	const int	scrolledX= 0;
 	const int	scrolledY= 0;
@@ -272,13 +262,25 @@ void tedPrimaryLost(	APP_WIDGET			w,
 	}
     }
 
+void tedPrimaryLost(	APP_WIDGET			w,
+			void *				voided,
+			APP_EVENT *			event )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    td->tdOwnsPrimarySelection= 0;
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Return the selection to a requestor.				*/
 /*									*/
 /************************************************************************/
 
-APP_GIVE_COPY( tedCopyPrimaryRtf, w, event, voided )
+APP_GIVE_COPY( tedCopyClipboardRtf, w, event, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -298,18 +300,58 @@ APP_GIVE_COPY( tedCopyPrimaryRtf, w, event, voided )
 
     sioOutClose( sos );
 
-#   if 0
-    sos= sioOutStdioOpen( "/tmp/copied.rtf" );
-    XDEB(sos);
-    if  ( sos )
+    if  ( getenv( "TED_SAVE_COPIES" ) )
 	{
-	if  ( sioOutWriteBytes( sos, td->tdCopiedSelection.mbBytes,
-				    td->tdCopiedSelection.mbSize ) <= 0 )
-	    { LDEB(td->tdCopiedSelection.mbSize); }
+	sos= sioOutStdioOpen( "/tmp/copied.rtf" );
+	XDEB(sos);
+	if  ( sos )
+	    {
+	    if  ( sioOutWriteBytes( sos, td->tdCopiedSelection.mbBytes,
+					td->tdCopiedSelection.mbSize ) <= 0 )
+		{ LDEB(td->tdCopiedSelection.mbSize); }
 
-	sioOutClose( sos );
+	    sioOutClose( sos );
+	    }
 	}
-#   endif
+
+    return;
+    }
+
+static void tedCopyString(	const MemoryBuffer *	mb,
+				int			closed,
+				APP_WIDGET		w,
+				APP_SELECTION_EVENT *	event,
+				EditDocument *		ed )
+    {
+    EditApplication *		ea= ed->edApplication;
+
+    SimpleOutputStream *	sos;
+
+    BufferDocument *		bd;
+    SimpleInputStream *		sis;
+    int				res;
+
+    if  ( mb->mbSize == 0 )
+	{ LDEB(mb->mbSize); return; }
+
+    sos= sioOutOpenCopy( w, event );
+    if  ( ! sos )
+	{ XDEB(sos); return;	}
+
+    sis= sioInMemoryOpen( mb );
+    if  ( ! sis )
+	{ XDEB(sis); sioOutClose( sos ); return;	}
+
+    bd= docRtfReadFile( sis, &(ea->eaPostScriptFontList) );
+    sioInClose( sis );
+
+    if  ( ! bd )
+	{ XDEB(bd); sioOutClose( sos ); return; }
+
+    res= docPlainSaveDocument( sos, bd, (DocumentSelection *)0, 0, closed );
+
+    docFreeDocument( bd );
+    sioOutClose( sos );
 
     return;
     }
@@ -319,45 +361,38 @@ APP_GIVE_COPY( tedCopyPrimaryString, w, event, voided )
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
 
-    SimpleOutputStream *	sos;
+    DocumentSelection		ds;
+    SelectionDescription	sd;
 
-    BufferDocument *		bd;
-    SimpleInputStream *		sis;
-    int				res;
+    MemoryBuffer		mb;
+    int				closed= 1;
 
-    if  ( td->tdCopiedSelection.mbSize == 0 )
-	{ LDEB(td->tdCopiedSelection.mbSize); return; }
+    LayoutContext		lc;
 
-    sos= sioOutOpenCopy( w, event );
-    if  ( ! sos )
-	{ XDEB(sos); return;	}
+    layoutInitContext( &lc );
+    tedSetLayoutContext( &lc, ed );
 
-    sis= sioInMemoryOpen( &td->tdCopiedSelection );
-    if  ( ! sis )
-	{ XDEB(sis); sioOutClose( sos ); return;	}
+    utilInitMemoryBuffer( &mb );
 
-    bd= docRtfReadFile( sis, td->tdDocument->bdProperties.dpAnsiCodepage );
+    if  ( ! tedDocSaveSelection( &mb, &closed, &ds, &sd, &lc, td ) )
+	{ tedCopyString( &mb, closed, w, event, ed );	}
 
-    sioInClose( sis );
-
-    if  ( ! bd )
-	{ XDEB(bd); sioOutClose( sos ); return; }
-
-    res= docPlainSaveDocument( sos, bd,
-				    (DocumentSelection *)0,
-				    0, td->tdCopiedSelectionClosed );
-
-    docFreeDocument( bd );
-
-    if  ( res )
-	{ LDEB(res); sioOutClose( sos ); return; }
-
-    sioOutClose( sos );
+    utilCleanMemoryBuffer( &mb );
 
     return;
     }
 
-APP_GIVE_COPY( tedCopyPrimaryPng, w, event, voided )
+APP_GIVE_COPY( tedCopyClipboardString, w, event, voided )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    tedCopyString( &(td->tdCopiedSelection),
+				td->tdCopiedSelectionClosed, w, event, ed );
+    return;
+    }
+
+APP_GIVE_COPY( tedCopyClipboardPng, w, event, voided )
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
@@ -380,10 +415,11 @@ APP_GIVE_COPY( tedCopyPrimaryPng, w, event, voided )
     return;
     }
 
-APP_GIVE_COPY( tedCopyPrimaryPixmap, w, event, voided )
+APP_GIVE_COPY( tedCopyClipboardPixmap, w, event, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
-    AppDrawingData *	add= &(ed->edDrawingData);
+    AppDrawingData *	add= &(ed->edDocumentWidget.dwDrawingData);
+    AppColors *		ac= &(ed->edDocumentWidget.dwColors);
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
 
     APP_BITMAP_IMAGE	pixmapCopied;
@@ -391,12 +427,11 @@ APP_GIVE_COPY( tedCopyPrimaryPixmap, w, event, voided )
     int			wide= td->tdCopiedImage.abiBitmap.bdPixelsWide;
     int			high= td->tdCopiedImage.abiBitmap.bdPixelsHigh;
 
-    if  ( appImgMakePixmap( add,
-			    &pixmapCopied, wide, high,
-			    &(ed->edColors), &(td->tdCopiedImage) ) )
+    if  ( appImgMakePixmap( add, &pixmapCopied, wide, high,
+		    ac, &(td->tdCopiedImage), (const DocumentRectangle *)0 ) )
 	{
-	appCleanBitmapImage( &(td->tdCopiedImage) );
-	appInitBitmapImage( &(td->tdCopiedImage) );
+	bmCleanBitmapImage( &(td->tdCopiedImage) );
+	bmInitBitmapImage( &(td->tdCopiedImage) );
 	return;
 	}
 
@@ -428,6 +463,13 @@ APP_GIVE_COPY( tedCopyFontTed, w, event, voided )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Copy the ruler as it was saved on the document struct when the	*/
+/*  menu option was clicked.						*/
+/*									*/
+/************************************************************************/
+
 APP_GIVE_COPY( tedCopyRulerTed, w, event, voided )
     {
     EditDocument *		ed= (EditDocument *)voided;
@@ -459,31 +501,31 @@ APP_GIVE_COPY( tedCopyRulerTed, w, event, voided )
 
 typedef BufferDocument * (*TED_READ) (
 				SimpleInputStream *		sis,
-				int				ansiCodepage,
+				EditApplication *		ea,
 				const DocumentGeometry *	dg );
 
-typedef int	(*TED_INCLUDE) (	APP_WIDGET		w,
-					EditDocument *		ed,
+typedef int	(*TED_INCLUDE) (	EditDocument *		ed,
 					BufferDocument *	bd );
 
 static BufferDocument * tedPasteRtfReadFile(
 				SimpleInputStream *		sis,
-				int				ansiCodepage,
+				EditApplication *		ea,
 				const DocumentGeometry *	dg )
-    { return docRtfReadFile( sis, ansiCodepage );	}
+    {
+    return docRtfReadFile( sis, &(ea->eaPostScriptFontList) );
+    }
 
 static BufferDocument * tedPastePlainReadFile(
 				SimpleInputStream *		sis,
-				int				ansiCodepage,
+				EditApplication *		ea,
 				const DocumentGeometry *	dg )
     {
     int		longestPara;
 
-    return docPlainReadFile( sis, &longestPara,
-					DOCcharsetANSI, ansiCodepage, dg );
+    return docPlainReadFile( sis, &longestPara, dg );
     }
 
-static void tedPastePrimaryGeneric(	APP_WIDGET		w,
+static void tedPasteClipboardGeneric(	APP_WIDGET		w,
 					EditDocument *		ed,
 					APP_SELECTION_EVENT *	event,
 					TED_READ		readDoc,
@@ -503,15 +545,22 @@ static void tedPastePrimaryGeneric(	APP_WIDGET		w,
     if  ( ! sis )
 	{ XDEB(sis); return;	}
 
-    bd= (*readDoc)( sis, tar->tarDefaultAnsicpgInt,
-					&(ea->eaDefaultDocumentGeometry) );
+    bd= (*readDoc)( sis, ea, &(ea->eaDefaultDocumentGeometry) );
 
     sioInClose( sis );
 
     if  ( ! bd )
 	{ XDEB( bd ); return; }
 
-    if  ( (*includeDoc)( w, ed, bd ) )
+    if  ( getenv( "TED_SAVE_COPIES" ) )
+	{
+	const int			rtfFlags= 0;
+	const DocumentSelection * const	ds= (const DocumentSelection *)0;
+
+	tedSaveSelectionToFile( bd, ds, rtfFlags, "/tmp/pasted.rtf" );
+	}
+
+    if  ( (*includeDoc)( ed, bd ) )
 	{ LDEB(1); docFreeDocument( bd ); return;	}
 
     docFreeDocument( bd );
@@ -521,23 +570,31 @@ static void tedPastePrimaryGeneric(	APP_WIDGET		w,
     return;
     }
 
-APP_PASTE_REPLY( tedPastePrimaryRtf, w, event, voided )
+APP_PASTE_REPLY( tedPasteClipboardRtf, w, event, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
 
-    tedPastePrimaryGeneric( w, ed, event,
+    tedPasteClipboardGeneric( w, ed, event,
 			    tedPasteRtfReadFile, tedIncludeRtfDocument );
     }
 
+APP_PASTE_REPLY( tedPasteClipboardString, w, event, voided )
+    {
+    EditDocument *	ed= (EditDocument *)voided;
+
+    tedPasteClipboardGeneric( w, ed, event,
+			    tedPastePlainReadFile, tedIncludePlainDocument );
+    }
+
 APP_PASTE_REPLY( tedPastePrimaryString, w, event, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
 
-    tedPastePrimaryGeneric( w, ed, event,
+    tedPasteClipboardGeneric( w, ed, event,
 			    tedPastePlainReadFile, tedIncludePlainDocument );
     }
 
-APP_PASTE_REPLY( tedPastePrimaryPng, w, event, voided )
+APP_PASTE_REPLY( tedPasteClipboardPng, w, event, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -552,7 +609,7 @@ APP_PASTE_REPLY( tedPastePrimaryPng, w, event, voided )
     abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
     if  ( ! abi )
 	{ XDEB(abi); return;	}
-    appInitBitmapImage( abi );
+    bmInitBitmapImage( abi );
 
     sis= sioInOpenPaste( w, event );
     if  ( ! sis )
@@ -569,7 +626,7 @@ APP_PASTE_REPLY( tedPastePrimaryPng, w, event, voided )
     return;
     }
 
-APP_PASTE_REPLY( tedPastePrimaryPixmap, w, event, voided )
+APP_PASTE_REPLY( tedPasteClipboardPixmap, w, event, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -582,7 +639,7 @@ APP_PASTE_REPLY( tedPastePrimaryPixmap, w, event, voided )
     abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
     if  ( ! abi )
 	{ XDEB(abi); return;	}
-    appInitBitmapImage( abi );
+    bmInitBitmapImage( abi );
 
     if  ( appImgPastePixmap( abi, w, event ) )
 	{ LDEB(1); return;	}
@@ -593,84 +650,175 @@ APP_PASTE_REPLY( tedPastePrimaryPixmap, w, event, voided )
     return;
     }
 
-APP_PASTE_REPLY( tedPasteFontTed, w, event, voided )
+static int tedApplyPastedRuler(		EditDocument *		ed,
+					BufferDocument *	bdFrom )
     {
-    EditDocument *	ed= (EditDocument *)voided;
+    int				rval= 0;
 
-#   define		FONTSIZ	200
+    DocumentPosition		dp;
 
-    SimpleInputStream *		sis;
-
-    char			attrString[FONTSIZ+1];
-    int				l;
-
-    sis= sioInOpenPaste( w, event );
-    if  ( ! sis )
-	{ XDEB(sis); return;	}
-
-    if  ( ! sioInGetString( attrString, FONTSIZ, sis ) )
-	{ LDEB(1); sioInClose( sis ); return; }
-
-    l= strlen( attrString );
-    if  ( l > 0 && attrString[l-1] == '\n' )
-	{ attrString[l-1]= '\0';	}
-
-    if  ( tedAppChangeSelectionAttributeString( ed, attrString ) )
-	{
-	SDEB(attrString);
-	sioInClose( sis );
-	return;
-	}
-
-    sioInClose( sis );
-
-    appDocumentChanged( ed, 1 );
-
-    return;
-    }
-
-APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    SimpleInputStream *		sis;
-
-    ParagraphProperties		ppNew;
     TextAttribute		taSet;
+    ParagraphProperties		ppSetMapped;
 
     PropertyMask		taSetMask;
     PropertyMask		ppUpdMask;
     PropertyMask		spUpdMask;
+    PropertyMask		ppMappedMask;
 
-    PROPmaskCLEAR( &taSetMask );
-    PROPmaskCLEAR( &spUpdMask );
+    TedEditOperation		teo;
+    EditOperation *		eo= &(teo.teoEo);
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+    DocumentCopyJob		dcj;
+
+    docInitDocumentCopyJob( &dcj );
+    docInitParagraphProperties( &ppSetMapped );
+
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
+
+    if  ( docSet2DocumentCopyJob( &dcj, eo, bdFrom, ed->edFilename ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    if  ( docFirstDocumentPosition( &dp, bdFrom ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    utilPropMaskClear( &taSetMask );
+    utilPropMaskClear( &spUpdMask );
+    utilPropMaskClear( &ppMappedMask );
 
-    docInitParagraphProperties( &ppNew );
     utilInitTextAttribute( &taSet );
 
-    sis= sioInOpenPaste( w, event );
-    if  ( ! sis )
-	{ XDEB(sis); return;	}
-
-    if  ( docRtfReadRuler( sis, &ppNew ) )
-	{ LDEB(1); sioInClose( sis ); return; }
-
-    sioInClose( sis );
-
-    PROPmaskCLEAR( &ppUpdMask );
+    utilPropMaskClear( &ppUpdMask );
     PROPmaskADD( &ppUpdMask, PPpropLEFT_INDENT );
     PROPmaskADD( &ppUpdMask, PPpropFIRST_INDENT );
     PROPmaskADD( &ppUpdMask, PPpropRIGHT_INDENT );
     PROPmaskADD( &ppUpdMask, PPpropALIGNMENT );
     PROPmaskADD( &ppUpdMask, PPpropTAB_STOPS );
 
+    PROPmaskADD( &ppUpdMask, PPpropLISTOVERRIDE );
+    PROPmaskADD( &ppUpdMask, PPpropOUTLINELEVEL );
+    PROPmaskADD( &ppUpdMask, PPpropLISTLEVEL );
+
+    if  ( docUpdParaProperties( &ppMappedMask, &ppSetMapped,
+				    &ppUpdMask, &(dp.dpBi->biParaProperties),
+						    &(dcj.dcjAttributeMap ) ) )
+	{ LDEB(1); rval= -1; goto ready; }
+
     if  ( tedChangeSelectionProperties( ed,
 				    &taSetMask, &taSet,
-				    &ppUpdMask, &ppNew,
+				    &ppUpdMask, &ppSetMapped,
 				    &spUpdMask, (SectionProperties *)0 ) )
-	{ LDEB(1); docCleanParagraphProperties( &ppNew ); return; }
+	{ LDEB(1); rval= -1; goto ready; }
 
-    docCleanParagraphProperties( &ppNew );
+  ready:
 
+    docCleanParagraphProperties( &ppSetMapped );
+    docCleanDocumentCopyJob( &dcj );
+
+    return rval;
+    }
+
+APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
+    {
+    EditDocument *	ed= (EditDocument *)voided;
+
+    tedPasteClipboardGeneric( w, ed, event,
+			    tedPasteRtfReadFile, tedApplyPastedRuler );
+    return;
+    }
+
+/************************************************************************/
+
+static int tedApplyPastedFont(		EditDocument *		ed,
+					BufferDocument *	bdFrom )
+    {
+    int				rval= 0;
+
+    DocumentSelection		dsAll;
+
+    TextAttribute		taSetFrom;
+    TextAttribute		taSetTo;
+    ParagraphProperties		ppSet;
+
+    PropertyMask		taSetMask;
+    PropertyMask		ppUpdMask;
+    PropertyMask		spUpdMask;
+    PropertyMask		ppMappedMask;
+
+    TedEditOperation		teo;
+    EditOperation *		eo= &(teo.teoEo);
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+    DocumentCopyJob		dcj;
+
+    docInitDocumentSelection( &dsAll );
+
+    docInitDocumentCopyJob( &dcj );
+    docInitParagraphProperties( &ppSet );
+
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
+
+    if  ( docSet2DocumentCopyJob( &dcj, eo, bdFrom, ed->edFilename ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    if  ( docSelectWholeTree( &dsAll, bdFrom ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    utilPropMaskClear( &ppUpdMask );
+    utilPropMaskClear( &taSetMask );
+    utilPropMaskClear( &spUpdMask );
+    utilPropMaskClear( &ppMappedMask );
+
+    utilInitTextAttribute( &taSetFrom );
+    utilInitTextAttribute( &taSetTo );
+
+    docGetSelectionAttributes( bdFrom, &dsAll, &taSetMask, &taSetFrom );
+    docMapTextAttribute( &taSetTo, &taSetFrom, &dcj );
+
+    {
+    PropertyMask		taOnlyMask;
+
+    utilPropMaskClear( &taOnlyMask );
+
+    PROPmaskADD( &taOnlyMask, TApropDOC_FONT_NUMBER );
+    PROPmaskADD( &taOnlyMask, TApropFONTSIZE );
+    PROPmaskADD( &taOnlyMask, TApropFONTBOLD );
+    PROPmaskADD( &taOnlyMask, TApropFONTSLANTED );
+    PROPmaskADD( &taOnlyMask, TApropTEXTUNDERLINED );
+    PROPmaskADD( &taOnlyMask, TApropSTRIKETHROUGH );
+
+    /* PROPmaskADD( &taOnlyMask, TApropSUPERSUB ); */
+    /* PROPmaskADD( &taOnlyMask, TApropSMALLCAPS ); */
+    /* PROPmaskADD( &taOnlyMask, TApropCAPITALS ); */
+
+    PROPmaskADD( &taOnlyMask, TApropTEXT_COLOR );
+    PROPmaskADD( &taOnlyMask, TApropTEXT_STYLE );
+    PROPmaskADD( &taOnlyMask, TApropBORDER );
+    PROPmaskADD( &taOnlyMask, TApropSHADING );
+
+    utilPropMaskAnd( &taSetMask, &taSetMask, &taOnlyMask );
+    }
+
+    if  ( tedChangeSelectionProperties( ed,
+				    &taSetMask, &taSetTo,
+				    &ppUpdMask, &ppSet,
+				    &spUpdMask, (SectionProperties *)0 ) )
+	{ LDEB(1); rval= -1; goto ready; }
+
+  ready:
+
+    docCleanParagraphProperties( &ppSet );
+    docCleanDocumentCopyJob( &dcj );
+
+    return rval;
+    }
+
+APP_PASTE_REPLY( tedPasteFontTed, w, event, voided )
+    {
+    EditDocument *	ed= (EditDocument *)voided;
+
+    tedPasteClipboardGeneric( w, ed, event,
+			    tedPasteRtfReadFile, tedApplyPastedFont );
     return;
     }
 
@@ -678,55 +826,147 @@ APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
 /*									*/
 /*  Custom 'Paste' functionality.					*/
 /*									*/
-/*  1)  The regular X11 manner.						*/
-/*  2)  Try to get a possible 'paste' from xv. This is just to make it	*/
-/*	possible to cooperate with John Bradley's XV. As XV is a little	*/
-/*	outdated, only support this if the appropriate resource is set.	*/
-/*									*/
 /************************************************************************/
 
-void tedDocEditPaste(		APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs )
+APP_MENU_CALLBACK_H( tedDocEditPaste, option, voided, e )
     {
     EditDocument *			ed= (EditDocument *)voided;
-    EditApplication *			ea= ed->edApplication;
     TedDocument *			td= (TedDocument *)ed->edPrivateData;
 
-    int					res;
-    BitmapDescription			bd;
-    unsigned char *			buffer;
-
     if  ( ! td->tdCanReplaceSelection )
 	{ LDEB(td->tdCanReplaceSelection); return;	}
 
     /*  1  */
-    res= appDocAskForPaste( ed, "PRIMARY" );
-    if  ( ! res )
-	{ return;	}
+    appDocAskForPaste( ed, "CLIPBOARD" );
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  'Copy Font' menu option.						*/
+/*									*/
+/************************************************************************/
+
+APP_MENU_CALLBACK_H( tedDocFontCopy, fontsOption, voided, e )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    SimpleOutputStream *	sos;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    const int			rtfFlags= RTFflagNO_BOOKMARKS|RTFflagCLOSE_LAST;
+
+    if  ( tedGetSelection( &ds, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td ) )
+	{ LDEB(1); return;	}
 
     /*  2  */
-    if  ( ea->eaSupportXvCopyPaste					&&
-	  ! appImgGetXvClipboard( &bd, &buffer,
-					ea->eaToplevel.atTopWidget )	)
+    sos= sioOutMemoryOpen( &(td->tdCopiedFont) );
+    if  ( ! sos )
+	{ XDEB(sos); return;	}
+
+    if  ( docRtfSaveDocument( sos, td->tdDocument, &ds, rtfFlags ) )
+	{ LDEB(1); sioOutClose( sos ); return;	}
+
+    if  ( sioOutClose( sos ) )
+	{ LDEB(1); return;	}
+
+    if  ( getenv( "TED_SAVE_COPIES" ) )
 	{
-	AppBitmapImage *	abi;
+	tedSaveSelectionToFile( td->tdDocument, &ds,
+					    rtfFlags, "/tmp/font.rtf" );
+	}
 
-	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
-	if  ( ! abi )
-	    { XDEB(abi); return;	}
-	appInitBitmapImage( abi );
+    appDocOwnSelection( ed, "RTFFONT", TedFontTargets, TedFontTargetCount );
+    }
 
-	if  ( bmCopyDescription( &(abi->abiBitmap), &bd ) )
-	    { bmCleanDescription( &bd ); return; }
+/************************************************************************/
+/*									*/
+/*  Callbacks for the Copy/Paste ruler menu options.			*/
+/*									*/
+/*  1)  The ruler paste is a selection paste with the selection		*/
+/*	constrained to the last position of the first paragraph in the	*/
+/*	selection.							*/
+/*  2)  Remember the bytes expecting that someone will ask for them.	*/
+/*  3)  Tell that we have a ruler paste available.			*/
+/*									*/
+/************************************************************************/
 
-	bmCleanDescription( &bd );
-	abi->abiBuffer= buffer;
+APP_MENU_CALLBACK_H( tedDocFormatCopyRul, fontsOption, voided, e )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    SimpleOutputStream *	sos;
 
-	if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
-	    { LDEB(1); return; }
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-	return;
+    DocumentSelection		dsPara;
+
+    const int			rtfFlags= RTFflagNO_BOOKMARKS|RTFflagCLOSE_LAST;
+
+    if  ( tedGetSelection( &ds, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td ) )
+	{ LDEB(1); return;	}
+
+    /*  1  */
+    dsPara= ds;
+    docLastPosition( &(dsPara.dsHead), dsPara.dsHead.dpBi );
+    docSetIBarSelection( &dsPara, &(dsPara.dsHead) );
+
+    /*  2  */
+    sos= sioOutMemoryOpen( &(td->tdCopiedRuler) );
+    if  ( ! sos )
+	{ XDEB(sos); return;	}
+
+    if  ( docRtfSaveDocument( sos, td->tdDocument, &dsPara, rtfFlags ) )
+	{ LDEB(1); sioOutClose( sos ); return;	}
+
+    if  ( sioOutClose( sos ) )
+	{ LDEB(1); return;	}
+
+    if  ( getenv( "TED_SAVE_COPIES" ) )
+	{
+	tedSaveSelectionToFile( td->tdDocument, &dsPara,
+						rtfFlags, "/tmp/ruler.rtf" );
+	}
+
+    /*  2  */
+    appDocOwnSelection( ed, "RTFRULER", TedRulerTargets, TedRulerTargetCount );
+    }
+
+APP_MENU_CALLBACK_H( tedDocFormatPasteRul, fontsOption, voided, e )
+    {
+    EditDocument *			ed= (EditDocument *)voided;
+
+    appDocAskForPaste( ed, "RTFRULER" );
+    }
+
+/************************************************************************/
+
+void tedManagePrimarySelection(	EditDocument *		ed )
+    {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    if  ( ! tedHasIBarSelection( td ) )
+	{
+	if  ( appDocOwnSelection( ed, "PRIMARY",
+					    TedPrimaryTargets,
+					    TedPrimaryTargetCount ) )
+	    { LDEB(1);				}
+	else{ td->tdOwnsPrimarySelection= 1;	}
+	}
+    else{
+	if  ( td->tdOwnsPrimarySelection )
+	    {
+	    appDocReleaseSelection( ed, "PRIMARY" );
+	    td->tdOwnsPrimarySelection= 0;
+	    }
 	}
 
     return;
