@@ -7,15 +7,25 @@
 #   include	<ctype.h>
 
 #   include	<sioStdio.h>
+#   include	<sioStdin.h>
 #   include	<appSystem.h>
+#   include	<appMatchFont.h>
+#   include	<utilOfficeCharset.h>
 
 #   include	<appSpellTool.h>
 
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
+#   include	"tedLayout.h"
 
-#   include	"docLayout.h"
+#   include	"docPsPrint.h"
+#   include	"docScreenLayout.h"
+#   include	"docEvalField.h"
 #   include	"docEdit.h"
+#   include	"docFind.h"
+#   include	"docRtfReadWrite.h"
+
+#   include	<appPrintJob.h>
 
 #   include	<appDebugon.h>
 
@@ -23,25 +33,7 @@
 #	include	<X11/cursorfont.h>
 #   endif
 
-/************************************************************************/
-/*									*/
-/*  Ted, callbacks for the DrawingArea.					*/
-/*									*/
-/************************************************************************/
-
-void tedDrawRectangle(	APP_WIDGET		w,
-			EditDocument *		ed,
-			DocumentRectangle *	drClip,
-			int			ox,
-			int			oy )
-    {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    AppDrawingData *		add= &(ed->edDrawingData);
-    AppColors *			ac= &(ed->edColors);
-
-    /*  2,3,4  */
-    tedRedrawRectangle( w, td, drClip, add, ac, ox, oy );
-    }
+#   define VALIDATE_TREE	0
 
 /************************************************************************/
 /*									*/
@@ -51,51 +43,53 @@ void tedDrawRectangle(	APP_WIDGET		w,
 
 void tedMoveObjectWindows(	EditDocument *		ed )
     {
-    AppDrawingData *		add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
 
     InsertedObject *		io;
 
-    int				ox= ed->edVisibleRect.drX0;
-    int				oy= ed->edVisibleRect.drY0;
-
-    DocumentPosition		dp;
+    DocumentPosition		dpObject;
     PositionGeometry		pg;
+    int				part;
+    const int			lastOne= 1;
 
-    docInitDocumentPosition( &dp );
+    const BufferItem *		bodySectBi= (const BufferItem *)0;
 
-    if  ( tedGetObjectSelection( td, &dp, &io ) )
+    LayoutContext		lc;
+
+    layoutInitContext( &lc );
+    tedSetLayoutContext( &lc, ed );
+
+    docInitDocumentPosition( &dpObject );
+
+    if  ( tedGetObjectSelection( td, &bodySectBi, &part, &dpObject, &io ) )
 	{ LDEB(1); return;	}
 
-    tedPositionGeometry( &pg, &dp, bd, add );
+    tedPositionGeometry( &pg, &dpObject, bodySectBi, lastOne, &lc );
 
-    tedSetObjectWindows( ed, &pg, io, ox, oy );
+    tedSetObjectWindows( ed, &pg, io, &lc );
     }
 
-void tedDocHorizontalScrollbarCallback(	APP_WIDGET	w,
-					void *		voided,
-					void *		voidscbs )
+APP_SCROLLBAR_CALLBACK_H( tedDocHorizontalScrollbarCallback, w, voided, e )
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    int				x= appGuiGetScrollbarValueFromCallback( w, e );
 
-    appDocHorizontalScrollbarCallback( w, voided, voidscbs );
+    appDocScrollToX( ed, x );
 
-    if  ( td->tdObjectSelected )
+    if  ( td->tdSelectionDescription.sdIsObjectSelection )
 	{ tedMoveObjectWindows( ed );	}
     }
 
-void tedDocVerticalScrollbarCallback(	APP_WIDGET	w,
-					void *		voided,
-					void *		voidscbs )
+APP_SCROLLBAR_CALLBACK_H( tedDocVerticalScrollbarCallback, w, voided, e )
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    int				y= appGuiGetScrollbarValueFromCallback( w, e );
 
-    appDocVerticalScrollbarCallback( w, voided, voidscbs );
+    appDocScrollToY( ed, y );
 
-    if  ( td->tdObjectSelected )
+    if  ( td->tdSelectionDescription.sdIsObjectSelection )
 	{ tedMoveObjectWindows( ed );	}
     }
 
@@ -105,103 +99,131 @@ void tedDocVerticalScrollbarCallback(	APP_WIDGET	w,
 /*									*/
 /*  1)  Set the position by hand, because the regular routine sends an	*/
 /*	expose to a window that does not exist yet.			*/
+/*  2)  Forget the name of document templates to force a save-as.	*/
+/*	Also forget it is a template, to make sure that it is saved as	*/
+/*	an ordinary document.						*/
 /*									*/
 /************************************************************************/
 
-int tedFinishDocumentSetup(	EditApplication *	ea,
-				EditDocument *		ed )
+int tedFinishDocumentSetup(	EditDocument *		ed )
     {
-    AppDrawingData *		add= &(ed->edDrawingData);
-    AppColors *			ac= &(ed->edColors);
+    EditApplication *		ea= ed->edApplication;
+    DocumentWidget *		dw= &(ed->edDocumentWidget);
+    AppDrawingData *		add= &(dw->dwDrawingData);
 
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
+    DocumentProperties *	dp= &(bd->bdProperties);
 
     const TedAppResources *	tar= (TedAppResources *)ea->eaResourceData;
     const char *		selColorName= tar->tarSelectionColor;
     const char *		xselColorName= tar->tarCopiedSelectionColor;
+    int				pixelsPerM;
+    double			pixelsPerTwip;
+    double			xfac;
     /*
     char *			tableColorName= "gray80";
     */
 
+    AffineTransform2D		at2;
+    LayoutContext		lc;
+
+    layoutInitContext( &lc );
+    tedSetLayoutContext( &lc, ed );
+
+    pixelsPerM= guiScreenWidgetPixelsPerMeter( ed->edScreenWidget );
+    pixelsPerTwip= pixelsPerM/ TWIPS_PER_M;
+    xfac= ea->eaMagnification* pixelsPerTwip;
+    geoScaleAffineTransform2D( &at2, xfac, xfac );
+
+    drawSetTransform( ed->edDrawingSurface, &at2 );
+
+    td->tdPageGapTwips= ( TWIPS_PER_M * ea->eaPageGapMM )/ 1000;
+
     {
-    DocumentPosition	dp;
+    DocumentPosition		dpFirst;
+    const BufferItem *		bodySectBi= (const BufferItem *)0;
 
     /*  1  */
-    if  ( ! docFirstDocumentPosition( &dp, bd ) )
+    if  ( ! docFirstDocumentPosition( &dpFirst, bd ) )
 	{
-	const TextParticule *		tp;
+	const int			lastLine= 0;
+	BufferItem *			bodyBi= bd->bdBody.eiRoot;
 
-	docSetIBarSelection( &(td->tdDocumentSelection), &dp );
+	if  ( tar->tarFindPattern					&&
+	      ! docFindSetPattern( &(td->tdFindProg),
+			    tar->tarFindPattern, tar->tarFindRegex )	&&
+	      ! docFindFindNextInDocument( &(td->tdDocumentSelection),
+		    &dpFirst, bd,
+		    docFindParaFindNext, td->tdFindProg )	)
+	    { bodySectBi= docGetBodySectBi( dpFirst.dpBi, bd );	}
+	else{
+	    docAvoidParaHeadField( &dpFirst, (int *)0, bd );
+
+	    docSetIBarSelection( &(td->tdDocumentSelection), &dpFirst );
+	    bodySectBi= bodyBi->biChildren[0];
+	    }
 
 	tedSelectionGeometry( &(td->tdSelectionGeometry),
-				     &(td->tdDocumentSelection), bd, add );
+			     &(td->tdDocumentSelection), bodySectBi, lastLine,
+			     &lc );
 
-	tp= dp.dpBi->biParaParticules+ dp.dpParticule;
-
-	utilGetTextAttributeByNumber( &(td->tdCurrentTextAttribute),
-						&(bd->bdTextAttributeList),
-						tp->tpTextAttributeNumber );
-	td->tdCurrentTextAttributeNumber= tp->tpTextAttributeNumber;
-
-	td->tdCurrentPhysicalFont= tp->tpPhysicalFont;
+	tedDescribeSelection( ed );
 	}
-    else{ docListItem( 0, &(bd->bdItem) );	}
+    else{ docListItem( 0, bd->bdBody.eiRoot, 0 );	}
 
-    /*
+    /*  2  */
+    if  ( dp->dpIsDocumentTemplate )
+	{
+	appSetDocumentFilename( ed, (const char *)0 );
+	dp->dpIsDocumentTemplate= 0;
+	}
+
+#   if VALIDATE_TREE
     LDEB(1);
-    if  ( docCheckItem( &(bd->bdItem) ) )
-	{ LDEB(2); docListItem( 0, &(bd->bdItem) ); abort();	}
-    */
+    if  ( docCheckItem( bodyBi ) )
+	{ LDEB(2); docListItem( 0, bodyBi ); abort();	}
+#   endif
 
     }
 
-    if  ( appFinishDocumentSetup( ea, ed ) )
+    if  ( appFinishDocumentSetup( ed ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( appAllocateColors( add, ac ) )
-	{ LDEB(1); return -1;	}
-
-    if  ( ed->edColors.acAllocator.caDepth < 4 )
+    if  ( dw->dwColors.acAllocator.caDepth < 4 )
 	{
-	if  ( appColorRgb( &(td->tdFieldColor), &(ed->edColors), 0, 0, 0 ) )
+	td->tdDrawMonochrome= 1;
+
+	if  ( appColorRgb( &(td->tdFieldColor), &(dw->dwColors), 0, 0, 0 ) )
 	    { LDEB(1); return -1;	}
 
-	td->tdSelColor.rgb8Red= 0;
-	td->tdSelColor.rgb8Green= 0;
-	td->tdSelColor.rgb8Blue= 0;
-
-	td->tdCopiedSelColor.rgb8Red= 0;
-	td->tdCopiedSelColor.rgb8Green= 0;
-	td->tdCopiedSelColor.rgb8Blue= 0;
-
-	td->tdTableColor.rgb8Red= 0;
-	td->tdTableColor.rgb8Green= 0;
-	td->tdTableColor.rgb8Blue= 0;
+	bmRGB8SolidBlack( &(td->tdSelColor) );
+	bmRGB8SolidBlack( &(td->tdCopiedSelColor) );
+	bmRGB8SolidBlack( &(td->tdTableColor) );
 	}
     else{
 	APP_COLOR_RGB	acSel;
 	APP_COLOR_RGB	acCopiedSel;
 
-	if  ( appColorNamed( &acSel, &ed->edColors, selColorName ) )
+	td->tdDrawMonochrome= 0;
+
+	if  ( appColorNamed( &acSel, &(dw->dwColors), selColorName ) )
 	    {
-	    if  ( appColorFindRgb( &acSel,
-					   &(ed->edColors), 176, 196, 222 ) )
+	    if  ( appColorFindRgb( &acSel, &(dw->dwColors), 176, 196, 222 ) )
 		{ SDEB(selColorName); return -1;	}
 	    }
 
-	if  ( appColorNamed( &acCopiedSel,
-					    &(ed->edColors), xselColorName ) )
+	if  ( appColorNamed( &acCopiedSel, &(dw->dwColors), xselColorName ) )
 	    {
 	    if  ( appColorFindRgb( &acCopiedSel,
-					    &(ed->edColors), 176, 176, 176 ) )
+					    &(dw->dwColors), 176, 176, 176 ) )
 		{ SDEB(xselColorName); return -1;	}
 	    }
 
-	if  ( appColorRgb( &(td->tdFieldColor), &ed->edColors, 0, 0, 200 ) )
+	if  ( appColorRgb( &(td->tdFieldColor), &(dw->dwColors), 0, 0, 200 ) )
 	    {
 	    if  ( appColorFindRgb( &(td->tdFieldColor),
-					       &(ed->edColors), 0, 0, 255 ) )
+					       &(dw->dwColors), 0, 0, 255 ) )
 		{ LDEB(1); return -1;	}
 	    }
 
@@ -234,8 +256,7 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
 #	endif
 	}
 
-    if  ( tedOpenItemObjects( &(bd->bdItem), &(ed->edColors),
-						    &(ed->edDrawingData) ) )
+    if  ( tedOpenExternalItemObjects( &(bd->bdBody), &lc ) )
 	{ LDEB(1);	}
 
     td->tdDrawTableGrid= tar->tarShowTableGridInt;
@@ -252,6 +273,41 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Scroll to the selection the first time the document becomes visible	*/
+/*									*/
+/************************************************************************/
+
+void tedDocumentFirstVisible(	EditDocument *	ed )
+    {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    DocumentRectangle		dr= ed->edVisibleRect;
+
+    int				pScrolledX= 0;
+    int				pScrolledY= 0;
+
+    if  ( td->tdSelectionGeometry.sgRectangle.drX1 > dr.drX1 )
+	{
+	int sh= td->tdSelectionGeometry.sgRectangle.drX1- dr.drX1;
+
+	dr.drX0 += sh;
+	dr.drX1 += sh;
+	}
+    if  ( td->tdSelectionGeometry.sgRectangle.drY1 > dr.drY1 )
+	{
+	int sh= td->tdSelectionGeometry.sgRectangle.drY1- dr.drY1;
+
+	dr.drY0 += sh;
+	dr.drY1 += sh;
+	}
+
+    appScrollToRectangle( ed, dr.drX0, dr.drY0, dr.drX1, dr.drY1,
+						    &pScrolledX, &pScrolledY );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Make the document widget and the rulers.				*/
@@ -267,8 +323,7 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
 /************************************************************************/
 
 void tedFreeDocument(		void *			voidtd,
-				int			format,
-				AppDrawingData *	add )
+				int			format )
     {
     TedDocument *	td= (TedDocument *)voidtd;
 
@@ -276,22 +331,19 @@ void tedFreeDocument(		void *			voidtd,
 
     if  ( td->tdDocument )
 	{
-	BufferDocument *	bd= td->tdDocument;
-	int			noteCount= 0;
-	int			paragraphCount= 0;
-
-	docCloseItemObjects( &noteCount, &paragraphCount,
-					    bd, &(bd->bdItem),
-					    (void *)add, tedCloseObject );
-
-	docFreeDocument( bd );
+	docCleanDocumentObjects( td->tdDocument, docScreenCloseObject );
+	docFreeDocument( td->tdDocument );
 	}
 
+    utilCleanIndexMapping( &(td->tdAttributeToScreenFont) );
+
     utilCleanMemoryBuffer( &(td->tdCopiedSelection) );
     utilCleanMemoryBuffer( &(td->tdCopiedFont) );
     utilCleanMemoryBuffer( &(td->tdCopiedRuler) );
 
-    appCleanBitmapImage( &(td->tdCopiedImage) );
+    bmCleanBitmapImage( &(td->tdCopiedImage) );
+
+    docFindSetPattern( &(td->tdFindProg), (const char *)0, 0 );
 
     free( td );
     }
@@ -322,53 +374,27 @@ int tedMakeDocumentWidget(	EditApplication *	ea,
     }
 
 /************************************************************************/
-/*									*/
-/*  Find out whether a document uses just one code page.		*/
-/*									*/
-/************************************************************************/
 
-static int tedDetermineCodepage(	BufferDocument *	bd,
-					const AppDrawingData *	add )
+static int tedSetGenerator(		DocumentProperties *	dp,
+					const EditApplication *	ea )
     {
-    PostScriptFaceList		psfl;
-    PostScriptFace *		psf;
-    int				i;
+    int		l= 0;
 
-    int				charset= -1;
-    int				charsetCount= 0;
+    l += strlen( ea->eaNameAndVersion );
+    l += 2;
+    l += strlen( ea->eaReference );
+    l += 2;
+    l += 1;
 
-    utilInitPostScriptFaceList( &psfl );
-
-    if  ( docPsPrintGetDocumentFonts( bd, &psfl, &(add->addPhysicalFontList) ) )
-	{ LDEB(1); return -1;	}
-
-    psf= psfl.psflFaces;
-    for ( i= 0; i < psfl.psflFaceCount; psf++, i++ )
-	{
-	if  ( charset < 0 )
-	    { charset= psf->psfEncodingUsed; charsetCount= 1;	}
-	else{
-	    if  ( charset != psf->psfEncodingUsed )
-		{ charsetCount++; }
-	    }
-	}
-
-    utilCleanPostScriptFaceList( &psfl );
-
-    if  ( charsetCount != 1 )
-	{ return -1;	}
-
-    switch( charset )
-	{
-	case ENCODINGpsISO_8859_1:
-	    return 1252;
-
-	case ENCODINGpsISO_8859_2:
-	    return 1250;
-
-	default:
-	    LDEB(charset); return -1;
+    dp->dpGeneratorWrite= malloc( l );
+    if  ( ! dp->dpGeneratorWrite )
+	{ LXDEB(l,dp->dpGeneratorWrite); return -1;	}
+    else{
+	sprintf( (char *)dp->dpGeneratorWrite, "%s (%s);",
+				ea->eaNameAndVersion, ea->eaReference );
 	}
+    
+    return 0;
     }
 
 /************************************************************************/
@@ -384,9 +410,6 @@ int tedNewDocument(	EditApplication *	ea,
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd;
     DocumentProperties *	dp;
-
-    PropertyMask		newMask;
-    ExpandedTextAttribute	etaNew;
     TextAttribute		ta;
 
     time_t			now;
@@ -394,26 +417,18 @@ int tedNewDocument(	EditApplication *	ea,
     TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
     int				rval= 0;
 
+    const PostScriptFontList *	psfl= (const PostScriptFontList *)0;
+
     tedDetermineDefaultSettings( tar );
 
-    PROPmaskCLEAR( &newMask );
-    docInitExpandedTextAttribute( &etaNew );
+    /*  2  */
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory); return -1;	}
 
-    if  ( docExpandedAttributeFromString( ea->eaDefaultFont,
-							&newMask, &etaNew ) )
-	{ SDEB(ea->eaDefaultFont); rval= -1; goto ready;	}
+    if  ( ! ea->eaPreferBase35Fonts )
+	{ psfl= &(ea->eaPostScriptFontList);	}
 
-    if  ( ! PROPmaskISSET( &newMask, TApropFONTSIZE )	||
-	  etaNew.etaFontSizeHalfPoints < 6		)
-	{
-	LDEB(etaNew.etaFontSizeHalfPoints);
-	etaNew.etaFontSizeHalfPoints= 24;
-	PROPmaskADD( &newMask, TApropFONTSIZE );
-	}
-
-    bd= docNewFile( &ta, etaNew.etaFontFamilyName,
-				    etaNew.etaFontSizeHalfPoints,
-				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+    bd= docNewFile( &ta, "Helvetica", 2* 12, psfl,
 				    &(ea->eaDefaultDocumentGeometry) );
     if  ( ! bd )
 	{ XDEB(bd); rval= -1; goto ready;	}
@@ -422,6 +437,8 @@ int tedNewDocument(	EditApplication *	ea,
     td->tdDocument= bd;
     dp= &(bd->bdProperties);
 
+    tedSetGenerator( dp, ea );
+
     if  ( filename )
 	{
 	const char *	ext= appFileExtensionOfName( filename );
@@ -433,15 +450,13 @@ int tedNewDocument(	EditApplication *	ea,
 	}
 
     if  ( ea->eaAuthor )
-	{ dp->dpAuthor= (unsigned char *)strdup( ea->eaAuthor ); }
+	{ dp->dpAuthor= strdup( ea->eaAuthor ); }
 
     now= time( (time_t *)0 );
     dp->dpCreatim= *localtime( &now );
 
   ready:
 
-    docCleanExpandedTextAttribute( &etaNew );
-
     return rval;
     }
 
@@ -456,6 +471,16 @@ int tedNewDocument(	EditApplication *	ea,
 /*									*/
 /************************************************************************/
 
+static void tedSetLayoutContext2(	LayoutContext *		lc,
+					AppDrawingData *	add,
+					TedDocument *		td )
+    {
+    lc->lcAdd= add;
+    lc->lcPostScriptFontList= add->addPostScriptFontList;
+    lc->lcDocument= td->tdDocument;
+    lc->lcAttributeToScreenFont= &(td->tdAttributeToScreenFont);
+    }
+
 int tedLayoutDocument(		void *				privateData,
 				int				format,
 				AppDrawingData *		add,
@@ -466,24 +491,33 @@ int tedLayoutDocument(		void *				privateData,
     const DocumentProperties *	dp= &(bd->bdProperties);
     const DocumentGeometry *	dgDoc= &(dp->dpGeometry);
 
+    const int			page= 0;
     int				noteNumbersChanged= 0;
 
     RecalculateFields		rf;
 
-    docInitRecalculateFields( &rf );
+    LayoutContext		lc;
 
-    rf.rfBd= bd;
-    rf.rfVoidadd= (void *)add;
-    rf.rfCloseObject= tedCloseObject;
-    rf.rfUpdateFlags= FIELDdoDOC_INFO|FIELDdoDOC_COMPLETE|FIELDdoCHFTN;
-    rf.rfFieldsUpdated= 0;
+    layoutInitContext( &lc );
+    tedSetLayoutContext2( &lc, add, td );
 
     tedScreenRectangles( add, dgDoc );
 
-    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
+    docInitRecalculateFields( &rf );
+
+    rf.rfBd= bd;
+    rf.rfCloseObject= docScreenCloseObject;
+    rf.rfUpdateFlags= FIELDdoDOC_INFO|FIELDdoDOC_COMPLETE|FIELDdoCHFTN;
+    rf.rfFieldsUpdated= 0;
+
+    if  ( docRecalculateTocFields( &rf ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( tedLayoutDocumentTree( td, add ) )
+    if  ( docRecalculateTextLevelFieldsInExternalItem( &rf, &(bd->bdBody),
+				    bd->bdBody.eiRoot->biChildren[0], page ) )
+	{ LDEB(1); return -1;	}
+
+    if  ( tedLayoutDocumentTree( td, &lc ) )
 	{ LDEB(1); return -1;	}
 
     docRenumberNotes( &noteNumbersChanged, bd );
@@ -494,21 +528,14 @@ int tedLayoutDocument(		void *				privateData,
     if  ( noteNumbersChanged )
 	{ rf.rfUpdateFlags |= FIELDdoCHFTN;	}
 
-    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
+    if  ( docRecalculateTextLevelFieldsInExternalItem( &rf, &(bd->bdBody),
+				    bd->bdBody.eiRoot->biChildren[0], page ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( rf.rfFieldsUpdated > 0 && tedLayoutDocumentTree( td, add ) )
+    if  ( rf.rfFieldsUpdated > 0 && tedLayoutDocumentTree( td, &lc ) )
 	{ LDEB(1); return -1;	}
 
-    /*  8  */
-    if  ( bd->bdProperties.dpAnsiCodepage < 0 )
-	{
-	bd->bdProperties.dpAnsiCodepage= tedDetermineCodepage( bd, add );
-
-	/*  9  */
-	if  ( bd->bdProperties.dpAnsiCodepage == 1252 )
-	    { bd->bdProperties.dpAnsiCodepage= -1;	}
-	}
+    /* LDEB(1); docListItem(0,bd->bdBody.eiRoot,1); */
 
     return 0;
     }
@@ -530,8 +557,6 @@ int tedSaveDocument(	const void *		privateData,
     DocumentProperties *	dp= &(bd->bdProperties);
     SimpleOutputStream *	sos;
 
-    const int			saveBookmarks= 1;
-
     const int			asMimeAggr= 0;
 
     time_t			now;
@@ -546,8 +571,8 @@ int tedSaveDocument(	const void *		privateData,
 	    now= time( (time_t *)0 );
 	    dp->dpRevtim= *localtime( &now );
 
-	    if  ( docRtfSaveDocument( sos, bd, (const DocumentSelection *)0,
-							    saveBookmarks ) )
+	    if  ( docRtfSaveDocument( sos, bd,
+					(const DocumentSelection *)0, 0 ) )
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
@@ -555,13 +580,6 @@ int tedSaveDocument(	const void *		privateData,
 	    LDEB(format);
 	    /*FALLTHROUGH*/
 
-	case TEDdockindTEXT_SAVE_FOLDED:
-	    if  ( docPlainSaveDocument( sos, bd,
-					    (const DocumentSelection *)0,
-					    1, 1 ) )
-		{ SDEB(filename); sioOutClose( sos ); return -1;	}
-	    break;
-
 	case TEDdockindTEXT_SAVE_WIDE:
 	    if  ( docPlainSaveDocument( sos, bd,
 					    (const DocumentSelection *)0,
@@ -569,6 +587,13 @@ int tedSaveDocument(	const void *		privateData,
 		{ SDEB(filename); sioOutClose( sos ); return -1;	}
 	    break;
 
+	case TEDdockindTEXT_SAVE_FOLDED:
+	    if  ( docPlainSaveDocument( sos, bd,
+					    (const DocumentSelection *)0,
+					    1, 1 ) )
+		{ SDEB(filename); sioOutClose( sos ); return -1;	}
+	    break;
+
 	case TEDdockindHTML_FILES:
 	    if  ( docHtmlSaveDocument( sos, bd, asMimeAggr,
 						(const char *)0, filename ) )
@@ -584,25 +609,32 @@ int tedSaveDocument(	const void *		privateData,
     return 0;
     }
 
-
 /************************************************************************/
 /*									*/
 /*  Allocate private data for a document.				*/
 /*									*/
 /************************************************************************/
 
-void * tedMakePrivateData()
+void * tedMakePrivateData( void )
     {
+    int			i;
     TedDocument *	td;
 
     td= (TedDocument *)malloc( sizeof(TedDocument) );
     if  ( ! td )
 	{ XDEB(td); return (void *)0;	}
 
+    docInitSelectionDescription( &(td->tdSelectionDescription) );
+    td->tdBodySectionNumber= -1;
+
     td->tdDocument= (BufferDocument *)0;
     utilInitTextAttribute( &(td->tdCurrentTextAttribute) );
     td->tdCurrentTextAttributeNumber= -1;
-    td->tdCurrentPhysicalFont= -1;
+    td->tdCurrentScreenFont= -1;
+
+    td->tdPageGapTwips= 0;
+
+    utilInitIndexMapping( &(td->tdAttributeToScreenFont) );
 
     td->tdFormatMenu= (APP_WIDGET)0;
     td->tdFormatMenuButton= (APP_WIDGET)0;
@@ -626,10 +658,12 @@ void * tedMakePrivateData()
     td->tdInsInsertPageNumberOption= (APP_WIDGET)0;
     td->tdInsInsertLineBreakOption= (APP_WIDGET)0;
     td->tdInsInsertPageBreakOption= (APP_WIDGET)0;
+    td->tdInsInsertColumnBreakOption= (APP_WIDGET)0;
     td->tdInsInsertSectBreakOption= (APP_WIDGET)0;
 
     td->tdTableMenu= (APP_WIDGET)0;
     td->tdTableMenuButton= (APP_WIDGET)0;
+
     td->tdTabInsertTableOption= (APP_WIDGET)0;
     td->tdTabAddRowOption= (APP_WIDGET)0;
     td->tdTabAddColumnOption= (APP_WIDGET)0;
@@ -637,6 +671,11 @@ void * tedMakePrivateData()
     td->tdSelectTableWidget= (APP_WIDGET)0;
     td->tdSelectRowWidget= (APP_WIDGET)0;
     td->tdSelectColumnOption= (APP_WIDGET)0;
+
+    td->tdDeleteTableWidget= (APP_WIDGET)0;
+    td->tdDeleteRowWidget= (APP_WIDGET)0;
+    td->tdDeleteColumnOption= (APP_WIDGET)0;
+
     td->tdDrawTableGridOption= (APP_WIDGET)0;
 
     td->tdFontMenu= (APP_WIDGET)0;
@@ -661,7 +700,10 @@ void * tedMakePrivateData()
     utilInitMemoryBuffer( &(td->tdCopiedFont) );
     utilInitMemoryBuffer( &(td->tdCopiedRuler) );
 
-    appInitBitmapImage( &(td->tdCopiedImage) );
+    bmInitBitmapImage( &(td->tdCopiedImage) );
+
+    td->tdFindProg= (void *)0;
+    td->tdOwnsPrimarySelection= 0;
 
 #   ifdef USE_MOTIF
     td->tdHideIBarId= (XtIntervalId)0;
@@ -673,12 +715,17 @@ void * tedMakePrivateData()
 #   endif
 
     td->tdObjectWindow= (APP_WINDOW)0;
-    td->tdObjectBottomWindow= (APP_WINDOW)0;
-    td->tdObjectRightWindow= (APP_WINDOW)0;
-    td->tdObjectCornerWindow= (APP_WINDOW)0;
-    td->tdObjectSelected= 0;
+    for ( i= 0; i < RESIZE_COUNT; i++ )
+	{ td->tdObjectResizeWindows[i]= (APP_WINDOW)0;	}
+    td->tdObjectResizeCorner= -1;
+    td->tdObjectCornerMovedX= 0;
+    td->tdObjectCornerMovedY= 0;
+    td->tdScaleChangedX= 0;
+    td->tdScaleChangedY= 0;
 
     td->tdDrawTableGrid= 1;
+    td->tdDrawMonochrome= 0;
+    td->tdInProgrammaticChange= 0;
 
     return (void *)td;
     }
@@ -706,15 +753,9 @@ APP_EVENT_HANDLER_H( tedObserveFocus, w, voided, event )
 	    {
 	    appSetCurrentDocument( ea, ed );
 
-	    if  ( ea->eaFindTool && ed->edIsReadonly )
-		{ appFindToolDisableReplace( ea->eaFindTool ); }
-
-	    if  ( ea->eaSpellTool )
-		{ appEnableSpellTool( ea->eaSpellTool, ! ed->edIsReadonly ); }
-
 	    tedAdaptPageToolToDocument( ea, ed );
 
-	    tedAdaptFormatToolToDocument( ed );
+	    tedAdaptFormatToolToDocument( ed, 0 );
 	    }
 
 	if  ( tedHasIBarSelection( td ) )
@@ -733,49 +774,78 @@ APP_EVENT_HANDLER_H( tedObserveFocus, w, voided, event )
 
 int tedPrintDocument(	SimpleOutputStream *		sos,
 			const PrintJob *		pj,
-			const PrintGeometry *		pg,
-			int				firstPage,
-			int				lastPage )
+			const PrintGeometry *		pg )
     {
     EditApplication *		ea= pj->pjApplication;
     TedDocument *		td= (TedDocument *)pj->pjPrivateData;
     BufferDocument *		bd= td->tdDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
+    const int			page= 0;
 
     time_t			now;
 
+    double			shadingMesh= -1;
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+
     RecalculateFields		rf;
 
+    LayoutContext		lc;
+
+    layoutInitContext( &lc );
+    tedSetLayoutContext2( &lc, pj->pjDrawingData, td );
+
+    if  ( tar )
+	{ shadingMesh= tar->tarShadingMeshPointsDouble;	}
+
     docInitRecalculateFields( &rf );
 
     rf.rfBd= bd;
-    rf.rfVoidadd= (void *)pj->pjDrawingData;
-    rf.rfCloseObject= tedCloseObject;
+    rf.rfCloseObject= docScreenCloseObject;
     rf.rfUpdateFlags= FIELDdoDOC_FORMATTED|FIELDdoDOC_COMPLETE|FIELDdoDOC_INFO;
     rf.rfFieldsUpdated= 0;
 
     now= time( (time_t *)0 );
     dp->dpPrintim= *localtime( &now );
 
-    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
+    if  ( docRecalculateTocFields( &rf ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( rf.rfFieldsUpdated > 0				&&
-	  tedLayoutDocumentTree( td, pj->pjDrawingData )	)
+    if  ( docRecalculateTextLevelFieldsInExternalItem( &rf, &(bd->bdBody),
+				    bd->bdBody.eiRoot->biChildren[0], page ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( docPsPrintDocument( sos, pj->pjTitle, ea->eaApplicationName,
-				ea->eaReference, pj->pjDrawingData,
-				td->tdDocument, pg,
-				pj->pjUsePostScriptFilters,
-				pj->pjUsePostScriptIndexedImages,
-				firstPage, lastPage,
-				tedLayoutExternalItem, tedCloseObject ) )
+    if  ( rf.rfFieldsUpdated > 0		&&
+	  tedLayoutDocumentTree( td, &lc )	)
+	{ LDEB(1); return -1;	}
+
+    if  ( docPsPrintDocument( sos, pj->pjTitle,
+				    ea->eaApplicationName,
+				    ea->eaReference,
+				    ea->eaFontDirectory,
+				    shadingMesh, &lc, pg ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
     }
 
+void tedSuggestNup(	PrintGeometry *	pg,
+			void *		privateData )
+    {
+    TedDocument *		td= (TedDocument *)privateData;
+    BufferDocument *		bd= td->tdDocument;
+    DocumentProperties *	dp= &(bd->bdProperties);
+
+    if  ( dp->dpTwoOnOne )
+	{
+	pg->pgGridRows= 2;
+	pg->pgGridCols= 1;
+	pg->pgGridHorizontal= 1;
+	pg->pgScalePagesToFit= 1;
+	}
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Open a document.							*/
@@ -807,6 +877,47 @@ int tedOpenDocumentFile(	EditApplication *	ea,
 
     tedDetermineDefaultSettings( tar );
 
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory); return -1;	}
+
+    if  ( ! strcmp( filename, "-rtf-" ) )
+	{
+	sis= sioInStdinOpen();
+	if  ( ! sis )
+	    { XDEB(sis); return -1;	}
+
+	bd= docRtfReadFile( sis, &(ea->eaPostScriptFontList) );
+
+	sioInClose( sis );
+
+	if  ( bd )
+	    { *pFormat= TEDdockindRTF; *pBd= bd; return 0;	}
+
+	return -1;
+	}
+
+    if  ( ! strcmp( filename, "-txt-" ) )
+	{
+	sis= sioInStdinOpen();
+	if  ( ! sis )
+	    { XDEB(sis); return -1;	}
+
+	bd= docPlainReadFile( sis, &longestPara,
+				    &(ea->eaDefaultDocumentGeometry) );
+
+	sioInClose( sis );
+
+	if  ( bd )
+	    {
+	    if  ( longestPara > 76 )
+		{ *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	    else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+	    *pBd= bd; return 0;
+	    }
+
+	return -1;
+	}
+
     ext= appFileExtensionOfName( filename );
 
     /*  1  */
@@ -822,10 +933,8 @@ int tedOpenDocumentFile(	EditApplication *	ea,
     if  ( ext && ! strcmp( ext, "rtf" ) )
 	{
 	/*  2  */
-	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
-
+	bd= docRtfReadFile( sis, &(ea->eaPostScriptFontList) );
 	sioInClose( sis );
-	
 	if  ( bd )
 	    {
 	    *pFormat= TEDdockindRTF; *pBd= bd;
@@ -833,13 +942,14 @@ int tedOpenDocumentFile(	EditApplication *	ea,
 	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
 		{ LDEB(1);	}
 
+	    tedSetGenerator( &(bd->bdProperties), ea );
+
 	    return 0;
 	    }
 	else{
 	    resp= appQuestionRunSubjectOkCancelDialog( ea,
 				    relative, option,
-				    filename, tar->tarFileNotRtf,
-				    (char *)0, (char *)0 );
+				    filename, tar->tarFileNotRtf );
 
 	    if  ( resp != AQDrespOK )
 		{ return -1;	}
@@ -851,9 +961,7 @@ int tedOpenDocumentFile(	EditApplication *	ea,
     if  ( ext && ! strcmp( ext, "txt" ) )
 	{
 	bd= docPlainReadFile( sis, &longestPara,
-				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
 				    &(ea->eaDefaultDocumentGeometry) );
-
 	sioInClose( sis );
 
 	if  ( bd )
@@ -867,6 +975,8 @@ int tedOpenDocumentFile(	EditApplication *	ea,
 	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
 		{ LDEB(1);	}
 
+	    tedSetGenerator( &(bd->bdProperties), ea );
+
 	    return 0;
 	    }
 	else{ SXDEB(filename,bd); return -1;	}
@@ -875,7 +985,8 @@ int tedOpenDocumentFile(	EditApplication *	ea,
     /*  2  */
     if  ( ! triedRtf )
 	{
-	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
+	bd= docRtfReadFile( sis, &(ea->eaPostScriptFontList) );
+	sioInClose( sis );
 	if  ( bd )
 	    {
 	    *pFormat= TEDdockindRTF; *pBd= bd;
@@ -883,13 +994,14 @@ int tedOpenDocumentFile(	EditApplication *	ea,
 	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
 		{ LDEB(1);	}
 
+	    tedSetGenerator( &(bd->bdProperties), ea );
+
 	    return 0;
 	    }
 
 	resp= appQuestionRunSubjectOkCancelDialog( ea,
 					ea->eaToplevel.atTopWidget, option,
-					filename, tar->tarFileNotRtf,
-					(char *)0, (char *)0 );
+					filename, tar->tarFileNotRtf );
 
 	if  ( resp != AQDrespOK )
 	    { return -1;	}
@@ -900,7 +1012,6 @@ int tedOpenDocumentFile(	EditApplication *	ea,
 	{ SXDEB(filename,sis); return -1;	}
 
     bd= docPlainReadFile( sis, &longestPara,
-				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
 				    &(ea->eaDefaultDocumentGeometry) );
 
     sioInClose( sis );
@@ -916,6 +1027,8 @@ int tedOpenDocumentFile(	EditApplication *	ea,
 	if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
 	    { LDEB(1);	}
 
+	tedSetGenerator( &(bd->bdProperties), ea );
+
 	return 0;
 	}
 
