@@ -30,6 +30,9 @@ static void tedFormatToolRefreshHeadFootPage(	HeaderFooterTool *	hft )
 
     int				i;
 
+    appIntegerToTextWidget( hft->hftSectionNumberText,
+						hft->hftSectionNumber+ 1 );
+
     appGuiSetToggleState( hft->hftTitlepgToggle, sp->spHasTitlePage );
 
     appGuiSetToggleState( hft->hftFacingpToggle, dp->dpHasFacingPages );
@@ -64,46 +67,62 @@ static void tedFormatToolRefreshHeadFootPage(	HeaderFooterTool *	hft )
 /*									*/
 /************************************************************************/
 
-void tedFormatToolRefreshHeaderFooterTool(
+void tedRefreshHeaderFooterTool(
 				HeaderFooterTool *		hft,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
+				const SelectionDescription *	sd,
 				const SelectionGeometry *	sg,
-				const DocumentProperties *	dpSet )
+				BufferDocument *		bd )
     {
+    const DocumentProperties *	dpSet= &(bd->bdProperties);
+
     PropertyMask		changed;
     PropertyMask		update;
 
-    BufferItem *		sectBi;
+    BufferItem *		selSectBi;
     const SectionProperties *	spSet;
 
-    int				inHeaderFooter;
+    int				whatHeaderFooter;
 
-    PROPmaskCLEAR( &changed );
+    utilPropMaskClear( &changed );
 
-    sectBi= ds->dsBegin.dpBi;
-    while( sectBi && sectBi->biLevel > DOClevSECT )
-	{ sectBi= sectBi->biParent;	}
+    selSectBi= docGetSectItem( ds->dsHead.dpBi );
+    if  ( ! selSectBi )
+	{ XDEB(selSectBi); return;	}
 
-    if  ( ! sectBi )
-	{ XDEB(sectBi); return;	}
-
-    inHeaderFooter= sectBi->biInExternalItem;
-    if  ( inHeaderFooter == DOCinBODY )
+    whatHeaderFooter= selSectBi->biInExternalItem;
+    if  ( whatHeaderFooter == DOCinBODY )
 	{
-	ExternalItem *	ei;
+	DocumentTree *	ei;
+	int		isEmpty;
 	int		page= sg->sgBegin.pgTopPosition.lpPage;
 
-	inHeaderFooter= docWhatPageHeader( &ei, sectBi, page, dpSet );
-	}
+	whatHeaderFooter= docWhatPageHeader( &ei, &isEmpty,
+						    selSectBi, page, bd );
 
-    hft->hftSectionNumber= sectBi->biNumberInParent;
-    spSet= &(sectBi->biSectProperties);
+	hft->hftSectionNumber= selSectBi->biNumberInParent;
+	spSet= &(selSectBi->biSectProperties);
+	}
+    else{
+	const BufferItem *	bodySectBi;
+
+	if  ( ! sd->sdInHeaderFooter )
+	    { *pEnabled= 0; return;	}
+
+	bodySectBi= docGetBodySectBi( selSectBi, bd );
+
+	hft->hftSectionNumber= bodySectBi->biNumberInParent;
+	spSet= &(bodySectBi->biSectProperties);
+
+	(*pPref) += 2;
+	}
 
     /**********/
 
-    PROPmaskCLEAR( &update );
+    utilPropMaskClear( &update );
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
     if  ( docUpdDocumentProperties( &changed, &(hft->hftDocPropertiesChosen),
@@ -113,7 +132,7 @@ void tedFormatToolRefreshHeaderFooterTool(
 							&update, dpSet ) )
 	{ LDEB(1); return;	}
 
-    PROPmaskCLEAR( &update );
+    utilPropMaskClear( &update );
     PROPmaskADD( &update, SPpropTITLEPG );
 
     if  ( docUpdSectProperties( &changed, &(hft->hftSectPropertiesChosen),
@@ -125,46 +144,51 @@ void tedFormatToolRefreshHeaderFooterTool(
 
     /**********/
 
-    hft->hftPagesOptionsEnabled[PAGES_FIRST_PAGE]=
+    if  ( sd->sdInHeaderFooter )
+	{
+	int		i;
+
+	for ( i= 0; i < PAGES__COUNT; i++ )
+	    { hft->hftPagesOptionsEnabled[i]= 0; }
+
+	appGuiEnableWidget( hft->hftSectionFrame, 0 );
+	appGuiEnableWidget( hft->hftDocumentFrame, 0 );
+	}
+    else{
+	hft->hftPagesOptionsEnabled[PAGES_FIRST_PAGE]=
 					    spSet->spHasTitlePage;
-    hft->hftPagesOptionsEnabled[PAGES_SUBSEQUENT_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_SUBSEQUENT_PAGES]=
 					    spSet->spHasTitlePage	&&
 					    ! dpSet->dpHasFacingPages;
-    hft->hftPagesOptionsEnabled[PAGES_ALL_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_ALL_PAGES]=
 					    ! spSet->spHasTitlePage	&&
 					    ! dpSet->dpHasFacingPages;
-    hft->hftPagesOptionsEnabled[PAGES_ODD_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_ODD_PAGES]=
 					    dpSet->dpHasFacingPages;
-    hft->hftPagesOptionsEnabled[PAGES_EVEN_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_EVEN_PAGES]=
 					    dpSet->dpHasFacingPages;
 
-    switch( inHeaderFooter )
+	appGuiEnableWidget( hft->hftSectionFrame, 1 );
+	appGuiEnableWidget( hft->hftDocumentFrame, 1 );
+	}
+
+    switch( whatHeaderFooter )
 	{
-	case DOCinSECT_HEADER:
-	case DOCinSECT_FOOTER:
-	    if  ( spSet->spHasTitlePage )
-		{ hft->hftPagesChosen= PAGES_SUBSEQUENT_PAGES;	}
-	    else{ hft->hftPagesChosen= PAGES_ALL_PAGES;		}
-
-	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
-
-	    break;
+	int	pages;
 
 	case DOCinFIRST_HEADER:
-	case DOCinFIRST_FOOTER:
-	    hft->hftPagesChosen= PAGES_FIRST_PAGE;
-	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
-	    break;
-
 	case DOCinLEFT_HEADER:
-	case DOCinLEFT_FOOTER:
-	    hft->hftPagesChosen= PAGES_EVEN_PAGES;
-	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
-	    break;
-
 	case DOCinRIGHT_HEADER:
+
+	case DOCinFIRST_FOOTER:
+	case DOCinLEFT_FOOTER:
 	case DOCinRIGHT_FOOTER:
-	    hft->hftPagesChosen= PAGES_ODD_PAGES;
+	    pages= docWhatPagesForHeaderFooter( dpSet, spSet,
+							    whatHeaderFooter );
+	    if  ( pages < 0 )
+		{ LLDEB(whatHeaderFooter,pages); *pEnabled= 0; return;	}
+	    hft->hftPagesChosen= pages;
+	    hft->hftPagesOptionsEnabled[hft->hftPagesChosen]= 1;
 	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
 	    break;
 
@@ -183,7 +207,7 @@ void tedFormatToolRefreshHeaderFooterTool(
 	    *pEnabled= 0; return;
 
 	default:
-	    LDEB(inHeaderFooter);
+	    LDEB(whatHeaderFooter);
 	    *pEnabled= 0; return;
 	}
 
@@ -191,7 +215,7 @@ void tedFormatToolRefreshHeaderFooterTool(
 
     tedFormatToolRefreshHeadFootPage( hft );
 
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
@@ -208,10 +232,10 @@ static APP_BUTTON_CALLBACK_H( tedHeadFootChangeDocPushed, w, voidhft )
 
     PropertyMask		update;
 
-    PROPmaskCLEAR( &update );
+    utilPropMaskClear( &update );
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
-    tedSetDocumentProperties( hft->hftApplication, dp, &update );
+    tedAppSetDocumentProperties( hft->hftApplication, dp, &update );
 
     return;
     }
@@ -223,7 +247,7 @@ static APP_BUTTON_CALLBACK_H( tedHeadFootChangeSectPushed, w, voidhft )
 
     PropertyMask		update;
 
-    PROPmaskCLEAR( &update );
+    utilPropMaskClear( &update );
     PROPmaskADD( &update, SPpropTITLEPG );
 
     if  ( tedAppChangeSectionProperties( hft->hftApplication, &update, sp ) )
@@ -240,9 +264,9 @@ static APP_BUTTON_CALLBACK_H( tedHeadFootRevertDocPushed, w, voidhft )
     PropertyMask		update;
     PropertyMask		changed;
 
-    PROPmaskCLEAR( &changed );
+    utilPropMaskClear( &changed );
 
-    PROPmaskCLEAR( &update );
+    utilPropMaskClear( &update );
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
     if  ( docUpdDocumentProperties( &changed, dp,
@@ -262,9 +286,9 @@ static APP_BUTTON_CALLBACK_H( tedHeadFootRevertSectPushed, w, voidhft )
     PropertyMask		update;
     PropertyMask		changed;
 
-    PROPmaskCLEAR( &changed );
+    utilPropMaskClear( &changed );
 
-    PROPmaskCLEAR( &update );
+    utilPropMaskClear( &update );
     PROPmaskADD( &update, SPpropTITLEPG );
 
     if  ( docUpdSectProperties( &changed, sp,
@@ -412,6 +436,7 @@ void tedFormatFillHeaderFooterPage( HeaderFooterTool *		hft,
     APP_WIDGET	row= (APP_WIDGET )0;
 
     const int	heightResizable= 0;
+    const int	textColumns= 5;
 
     /**/
     hft->hftPageResources= hfpr;
@@ -428,9 +453,14 @@ void tedFormatFillHeaderFooterPage( HeaderFooterTool *		hft,
 			    &(hft->hftSectionPaned),
 			    pageWidget, hfpr->hfprSectionHeaderText );
 
+    appMakeLabelAndTextRow( &row, &(hft->hftSectionNumberLabel),
+			    &(hft->hftSectionNumberText),
+			    hft->hftSectionPaned, hfpr->hfprSectionNumberText,
+			    textColumns, 0 );
+
     row= appMakeRowInColumn( hft->hftSectionPaned, 1, heightResizable );
     hft->hftTitlepgToggle= appMakeToggleInRow( row, hfpr->hfprTitlepgText,
-				tedHeaderFooterToggleTitlepg, (void *)hft, 0 );
+				tedHeaderFooterToggleTitlepg, (void *)hft, 0, 1 );
 
     appInspectorMakeButtonRow( &row, hft->hftSectionPaned,
 		&(is->isRevertButton), &(is->isApplyButton),
@@ -446,7 +476,7 @@ void tedFormatFillHeaderFooterPage( HeaderFooterTool *		hft,
 
     row= appMakeRowInColumn( hft->hftDocumentPaned, 1, heightResizable );
     hft->hftFacingpToggle= appMakeToggleInRow( row, hfpr->hfprFacingpText,
-				tedHeaderFooterToggleFacingp, (void *)hft, 0 );
+				tedHeaderFooterToggleFacingp, (void *)hft, 0, 1 );
 
     appInspectorMakeButtonRow( &row, hft->hftDocumentPaned,
 		&(hft->hftRevertDocButton), &(hft->hftChangeDocButton),
@@ -549,6 +579,9 @@ static AppConfigurableResource TED_TedHeaderFooterToolResourceTable[]=
     APP_RESOURCE( "formatToolHeadFootSectionHeader",
 		offsetof(HeaderFooterPageResources,hfprSectionHeaderText),
 		"Section Properties" ),
+    APP_RESOURCE( "formatToolHeadFootSectionNumber",
+		offsetof(HeaderFooterPageResources,hfprSectionNumberText),
+		"Section" ),
     APP_RESOURCE( "formatToolHeadFootTitlepgText",
 		offsetof(HeaderFooterPageResources,hfprTitlepgText),
 		"Different first Page" ),
@@ -609,15 +642,24 @@ void tedFormatToolGetHeaderFooterResourceTable( EditApplication *	ea,
 					HeaderFooterPageResources *	hfpr,
 					InspectorSubjectResources *	isr )
     {
-    appGuiGetResourceValues( ea, (void *)hfpr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)hfpr,
 				TED_TedHeaderFooterToolResourceTable,
 				sizeof(TED_TedHeaderFooterToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedHeaderFooterSubjectResourceTable,
 				sizeof(TED_TedHeaderFooterSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
