@@ -11,7 +11,7 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	"docEvalField.h"
 
 #   include	"tedApp.h"
@@ -58,19 +58,18 @@ static const int TED_EndnotePositions[]=
 
 static void tedSetNotesMenu(	AppOptionmenu *		aom,
 				int			val,
-				APP_WIDGET *		options )
+				const int *		values,
+				int			valueCount )
     {
-    int		n= 0;
     int		i;
 
-    for ( i= 0; i < val; i++ )
+    for ( i= 0; i < valueCount; i++ )
 	{
-	if  ( options[i] )
-	    { n++;	}
+	if  ( values[i] == val )
+	    { appSetOptionmenu( aom, i ); return;	}
 	}
 
-    appSetOptionmenu( aom, n );
-
+    LLDEB(val,valueCount);
     return;
     }
 
@@ -90,17 +89,23 @@ static void tedNotesRefreshNotesStartNumber(	NotePropertiesTool *	npt,
     return;
     }
 
-static void tedNotesRefreshNotePropertiesTool(	NotePropertiesTool *	npt,
-						const NotesProperties *	np )
+static void tedNotesRefreshNotePropertiesTool(
+					NotePropertiesTool *	npt,
+					const NotesProperties *	np,
+					const int *		restarts,
+					int			restartCount,
+					const int *		positions,
+					int			positionCount )
     {
     tedNotesRefreshNotesStartNumber( npt, np );
 
     tedSetNotesMenu( &(npt->nptPositionOptionmenu),
 					    np->npPosition,
-					    npt->nptPositionOptions );
+					    positions, positionCount );
     tedSetNotesMenu( &(npt->nptRestartOptionmenu),
 					    np->npRestart,
-					    npt->nptRestartOptions );
+					    restarts, restartCount );
+
     appSetOptionmenu( &(npt->nptStyleOptionmenu),
 					    np->npNumberStyle );
 
@@ -109,24 +114,57 @@ static void tedNotesRefreshNotePropertiesTool(	NotePropertiesTool *	npt,
 
 static void tedNotesToolRefreshDocWidgets(	NotesTool *	nt )
     {
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     tedNotesRefreshNotePropertiesTool( &(nt->ntFootnotePropertiesTool),
-						&(dp->dpFootnoteProperties) );
+				&(dp->dpFootnoteProperties),
+				TED_FootnoteRestarts,
+				sizeof(TED_FootnoteRestarts)/sizeof(int),
+				TED_FootnotePositions,
+				sizeof(TED_FootnotePositions)/sizeof(int) );
 
     tedNotesRefreshNotePropertiesTool( &(nt->ntEndnotePropertiesTool),
-						&(dp->dpEndnoteProperties) );
+				&(dp->dpEndnoteProperties),
+				TED_EndnoteRestarts,
+				sizeof(TED_EndnoteRestarts)/sizeof(int),
+				TED_EndnotePositions,
+				sizeof(TED_EndnotePositions)/sizeof(int) );
 
     return;
     }
 
+static void tedNotesToolRefreshFixedTextSet(	NotesTool *	nt )
+    {
+    if  ( docFormatChftnField( &(nt->ntNotePropertiesSet.npFixedText),
+					&(nt->ntDocPropertiesChosen),
+					&(nt->ntNotePropertiesSet),
+					nt->ntNoteNumber ) )
+	{ LDEB(1);	}
+
+    if  ( utilCopyMemoryBuffer( &(nt->ntNotePropertiesChosen.npFixedText),
+				&(nt->ntNotePropertiesSet.npFixedText) ) )
+	{ LDEB(2);	}
+    }
+
+static void tedNotesToolRefreshFixedTextChosen(	NotesTool *	nt )
+    {
+    if  ( docFormatChftnField( &(nt->ntNotePropertiesChosen.npFixedText),
+					&(nt->ntDocPropertiesChosen),
+					&(nt->ntNotePropertiesChosen),
+					nt->ntNoteNumber ) )
+	{ LDEB(1);	}
+
+    appStringToTextWidget( nt->ntFixedTextText,
+		    (char *)nt->ntNotePropertiesChosen.npFixedText.mbBytes );
+    }
+
 static void tedNotesToolRefreshNoteWidgets(	NotesTool *	nt )
     {
     int		enabled= 0;
 
-    if  ( nt->ntNoteKindChosen == DOCinFOOTNOTE )
+    if  ( nt->ntNotePropertiesChosen.npExternalItemKind == DOCinFOOTNOTE )
 	{ enabled= 1;	}
-    if  ( nt->ntNoteKindChosen == DOCinENDNOTE )
+    if  ( nt->ntNotePropertiesChosen.npExternalItemKind == DOCinENDNOTE )
 	{ enabled= 1;	}
 
     appGuiEnableWidget( nt->ntCurrentNoteFrame, enabled );
@@ -135,25 +173,29 @@ static void tedNotesToolRefreshNoteWidgets(	NotesTool *	nt )
     appGuiEnableWidget( nt->ntToNoteButton, ! nt->ntInsideNote );
 
     appGuiSetToggleState( nt->ntFootnoteToggle,
-				nt->ntNoteKindChosen == DOCinFOOTNOTE );
+	    nt->ntNotePropertiesChosen.npExternalItemKind == DOCinFOOTNOTE );
 
     appGuiSetToggleState( nt->ntEndnoteToggle,
-				nt->ntNoteKindChosen == DOCinENDNOTE );
+	    nt->ntNotePropertiesChosen.npExternalItemKind == DOCinENDNOTE );
 
-    appGuiSetToggleState( nt->ntFixedTextToggle, nt->ntFixedTextChosen );
+    appGuiSetToggleState( nt->ntFixedTextToggle,
+				    ! nt->ntNotePropertiesChosen.npAutoNumber );
 
-    appEnableText( nt->ntFixedTextText, nt->ntFixedTextChosen );
-    appStringToTextWidget( nt->ntFixedTextText, (char *)nt->ntNoteTextChosen );
+    appEnableText( nt->ntFixedTextText,
+				    ! nt->ntNotePropertiesChosen.npAutoNumber );
+
+    appStringToTextWidget( nt->ntFixedTextText,
+		    (char *)nt->ntNotePropertiesChosen.npFixedText.mbBytes );
 
     return;
     }
 
-void tedFormatToolRefreshNotesTool(
-				NotesTool *			nt,
+void tedRefreshNotesTool(	NotesTool *			nt,
 				int *				pEnabled,
 				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
+				const SelectionDescription *	sd,
 				BufferDocument *		bd )
     {
     const DocumentProperties *	dp= &(bd->bdProperties);
@@ -162,15 +204,12 @@ void tedFormatToolRefreshNotesTool(
     PropertyMask		updMask;
 
     int				enabled= 0;
-    int				noteIndex= -1;
-    int				noteFieldNr= -1;
+    DocumentField *		dfNote= (DocumentField *)0;
 
     nt->ntInsideNote= 0;
-    nt->ntNoteKindSet= DOCinBODY;
-    nt->ntFixedTextSet= 0;
-
-    nt->ntNoteTextSet[0]= '\0';
-    nt->ntNoteTextChosen[0]= '\0';
+    docCleanNoteProperties( &(nt->ntNotePropertiesSet) );
+    docInitNoteProperties( &(nt->ntNotePropertiesSet) );
+    nt->ntNotePropertiesSet.npExternalItemKind= DOCinBODY;
 
     nt->ntNoteNumber= 0;
 
@@ -178,31 +217,33 @@ void tedFormatToolRefreshNotesTool(
 	{
 	DocumentNote *		dn= (DocumentNote *)0;
 
-	noteIndex= docGetSelectedNote( &dn, &noteFieldNr,
-						nt->ntNoteTextSet,
-						sizeof(nt->ntNoteTextSet)- 1,
-						bd, ds );
-	if  ( noteIndex >= 0 )
+	dfNote= docGetSelectedNote( &dn, bd, ds );
+	if  ( dfNote )
 	    {
-	    nt->ntNoteKindSet= dn->dnExternalItemKind;
+	    if  ( docCopyNoteProperties( &(nt->ntNotePropertiesSet),
+					    &(dn->dnNoteProperties) ) )
+		{ LDEB(1);	}
 	    nt->ntNoteNumber= dn->dnNoteNumber;
 
 	    if  ( ds->dsSelectionScope.ssInExternalItem == DOCinFOOTNOTE  ||
 		  ds->dsSelectionScope.ssInExternalItem == DOCinENDNOTE   )
 		{ nt->ntInsideNote= 1;	}
 
-	    if  ( noteFieldNr < 0 )
-		{ nt->ntFixedTextSet= 1;	}
+	    if  ( docFormatChftnField( &(nt->ntNotePropertiesSet.npFixedText),
+					    &(bd->bdProperties),
+					    &(nt->ntNotePropertiesSet),
+					    nt->ntNoteNumber ) )
+		{ LDEB(1);	}
 
 	    (*pPref) += 4;
 	    }
 	}
 
-    nt->ntNoteKindChosen=  nt->ntNoteKindSet;
-    nt->ntFixedTextChosen=  nt->ntFixedTextSet;
-    strcpy( (char *)nt->ntNoteTextChosen, (char *)nt->ntNoteTextSet );
+    if  ( docCopyNoteProperties( &(nt->ntNotePropertiesChosen),
+					    &(nt->ntNotePropertiesSet) ) )
+	{ LDEB(1);	}
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
 
     PROPmaskADD( &updMask, DPpropFOOTNOTE_POSITION );
     PROPmaskADD( &updMask, DPpropFOOTNOTE_RESTART );
@@ -214,34 +255,25 @@ void tedFormatToolRefreshNotesTool(
     PROPmaskADD( &updMask, DPpropENDNOTE_STYLE );
     PROPmaskADD( &updMask, DPpropENDNOTE_STARTNR );
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
-    if  ( docUpdDocumentProperties( &chgMask, &(nt->ntPropertiesChosen),
+    if  ( docUpdDocumentProperties( &chgMask, &(nt->ntDocPropertiesChosen),
 							&updMask, dp ) )
 	{ LDEB(1); return;	}
-    if  ( docUpdDocumentProperties( &chgMask, &(nt->ntPropertiesSet),
+    if  ( docUpdDocumentProperties( &chgMask, &(nt->ntDocPropertiesSet),
 							&updMask, dp ) )
 	{ LDEB(1); return;	}
 
-    if  ( ! nt->ntFixedTextSet			&&
-	  nt->ntNoteKindSet != DOCinBODY	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-
-	strcpy( (char *)nt->ntNoteTextSet, (char *)nt->ntNoteTextChosen );
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind != DOCinBODY	)
+	{ tedNotesToolRefreshFixedTextSet( nt );	}
 
     tedNotesToolRefreshDocWidgets( nt );
     tedNotesToolRefreshNoteWidgets( nt );
 
     if  ( ds->dsSelectionScope.ssInExternalItem == DOCinBODY		||
-	  noteIndex >= 0						)
-	{ enabled= 1;	}
+	  dfNote							)
+	{ enabled= ! sd->sdDocumentReadonly;	}
 
     *pEnabled= enabled;
     return;
@@ -293,7 +325,7 @@ static int tedNotesToolGetStartNumber(	NotePropertiesTool *	npt,
 static APP_TXACTIVATE_CALLBACK_H( tedNotesFootRestartChanged, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
     NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
     NotesProperties *		np= &(dp->dpFootnoteProperties);
 
@@ -301,20 +333,9 @@ static APP_TXACTIVATE_CALLBACK_H( tedNotesFootRestartChanged, w, voidnt )
 	  tedNotesToolGetStartNumber( npt, np )		)
 	{ return;	}
 
-    if  ( ! nt->ntFixedTextChosen		&&
-	  nt->ntNoteKindChosen == DOCinFOOTNOTE	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	else{
-	    appStringToTextWidget( nt->ntFixedTextText,
-					    (char *)nt->ntNoteTextChosen );
-	    }
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind == DOCinFOOTNOTE	)
+	{ tedNotesToolRefreshFixedTextChosen( nt );	}
 
     return;
     }
@@ -322,7 +343,7 @@ static APP_TXACTIVATE_CALLBACK_H( tedNotesFootRestartChanged, w, voidnt )
 static APP_TXACTIVATE_CALLBACK_H( tedNotesEndRestartChanged, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
     NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
     NotesProperties *		np= &(dp->dpEndnoteProperties);
 
@@ -330,20 +351,9 @@ static APP_TXACTIVATE_CALLBACK_H( tedNotesEndRestartChanged, w, voidnt )
 	  tedNotesToolGetStartNumber( npt, np )		)
 	{ return;	}
 
-    if  ( ! nt->ntFixedTextChosen		&&
-	  nt->ntNoteKindChosen == DOCinENDNOTE	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	else{
-	    appStringToTextWidget( nt->ntFixedTextText,
-					    (char *)nt->ntNoteTextChosen );
-	    }
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind == DOCinENDNOTE	)
+	{ tedNotesToolRefreshFixedTextChosen( nt );	}
 
     return;
     }
@@ -351,7 +361,7 @@ static APP_TXACTIVATE_CALLBACK_H( tedNotesEndRestartChanged, w, voidnt )
 static APP_BUTTON_CALLBACK_H( tedChangeNotesPushed, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     PropertyMask		dpUpdMask;
     PropertyMask		spUpdMask;
@@ -360,7 +370,7 @@ static APP_BUTTON_CALLBACK_H( tedChangeNotesPushed, w, voidnt )
 
     docInitSectionProperties( &sp );
 
-    PROPmaskCLEAR( &dpUpdMask );
+    utilPropMaskClear( &dpUpdMask );
 
     PROPmaskADD( &dpUpdMask, DPpropFOOTNOTE_POSITION );
     PROPmaskADD( &dpUpdMask, DPpropFOOTNOTE_RESTART );
@@ -372,7 +382,7 @@ static APP_BUTTON_CALLBACK_H( tedChangeNotesPushed, w, voidnt )
     PROPmaskADD( &dpUpdMask, DPpropENDNOTE_STYLE );
     PROPmaskADD( &dpUpdMask, DPpropENDNOTE_STARTNR );
 
-    PROPmaskCLEAR( &spUpdMask );
+    utilPropMaskClear( &spUpdMask );
 
     PROPmaskADD( &spUpdMask, SPpropFOOTNOTE_POSITION );
     PROPmaskADD( &spUpdMask, SPpropFOOTNOTE_RESTART );
@@ -414,7 +424,7 @@ static APP_BUTTON_CALLBACK_H( tedRevertNotesPushed, w, voidnt )
     PropertyMask	chgMask;
     PropertyMask	updMask;
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
 
     PROPmaskADD( &updMask, DPpropFOOTNOTE_POSITION );
     PROPmaskADD( &updMask, DPpropFOOTNOTE_RESTART );
@@ -426,10 +436,10 @@ static APP_BUTTON_CALLBACK_H( tedRevertNotesPushed, w, voidnt )
     PROPmaskADD( &updMask, DPpropENDNOTE_STYLE );
     PROPmaskADD( &updMask, DPpropENDNOTE_STARTNR );
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
-    if  ( docUpdDocumentProperties( &chgMask, &(nt->ntPropertiesChosen),
-					&updMask, &(nt->ntPropertiesSet) ) )
+    if  ( docUpdDocumentProperties( &chgMask, &(nt->ntDocPropertiesChosen),
+					&updMask, &(nt->ntDocPropertiesSet) ) )
 	{ LDEB(1); return;	}
 
     tedNotesToolRefreshDocWidgets( nt );
@@ -451,9 +461,11 @@ static APP_BUTTON_CALLBACK_H( tedChangeNotePushed, w, voidnt )
 
     fixedText= appGetStringFromTextWidget( nt->ntFixedTextText );
 
-    tedChangeCurrentNote( nt->ntApplication, ! nt->ntFixedTextChosen,
-						(unsigned char *)fixedText,
-						nt->ntNoteKindChosen );
+    if  ( utilSetMemoryBuffer( &(nt->ntNotePropertiesChosen.npFixedText),
+		(unsigned char *)fixedText, strlen( fixedText ) ) )
+	{ LDEB(1);	}
+
+    tedChangeCurrentNote( nt->ntApplication, &(nt->ntNotePropertiesChosen) );
 
     appFreeStringFromTextWidget( fixedText );
 
@@ -464,10 +476,9 @@ static APP_BUTTON_CALLBACK_H( tedRevertNotePushed, w, voidnt )
     {
     NotesTool *		nt= (NotesTool *)voidnt;
 
-    nt->ntNoteKindChosen= nt->ntNoteKindSet;
-    nt->ntFixedTextChosen= nt->ntFixedTextSet;
-
-    strcpy( (char *)nt->ntNoteTextChosen, (char *)nt->ntNoteTextSet );
+    if  ( docCopyNoteProperties( &(nt->ntNotePropertiesChosen),
+					    &(nt->ntNotePropertiesSet) ) )
+	{ LDEB(1);	}
 
     tedNotesToolRefreshNoteWidgets( nt );
 
@@ -489,19 +500,12 @@ static APP_TOGGLE_CALLBACK_H( tedFootnoteToggled, w, voidnt, voidtbcs )
 
     set= appGuiGetToggleStateFromCallback( w, voidtbcs );
 
-    if  ( set && nt->ntNoteKindChosen != DOCinFOOTNOTE )
+    if  ( set && nt->ntNotePropertiesSet.npExternalItemKind != DOCinFOOTNOTE )
 	{
-	nt->ntNoteKindChosen= DOCinFOOTNOTE;
+	nt->ntNotePropertiesSet.npExternalItemKind= DOCinFOOTNOTE;
 
-	if  ( ! nt->ntFixedTextChosen )
-	    {
-	    if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					    sizeof(nt->ntNoteTextChosen)- 1,
-					    &(nt->ntPropertiesChosen),
-					    nt->ntNoteNumber,
-					    nt->ntNoteKindChosen ) )
-		{ LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	    }
+	if  ( nt->ntNotePropertiesChosen.npAutoNumber )
+	    { tedNotesToolRefreshFixedTextChosen( nt );	}
 
 	tedNotesToolRefreshNoteWidgets( nt );
 	}
@@ -517,19 +521,12 @@ static APP_TOGGLE_CALLBACK_H( tedEndnoteToggled, w, voidnt, voidtbcs )
 
     set= appGuiGetToggleStateFromCallback( w, voidtbcs );
 
-    if  ( set && nt->ntNoteKindChosen != DOCinENDNOTE )
+    if  ( set && nt->ntNotePropertiesSet.npExternalItemKind != DOCinENDNOTE )
 	{
-	nt->ntNoteKindChosen= DOCinENDNOTE;
+	nt->ntNotePropertiesSet.npExternalItemKind= DOCinENDNOTE;
 	
-	if  ( ! nt->ntFixedTextChosen )
-	    {
-	    if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					    sizeof(nt->ntNoteTextChosen)- 1,
-					    &(nt->ntPropertiesChosen),
-					    nt->ntNoteNumber,
-					    nt->ntNoteKindChosen ) )
-		{ LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	    }
+	if  ( nt->ntNotePropertiesChosen.npAutoNumber )
+	    { tedNotesToolRefreshFixedTextChosen( nt );	}
 
 	tedNotesToolRefreshNoteWidgets( nt );
 	}
@@ -545,19 +542,12 @@ static APP_TOGGLE_CALLBACK_H( tedFixedNoteTextToggled, w, voidnt, voidtbcs )
 
     set= appGuiGetToggleStateFromCallback( w, voidtbcs );
 
-    if  ( nt->ntFixedTextChosen != set )
+    if  ( nt->ntNotePropertiesChosen.npAutoNumber == set )
 	{
-	if  ( ! set )
-	    {
-	    if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-		{ LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	    }
+	nt->ntNotePropertiesChosen.npAutoNumber= ! set;
 
-	nt->ntFixedTextChosen= set;
+	if  ( nt->ntNotePropertiesChosen.npAutoNumber )
+	    { tedNotesToolRefreshFixedTextChosen( nt );	}
 	}
 
     tedNotesToolRefreshNoteWidgets( nt );
@@ -623,8 +613,11 @@ void tedFormatFillNotesPage(	NotesTool *			nt,
     nt->ntPageResources= npr;
 
     /**/
-    docInitDocumentProperties( &(nt->ntPropertiesSet) );
-    docInitDocumentProperties( &(nt->ntPropertiesChosen) );
+    docInitNoteProperties( &(nt->ntNotePropertiesSet) );
+    docInitNoteProperties( &(nt->ntNotePropertiesChosen) );
+
+    docInitDocumentProperties( &(nt->ntDocPropertiesSet) );
+    docInitDocumentProperties( &(nt->ntDocPropertiesChosen) );
 
     /**/
     appMakeColumnFrameInColumn( &(nt->ntCurrentNoteFrame),
@@ -684,7 +677,7 @@ void tedFormatFillNotesPage(	NotesTool *			nt,
 static APP_OITEM_CALLBACK_H( tedFootnoteStyleChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
     NotesProperties *		np= &(dp->dpFootnoteProperties);
@@ -697,20 +690,9 @@ static APP_OITEM_CALLBACK_H( tedFootnoteStyleChosen, w, voidnt )
 
     np->npNumberStyle= n;
 
-    if  ( ! nt->ntFixedTextChosen		&&
-	  nt->ntNoteKindChosen == DOCinFOOTNOTE	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	else{
-	    appStringToTextWidget( nt->ntFixedTextText,
-					    (char *)nt->ntNoteTextChosen );
-	    }
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind == DOCinFOOTNOTE	)
+	{ tedNotesToolRefreshFixedTextChosen( nt );	}
 
     return;
     }
@@ -718,7 +700,7 @@ static APP_OITEM_CALLBACK_H( tedFootnoteStyleChosen, w, voidnt )
 static APP_OITEM_CALLBACK_H( tedEndnoteStyleChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
     NotesProperties *		np= &(dp->dpEndnoteProperties);
@@ -731,20 +713,9 @@ static APP_OITEM_CALLBACK_H( tedEndnoteStyleChosen, w, voidnt )
 
     np->npNumberStyle= n;
 
-    if  ( ! nt->ntFixedTextChosen		&&
-	  nt->ntNoteKindChosen == DOCinENDNOTE	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	else{
-	    appStringToTextWidget( nt->ntFixedTextText,
-					    (char *)nt->ntNoteTextChosen );
-	    }
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind == DOCinENDNOTE	)
+	{ tedNotesToolRefreshFixedTextChosen( nt );	}
 
     return;
     }
@@ -752,7 +723,7 @@ static APP_OITEM_CALLBACK_H( tedEndnoteStyleChosen, w, voidnt )
 static APP_OITEM_CALLBACK_H( tedFootnoteRestartChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
     NotesProperties *		np= &(dp->dpFootnoteProperties);
@@ -767,27 +738,17 @@ static APP_OITEM_CALLBACK_H( tedFootnoteRestartChosen, w, voidnt )
 
     tedNotesRefreshNotesStartNumber( npt, np );
 
-    if  ( ! nt->ntFixedTextChosen		&&
-	  nt->ntNoteKindChosen == DOCinFOOTNOTE	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	else{
-	    appStringToTextWidget( nt->ntFixedTextText,
-					    (char *)nt->ntNoteTextChosen );
-	    }
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind == DOCinFOOTNOTE	)
+	{ tedNotesToolRefreshFixedTextChosen( nt );	}
+
     return;
     }
 
 static APP_OITEM_CALLBACK_H( tedEndnoteRestartChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
     NotesProperties *		np= &(dp->dpEndnoteProperties);
@@ -802,20 +763,9 @@ static APP_OITEM_CALLBACK_H( tedEndnoteRestartChosen, w, voidnt )
 
     tedNotesRefreshNotesStartNumber( npt, np );
 
-    if  ( ! nt->ntFixedTextChosen		&&
-	  nt->ntNoteKindChosen == DOCinENDNOTE	)
-	{
-	if  ( docFormatChftnField( nt->ntNoteTextChosen,
-					sizeof(nt->ntNoteTextChosen)- 1,
-					&(nt->ntPropertiesChosen),
-					nt->ntNoteNumber,
-					nt->ntNoteKindChosen ) )
-	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
-	else{
-	    appStringToTextWidget( nt->ntFixedTextText,
-					    (char *)nt->ntNoteTextChosen );
-	    }
-	}
+    if  ( nt->ntNotePropertiesSet.npAutoNumber				&&
+	  nt->ntNotePropertiesSet.npExternalItemKind == DOCinENDNOTE	)
+	{ tedNotesToolRefreshFixedTextChosen( nt );	}
 
     return;
     }
@@ -823,7 +773,7 @@ static APP_OITEM_CALLBACK_H( tedEndnoteRestartChosen, w, voidnt )
 static APP_OITEM_CALLBACK_H( tedFootnotePositionChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
     NotesProperties *		np= &(dp->dpFootnoteProperties);
@@ -842,7 +792,7 @@ static APP_OITEM_CALLBACK_H( tedFootnotePositionChosen, w, voidnt )
 static APP_OITEM_CALLBACK_H( tedEndnotePositionChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    DocumentProperties *	dp= &(nt->ntDocPropertiesChosen);
 
     NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
     NotesProperties *		np= &(dp->dpEndnoteProperties);
@@ -973,8 +923,11 @@ void tedFormatFinishNotesPage(		NotesTool *			nt,
 
 void tedFormatCleanNotesTool(	NotesTool *	nt )
     {
-    docCleanDocumentProperties( &(nt->ntPropertiesSet) );
-    docCleanDocumentProperties( &(nt->ntPropertiesChosen) );
+    docCleanNoteProperties( &(nt->ntNotePropertiesSet) );
+    docCleanNoteProperties( &(nt->ntNotePropertiesChosen) );
+
+    docCleanDocumentProperties( &(nt->ntDocPropertiesSet) );
+    docCleanDocumentProperties( &(nt->ntDocPropertiesChosen) );
 
     return;
     }
