@@ -15,13 +15,14 @@
 /*									*/
 /************************************************************************/
 
-extern void tedFormatToolRefreshPageLayoutTool(
+extern void tedRefreshPageLayoutTool(
 				PageLayoutTool *		plt,
 				int *				pEnabled,
 				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
-				const DocumentProperties *	dp );
+				const SelectionDescription *	sd,
+				const BufferDocument *		bd );
 
 extern void tedFormatCleanPageLayoutTool(	PageLayoutTool *	plt );
 
