@@ -11,7 +11,7 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
@@ -29,8 +29,6 @@ static void tedFormatToolRefreshSpaceAround(	APP_WIDGET	text,
 						APP_WIDGET	toggle,
 						int		space )
     {
-    char		scratch[50];
-
     if  ( space == 0 )
 	{
 	appStringToTextWidget( text, "" );
@@ -39,8 +37,7 @@ static void tedFormatToolRefreshSpaceAround(	APP_WIDGET	text,
 	appGuiSetToggleState( toggle, 0 );
 	}
     else{
-	appGeoLengthToString( scratch, space, UNITtyPOINTS );
-	appStringToTextWidget( text, scratch );
+	appLengthToTextWidget( text, space, UNITtyPOINTS );
 	appEnableText( text, 1 );
 
 	appGuiSetToggleState( toggle, 1 );
@@ -51,17 +48,28 @@ static void tedFormatToolRefreshParagraphPage(	ParagraphLayoutTool *	plt )
     {
     ParagraphProperties *	pp= &(plt->ptPropertiesChosen);
 
-    char		scratch[50];
+    if  ( pp->ppListOverride > 0 )
+	{
+	appIntegerToTextWidget( plt->ptListLevelText, pp->ppListLevel+ 1 );
 
-    appGeoLengthToString( scratch,
+	appEnableText( plt->ptFirstIndentText, 0 );
+	appEnableText( plt->ptLeftIndentText, 0 );
+	}
+    else{
+	appStringToTextWidget( plt->ptListLevelText, "" );
+
+	appEnableText( plt->ptFirstIndentText, 1 );
+	appEnableText( plt->ptLeftIndentText, 1 );
+	}
+
+    appLengthToTextWidget( plt->ptFirstIndentText,
 	    pp->ppLeftIndentTwips+ pp->ppFirstIndentTwips, UNITtyPOINTS );
-    appStringToTextWidget( plt->ptFirstIndentText, scratch );
 
-    appGeoLengthToString( scratch, pp->ppLeftIndentTwips, UNITtyPOINTS );
-    appStringToTextWidget( plt->ptLeftIndentText, scratch );
+    appLengthToTextWidget( plt->ptLeftIndentText,
+				    pp->ppLeftIndentTwips, UNITtyPOINTS );
 
-    appGeoLengthToString( scratch, pp->ppRightIndentTwips, UNITtyPOINTS );
-    appStringToTextWidget( plt->ptRightIndentText, scratch );
+    appLengthToTextWidget( plt->ptRightIndentText,
+				    pp->ppRightIndentTwips, UNITtyPOINTS );
 
     tedFormatToolRefreshSpaceAround( plt->ptSpaceAboveText,
 			    plt->ptSpaceAboveToggle, pp->ppSpaceBeforeTwips );
@@ -74,7 +82,8 @@ static void tedFormatToolRefreshParagraphPage(	ParagraphLayoutTool *	plt )
 
     appSetOptionmenu( &(plt->ptAlignOptionmenu), pp->ppAlignment );
 
-    appGuiSetToggleState( plt->ptOnNewPageToggle, pp->ppStartsOnNewPage );
+    appGuiSetToggleState( plt->ptOnNewPageToggle,
+					pp->ppBreakKind == DOCibkPAGE );
     appGuiSetToggleState( plt->ptKeepOnPageToggle, pp->ppKeepOnPage );
 
     appGuiSetToggleState( plt->ptWidowControlToggle, pp->ppWidowControl );
@@ -84,40 +93,47 @@ static void tedFormatToolRefreshParagraphPage(	ParagraphLayoutTool *	plt )
     }
 
 
-void tedFormatToolRefreshParaLayoutTool(
-				ParagraphLayoutTool *		plt,
+void tedRefreshParaLayoutTool(	ParagraphLayoutTool *		plt,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	ds )
+				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
+				const SelectionDescription *	sd )
     {
     const ParagraphProperties *		pp;
 
     PropertyMask			ppChgMask;
     PropertyMask			ppUpdMask;
 
-    BufferItem *			bi= ds->dsBegin.dpBi;
+    BufferItem *			bi= ds->dsHead.dpBi;
 
     pp= &(bi->biParaProperties);
 
-    PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
-    PROPmaskUNSET( &ppUpdMask, PPpropIN_TABLE );
+    utilPropMaskClear( &ppUpdMask );
+    utilPropMaskFill( &ppUpdMask, PPprop_FULL_COUNT );
+    PROPmaskUNSET( &ppUpdMask, PPpropTABLE_NESTING );
 
-    PROPmaskCLEAR( &ppChgMask );
+    utilPropMaskClear( &ppChgMask );
 
     if  ( docUpdParaProperties( &ppChgMask, &(plt->ptPropertiesChosen),
-					    &ppUpdMask, pp, (const int *)0 ) )
+					    &ppUpdMask, pp,
+					    (const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return ;	}
 
-    PROPmaskCLEAR( &ppChgMask );
+    utilPropMaskClear( &ppChgMask );
 
     if  ( docUpdParaProperties( &ppChgMask, &(plt->ptPropertiesSet),
-					    &ppUpdMask, pp, (const int *)0 ) )
+					    &ppUpdMask, pp,
+					    (const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return ;	}
 
     appGuiEnableWidget( plt->ptOnNewPageToggle,
 			bi->biInExternalItem == DOCinBODY &&
-			! pp->ppInTable );
+			pp->ppTableNesting == 0 );
+
+    appGuiEnableWidget( is->isDeleteButton,
+			! sd->sdDocumentReadonly && sd->sdIsSingleCell );
 
     appGuiEnableWidget( is->isPrevButton,
 				docPrevParagraph( bi ) != (BufferItem *)0 );
@@ -126,7 +142,7 @@ void tedFormatToolRefreshParaLayoutTool(
 
     tedFormatToolRefreshParagraphPage( plt );
 
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
@@ -143,13 +159,14 @@ static APP_BUTTON_CALLBACK_H( tedFormatParaRevertPushed, w, voidplt )
     PropertyMask		ppChgMask;
     PropertyMask		ppUpdMask;
 
-    PROPmaskCLEAR( &ppChgMask );
+    utilPropMaskClear( &ppChgMask );
 
-    PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+    utilPropMaskClear( &ppUpdMask );
+    utilPropMaskFill( &ppUpdMask, PPprop_FULL_COUNT );
 
     docUpdParaProperties( &ppChgMask, &(plt->ptPropertiesChosen),
-			&ppUpdMask, &(plt->ptPropertiesSet), (const int *)0 );
+					&ppUpdMask, &(plt->ptPropertiesSet),
+					(const DocumentAttributeMap *)0 );
 
     tedFormatToolRefreshParagraphPage( plt );
 
@@ -174,7 +191,6 @@ static APP_TXACTIVATE_CALLBACK_H( tedParaFirstIndentChanged, w, voidplt )
 
     tedFormatValidateDimension( &lipfi, &changed,
 					    plt->ptFirstIndentText, lipfi );
-
     return;
     }
 
@@ -242,9 +258,8 @@ static APP_TXACTIVATE_CALLBACK_H( tedParaLineDistanceChanged, w, voidplt )
     int				value;
     int				changed;
 
-    tedFormatValidateDimension( &value, &changed,
-					    plt->ptLineDistChooser.hcText,
-					    pp->ppLineSpacingTwips );
+    tedHeightToolValidateDimension( &value, &changed,
+			&(plt->ptLineDistChooser), pp->ppLineSpacingTwips );
 
     return;
     }
@@ -285,15 +300,15 @@ static APP_BUTTON_CALLBACK_H( tedFormatSelectPara, w, voidplt )
     return;
     }
 
-static APP_BUTTON_CALLBACK_H( tedFormatDeletePara, w, voidplt )
+static APP_BUTTON_CALLBACK_H( tedLayoutDeletePara, w, voidplt )
     {
     ParagraphLayoutTool *	plt= (ParagraphLayoutTool *)voidplt;
     EditApplication *		ea= plt->ptApplication;
 
-    tedDeleteCurrentParagraph( ea );
+    tedAppDeleteSelectedParagraphs( ea );
     }
 
-static APP_BUTTON_CALLBACK_H( tedFormatInsertPara, w, voidplt )
+static APP_BUTTON_CALLBACK_H( tedLayoutInsertPara, w, voidplt )
     {
     ParagraphLayoutTool *	plt= (ParagraphLayoutTool *)voidplt;
     EditApplication *		ea= plt->ptApplication;
@@ -303,7 +318,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatInsertPara, w, voidplt )
     return;
     }
 
-static APP_BUTTON_CALLBACK_H( tedFormatAppendPara, w, voidplt )
+static APP_BUTTON_CALLBACK_H( tedLayoutAppendPara, w, voidplt )
     {
     ParagraphLayoutTool *	plt= (ParagraphLayoutTool *)voidplt;
     EditApplication *		ea= plt->ptApplication;
@@ -349,7 +364,7 @@ static int tedFormatToolGetSpaceAround(		int *		pValue,
     *pChanged= changed; *pValue=value; return 0;
     }
 
-static APP_BUTTON_CALLBACK_H( tedFormatChangePara, w, voidplt )
+static APP_BUTTON_CALLBACK_H( tedLayoutChangePara, w, voidplt )
     {
     ParagraphLayoutTool *	plt= (ParagraphLayoutTool *)voidplt;
     EditApplication *		ea= plt->ptApplication;
@@ -363,12 +378,13 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangePara, w, voidplt )
 
     PropertyMask		ppUpdMask;
 
+    const int			minValue= 0;
     const int			maxValue= INT_MAX;
     const int			adaptToMax= 0;
 
-    PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
-    PROPmaskUNSET( &ppUpdMask, PPpropIN_TABLE );
+    utilPropMaskClear( &ppUpdMask );
+    utilPropMaskFill( &ppUpdMask, PPprop_FULL_COUNT );
+    PROPmaskUNSET( &ppUpdMask, PPpropTABLE_NESTING );
     PROPmaskUNSET( &ppUpdMask, PPpropTAB_STOPS );
     PROPmaskUNSET( &ppUpdMask, PPpropTOP_BORDER );
     PROPmaskUNSET( &ppUpdMask, PPpropBOTTOM_BORDER );
@@ -382,7 +398,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangePara, w, voidplt )
     li= pp->ppLeftIndentTwips;
     if  ( appGetLengthFromTextWidget( plt->ptLeftIndentText,
 					&li, &changed, UNITtyPOINTS,
-					1, 1, maxValue, adaptToMax ) )
+					minValue, 1, maxValue, adaptToMax ) )
 	{ return;	}
 
     pp->ppLeftIndentTwips= li;
@@ -391,7 +407,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangePara, w, voidplt )
     value= pp->ppRightIndentTwips;
     if  ( appGetLengthFromTextWidget( plt->ptRightIndentText,
 					&value, &changed, UNITtyPOINTS,
-					1, 1, maxValue, adaptToMax ) )
+					minValue, 1, maxValue, adaptToMax ) )
 	{ return;	}
     pp->ppRightIndentTwips= value;
 
@@ -548,7 +564,9 @@ static APP_TOGGLE_CALLBACK_H( tedParaOnNewPageToggled, w, voidplt, voidtbcs )
 
     set= appGuiGetToggleStateFromCallback( w, voidtbcs );
 
-    pp->ppStartsOnNewPage= ( set != 0 ) && ! pp->ppInTable;
+    pp->ppBreakKind= DOCibkNONE;
+    if  ( ( set != 0 ) && pp->ppTableNesting == 0 )
+	{ pp->ppBreakKind= DOCibkPAGE;	}
 
     return;
     }
@@ -624,21 +642,25 @@ void tedFormatFillParagraphLayoutPage(
     docInitParagraphProperties( &(plt->ptPropertiesSet) );
     docInitParagraphProperties( &(plt->ptPropertiesChosen) );
 
+    /**/
+    appMakeLabelAndTextRow( &row, &firstLabel, &(plt->ptListLevelText),
+		    pageWidget, plpr->plprParaListLevel, textColumns, 0 );
+
     /**/
     appMakeLabelAndTextRow( &row, &firstLabel, &(plt->ptFirstIndentText),
-		    pageWidget, plpr->pprParaFirstIndent, textColumns, 1 );
+		    pageWidget, plpr->plprParaFirstIndent, textColumns, 1 );
 
     appGuiSetGotValueCallbackForText( plt->ptFirstIndentText,
 				    tedParaFirstIndentChanged, (void *)plt );
     /**/
     appMakeLabelAndTextRow( &row, &leftLabel, &(plt->ptLeftIndentText),
-		    pageWidget, plpr->pprParaLeftIndent, textColumns, 1 );
+		    pageWidget, plpr->plprParaLeftIndent, textColumns, 1 );
 
     appGuiSetGotValueCallbackForText( plt->ptLeftIndentText,
 				    tedParaLeftIndentChanged, (void *)plt );
     /**/
     appMakeLabelAndTextRow( &row, &rightLabel, &(plt->ptRightIndentText),
-		    pageWidget, plpr->pprParaRightIndent, textColumns, 1 );
+		    pageWidget, plpr->plprParaRightIndent, textColumns, 1 );
 
     appGuiSetGotValueCallbackForText( plt->ptRightIndentText,
 				    tedParaRightIndentChanged, (void *)plt );
@@ -692,17 +714,17 @@ void tedFormatFillParagraphLayoutPage(
     appInspectorMakeButtonRow( &row, pageWidget,
 			&(is->isSelectButton), &(is->isDeleteButton),
 			isr->isrSelectButtonText, isr->isrDeleteButtonText,
-			tedFormatSelectPara, tedFormatDeletePara, plt );
+			tedFormatSelectPara, tedLayoutDeletePara, plt );
 
     appInspectorMakeButtonRow( &row, pageWidget,
 			&(is->isInsertButton), &(is->isAppendButton),
 			isr->isrInsertButtonText, isr->isrAppendButtonText,
-			tedFormatInsertPara, tedFormatAppendPara, plt );
+			tedLayoutInsertPara, tedLayoutAppendPara, plt );
 
     appInspectorMakeButtonRow( &row, pageWidget,
 			&(is->isRevertButton), &(is->isApplyButton),
 			isr->isrRevert, isr->isrApplyToSubject,
-			tedFormatParaRevertPushed, tedFormatChangePara, plt );
+			tedFormatParaRevertPushed, tedLayoutChangePara, plt );
 
     return;
     }
@@ -713,7 +735,7 @@ void tedFormatFillParagraphLayoutPage(
 /*									*/
 /************************************************************************/
 
-void tedFormatFillParagraphLayoutChoosers(
+void tedParaLayoutToolFillChoosers(
 			    ParagraphLayoutTool *			plt,
 			    const ParagraphLayoutPageResources *	plpr )
     {
@@ -731,7 +753,7 @@ void tedFormatFillParagraphLayoutChoosers(
     return;
     }
 
-void tedFormatFinishParaLayoutPage(
+void tedFinishParaLayoutPage(
 				ParagraphLayoutTool *			plt,
 				TedFormatTool *				tft,
 				const ParagraphLayoutPageResources *	plpr )
@@ -797,14 +819,17 @@ static AppConfigurableResource TED_TedParagraphSubjectResourceTable[]=
 static AppConfigurableResource TED_TedParagraphToolResourceTable[]=
     {
     /**/
+    APP_RESOURCE( "formatToolParaListLevel",
+	offsetof(ParagraphLayoutPageResources,plprParaListLevel),
+	"List Level" ),
     APP_RESOURCE( "formatToolParaFirstIndent",
-	offsetof(ParagraphLayoutPageResources,pprParaFirstIndent),
+	offsetof(ParagraphLayoutPageResources,plprParaFirstIndent),
 	"First Line Indent" ),
     APP_RESOURCE( "formatToolParaLeftIndent",
-	offsetof(ParagraphLayoutPageResources,pprParaLeftIndent),
+	offsetof(ParagraphLayoutPageResources,plprParaLeftIndent),
 	"Left Indent" ),
     APP_RESOURCE( "formatToolParaRightIndent",
-	offsetof(ParagraphLayoutPageResources,pprParaRightIndent),
+	offsetof(ParagraphLayoutPageResources,plprParaRightIndent),
 	"Right Indent" ),
 
     /**/
@@ -867,15 +892,24 @@ void tedFormatToolGetParaLayoutResourceTable(
 					ParagraphLayoutPageResources *	plpr,
 					InspectorSubjectResources *	isr )
     {
-    appGuiGetResourceValues( ea, (void *)plpr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)plpr,
 				TED_TedParagraphToolResourceTable,
 				sizeof(TED_TedParagraphToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedParagraphSubjectResourceTable,
 				sizeof(TED_TedParagraphSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
