@@ -11,7 +11,7 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
@@ -29,63 +29,68 @@ static void tedFormatToolRefreshParaOrnamentsPage(
 					ParagraphOrnamentsTool *	pot )
     {
     ParagraphProperties *	pp= &(pot->potPropertiesChosen);
-    EditApplication *		ea= pot->potApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, pot->potApplication );
 
-    TedDocument *		td;
-    BufferDocument *		bd;
-    DocumentProperties *	dp;
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
 
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
+    tedBorderToolSetPropertiesByNumber( &(pot->potTopBorderTool), bd,
+						pp->ppTopBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(pot->potBottomBorderTool), bd,
+						pp->ppBottomBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(pot->potLeftBorderTool), bd,
+						pp->ppLeftBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(pot->potRightBorderTool), bd,
+						pp->ppRightBorderNumber );
 
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
-    dp= &(bd->bdProperties);
-
-    tedBorderToolSetProperties( &(pot->potTopBorderTool), dp,
-						&(pp->ppTopBorder) );
-    tedBorderToolSetProperties( &(pot->potBottomBorderTool), dp,
-						&(pp->ppBottomBorder) );
-
-    tedSetShadingTool( &(pot->potShadingTool), dp, &(pp->ppShading) );
+    tedSetShadingToolByNumber( &(pot->potShadingTool),
+						bd, pp->ppShadingNumber );
 
     return;
     }
 
 
-void tedFormatToolRefreshParaOrnamentsTool(
+void tedRefreshParaOrnamentsTool(
 				ParagraphOrnamentsTool *	pot,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	ds )
+				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
+				const SelectionDescription *	sd )
     {
     const ParagraphProperties *		pp;
 
     PropertyMask			ppChgMask;
     PropertyMask			ppUpdMask;
 
-    BufferItem *			bi= ds->dsBegin.dpBi;
+    BufferItem *			bi= ds->dsHead.dpBi;
 
     pp= &(bi->biParaProperties);
 
-    PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
-    PROPmaskUNSET( &ppUpdMask, PPpropIN_TABLE );
+    utilPropMaskClear( &ppUpdMask );
+    utilPropMaskFill( &ppUpdMask, PPprop_FULL_COUNT );
+    PROPmaskUNSET( &ppUpdMask, PPpropTABLE_NESTING );
     PROPmaskUNSET( &ppUpdMask, PPpropTAB_STOPS );
 
-    PROPmaskCLEAR( &ppChgMask );
+    utilPropMaskClear( &ppChgMask );
 
     if  ( docUpdParaProperties( &ppChgMask, &(pot->potPropertiesChosen),
-					    &ppUpdMask, pp, (const int *)0 ) )
+					    &ppUpdMask, pp,
+					    (const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return ;	}
 
-    PROPmaskCLEAR( &ppChgMask );
+    utilPropMaskClear( &ppChgMask );
 
     if  ( docUpdParaProperties( &ppChgMask, &(pot->potPropertiesSet),
-					    &ppUpdMask, pp, (const int *)0 ) )
+					    &ppUpdMask, pp,
+					    (const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return ;	}
 
+    appGuiEnableWidget( is->isDeleteButton,
+			! sd->sdDocumentReadonly && sd->sdIsSingleCell );
+
     appGuiEnableWidget( is->isPrevButton,
 				docPrevParagraph( bi ) != (BufferItem *)0 );
     appGuiEnableWidget( is->isNextButton,
@@ -93,7 +98,7 @@ void tedFormatToolRefreshParaOrnamentsTool(
 
     tedFormatToolRefreshParaOrnamentsPage( pot );
 
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
@@ -110,13 +115,14 @@ static APP_BUTTON_CALLBACK_H( tedFormatParaRevertPushed, w, voidpot )
     PropertyMask		ppChgMask;
     PropertyMask		ppUpdMask;
 
-    PROPmaskCLEAR( &ppChgMask );
+    utilPropMaskClear( &ppChgMask );
 
-    PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+    utilPropMaskClear( &ppUpdMask );
+    utilPropMaskFill( &ppUpdMask, PPprop_FULL_COUNT );
 
     docUpdParaProperties( &ppChgMask, &(pot->potPropertiesChosen),
-			&ppUpdMask, &(pot->potPropertiesSet), (const int *)0 );
+			&ppUpdMask, &(pot->potPropertiesSet),
+			(const DocumentAttributeMap *)0 );
 
     tedFormatToolRefreshParaOrnamentsPage( pot );
 
@@ -164,7 +170,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatDeletePara, w, voidpot )
     ParagraphOrnamentsTool *	pot= (ParagraphOrnamentsTool *)voidpot;
     EditApplication *		ea= pot->potApplication;
 
-    tedDeleteCurrentParagraph( ea );
+    tedAppDeleteSelectedParagraphs( ea );
     }
 
 static APP_BUTTON_CALLBACK_H( tedFormatInsertPara, w, voidpot )
@@ -200,72 +206,49 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangePara, w, voidpot )
     ParagraphProperties *	pp= &(pot->potPropertiesChosen);
 
     PropertyMask		ppSetMask;
-    PropertyMask		xxSetMask;
+    int				changed;
 
     /**/
-    EditApplication *		ea= pot->potApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, pot->potApplication );
 
-    TedDocument *		td;
-    BufferDocument *		bd;
-    DocumentProperties *	dp;
-
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
-    dp= &(bd->bdProperties);
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
 
     /**/
-    PROPmaskCLEAR( &ppSetMask );
+    utilPropMaskClear( &ppSetMask );
     PROPmaskADD( &ppSetMask, PPpropTOP_BORDER );
     PROPmaskADD( &ppSetMask, PPpropBOTTOM_BORDER );
+    PROPmaskADD( &ppSetMask, PPpropLEFT_BORDER );
+    PROPmaskADD( &ppSetMask, PPpropRIGHT_BORDER );
 
-    PROPmaskADD( &ppSetMask, PPpropSHADE_FORE_COLOR );
-    PROPmaskADD( &ppSetMask, PPpropSHADE_BACK_COLOR );
-    PROPmaskADD( &ppSetMask, PPpropSHADE_LEVEL );
-    PROPmaskADD( &ppSetMask, PPpropSHADE_PATTERN );
+    PROPmaskADD( &ppSetMask, PPpropSHADING );
 
     /**/
-    PROPmaskCLEAR( &xxSetMask );
-    if  ( tedBorderToolGetProperties( &(pp->ppTopBorder), &xxSetMask,
-					    &(pot->potTopBorderTool), dp ) )
-	{ return;	}
-
-    PROPmaskCLEAR( &xxSetMask );
-    if  ( tedBorderToolGetProperties( &(pp->ppBottomBorder), &xxSetMask,
-					    &(pot->potBottomBorderTool), dp ) )
-	{ return;	}
+    if  ( tedBorderToolGetNumber( &(pp->ppTopBorderNumber), &changed,
+					    &(pot->potTopBorderTool), bd ) )
+	{ LDEB(1); return;	}
+    if  ( tedBorderToolGetNumber( &(pp->ppBottomBorderNumber), &changed,
+					    &(pot->potBottomBorderTool), bd ) )
+	{ LDEB(1); return;	}
+    if  ( tedBorderToolGetNumber( &(pp->ppLeftBorderNumber), &changed,
+					    &(pot->potLeftBorderTool), bd ) )
+	{ LDEB(1); return;	}
+    if  ( tedBorderToolGetNumber( &(pp->ppRightBorderNumber), &changed,
+					    &(pot->potRightBorderTool), bd ) )
+	{ LDEB(1); return;	}
 
     {
     PropertyMask		isSetMask;
 
-    ExpandedItemShading		eis;
+    utilPropMaskClear( &isSetMask );
 
-    docInitExpandedItemShading( &eis );
-
-    PROPmaskCLEAR( &isSetMask );
-
-    if  ( tedShadingToolGetShading( &eis, &isSetMask,
-						    &(pot->potShadingTool) ) )
+    if  ( tedShadingToolGetShadingNumber( &(pp->ppShadingNumber), &isSetMask,
+						&(pot->potShadingTool), bd ) )
 	{ return;	}
 
-    if  ( ! PROPmaskISEMPTY( &isSetMask ) )
-	{
-	PropertyMask	isDoneMask;
-
-	PROPmaskCLEAR( &isDoneMask );
-
-	if  ( docIndirectItemShading( &isDoneMask,
-				    &(pp->ppShading), &isSetMask, &eis,
-				    &(dp->dpColors), &(dp->dpColorCount) ) )
-	    { LDEB(1); return;	}
-
-	docShadingMaskToParagraphMask( &isSetMask, &isSetMask );
-
-	utilPropMaskOr( &ppSetMask, &ppSetMask, &isSetMask );
-	}
+    if  ( ! utilPropMaskIsEmpty( &isSetMask ) )
+	{ PROPmaskADD( &ppSetMask, PPpropSHADING );	}
     }
 
     /**/
@@ -281,6 +264,10 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangePara, w, voidpot )
 /*									*/
 /************************************************************************/
 
+/* values are arbitrary but cannot be equal to real PPpropSOMETHING values */
+# define PPprop_SHADE_FORE_COLOR PPprop_FULL_COUNT+ 1
+# define PPprop_SHADE_BACK_COLOR PPprop_FULL_COUNT+ 2
+
 static void tedParaOrnamentsGotColor(	void *			voidpot,
 					int 			which,
 					const RGB8Color *	rgb8 )
@@ -293,19 +280,20 @@ static void tedParaOrnamentsGotColor(	void *			voidpot,
 	    tedBorderSetExplicitColorChoice(
 					&(pot->potTopBorderTool), rgb8 );
 	    break;
-
 	case PPpropBOTTOM_BORDER:
 	    tedBorderSetExplicitColorChoice(
 					&(pot->potBottomBorderTool), rgb8 );
 	    break;
-
 	case PPpropLEFT_BORDER:
-	    LDEB(which); return;
+	    tedBorderSetExplicitColorChoice(
+					&(pot->potLeftBorderTool), rgb8 );
 	    break;
 	case PPpropRIGHT_BORDER:
-	    LDEB(which); return;
+	    tedBorderSetExplicitColorChoice(
+					&(pot->potRightBorderTool), rgb8 );
 	    break;
-	case PPpropBOX_BORDER:
+
+	case PPprop_BOX_BORDER:
 	    LDEB(which); return;
 	    break;
 	case PPpropBETWEEN_BORDER:
@@ -315,8 +303,8 @@ static void tedParaOrnamentsGotColor(	void *			voidpot,
 	    LDEB(which); return;
 	    break;
 
-	case PPpropSHADE_BACK_COLOR:
-	case PPpropSHADE_FORE_COLOR:
+	case PPprop_SHADE_BACK_COLOR:
+	case PPprop_SHADE_FORE_COLOR:
 	    tedShadeSetExplicitColorChoice( &(pot->potShadingTool),
 							    which, rgb8 );
 	    break;
@@ -356,15 +344,20 @@ void tedFormatFillParagraphOrnamentsPage(
     tedMakeBorderTool( &(pot->potTopBorderTool), ai, pageWidget,
 	    popr->poprTopBorder, &(popr->poprBorderToolResources),
 	    subjectPage, PPpropTOP_BORDER );
-
     tedMakeBorderTool( &(pot->potBottomBorderTool), ai, pageWidget,
 	    popr->poprBottomBorder, &(popr->poprBorderToolResources),
 	    subjectPage, PPpropBOTTOM_BORDER );
+    tedMakeBorderTool( &(pot->potLeftBorderTool), ai, pageWidget,
+	    popr->poprLeftBorder, &(popr->poprBorderToolResources),
+	    subjectPage, PPpropLEFT_BORDER );
+    tedMakeBorderTool( &(pot->potRightBorderTool), ai, pageWidget,
+	    popr->poprRightBorder, &(popr->poprBorderToolResources),
+	    subjectPage, PPpropRIGHT_BORDER );
 
     /**/
     tedFormatMakeShadingTool( &(pot->potShadingTool), ai, pageWidget,
 		popr->poprShading, &(popr->poprShadingResources),
-		subjectPage, PPpropSHADE_FORE_COLOR, PPpropSHADE_BACK_COLOR,
+		subjectPage, PPprop_SHADE_FORE_COLOR, PPprop_SHADE_BACK_COLOR,
 		(TedShadingToolCallback)0, (void *)pot );
 
 
@@ -398,7 +391,7 @@ void tedFormatFillParagraphOrnamentsPage(
 /*									*/
 /************************************************************************/
 
-void tedFormatFillParagraphOrnamentsChoosers(
+void tedParaOrnamentsToolFillChoosers(
 			    ParagraphOrnamentsTool *			pot,
 			    const ParagraphOrnamentsPageResources *	popr )
     {
@@ -412,6 +405,8 @@ void tedFormatFinishParaOrnamentsPage(
     {
     tedFinishBorderTool( &(pot->potTopBorderTool) );
     tedFinishBorderTool( &(pot->potBottomBorderTool) );
+    tedFinishBorderTool( &(pot->potLeftBorderTool) );
+    tedFinishBorderTool( &(pot->potRightBorderTool) );
 
     tedFinishShadingTool( &(pot->potShadingTool) );
 
@@ -435,6 +430,8 @@ void tedInitParaOrnamentsTool(	ParagraphOrnamentsTool *	pot )
 
     tedInitBorderTool( &(pot->potTopBorderTool) );
     tedInitBorderTool( &(pot->potBottomBorderTool) );
+    tedInitBorderTool( &(pot->potLeftBorderTool) );
+    tedInitBorderTool( &(pot->potRightBorderTool) );
     
     tedInitShadingTool( &(pot->potShadingTool) );
 
@@ -448,6 +445,8 @@ void tedCleanParaOrnamentsTool(	ParagraphOrnamentsTool *	pot )
 
     tedCleanBorderTool( &(pot->potTopBorderTool) );
     tedCleanBorderTool( &(pot->potBottomBorderTool) );
+    tedCleanBorderTool( &(pot->potLeftBorderTool) );
+    tedCleanBorderTool( &(pot->potRightBorderTool) );
     
     tedCleanShadingTool( &(pot->potShadingTool) );
 
@@ -500,6 +499,12 @@ static AppConfigurableResource TED_TedParagraphToolResourceTable[]=
     APP_RESOURCE( "formatToolParaBottomBorder",
 	offsetof(ParagraphOrnamentsPageResources,poprBottomBorder),
 	"Bottom Border" ),
+    APP_RESOURCE( "formatToolParaLeftBorder",
+	offsetof(ParagraphOrnamentsPageResources,poprLeftBorder),
+	"Left Border" ),
+    APP_RESOURCE( "formatToolParaRightBorder",
+	offsetof(ParagraphOrnamentsPageResources,poprRightBorder),
+	"Right Border" ),
 
     /**/
     APP_RESOURCE( "formatToolParaBorderWidth",
@@ -570,15 +575,24 @@ void tedFormatToolGetParaOrnamentsResourceTable(
 			    ParagraphOrnamentsPageResources *	popr,
 			    InspectorSubjectResources *		isr )
     {
-    appGuiGetResourceValues( ea, (void *)popr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)popr,
 				TED_TedParagraphToolResourceTable,
 				sizeof(TED_TedParagraphToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedParagraphSubjectResourceTable,
 				sizeof(TED_TedParagraphSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
