@@ -11,7 +11,7 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
@@ -30,67 +30,69 @@ static void tedFormatToolRefreshRowPage( RowTool *	rt )
     const RowProperties *	rp= &(rt->rtPropertiesChosen);
     const CellProperties *	cp;
     const TableRectangle *	tr= &(rt->rtTableRectangle);
-    char			scratch[50];
 
-    EditApplication *		ea= rt->rtApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, rt->rtApplication );
 
-    TedDocument *		td;
-    BufferDocument *		bd;
-    DocumentProperties *	dp;
-
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
-    dp= &(bd->bdProperties);
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
 
     appIntegerToTextWidget( rt->rtNumberText, tr->trRow0- tr->trRow00+ 1 );
 
     tedFormatRefreshHeightChooser( &(rt->rtHeightChooser),
 							rp->rpHeightTwips );
 
-    appGeoLengthToString( scratch, rp->rpHalfGapWidthTwips, UNITtyPOINTS );
-    appStringToTextWidget( rt->rtCellMarginText, scratch );
+    appLengthToTextWidget( rt->rtCellMarginText,
+				    rp->rpHalfGapWidthTwips, UNITtyPOINTS );
 
-    appGeoLengthToString( scratch, rp->rpLeftIndentTwips, UNITtyPOINTS );
-    appStringToTextWidget( rt->rtLeftIndentText, scratch );
+    appLengthToTextWidget( rt->rtLeftIndentText,
+				    rp->rpLeftIndentTwips, UNITtyPOINTS );
 
     cp= rp->rpCells;
-    tedBorderToolSetProperties( &(rt->rtTopBorderTool), dp,
-					&(cp[tr->trCol0].cpTopBorder) );
-    tedBorderToolSetProperties( &(rt->rtBottomBorderTool), dp,
-					&(cp[tr->trCol0].cpBottomBorder) );
+    tedBorderToolSetPropertiesByNumber( &(rt->rtTopBorderTool), bd,
+				    cp[tr->trCol0].cpTopBorderNumber );
+    tedBorderToolSetPropertiesByNumber( &(rt->rtBottomBorderTool), bd,
+				    cp[tr->trCol0].cpBottomBorderNumber );
+
+    tedSetShadingToolByNumber( &(rt->rtShadingTool), bd, rp->rpShadingNumber );
 
     appGuiSetToggleState( rt->rtKeepOnPageToggle,
 						rp->rpKeepOnOnePage );
     appGuiSetToggleState( rt->rtIsTableHeaderToggle,
 						rp->rpIsTableHeader );
+
+    if  ( rt->rtKeepfollowToggle )
+	{
+	appGuiSetToggleState( rt->rtKeepfollowToggle, rp->rp_Keepfollow );
+	}
+
     /*
     appGuiSetToggleState( rt->rtAutofitToggle, rp->rpAutofit );
     */
     }
 
-void tedFormatToolRefreshRowTool(
-				RowTool *			rt,
+void tedRefreshRowTool(		RowTool *			rt,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	ds )
+				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
+				const SelectionDescription *	sd )
     {
     TableRectangle *		tr= &(rt->rtTableRectangle);
 
+
     const RowProperties *	rp;
     const DocumentGeometry *	dg;
 
     const BufferItem *		rowBi;
     const BufferItem *		sectBi;
 
-    if  ( docGetTableRectangle( tr, ds )	||
-	  tr->trRow1 != tr->trRow0		)
-	{ docInitTableRectangle( tr ); *pEnabled= 0; return; }
+    rt->rtTableRectangle= sd->sdTableRectangle;
+    if  ( ! sd->sdIsTableRectangle )
+	{ *pEnabled= 0; return; }
 
-    rowBi= ds->dsBegin.dpBi;
+    rowBi= ds->dsHead.dpBi;
     rowBi= rowBi->biParent;
     rowBi= rowBi->biParent;
     sectBi= rowBi->biParent;
@@ -103,17 +105,17 @@ void tedFormatToolRefreshRowTool(
 
     rp= &(rowBi->biRowProperties);
 
-    if  ( docCopyRowProperties( &(rt->rtPropertiesChosen),
-						    rp, (const int *)0 ) )
+    if  ( docCopyRowProperties( &(rt->rtPropertiesChosen), rp,
+					(const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return;	}
-    if  ( docCopyRowProperties( &(rt->rtPropertiesSet),
-						    rp, (const int *)0 ) )
+    if  ( docCopyRowProperties( &(rt->rtPropertiesSet), rp,
+					(const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return;	}
 
-    appGuiEnableWidget( is->isPrevButton,
-				    tr->trRow0 > tr->trRow00 );
-    appGuiEnableWidget( is->isNextButton,
-				    tr->trRow1 < tr->trRow11 );
+    appGuiEnableWidget( is->isPrevButton, tr->trRow0 > tr->trRow00 );
+    appGuiEnableWidget( is->isNextButton, tr->trRow1 < tr->trRow11 );
+    appGuiEnableWidget( is->isMoveUpButton, tr->trRow0 > tr->trRow00 );
+    appGuiEnableWidget( is->isMoveDownButton, tr->trRow1 < tr->trRow11 );
 
     appGuiEnableWidget( rt->rtIsTableHeaderToggle,
 				rowBi->biInExternalItem == DOCinBODY	&&
@@ -122,7 +124,7 @@ void tedFormatToolRefreshRowTool(
 
     tedFormatToolRefreshRowPage( rt );
 
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
@@ -156,29 +158,22 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangeRowPushed, w, voidrt )
     PropertyMask		bpSetMask;
 
     /**/
-    EditApplication *		ea= rt->rtApplication;
-    EditDocument *		ed= ea->eaCurrentDocument;
-
-    TedDocument *		td;
-    BufferDocument *		bd;
+    EditDocument *		ed;
+    BufferDocument *		bd= tedFormatCurDoc( &ed, rt->rtApplication );
     DocumentProperties *	dp;
 
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-    bd= td->tdDocument;
+    if  ( ! bd )
+	{ XDEB(bd); return;	}
     dp= &(bd->bdProperties);
 
     /**/
-    PROPmaskCLEAR( &rpSetMask );
-    PROPmaskCLEAR( &cpSetMask );
+    utilPropMaskClear( &rpSetMask );
+    utilPropMaskClear( &cpSetMask );
 
     PROPmaskADD( &rpSetMask, RPpropKEEP_ON_ONE_PAGE );
     PROPmaskADD( &rpSetMask, RPpropIS_TABLE_HEADER );
-    /* No! not yet
-    PROPmaskADD( &rpSetMask, RPpropKEEP_WITH_NEXT );
-    */
+
+    PROPmaskADD( &rpSetMask, RPprop_KEEPFOLLOW );
 
     docInitBorderProperties( &bp );
 
@@ -206,30 +201,46 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangeRowPushed, w, voidrt )
     docExpandTableRectangleToWholeRows( &tr );
 
     /***/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 						&(rt->rtTopBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
-	docRowSetTopBorderInCols( rp, tr.trCol0, tr.trCol1, &bpSetMask, &bp );
+	docRowSetTopBorderInCols( rp, tr.trCol0, tr.trCol1,
+						&(bd->bdBorderPropertyList),
+						&bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropTOP_BORDER );
 	}
 
     /***/
-    PROPmaskCLEAR( &bpSetMask );
+    utilPropMaskClear( &bpSetMask );
     if  ( tedBorderToolGetProperties( &bp, &bpSetMask,
 					    &(rt->rtBottomBorderTool), dp ) )
 	{ goto ready;	}
 
-    if  ( ! PROPmaskISEMPTY( &bpSetMask ) )
+    if  ( ! utilPropMaskIsEmpty( &bpSetMask ) )
 	{
 	docRowSetBottomBorderInCols( rp, tr.trCol0, tr.trCol1,
-							&bpSetMask, &bp );
+						&(bd->bdBorderPropertyList),
+						&bpSetMask, &bp );
 	PROPmaskADD( &cpSetMask, CLpropBOTTOM_BORDER );
 	}
 
+    {
+    PropertyMask		isSetMask;
+
+    utilPropMaskClear( &isSetMask );
+
+    if  ( tedShadingToolGetShadingNumber( &(rp->rpShadingNumber), &isSetMask,
+						&(rt->rtShadingTool), bd ) )
+	{ return;	}
+
+    if  ( ! utilPropMaskIsEmpty( &isSetMask ) )
+	{ PROPmaskADD( &rpSetMask, RPpropSHADING );	}
+    }
+
     /***/
     tedAppSetTableProperties( rt->rtApplication, &tr,
 					    &rpSetMask, &cpSetMask, rp );
@@ -267,10 +278,7 @@ static APP_TXACTIVATE_CALLBACK_H( tedRowCellMarginChanged, w, voidrt )
 
     if  ( ! tedFormatToolGetGapWidth( rp, &width, rt->rtCellMarginText ) )
 	{
-	char	scratch[50];
-
-	appGeoLengthToString( scratch, width, UNITtyPOINTS );
-	appStringToTextWidget( rt->rtCellMarginText, scratch );
+	appLengthToTextWidget( rt->rtCellMarginText, width, UNITtyPOINTS );
 	}
     }
 
@@ -284,10 +292,7 @@ static APP_TXACTIVATE_CALLBACK_H( tedRowLeftIndentChanged, w, voidrt )
     if  ( ! tedFormatToolGetRowLeftIndent( rp, rt->rtPageLeftMargin, &value,
 						    rt->rtLeftIndentText ) )
 	{
-	char	scratch[50];
-
-	appGeoLengthToString( scratch, value, UNITtyPOINTS );
-	appStringToTextWidget( rt->rtLeftIndentText, scratch );
+	appLengthToTextWidget( rt->rtLeftIndentText, value, UNITtyPOINTS );
 	}
     }
 
@@ -301,8 +306,8 @@ static APP_BUTTON_CALLBACK_H( tedFormatRevertRowPushed, w, voidrt )
     {
     RowTool *	rt= (RowTool *)voidrt;
 
-    docCopyRowProperties( &(rt->rtPropertiesChosen),
-				&(rt->rtPropertiesSet), (const int *)0 );
+    docCopyRowProperties( &(rt->rtPropertiesChosen), &(rt->rtPropertiesSet),
+					(const DocumentAttributeMap *)0 );
 
     tedFormatToolRefreshRowPage( rt );
 
@@ -314,7 +319,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatPreviousRow, w, voidrt )
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -322,7 +326,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatPreviousRow, w, voidrt )
 	{ XDEB(ed); return;	}
 
     tr= rt->rtTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     docExpandTableRectangleToWholeRows( &tr );
 
@@ -339,7 +342,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatSelectRow, w, voidrt )
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -347,7 +349,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatSelectRow, w, voidrt )
 	{ XDEB(ed); return;	}
 
     tr= rt->rtTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     docExpandTableRectangleToWholeRows( &tr );
 
@@ -361,7 +362,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatNextRow, w, voidrt )
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
     EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
 
     TableRectangle	tr;
 
@@ -369,7 +369,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatNextRow, w, voidrt )
 	{ XDEB(ed); return;	}
 
     tr= rt->rtTableRectangle;
-    td= (TedDocument *)ed->edPrivateData;
 
     docExpandTableRectangleToWholeRows( &tr );
 
@@ -413,7 +412,6 @@ static APP_BUTTON_CALLBACK_H( tedFormatInsertRow, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
     EditApplication *	ea= rt->rtApplication;
-
     EditDocument *	ed= ea->eaCurrentDocument;
 
     if  ( ! ed )
@@ -438,6 +436,34 @@ static APP_BUTTON_CALLBACK_H( tedFormatAppendRow, w, voidrt )
     appDocumentChanged( ed, 1 );
     }
 
+static APP_BUTTON_CALLBACK_H( tedRowToolMoveDownRow, w, voidrt )
+    {
+    RowTool *		rt= (RowTool *)voidrt;
+    EditApplication *	ea= rt->rtApplication;
+    EditDocument *	ed= ea->eaCurrentDocument;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    tedRollRowsInTable( ed, +1 );
+
+    appDocumentChanged( ed, 1 );
+    }
+
+static APP_BUTTON_CALLBACK_H( tedRowToolMoveUpRow, w, voidrt )
+    {
+    RowTool *		rt= (RowTool *)voidrt;
+    EditApplication *	ea= rt->rtApplication;
+    EditDocument *	ed= ea->eaCurrentDocument;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    tedRollRowsInTable( ed, -1 );
+
+    appDocumentChanged( ed, 1 );
+    }
+
 static APP_OITEM_CALLBACK_H( tedRowHeightChosen, w, voidrt )
     {
     RowTool *		rt= (RowTool *)voidrt;
@@ -493,12 +519,29 @@ static APP_TOGGLE_CALLBACK_H( tedRowToggleIsTableHeader, w, voidrt, voidtbcs )
     return;
     }
 
+static APP_TOGGLE_CALLBACK_H( tedRowToggleKeepWithNext, w, voidrt, voidtbcs )
+    {
+    RowTool *				rt= (RowTool *)voidrt;
+    RowProperties *			rp= &(rt->rtPropertiesChosen);
+    int					set;
+
+    set= appGuiGetToggleStateFromCallback( w, voidtbcs );
+
+    rp->rp_Keepfollow= set;
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Callback from the general color chooser: a color was chosen.	*/
 /*									*/
 /************************************************************************/
 
+/* values are arbitrary but cannot be equal to real RPpropSOMETHING values */
+# define RPprop_SHADE_FORE_COLOR RPprop_COUNT+ 1
+# define RPprop_SHADE_BACK_COLOR RPprop_COUNT+ 2
+
 static void tedRowToolGotColor(		void *			voidrt,
 					int 			which,
 					const RGB8Color *	rgb8 )
@@ -515,6 +558,13 @@ static void tedRowToolGotColor(		void *			voidrt,
 	    tedBorderSetExplicitColorChoice( &(rt->rtBottomBorderTool), rgb8 );
 	    break;
 
+	case RPprop_SHADE_BACK_COLOR:
+	case RPprop_SHADE_FORE_COLOR:
+	    tedShadeSetExplicitColorChoice( &(rt->rtShadingTool),
+							    which, rgb8 );
+	    break;
+
+
 	default:
 	    LDEB(which); return;
 	}
@@ -579,12 +629,24 @@ void tedFormatFillRowPage(  RowTool *				rt,
 	    rpr->rprBottomBorder, &(rpr->rprBorderToolResources),
 	    subjectPage, CLpropBOTTOM_BORDER );
 
+    /**/
+    tedFormatMakeShadingTool( &(rt->rtShadingTool), ai, pageWidget,
+		rpr->rprShading, &(rpr->rprShadingResources),
+		subjectPage, RPprop_SHADE_FORE_COLOR, RPprop_SHADE_BACK_COLOR,
+		(TedShadingToolCallback)0, (void *)rt );
+
     /**************/
     appInspectorMakeToggleRow( &row, pageWidget,
 	    &(rt->rtKeepOnPageToggle), &(rt->rtIsTableHeaderToggle),
 	    rpr->rprKeepOnPage, rpr->rprIsTableHeader,
 	    tedRowToggleKeepOnPage, tedRowToggleIsTableHeader, (void *)rt );
 
+    appInspectorMakeToggleAndLabelRow( &row, pageWidget,
+	    &(rt->rtKeepfollowToggle), &(rt->rtKeepfollowLabel),
+	    rpr->rprKeepWithNext, "",
+	    tedRowToggleKeepWithNext, (void *)rt );
+
+    /**************/
     appInspectorMakeButtonRow( &row, pageWidget,
 	    &(is->isPrevButton), &(is->isNextButton),
 	    isr->isrPrevButtonText, isr->isrNextButtonText,
@@ -600,6 +662,11 @@ void tedFormatFillRowPage(  RowTool *				rt,
 	    isr->isrInsertButtonText, isr->isrAppendButtonText,
 	    tedFormatInsertRow, tedFormatAppendRow, (void *)rt );
 
+    appInspectorMakeButtonRow( &row, pageWidget,
+	    &(is->isMoveUpButton), &(is->isMoveDownButton),
+	    isr->isrMoveUpButtonText, isr->isrMoveDownButtonText,
+	    tedRowToolMoveUpRow, tedRowToolMoveDownRow, (void *)rt );
+
     appInspectorMakeButtonRow( &row, pageWidget,
 	    &(is->isRevertButton), &(is->isApplyButton),
 	    isr->isrRevert, isr->isrApplyToSubject,
@@ -608,7 +675,7 @@ void tedFormatFillRowPage(  RowTool *				rt,
     return;
     }
 
-void tedFormatFillRowChoosers(		RowTool *			rt,
+void tedRowToolFillChoosers(		RowTool *			rt,
 					const RowPageResources *	rpr )
     {
     tedFormatFillHeightChooser( &(rt->rtHeightChooser),
@@ -619,13 +686,14 @@ void tedFormatFillRowChoosers(		RowTool *			rt,
     }
 
 void tedFormatFinishRowPage( 	RowTool *			rt,
-				TedFormatTool *			tft,
 				const RowPageResources *	rpr )
     {
     appOptionmenuRefreshWidth( &(rt->rtHeightChooser.hcOptionmenu) );
 
     tedFinishBorderTool( &(rt->rtTopBorderTool) );
     tedFinishBorderTool( &(rt->rtBottomBorderTool) );
+
+    tedFinishShadingTool( &(rt->rtShadingTool) );
     }
 
 /************************************************************************/
@@ -656,6 +724,8 @@ void tedInitRowTool(	RowTool *	rt )
     tedInitBorderTool( &(rt->rtTopBorderTool) );
     tedInitBorderTool( &(rt->rtBottomBorderTool) );
 
+    tedInitShadingTool( &(rt->rtShadingTool) );
+
     rt->rtKeepOnPageToggle= (APP_WIDGET)0;
     rt->rtIsTableHeaderToggle= (APP_WIDGET)0;
 
@@ -670,6 +740,8 @@ void tedCleanRowTool(	RowTool *	rt )
     tedCleanBorderTool( &(rt->rtTopBorderTool) );
     tedCleanBorderTool( &(rt->rtBottomBorderTool) );
 
+    tedCleanShadingTool( &(rt->rtShadingTool) );
+
     return;
     }
 
@@ -708,6 +780,12 @@ static AppConfigurableResource TED_TedRowSubjectResourceTable[]=
     APP_RESOURCE( "tableToolAppendRow",
 		offsetof(InspectorSubjectResources,isrAppendButtonText),
 		"Append Row" ),
+    APP_RESOURCE( "rowToolMoveDown",
+		offsetof(InspectorSubjectResources,isrMoveDownButtonText),
+		"Move Down" ),
+    APP_RESOURCE( "rowToolMoveUp",
+		offsetof(InspectorSubjectResources,isrMoveUpButtonText),
+		"Move Up" ),
     };
 
 static AppConfigurableResource TED_TedRowToolResourceTable[]=
@@ -745,6 +823,44 @@ static AppConfigurableResource TED_TedRowToolResourceTable[]=
 			    btrColorChooserResources.ccrMoreColors),
 	    "More Colors..." ),
 
+    /**/
+    APP_RESOURCE( "formatToolRowShading",
+	    offsetof(RowPageResources,rprShading),
+	    "Shading" ),
+
+    APP_RESOURCE( "formatToolRowShadingPattern",
+	    offsetof(RowPageResources,rprShadingResources.strPattern),
+	    "Pattern" ),
+    APP_RESOURCE( "formatToolRowShadingLevel",
+	    offsetof(RowPageResources,rprShadingResources.strLevel),
+	    "Level" ),
+    APP_RESOURCE( "formatToolRowShadingForeColor",
+	    offsetof(RowPageResources,rprShadingResources.strForeColor),
+	    "Foreground" ),
+    APP_RESOURCE( "formatToolRowShadingBackColor",
+	    offsetof(RowPageResources,rprShadingResources.strBackColor),
+	    "Background" ),
+
+    /**/
+    APP_RESOURCE( "formatToolRowShadingBackColorChooserNoBackground",
+	    offsetof(RowPageResources,rprShadingResources.
+			    strBackColorChooserResources.ccrAutomaticColor),
+	    "No Background" ),
+    APP_RESOURCE( "formatToolRowShadingBackColorChooserMoreColors",
+	    offsetof(RowPageResources,rprShadingResources.
+			    strBackColorChooserResources.ccrMoreColors),
+	    "More Colors..." ),
+
+    APP_RESOURCE( "formatToolRowShadingForeColorChooserAutomatic",
+	    offsetof(RowPageResources,rprShadingResources.
+			    strForeColorChooserResources.ccrAutomaticColor),
+	    "Automatic" ),
+    APP_RESOURCE( "formatToolRowShadingForeColorChooserMoreColors",
+	    offsetof(RowPageResources,rprShadingResources.
+			    strForeColorChooserResources.ccrMoreColors),
+	    "More Colors..." ),
+
+
     /**/
     APP_RESOURCE( "formatToolRowKeepOnPage",
 	    offsetof(RowPageResources,rprKeepOnPage),
@@ -753,6 +869,10 @@ static AppConfigurableResource TED_TedRowToolResourceTable[]=
 	    offsetof(RowPageResources,rprIsTableHeader),
 	    "Is Table Header" ),
 
+    APP_RESOURCE( "formatToolRowKeepWithNext",
+	    offsetof(RowPageResources,rprKeepWithNext),
+	    "Keep with Next" ),
+
     /**/
     APP_RESOURCE( "formatToolRowBorderWidth",
 	    offsetof(RowPageResources, rprBorderToolResources.btrWidth),
@@ -769,15 +889,24 @@ void tedFormatToolGetRowResourceTable(	EditApplication *		ea,
 					RowPageResources *		rpr,
 					InspectorSubjectResources *	isr )
     {
-    appGuiGetResourceValues( ea, (void *)rpr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)rpr,
 				TED_TedRowToolResourceTable,
 				sizeof(TED_TedRowToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedRowSubjectResourceTable,
 				sizeof(TED_TedRowSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
