@@ -9,7 +9,7 @@
 #   include	<appFrame.h>
 #   include	<appRuler.h>
 
-#   include	"docBuf.h"
+#   include	<docBuf.h>
 
 typedef struct ColumnSeparator
     {
@@ -39,56 +39,42 @@ extern void * tedMakeTopRuler(	int			height,
 				const char *		rulerFont,
 				int			whatUnit );
 
-extern void * tedMakeLeftRuler(	int			width,
-				double			magnifiedPixelsPerTwip,
-				double			magnification,
-
-				int			minUnused,
-				int			maxUnused,
-				int			docY0,
-				int			docY1,
-
-				int			docVisY0,
-				int			docVisY1,
-
-				int			rulerC1,
+extern void * tedMakeBottomRuler( int			height,
+				int			leftRulerWidth,
+				int			rightRulerWidth,
 				const char *		rulerFont,
-				int			whatUnit,
-				int			pageStep,
-				const char *		tlrPageFormat );
+				const char *		pageFormat );
 
-extern void tedFreeHorizontalRuler(	void *		voidttr );
-extern void tedFreeVerticalRuler(	void *		voidtlr );
+extern void tedFreeTopRuler(	void *		voidttr );
+extern void tedFreeBottomRuler(	void *		voidtlr );
 
-extern APP_EVENT_HANDLER_H( tedRedrawHorizontalRuler, w, voidttr, event );
-extern APP_EVENT_HANDLER_H( tedRedrawVerticalRuler, w, voidtlr, event );
+extern APP_EVENT_HANDLER_H( tedRedrawTopRuler, w, voidttr, event );
+extern APP_EVENT_HANDLER_H( tedRedrawBottomRuler, w, voidtlr, event );
 
-extern void tedScrollHorizontalRuler(	void *			voidttr,
+extern void tedScrollTopRuler(	void *			voidttr,
 					APP_WIDGET		w,
 					int			d	);
 
-extern void tedSetHorizontalRulerRange(	void *		voidttr,
+extern void tedSetTopRulerRange(	void *		voidttr,
 					APP_WIDGET	w,
 					int		docVisX0,
 					int		docVisX1,
 					int		docBackX1 );
 
-extern void tedAdaptHorizontalRuler(	void *		voidttr,
-					APP_WIDGET	w,
-					int		docX0,
-					int		docX1,
-					int		firstIndent,
-					int		leftIndent,
-					int		rightIndent,
-					int		rulerC1,
-					int		tabCount,
-					TabStop *	tabStops );
+extern void tedAdaptTopRuler(	void *			voidttr,
+					APP_WIDGET		w,
+					int			docX0,
+					int			docX1,
+					int			firstIndent,
+					int			leftIndent,
+					int			rightIndent,
+					int			rulerC1,
+					const TabStopList *	tslSet );
 
-extern void tedHorizontalRulerTrackMouse( int *			pFirstIndent,
+extern void tedTopRulerTrackMouse( int *			pFirstIndent,
 					int *			pLeftIndent,
 					int *			pRightIndent,
-					int *			pTabCount,
-					TabStop **		pTabStops,
+					TabStopList *		tsl,
 					int *			pCsCount,
 					ColumnSeparator **	pCs,
 					int *			pProperty,
@@ -96,31 +82,24 @@ extern void tedHorizontalRulerTrackMouse( int *			pFirstIndent,
 					EditApplication *	ea,
 					APP_EVENT *		downEvent,
 					void *			voidttr,
-					void *			voided,
-					DragHair		dragHair );
+					EditDocument *		ed );
 
-extern APP_EVENT_HANDLER_H( tedHorizontalRulerConfigure, w, voidttr, event );
-extern APP_EVENT_HANDLER_H( tedVerticalRulerConfigure, w, voidtlr, event );
+extern APP_EVENT_HANDLER_H( tedTopRulerConfigure, w, voidttr, event );
+extern APP_EVENT_HANDLER_H( tedBottomRulerConfigure, w, voidtlr, event );
 
 extern int tedSetRulerColumns(	APP_WIDGET		w,
 				void *			voidttr,
-				int			colX0,
-				int			colX1,
+				int			bfX0Pixels,
+				int			bfX1Pixels,
+				int			pfX0Pixels,
+				int			pfX1Pixels,
 				ColumnSeparator *	cs,
 				int			csCount );
 
-extern void tedSetLeftRulerRange(	void *			voidtr,
-					APP_WIDGET		w,
-					int			docVisV0,
-					int			docVisV1,
-					int			docBackV1 );
+extern void tedSetBottomRulerValues(
+				void *			voidtbr,
+				int			page,
+				int			pages,
+				int			section,
+				int			sections );
 
-extern void tedScrollLeftRuler(		void *			voidtr,
-					APP_WIDGET		w,
-					int			d );
-
-extern void tedAdaptLeftRuler(		void *		voidtlr,
-					APP_WIDGET	w,
-					int		visisbleC0,
-					int		visisbleC1,
-					int		pageStep );
