@@ -15,19 +15,23 @@
 
 typedef struct SectionPageResources
     {
-    char *		sprUnitType;
+    const char *	sprUnitType;
 
-    char *		sprNumberingRestarts;
+    const char *	sprNumberingRestarts;
 
-    char *		sprPageNumberStyle;
-    char *		sprNumberStyleMenuTexts[DOCpgn_COUNT];
+    const char *	sprPageNumberStyle;
+    const char *	sprNumberStyleMenuTexts[DOCpgn_COUNT];
 
-    char *		sprBreakKind;
-    char *		sprBreakKindMenuTexts[DOCsbk_COUNT];
+    const char *	sprBreakKind;
+    const char *	sprBreakKindMenuTexts[DOCibk_COUNT];
 
-    char *		sprColumnCount;
-    char *		sprColumnSpacing;
-    char *		sprColumnWidth;
+    const char *	sprColumns;
+    const char *	sprColumnCount;
+    const char *	sprColumn;
+    const char *	sprColumnWidth;
+    const char *	sprColumnSpacing;
+    const char *	sprLineBetweenColumns;
+    const char *	sprColumnsEqualWidth;
     } SectionPageResources;
 
 /************************************************************************/
@@ -36,16 +40,19 @@ typedef struct SectionPageResources
 /*									*/
 /************************************************************************/
 
+# define SECT_MAX_COLUMNS	4
+
 typedef struct SectionTool
     {
     EditApplication *			stApplication;
     AppInspector *			stInspector;
     const SectionPageResources *	stPageResources;
 
-    int					stUnitType;
+    int					stUnitInt;
 
     SectionProperties			stPropertiesSet;
     SectionProperties			stPropertiesChosen;
+    int					stFixedCoulumnWidth;
 
     int					stSectionNumber;
 
@@ -58,11 +65,21 @@ typedef struct SectionTool
     APP_WIDGET				stNumberStyleItems[DOCpgn_COUNT];
 
     AppOptionmenu			stBreakKindOptionmenu;
-    APP_WIDGET				stBreakKindItems[DOCsbk_COUNT];
+    APP_WIDGET				stBreakKindItems[DOCibk_COUNT];
 
-    APP_WIDGET				stColumnsText;
-    APP_WIDGET				stColumnSpacingText;
-    APP_WIDGET				stColumnWidthText;
+    APP_WIDGET				stColumnsFrame;
+    APP_WIDGET				stColumnsPaned;
+
+    AppOptionmenu			stColumnCountMenu;
+    APP_WIDGET				stColumnCountItems[SECT_MAX_COLUMNS+1];
+
+    APP_WIDGET				stLineBetweenColumnsToggle;
+    APP_WIDGET				stEqualWidthToggle;
+
+    APP_WIDGET				stColumnRows[SECT_MAX_COLUMNS];
+    APP_WIDGET				stColnoLabels[SECT_MAX_COLUMNS];
+    APP_WIDGET				stColwTexts[SECT_MAX_COLUMNS];
+    APP_WIDGET				stColsrTexts[SECT_MAX_COLUMNS];
     } SectionTool;
 
 /************************************************************************/
@@ -75,11 +92,14 @@ extern void tedFormatToolGetSectResourceTable(	EditApplication *	ea,
 					SectionPageResources *		spr,
 					InspectorSubjectResources *	isr );
 
-extern void tedFormatToolRefreshSectionTool(
+extern void tedRefreshSectionTool(
 				SectionTool *			st,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
-				const DocumentSelection *	bs );
+				const DocumentSelection *	ds,
+				const SelectionDescription *	sd,
+				const BufferDocument *		bd );
 
 extern void tedFormatCleanSectionTool(		SectionTool *		st );
 
