@@ -12,10 +12,49 @@
 #   include	<ctype.h>
 
 #   include	"tedEdit.h"
-#   include	"docLayout.h"
+#   include	"docPageGrid.h"
 
 #   include	<appDebugon.h>
 
+
+/************************************************************************/
+/*									*/
+/*  Table related menu option callbacks.				*/
+/*									*/
+/************************************************************************/
+
+void tedDocTableSelectTableRectangle(	EditDocument *		ed,
+					const TableRectangle *	tr )
+    {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+
+    DocumentSelection		dsNew;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    BufferItem *		selParentBi;
+
+    int				scrolledX= 0;
+    int				scrolledY= 0;
+
+    const int			lastLine= 0;
+
+    if  ( tedGetSelection( &dsNew, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td  ) )
+	{ LDEB(1); return;	}
+
+    if  ( docTableRectangleSelection( &dsNew, &selParentBi, bd, tr ) )
+	{ LDEB(1); return;	}
+
+    tedSetSelection( ed, &dsNew, lastLine, &scrolledX, &scrolledY );
+
+    tedAdaptToolsToSelection( ed );
+
+    return;
+    }
+
+
 /************************************************************************/
 /*									*/
 /*  Insert a table in the document.					*/
@@ -25,7 +64,7 @@
 /*  2)  If the selection was at the end of a paragraph, try to move to	*/
 /*	the beginning of the next one.					*/
 /*  3)  If the IBar is in the middle of a paragraph, split it.		*/
-/*	tedSplitParaContents() does the layout of both halves.		*/
+/*	tedSplitParaItem() does the layout of both halves.		*/
 /*  4)  If the paragraph is not the first one in the 'cell' that	*/
 /*	contains it, split the cell.					*/
 /*  5)  If the 'cell' is not the first one in the 'row' that contains	*/
@@ -39,9 +78,7 @@ int tedInsertTable(		EditDocument *		ed,
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
-    DocumentProperties *	dp= &(bd->bdProperties);
 
-    BufferItem *		newBi;
     BufferItem *		sectBi;
     BufferItem *		refRowBi;
     int				col;
@@ -53,43 +90,51 @@ int tedInsertTable(		EditDocument *		ed,
 
     RowProperties		rp;
 
-    EditOperation		eo;
-    DocumentSelection		dsOld;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     BufferItem *		bi;
+    DocumentRectangle		dr;
 
-    DocumentSelection		dsRep;
+    DocumentPosition		dpTail;
+    DocumentPosition		dpFirst;
 
-    tedStartEditOperation( &eo, &dsOld, &sg, ed, 1 );
-    bi= dsOld.dsBegin.dpBi;
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
     /*  0  */
-    eo.eoChangedRectangle.drY0--;
+    teo.teoChangedRect.drY0--;
 
     /*  1  */
-    if  ( tedEditReplaceSelection( &eo, &dsRep, ed,
-					    (const unsigned char *)0, 0 ) )
+    if  ( tedEditDeleteSelection( &teo ) )
 	{ LDEB(1); return -1;	}
 
+    dpTail= teo.teoEo.eoTailDp;
+
     /*  2  */
-    bi= dsRep.dsBegin.dpBi;
-    if  ( dsRep.dsBegin.dpStroff == bi->biParaStrlen )
+    bi= dpTail.dpBi;
+    if  ( dpTail.dpStroff > 0				&&
+	  dpTail.dpStroff == docParaStrlen( bi )	)
 	{
-	docNextPosition( &(dsRep.dsBegin) );
-	bi= dsRep.dsBegin.dpBi;
+	DocumentPosition	dpNext;
+
+	dpNext= dpTail;
+	docNextPosition( &dpNext );
+
+	if  ( docPositionsInsideCell( &dpNext, &(dpTail) ) )
+	    { dpTail= dpNext; bi= dpTail.dpBi; }
 	}
 
     /*  3  */
-    if  ( dsRep.dsBegin.dpStroff != 0 )
+    if  ( dpTail.dpStroff != 0 )
 	{
+	BufferItem *	newParaBi;
 	int		onNewPage= 0;
 
-	if  ( tedSplitParaContents( &eo, &newBi, bi, dsRep.dsBegin.dpStroff,
-						ed, DOClevROW, onNewPage ) )
+	if  ( tedSplitParaItem( &newParaBi, &teo, DOClevROW, onNewPage ) )
 	    { LDEB(1); return -1;	}
 
-	bi= newBi;
+	bi= newParaBi;
 	}
     else{
 	BufferItem *		aftBi;
@@ -98,23 +143,28 @@ int tedInsertTable(		EditDocument *		ed,
 	if  ( bi->biNumberInParent > 0			||
 	      bi->biParent->biNumberInParent > 0	)
 	    {
+	    BufferItem *	newBi;
+
 	    if  ( docSplitGroupItem( bd, &newBi, &aftBi, bi->biParent,
 					bi->biNumberInParent, DOClevROW ) )
 		{ LDEB(bi->biNumberInParent); return -1;	}
 
 	    if  ( aftBi && aftBi->biParent )
 		{
-		docEditIncludeItemInReformatRange( &eo, aftBi->biParent );
+		docEditIncludeItemInReformatRange( &(teo.teoEo),
+							    aftBi->biParent );
 		}
 	    else{ XDEB(aftBi);	}
 	    }
 	}
 
-    textAttributeNumber= bi->biParaParticules[0].tpTextAttributeNumber;
+    textAttributeNumber= bi->biParaParticules[0].tpTextAttrNr;
 
-    wide=   dp->dpGeometry.dgPageWideTwips;
-    wide -= dp->dpGeometry.dgLeftMarginTwips;
-    wide -= dp->dpGeometry.dgRightMarginTwips;
+    docParaBlockFrameRectangle( &dr, bi, teo.teoEo.eoBodySectBi, bd,
+					    bi->biTopPosition.lpPage,
+					    bi->biTopPosition.lpColumn );
+
+    wide= dr.drX1- dr.drX0;
 
     docInitRowProperties( &rp );
 
@@ -131,35 +181,29 @@ int tedInsertTable(		EditDocument *		ed,
     for ( col= 0; col < columns; col++ )
 	{
 	CellProperties		cp;
-	const int * const	colorMap= (const int *)0;
 
 	docInitCellProperties( &cp );
 	cp.cpRightBoundaryTwips= ( ( col+ 1 )* wide )/ columns;
 
-	if  ( docInsertRowColumn( &rp, col, &cp, colorMap ) )
+	if  ( docInsertRowColumn( &rp, col, &cp,
+					(const DocumentAttributeMap *)0 ) )
 	    { LDEB(col); return -1;	}
 
-	rp.rpHasTableParagraphs= 1;
-
 	docCleanCellProperties( &cp );
 	}
 
-    newBi= (BufferItem *)0;
     refRowBi= (BufferItem *)0;
     sectBi= bi->biParent->biParent->biParent;
     paraNr= docNumberOfParagraph( bi );
 
-    if  ( docInsertTableRows( sectBi, &newBi, refRowBi,
-			    &rp, textAttributeNumber,
-			    bi->biParent->biParent->biNumberInParent, paraNr,
-			    rows, &eo ) )
+    if  ( docInsertTableRows( &dpFirst, &(teo.teoEo), sectBi, refRowBi,
+		    &rp, textAttributeNumber,
+		    bi->biParent->biParent->biNumberInParent, paraNr, rows ) )
 	{ LDEB(rows); return -1;	}
 
     docCleanRowProperties( &rp );
 
-    docSectDelimitTables( sectBi );
-
-    tedEditFinishIBarSelection( ed, &eo, newBi, 0 );
+    tedEditFinishIBarSelection( &teo, dpFirst.dpBi, dpFirst.dpStroff );
 
     return 0;
     }
@@ -172,7 +216,6 @@ int tedInsertTable(		EditDocument *		ed,
 /*	any price. Every Item is supposed to contain at least one child	*/
 /*	or at least one particule.					*/
 /*  2)  Sanity check.							*/
-/*  4)  Section related administration.					*/
 /*  5)  Set an I-Bar selection of the first position in the first	*/
 /*	(empty) cell of the first fresh row.				*/
 /*									*/
@@ -180,7 +223,7 @@ int tedInsertTable(		EditDocument *		ed,
 
 static int tedInsertRowsInTable(
 			BufferItem *			sectBi,
-			EditOperation *			eo,
+			TedEditOperation *		teo,
 			int				paraNr,
 			int				textAttributeNumber,
 			BufferItem *			refRowBi,
@@ -188,35 +231,28 @@ static int tedInsertRowsInTable(
 			int				row,
 			int				rows )
     {
-    BufferItem *		newBi;
+    EditOperation *		eo= &(teo->teoEo);
     const RowProperties *	rp;
 
+    DocumentPosition		dpFirst;
     /*  1  */
     if  ( sectBi->biChildCount == 0 )
 	{ LDEB(sectBi->biChildCount); return -1;	}
 
     /*  2  */
-    if  ( refRowBi->biLevel != DOClevROW	||
-	  ! refRowBi->biRowHasTableParagraphs	)
+    if  ( ! docIsRowItem( refRowBi ) )
 	{
-	SDEB(docLevelStr(refRowBi->biLevel));
-	LDEB(refRowBi->biRowHasTableParagraphs);
+	SLDEB(docLevelStr(refRowBi->biLevel),refRowBi->biRowCellCount);
 	return -1;
 	}
 
-    newBi= (BufferItem *)0;
-
     rp= &(refRowBi->biRowProperties);
-    if  ( docInsertTableRows( sectBi, &newBi, refRowBi, rp,
-						textAttributeNumber,
-						row, paraNr, rows, eo ) )
+    if  ( docInsertTableRows( &dpFirst, eo, sectBi, refRowBi, rp,
+				textAttributeNumber, row, paraNr, rows ) )
 	{ LDEB(rows); return -1;	}
 
-    /*  4  */
-    docSectDelimitTables( sectBi );
-
     /*  5  */
-    tedEditFinishIBarSelection( ed, eo, newBi, 0 );
+    tedEditFinishIBarSelection( teo, dpFirst.dpBi, dpFirst.dpStroff );
 
     return 0;
     }
@@ -247,19 +283,21 @@ int tedInsertRowInTable(	EditDocument *		ed )
 
     const int			rows= 1;
 
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     DocumentPosition		dpRef;
+    int				part;
+    const int			lastOne= 1;
     int				paraNr;
     int				textAttributeNumber;
 
     /*  1  */
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
     /*  2  */
-    if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
+    if  ( docDelimitTable( teo.teoEo.eoHeadDp.dpBi, &sectBi,
 						&col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
@@ -267,14 +305,16 @@ int tedInsertRowInTable(	EditDocument *		ed )
     refRowBi= sectBi->biChildren[row];
     if  ( docFirstPosition( &dpRef, refRowBi ) )
 	{ LDEB(row); return -1;	}
+    if  ( docFindParticuleOfPosition( &part, &dpRef, lastOne ) )
+	{ LDEB(dpRef.dpStroff); return -1;	}
 
     /*  4  */
     paraNr= docNumberOfParagraph( dpRef.dpBi );
-    textAttributeNumber= dpRef.dpBi->biParaParticules[dpRef.dpParticule].
-							tpTextAttributeNumber;
+    textAttributeNumber= dpRef.dpBi->biParaParticules[part].
+						    tpTextAttrNr;
 
     /*  5  */
-    if  ( tedInsertRowsInTable( sectBi, &eo,
+    if  ( tedInsertRowsInTable( sectBi, &teo,
 		    paraNr, textAttributeNumber, refRowBi, ed, row, rows ) )
 	{ LDEB(row); return -1;	}
 
@@ -306,19 +346,21 @@ int tedAppendRowToTable(	EditDocument *		ed )
 
     const int			rows= 1;
 
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     DocumentPosition		dpRef;
+    int				part;
+    const int			lastOne= 0;
     int				paraNr;
     int				textAttributeNumber;
 
     /*  1  */
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
     /*  2  */
-    if  ( docDelimitTable( ds.dsEnd.dpBi, &sectBi,
+    if  ( docDelimitTable( teo.teoEo.eoTailDp.dpBi, &sectBi,
 						&col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
@@ -326,14 +368,16 @@ int tedAppendRowToTable(	EditDocument *		ed )
     refRowBi= sectBi->biChildren[row];
     if  ( docLastPosition( &dpRef, refRowBi ) )
 	{ LDEB(row); return -1;	}
+    if  ( docFindParticuleOfPosition( &part, &dpRef, lastOne ) )
+	{ LDEB(dpRef.dpStroff); return -1;	}
 
     /*  4  */
     paraNr= docNumberOfParagraph( dpRef.dpBi )+ 1;
-    textAttributeNumber= dpRef.dpBi->biParaParticules[dpRef.dpParticule].
-							tpTextAttributeNumber;
+    textAttributeNumber= dpRef.dpBi->biParaParticules[part].
+							tpTextAttrNr;
 
     /*  5  */
-    if  ( tedInsertRowsInTable( sectBi, &eo,
+    if  ( tedInsertRowsInTable( sectBi, &teo,
 		    paraNr, textAttributeNumber, refRowBi, ed, row+ 1, rows ) )
 	{ LDEB(row); return -1;	}
     
@@ -344,89 +388,99 @@ int tedAppendRowToTable(	EditDocument *		ed )
 /*									*/
 /*  Delete rows from a table.						*/
 /*									*/
-/*  4)  Sanity check.							*/
-/*									*/
 /************************************************************************/
 
 static int tedDeleteRowsFromTableOperation(
-					EditDocument *		ed,
-					EditOperation *		eo,
-					DocumentSelection *	ds,
-					int			delRow0,
-					int			delRow1 )
+					TedEditOperation *	teo,
+					int			row0,
+					int			row1 )
     {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
+    EditOperation *		eo= &(teo->teoEo);
+
+    int				rval= 0;
 
     BufferItem *		selSectBi;
 
     int				col;
     int				row;
-    int				row0;
-    int				row1;
+    int				tabRow0;
+    int				tabRow1;
 
     DocumentPosition		dpNew;
 
     int				paraNr;
+    int				sectionsDeleted= 0;
     int				firstParaDeleted= -1;
     int				paragraphsDeleted= 0;
 
-    const int			stroffFrom= 0;
-    const int			sectShift= 0;
-    const int			stroffShift= 0;
+    PropertyMask		ppDoneMask;
+    PropertyMask		ppSetMask;
 
-    if  ( docDelimitTable( ds->dsBegin.dpBi, &selSectBi,
-						&col, &row0, &row, &row1 ) )
-	{ LDEB(1); return -1;	}
+    DocumentSelection		dsRows;
 
-    if  ( delRow0 > delRow1 )
-	{ LLDEB(delRow0,delRow1); return -1;	}
-    if  ( delRow0 < row0 || delRow0 > row1 )
-	{ LLLDEB(row0,delRow0,row1); return -1;	}
-    if  ( delRow1 < row0 || delRow1 > row1 )
-	{ LLLDEB(row0,delRow1,row1); return -1;	}
+    ParagraphProperties		pp;
+
+    docInitParagraphProperties( &pp );
+
+    if  ( docDelimitTable( teo->teoEo.eoHeadDp.dpBi, &selSectBi,
+					    &col, &tabRow0, &row, &tabRow1 ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    if  ( row0 > row1 )
+	{ LLDEB(row0,row1); rval= -1; goto ready;	}
+    if  ( row0 < tabRow0 || row0 > tabRow1 )
+	{ LLLDEB(tabRow0,row0,tabRow1); rval= -1; goto ready;	}
+    if  ( row1 < tabRow0 || row1 > tabRow1 )
+	{ LLLDEB(tabRow0,row1,tabRow1); rval= -1; goto ready;	}
+
+    docInitDocumentSelection( &dsRows );
+    if  ( docFirstPosition( &dsRows.dsHead, selSectBi->biChildren[row0] ) )
+	{ LLDEB(row0,row1); rval= -1; goto ready;	}
+    if  ( docLastPosition( &dsRows.dsTail, selSectBi->biChildren[row1] ) )
+	{ LLDEB(row0,row1); rval= -1; goto ready;	}
+
+    paraNr= docNumberOfParagraph( dsRows.dsTail.dpBi )+ 1;
+
+    utilPropMaskClear( &ppDoneMask );
+    utilPropMaskClear( &ppSetMask );
+    utilPropMaskFill( &ppSetMask, PPprop_FULL_COUNT );
+    PROPmaskUNSET( &ppSetMask, PPpropTABLE_NESTING );
+
+    if  ( docUpdParaProperties( &ppDoneMask, &pp,
+			    &ppSetMask, &(dsRows.dsTail.dpBi->biParaProperties),
+			    (const DocumentAttributeMap *)0 ) )
+	{ LDEB(1); rval= -1; goto ready;	}
 
     docInitDocumentPosition( &dpNew );
-    if  ( docLastPosition( &dpNew, selSectBi->biChildren[delRow1] ) )
-	{ LLDEB(delRow0,delRow1); return -1;	}
-
-    paraNr= docNumberOfParagraph( dpNew.dpBi )+ 1;
-
-    docInitDocumentPosition( &dpNew );
-    if  ( docLastPosition( &dpNew, selSectBi->biChildren[delRow1] )	||
-	  docNextPosition( &dpNew )					)
+    dpNew= dsRows.dsTail;
+    if  ( docNextPosition( &dpNew ) )
 	{
-	docInitDocumentPosition( &dpNew );
-	if  ( docFirstPosition( &dpNew,
-				selSectBi->biChildren[delRow0] )	||
-	      docPrevPosition( &dpNew, /*lastOne=*/ 0 )			)
-	    {
-	    docInitDocumentPosition( &dpNew );
-	    }
+	dpNew= dsRows.dsHead;
+	if  ( docPrevPosition( &dpNew ) )
+	    { docInitDocumentPosition( &dpNew );	}
 	}
 
-    tedEditIncludeRowsInRedraw( eo, ed, selSectBi, delRow0, delRow1 );
+    tedEditIncludeRowsInRedraw( teo, selSectBi, row0, row1 );
+
+    if  ( docEditDeleteFieldsForBlockDelete( eo, &dsRows ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    docEditDeleteItems( eo, &sectionsDeleted,
+				&firstParaDeleted, &paragraphsDeleted,
+				selSectBi, row0, row1- row0+ 1 );
 
     docEditIncludeItemInReformatRange( eo, selSectBi );
 
-    docEditDeleteItems( eo, &firstParaDeleted, &paragraphsDeleted,
-				selSectBi, delRow0, delRow1- delRow0+ 1 );
-
-    /* NO! done by docEditDeleteItems()
-    docEditShiftReformatRangeParaNr( eo, paraNr+ 1, -paragraphsDeleted );
-    */
-
-    if  ( selSectBi->biInExternalItem == DOCinBODY )
-	{
-	docShiftNoteReferences( eo->eoBd, paraNr, stroffFrom,
-				    sectShift, -paragraphsDeleted,
-				    stroffShift );
-	}
-
     if  ( ! dpNew.dpBi )
 	{
-	dpNew.dpBi= docInsertEmptyParagraph( bd, selSectBi,
-					td->tdCurrentTextAttributeNumber );
+	const int	sectShift= 0;
+	EditDocument *	ed= teo->teoEditDocument;
+	TedDocument *	td= (TedDocument *)ed->edPrivateData;
+
+	if  ( docSectionParagraph( eo, &(dpNew.dpBi),
+				    selSectBi, sectShift,
+				    &pp, td->tdCurrentTextAttributeNumber ) )
+	    { LDEB(1); rval= -1; goto ready;	}
 
 	docEditIncludeItemInReformatRange( eo, selSectBi );
 
@@ -434,24 +488,156 @@ static int tedDeleteRowsFromTableOperation(
 	    { LDEB(1);	}
 	}
 
-    docSectDelimitTables( selSectBi );
+    docDelimitTables( selSectBi );
 
-    tedEditFinishIBarSelection( ed, eo, dpNew.dpBi, dpNew.dpStroff );
+    tedEditFinishIBarSelection( teo, dpNew.dpBi, dpNew.dpStroff );
+
+  ready:
+
+    docCleanParagraphProperties( &pp );
+
+    return rval;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Roll rows in a table.						*/
+/*									*/
+/************************************************************************/
+
+static int tedRollRowsInTableOperation(
+				EditDocument *			ed,
+				TedEditOperation *		teo,
+				const SelectionDescription  *	sd,
+				int				row0,
+				int				row1,
+				int				rowsdown )
+    {
+    EditOperation *		eo= &(teo->teoEo);
+    int				rval= 0;
+
+    BufferItem *		selSectBi;
+
+    int				col;
+    int				row;
+    int				tabRow0;
+    int				tabRow1;
+    int				rowsHigh= row1- row0+ 1;
+
+    DocumentPosition		dp;
+
+    EditRange			selectedRange= eo->eoSelectedRange;
+    int				paraNr;
+    int				paraOff0;
+    int				paraOff1;
+    int				selRow0= sd->sdTableRectangle.trRow0;
+    int				selRow1= sd->sdTableRectangle.trRow1;
+
+    if  ( docDelimitTable( teo->teoEo.eoHeadDp.dpBi, &selSectBi,
+					    &col, &tabRow0, &row, &tabRow1 ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    if  ( rowsdown < 0 )
+	{ rowsdown= rowsHigh+ rowsdown;	}
+
+    if  ( row0 >= row1 )
+	{ LLDEB(row0,row1); rval= -1; goto ready;	}
+    if  ( row0 < tabRow0 || row0 > tabRow1 )
+	{ LLLDEB(tabRow0,row0,tabRow1); rval= -1; goto ready;	}
+    if  ( row1 < tabRow0 || row1 > tabRow1 )
+	{ LLLDEB(tabRow0,row1,tabRow1); rval= -1; goto ready;	}
+    if  ( rowsdown < 1 || rowsdown >= rowsHigh )
+	{ LLDEB(rowsdown,rowsHigh); rval= -1; goto ready;	}
+
+    docInitDocumentPosition( &dp );
+    if  ( docFirstPosition( &dp, selSectBi->biChildren[selRow0] ) )
+	{ LDEB(selRow0); rval= -1; goto ready;	}
+    paraNr= docNumberOfParagraph( dp.dpBi );
+    paraOff0= selectedRange.erHead.epParaNr- paraNr;
+    if  ( docFirstPosition( &dp, selSectBi->biChildren[selRow1] ) )
+	{ LDEB(selRow1); rval= -1; goto ready;	}
+    paraNr= docNumberOfParagraph( dp.dpBi );
+    paraOff1= selectedRange.erTail.epParaNr- paraNr;
+    if  ( paraOff0 < 0 || paraOff1 < 0 )
+	{ LLDEB(paraOff0,paraOff1); rval= -1; goto ready;	}
+
+    tedEditIncludeRowsInRedraw( teo, selSectBi, row0, row1 );
+
+    if  ( docRollItemChildren( eo, selSectBi, row0, row1+ 1, rowsdown ) )
+	{ LLLDEB(row0,row1,rowsdown); rval= -1; goto ready;	}
+
+    docEditIncludeItemInReformatRange( eo, selSectBi );
+
+    docDelimitTables( selSectBi );
+
+    selRow0 += rowsdown;
+    if  ( selRow0 > row1 )
+	{ selRow0 -= rowsHigh;	}
+    selRow1 += rowsdown;
+    if  ( selRow1 > row1 )
+	{ selRow1 -= rowsHigh;	}
+
+    if  ( docFirstPosition( &dp, selSectBi->biChildren[selRow0] ) )
+	{ LDEB(selRow0); rval= -1; goto ready;	}
+    paraNr= docNumberOfParagraph( dp.dpBi );
+    selectedRange.erHead.epParaNr= paraNr+ paraOff0;
+    if  ( docFirstPosition( &dp, selSectBi->biChildren[selRow1] ) )
+	{ LDEB(selRow1); rval= -1; goto ready;	}
+    paraNr= docNumberOfParagraph( dp.dpBi );
+    selectedRange.erTail.epParaNr= paraNr+ paraOff1;
+    if  ( selectedRange.erHead.epParaNr < 0	||
+	  selectedRange.erTail.epParaNr < 0	)
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    eo->eoSelectedRange= selectedRange;
+    tedEditFinishOldSelection( teo );
+
+  ready:
+
+    return rval;
+    }
+
+/************************************************************************/
+
+int tedDeleteRowsFromTable(	EditDocument *		ed,
+				int			row0,
+				int			row1 )
+    {
+    TedEditOperation		teo;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
+
+    if  ( tedDeleteRowsFromTableOperation( &teo, row0, row1 ) )
+	{ LDEB(1); return -1;	}
+
+    appDocumentChanged( ed, 1 );
 
     return 0;
     }
 
-int tedDeleteRowsFromTable(	EditDocument *		ed,
-				int			delRow0,
-				int			delRow1 )
+int tedRollRowsInTable(		EditDocument *		ed,
+				int			rowsdown )
     {
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    int				row0;
+    int				row1;
 
-    if  ( tedDeleteRowsFromTableOperation( ed, &eo, &ds, delRow0, delRow1 ) )
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
+
+    row0= sd.sdTableRectangle.trRow0;
+    row1= sd.sdTableRectangle.trRow1;
+
+    if  ( rowsdown < 0 )
+	{ row0 += rowsdown;	}
+    if  ( rowsdown > 0 )
+	{ row1 += rowsdown;	}
+
+    if  ( tedRollRowsInTableOperation( ed, &teo, &sd, row0, row1, rowsdown ) )
 	{ LDEB(1); return -1;	}
 
     appDocumentChanged( ed, 1 );
@@ -465,26 +651,26 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
 /*									*/
 /************************************************************************/
 
-static int tedSplitColumnInRows(	EditDocument *	ed,
-					EditOperation *	eo,
-					BufferItem *	sectBi,
-					int		row0,
-					int		row,
-					int		row1,
-					int		col,
-					int		after )
+static int tedSplitColumnInRows(	TedEditOperation *	teo,
+					BufferItem *		sectBi,
+					int			row0,
+					int			row,
+					int			row1,
+					int			col,
+					int			after )
     {
+    EditOperation *		eo= &(teo->teoEo);
     BufferItem *		newParaBi= (BufferItem *)0;
 
-    tedEditIncludeRowsInRedraw( eo, ed, sectBi, row0, row1 );
+    tedEditIncludeRowsInRedraw( teo, sectBi, row0, row1 );
 
     if  ( docSplitColumnInRows( &newParaBi, eo,
 				    sectBi, row0, row, row1, col, after ) )
 	{ LDEB(1); return -1;	}
 
-    docSectDelimitTables( sectBi );
+    docDelimitTables( sectBi );
 
-    tedEditFinishIBarSelection( ed, eo, newParaBi, 0 );
+    tedEditFinishIBarSelection( teo, newParaBi, 0 );
 
     return 0;
     }
@@ -500,17 +686,17 @@ int tedInsertColumnInTable(	EditDocument *		ed )
 
     int				after= 0;
 
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
-    if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
+    if  ( docDelimitTable( teo.teoEo.eoHeadDp.dpBi, &sectBi,
 					    &col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
-    return tedSplitColumnInRows( ed, &eo, sectBi, row0, row, row1, col, after );
+    return tedSplitColumnInRows( &teo, sectBi, row0, row, row1, col, after );
     }
 
 int tedAppendColumnToTable(	EditDocument *		ed )
@@ -524,28 +710,28 @@ int tedAppendColumnToTable(	EditDocument *		ed )
 
     int				after= 1;
 
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
-    if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
+    if  ( docDelimitTable( teo.teoEo.eoHeadDp.dpBi, &sectBi,
 					    &col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
-    return tedSplitColumnInRows( ed, &eo, sectBi, row0, row, row1, col, after );
+    return tedSplitColumnInRows( &teo, sectBi, row0, row, row1, col, after );
     }
 
 static int tedDeleteColumnsFromRowsOperation(
-					EditDocument *		ed,
-					EditOperation *		eo,
-					DocumentSelection *	ds,
+					TedEditOperation *	teo,
 					int			delRow0,
 					int			delRow1,
 					int			delCol0,
 					int			delCol1 )
     {
+    EditOperation *		eo= &(teo->teoEo);
+
     BufferItem *		sectBi;
     BufferItem *		rowBi;
 
@@ -556,11 +742,11 @@ static int tedDeleteColumnsFromRowsOperation(
 
     DocumentPosition		dpNew;
 
-    if  ( docDelimitTable( ds->dsBegin.dpBi, &sectBi,
+    if  ( docDelimitTable( teo->teoEo.eoHeadDp.dpBi, &sectBi,
 					    &col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
-    tedEditIncludeRowsInRedraw( eo, ed, sectBi, delRow0, delRow1 );
+    tedEditIncludeRowsInRedraw( teo, sectBi, delRow0, delRow1 );
 
     docInitDocumentPosition( &dpNew );
     rowBi= sectBi->biChildren[delRow1];
@@ -572,7 +758,7 @@ static int tedDeleteColumnsFromRowsOperation(
 	rowBi= sectBi->biChildren[delRow0];
 	if  ( delCol0 <= 0						||
 	      docFirstPosition( &dpNew, rowBi->biChildren[delCol0] )	||
-	      docPrevPosition( &dpNew, /*lastOne=*/ 0 )			)
+	      docPrevPosition( &dpNew )					)
 	    { docInitDocumentPosition( &dpNew ); }
 	}
 
@@ -580,7 +766,7 @@ static int tedDeleteColumnsFromRowsOperation(
 				    delRow0, delRow1, delCol0, delCol1 ) )
 	{ LDEB(1); return -1;	}
 
-    tedEditFinishIBarSelection( ed, eo, dpNew.dpBi, dpNew.dpStroff );
+    tedEditFinishIBarSelection( teo, dpNew.dpBi, dpNew.dpStroff );
 
     return 0;
     }
@@ -591,14 +777,20 @@ int tedDeleteColumnsFromRows(	EditDocument *	ed,
 				int		delCol0,
 				int		delCol1 )
     {
-    EditOperation		eo;
-    DocumentSelection		ds;
+    int				rval;
+
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
-    return tedDeleteColumnsFromRowsOperation( ed, &eo, &ds,
+    rval= tedDeleteColumnsFromRowsOperation( &teo,
 					delRow0, delRow1, delCol0, delCol1 );
+
+    appDocumentChanged( ed, 1 );
+
+    return rval;
     }
 
 /************************************************************************/
@@ -623,11 +815,12 @@ void tedDocSetTableProperties(	EditDocument *		ed,
     TedDocument *		td;
     BufferDocument *		bd;
 
-    BufferItem *		selSectBi;
+    BufferItem *		selParentBi;
     int				row1Reformat;
 
     DocumentSelection		dsNew;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     PropertyMask		cellSpanMask;
     PropertyMask		cpSetMaskStripped;
@@ -637,17 +830,18 @@ void tedDocSetTableProperties(	EditDocument *		ed,
 
     /*  1  */
     docInitDocumentSelection( &dsNew );
-    if  ( tedGetSelection( &dsNew, &sg, td ) )
+    if  ( tedGetSelection( &dsNew, &sg, &sd, 
+			    (DocumentTree **)0, (const BufferItem **)0, td ) )
 	{ LDEB(1); return;	}
 
     /*  2  */
-    if  ( docTableRectangleSelection( &dsNew, &selSectBi, bd, trChange ) )
+    if  ( docTableRectangleSelection( &dsNew, &selParentBi, bd, trChange ) )
 	{ LDEB(1); return;	}
 
     row1Reformat= trChange->trRow1;
 
     /*  3  */
-    PROPmaskCLEAR( &cellSpanMask );
+    utilPropMaskClear( &cellSpanMask );
     PROPmaskADD( &cellSpanMask, CLprop_ROWSPAN );
     PROPmaskADD( &cellSpanMask, CLprop_COLSPAN );
 
@@ -657,7 +851,7 @@ void tedDocSetTableProperties(	EditDocument *		ed,
 	{
 	int		row1SpanChanged= trChange->trRow0- 1;
 
-	if  ( docChangeCellSpans( &row1SpanChanged, selSectBi,
+	if  ( docChangeCellSpans( &row1SpanChanged, selParentBi,
 					    trChange->trRow0, trChange->trCol0,
 					    trChange->trCellRowspan,
 					    trChange->trCellColspan ) )
@@ -673,7 +867,7 @@ void tedDocSetTableProperties(	EditDocument *		ed,
     PROPmaskUNSET( &cpSetMaskStripped, CLprop_COLSPAN );
 
     /*  5  */
-    tedChangeTableLayout( ed, selSectBi,
+    tedChangeTableLayout( ed, selParentBi,
 				    trChange->trRow0, trChange->trRow1,
 				    trChange->trCol0, trChange->trCol1,
 				    row1Reformat,
@@ -681,7 +875,7 @@ void tedDocSetTableProperties(	EditDocument *		ed,
 
     tedAdaptToolsToSelection( ed );
 
-    if  ( ! utilPropMaskIsEmpty( rpSetMask )	&&
+    if  ( ! utilPropMaskIsEmpty( rpSetMask )	||
 	  ! utilPropMaskIsEmpty( cpSetMask )	)
 	{ appDocumentChanged( ed, 1 );	}
 
@@ -721,13 +915,15 @@ void tedAppSetTableSelection(	EditDocument *		ed,
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     if  ( ! ed )
 	{ XDEB(ed); return;	}
 
     td= (TedDocument *)ed->edPrivateData;
 
-    tedGetSelection( &ds, &sg, td );
+    tedGetSelection( &ds, &sg, &sd,
+			    (DocumentTree **)0, (const BufferItem **)0, td );
 
     if  ( ! docGetTableRectangle( &tr, &ds ) )
 	{ tableRectangle= 1;	}
@@ -771,21 +967,22 @@ void tedChangeTableLayout(	EditDocument *		ed,
 				const PropertyMask *	cpSetMask,
 				const RowProperties *	rpSet )
     {
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
+    EditOperation *		eo= &(teo.teoEo);
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     /*  1  */
     if  ( row1Reformat < row1Change )
 	{ row1Reformat=  row1Change;	}
 
     /*  2  */
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
     /*  3  */
-    docEditIncludeRowsInReformatRange( &eo,
+    docEditIncludeRowsInReformatRange( eo,
 				    sectBi, row0Change, row1Reformat );
-    tedEditIncludeRowsInRedraw( &eo, ed,
+    tedEditIncludeRowsInRedraw( &teo,
 				    sectBi, row0Change, row1Reformat );
 
     /*  4  */
@@ -793,58 +990,30 @@ void tedChangeTableLayout(	EditDocument *		ed,
 	{
 	const BufferItem *	rowBi= sectBi->biChildren[row0Change];
 
-	if  ( row0Change != row1Change )
-	    { LLDEB(row0Change,row1Change);	}
-
-	docEditIncludeRowsInReformatRange( &eo,
+	docEditIncludeRowsInReformatRange( eo,
 				sectBi, row0Change, rowBi->biRowTablePast- 1 );
-	tedEditIncludeRowsInRedraw( &eo, ed,
+	tedEditIncludeRowsInRedraw( &teo,
 				sectBi, row0Change, rowBi->biRowTablePast- 1 );
 	}
 
     /*  5  */
-    docChangeTableLayout( &eo, sectBi,
+    docChangeTableLayout( eo, sectBi,
 			    row0Change, row1Change, col0Change, col1Change,
 			    rpSetMask, cpSetMask, rpSet );
 
     /*  6  */
-    if  ( tedEditFinishSelection( ed, &eo, &ds ) )
+    if  ( tedEditFinishOldSelection( &teo ) )
 	{ LDEB(1);	}
 
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Callback for the 'DrawTableGrid' toggle.				*/
-/*									*/
-/************************************************************************/
-
-void tedDocTableDrawGrid(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidcbs )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    int				set;
-
-    set= appGuiGetMenuToggleStateFromCallback( option, voidcbs );
-
-    if  ( set )
-	{ td->tdDrawTableGrid=  1;	}
-    else{ td->tdDrawTableGrid= -1;	}
-
-    appDocExposeRectangle( ed, (const DocumentRectangle *)0, 0,0 );
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Delete a slice from a table. Either horizontal or vertical or both:	*/
 /*  the table as a whole.						*/
 /*									*/
-/*  1)  See whether the ccurrent selection is a table slice.		*/
+/*  1)  See whether the current selection is a table slice.		*/
 /*  2)  If it is a row slice (Whole rows) delete the rows. This covers	*/
 /*	deleteing the whole table as well. [ Handling it in the column	*/
 /*	branch would leave empty rows.]					*/
@@ -859,26 +1028,31 @@ int tedDeleteTableSliceSelection(	EditDocument *		ed )
     int				isColSlice= 0;
     TableRectangle		tr;
 
-    EditOperation		eo;
-    DocumentSelection		ds;
+    TedEditOperation		teo;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     int				ret;
 
     if  ( ed->edIsReadonly )
 	{ return 1;	}
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &teo, &sg, &sd, ed, 1 );
 
     /*  1  */
+    {
+    DocumentSelection		ds;
+
+    docEditOperationGetSelection( &ds, &(teo.teoEo) );
+
     if  ( docGetTableSliceSelection( &isRowSlice, &isColSlice, &tr, &ds ) )
 	{ return 1;	}
+    }
 
     /*  2  */
     if  ( isRowSlice )
 	{
-	ret= tedDeleteRowsFromTableOperation( ed, &eo, &ds,
-							tr.trRow0, tr.trRow1 );
+	ret= tedDeleteRowsFromTableOperation( &teo, tr.trRow0, tr.trRow1 );
 	if  ( ret )
 	    { LDEB(ret); return -1;	}
 
@@ -889,7 +1063,7 @@ int tedDeleteTableSliceSelection(	EditDocument *		ed )
     /*  3  */
     if  ( isColSlice )
 	{
-	ret= tedDeleteColumnsFromRowsOperation( ed, &eo, &ds,
+	ret= tedDeleteColumnsFromRowsOperation( &teo,
 				tr.trRow0, tr.trRow1, tr.trCol0, tr.trCol1 );
 	if  ( ret )
 	    { LDEB(ret); return -1;	}
@@ -901,3 +1075,4 @@ int tedDeleteTableSliceSelection(	EditDocument *		ed )
     /*  4  */
     LLDEB(isRowSlice,isColSlice); return -1;
     }
+
