@@ -11,7 +11,7 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
@@ -31,10 +31,8 @@ static void tedTabToolRefreshCurrentTab(	TabsTool *	tt )
 
     if  ( ts->tsTwips >= 0 )
 	{
-	char			scratch[50];
-
-	appGeoLengthToString( scratch, ts->tsTwips, UNITtyPOINTS );
-	appStringToTextWidget( tt->ttTabPositionText, scratch );
+	appLengthToTextWidget( tt->ttTabPositionText,
+					    ts->tsTwips, UNITtyPOINTS );
 	}
     else{
 	appStringToTextWidget( tt->ttTabPositionText, "" );
@@ -108,8 +106,8 @@ static void tedFormatToolRefreshTabsPage(	TabsTool *	tt )
     ts= tsl->tslTabStops;
     for ( i= 0; i < tsl->tslTabStopCount; ts++, i++ )
 	{
-	appGeoLengthToString( scratch, ts->tsTwips, UNITtyPOINTS );
-	appGuiAddValueToListWidget( tt->ttTabPositionList, scratch );
+	geoLengthToString( scratch, ts->tsTwips, UNITtyPOINTS );
+	appGuiAddValueToListWidget( tt->ttTabPositionList, -1, scratch );
 
 	if  ( ts->tsTwips <= tt->ttTabStopValue.tsTwips )
 	    {
@@ -140,11 +138,14 @@ static void tedFormatToolRefreshTabsPage(	TabsTool *	tt )
 /*									*/
 /************************************************************************/
 
-void tedFormatToolRefreshTabsTool(
+void tedRefreshTabsTool(
 				TabsTool *			tt,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
+				const SelectionGeometry *	sg,
+				const SelectionDescription *	sd,
 				const DocumentProperties *	dp )
     {
     const ParagraphProperties *		pp;
@@ -152,21 +153,23 @@ void tedFormatToolRefreshTabsTool(
     PropertyMask			chgMask;
     PropertyMask			updMask;
 
-    pp= &(ds->dsBegin.dpBi->biParaProperties);
+    pp= &(ds->dsHead.dpBi->biParaProperties);
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
     if  ( docUpdParaProperties( &chgMask, &(tt->ttParaPropertiesChosen),
-					    &updMask, pp, (const int *)0 ) )
+				    &updMask, pp,
+				    (const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return ;	}
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
     if  ( docUpdParaProperties( &chgMask, &(tt->ttParaPropertiesSet),
-					    &updMask, pp, (const int *)0 ) )
+				    &updMask, pp,
+				    (const DocumentAttributeMap *)0 ) )
 	{ LDEB(1); return ;	}
 
     /**/
@@ -175,16 +178,16 @@ void tedFormatToolRefreshTabsTool(
 
     /**/
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, DPpropDEFTAB );
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
     if  ( docUpdDocumentProperties( &chgMask, &(tt->ttDocPropertiesChosen),
 							    &updMask, dp ))
 	{ LDEB(1); return ;	}
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
     if  ( docUpdDocumentProperties( &chgMask, &(tt->ttDocPropertiesSet),
 							    &updMask, dp ))
@@ -192,14 +195,10 @@ void tedFormatToolRefreshTabsTool(
 
     /**/
 
-    {
-    char		scratch[50];
+    appLengthToTextWidget( tt->ttTabDefaultText,
+				    dp->dpTabIntervalTwips, UNITtyPOINTS );
 
-    appGeoLengthToString( scratch, dp->dpTabIntervalTwips, UNITtyPOINTS );
-    appStringToTextWidget( tt->ttTabDefaultText, scratch );
-    }
-
-    *pEnabled= 1;
+    *pEnabled= ! sd->sdDocumentReadonly;
     return;
     }
 
@@ -211,18 +210,19 @@ void tedFormatToolRefreshTabsTool(
 
 static APP_BUTTON_CALLBACK_H( tedFormatTabsRevertParaPushed, w, voidtt )
     {
-    TabsTool *			tt= (TabsTool *)voidtt;
+    TabsTool *		tt= (TabsTool *)voidtt;
 
-    PropertyMask		chgMask;
-    PropertyMask		updMask;
+    PropertyMask	chgMask;
+    PropertyMask	updMask;
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
 
     docUpdParaProperties( &chgMask, &(tt->ttParaPropertiesChosen),
-			&updMask, &(tt->ttParaPropertiesSet), (const int *)0 );
+			&updMask, &(tt->ttParaPropertiesSet),
+			(const DocumentAttributeMap *)0 );
 
     tedFormatToolRefreshTabsPage( tt );
 
@@ -236,20 +236,19 @@ static APP_BUTTON_CALLBACK_H( tedFormatTabsRevertDocPushed, w, voidtt )
     PropertyMask		chgMask;
     PropertyMask		updMask;
 
-    PROPmaskCLEAR( &chgMask );
+    utilPropMaskClear( &chgMask );
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, DPpropDEFTAB );
 
     docUpdDocumentProperties( &chgMask, &(tt->ttDocPropertiesChosen),
 				    &updMask, &(tt->ttDocPropertiesSet) );
 
     {
-    char			scratch[50];
     const DocumentProperties *	dp= &(tt->ttDocPropertiesChosen);
 
-    appGeoLengthToString( scratch, dp->dpTabIntervalTwips, UNITtyPOINTS );
-    appStringToTextWidget( tt->ttTabDefaultText, scratch );
+    appLengthToTextWidget( tt->ttTabDefaultText,
+				    dp->dpTabIntervalTwips, UNITtyPOINTS );
     }
 
     return;
@@ -427,7 +426,7 @@ static APP_BUTTON_CALLBACK_H( tedTabToolAddTab, w, voidtt )
 	{
 	appQuestionRunErrorDialog( tt->ttApplication,
 					tt->ttInspector->aiTopWidget,
-					(APP_WIDGET)w,
+					w,
 					tpr->tprTooCloseMessage );
 	return;
 	}
@@ -440,7 +439,7 @@ static APP_BUTTON_CALLBACK_H( tedTabToolAddTab, w, voidtt )
     if  ( changed < 0 )
 	{ LDEB(changed); return;	}
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
 
     if  ( tedAppChangeParagraphProperties( ea, &updMask, pp ) )
@@ -491,7 +490,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangeParaTabs, w, voidtt )
 	{
 	appQuestionRunErrorDialog( tt->ttApplication,
 					tt->ttInspector->aiTopWidget,
-					(APP_WIDGET)w,
+					w,
 					tpr->tprTooCloseMessage );
 	return;
 	}
@@ -500,7 +499,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangeParaTabs, w, voidtt )
 	{ tt->ttTabStopValue.tsTwips= value;	}
 
     /*  4  */
-    if  ( tt->ttTabStopNumber >= 0		&&
+    if  ( tt->ttTabStopNumber >= 0			&&
 	  tt->ttTabStopNumber < tsl->tslTabStopCount	)
 	{ docParaDeleteTab( pp, tt->ttTabStopNumber );	}
 
@@ -509,7 +508,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatChangeParaTabs, w, voidtt )
     if  ( changed < 0 )
 	{ LDEB(changed); return;	}
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
 
     if  ( tedAppChangeParagraphProperties( ea, &updMask, pp ) )
@@ -550,7 +549,7 @@ static APP_BUTTON_CALLBACK_H( tedTabToolDeleteTab, w, voidtt )
     if  ( tt->ttTabStopNumber < tsl->tslTabStopCount )
 	{ tt->ttTabStopValue= tsl->tslTabStops[tt->ttTabStopNumber];	}
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
 
     if  ( tedAppChangeParagraphProperties( ea, &updMask, pp ) )
@@ -581,7 +580,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatTabsApplyDocPushed, w, voidtt )
 
     PropertyMask		updMask;
 
-    PROPmaskCLEAR( &updMask );
+    utilPropMaskClear( &updMask );
     PROPmaskADD( &updMask, DPpropDEFTAB );
 
     value= dp->dpTabIntervalTwips;
@@ -594,7 +593,7 @@ static APP_BUTTON_CALLBACK_H( tedFormatTabsApplyDocPushed, w, voidtt )
     if  ( changed )
 	{ dp->dpTabIntervalTwips= value;	}
 
-    if  ( tedSetDocumentProperties( ea, dp, &updMask ) )
+    if  ( tedAppSetDocumentProperties( ea, dp, &updMask ) )
 	{ LDEB(1);	}
 
     return;
@@ -703,8 +702,9 @@ void tedFormatFillTabsPage(	TabsTool *			tt,
 					tedFormatTabsRevertDocPushed,
 					tedFormatTabsApplyDocPushed, tt );
     /**/
-    appGuiMakeListInColumn( &(tt->ttTabPositionList), pageWidget,
-			    visibleItems, tedTabsToolTabChosen, (void *)tt );
+    appGuiMakeListInColumn( &(tt->ttTabPositionList),
+		pageWidget, visibleItems,
+		tedTabsToolTabChosen, (APP_BUTTON_CALLBACK_T)0, (void *)tt );
 
     /**/
     appMakeColumnFrameInColumn( &(tt->ttTabPropertyFrame),
@@ -749,11 +749,11 @@ void tedFormatFillTabsPage(	TabsTool *			tt,
 
 /************************************************************************/
 /*									*/
-/*  Finish the paragraph page.						*/
+/*  Finish the tabs page.						*/
 /*									*/
 /************************************************************************/
 
-void tedFormatFillTabsChoosers(		TabsTool *			tt )
+void tedTabsToolFillChoosers(		TabsTool *			tt )
     {
     const TabsPageResources *	tpr= tt->ttPageResources;
 
@@ -809,7 +809,7 @@ static AppConfigurableResource TED_TedTabsSubjectResourceTable[]=
     {
     APP_RESOURCE( "formatToolTabs",
 	offsetof(InspectorSubjectResources,isrSubjectName),
-	"Tabs" ),
+	"Tab Stops" ),
     APP_RESOURCE( "formatToolChangeTabs",
 	offsetof(InspectorSubjectResources,isrApplyToSubject),
 	"Change Tab" ),
@@ -871,7 +871,7 @@ static AppConfigurableResource TED_TedTabsToolResourceTable[]=
 	offsetof(TabsPageResources,tprAlignmentOptionTexts[DOCtaRIGHT]),
 	"Right" ),
     APP_RESOURCE( "formatToolTabAlignmentCenter",
-	offsetof(TabsPageResources,tprAlignmentOptionTexts[DOCtaCENTRE]),
+	offsetof(TabsPageResources,tprAlignmentOptionTexts[DOCtaCENTER]),
 	"Center" ),
     APP_RESOURCE( "formatToolTabAlignmentDecimal",
 	offsetof(TabsPageResources,tprAlignmentOptionTexts[DOCtaDECIMAL]),
@@ -914,15 +914,24 @@ void tedFormatToolGetTabsResourceTable(	EditApplication *		ea,
 					TabsPageResources *		tpr,
 					InspectorSubjectResources *	isr )
     {
-    appGuiGetResourceValues( ea, (void *)tpr,
+    static int	gotToolResources= 0;
+    static int	gotSubjectResources= 0;
+
+    if  ( ! gotToolResources )
+	{
+	appGuiGetResourceValues( &gotToolResources, ea, (void *)tpr,
 				TED_TedTabsToolResourceTable,
 				sizeof(TED_TedTabsToolResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
-    appGuiGetResourceValues( ea, (void *)isr,
+    if  ( ! gotSubjectResources )
+	{
+	appGuiGetResourceValues( &gotSubjectResources, ea, (void *)isr,
 				TED_TedTabsSubjectResourceTable,
 				sizeof(TED_TedTabsSubjectResourceTable)/
 				sizeof(AppConfigurableResource) );
+	}
 
     return;
     }
