@@ -17,24 +17,24 @@
 
 typedef struct TabsPageResources
     {
-    char *		tprDefaultTabStopsLabel;
-    char *		tprDefaultTabStopsEvery;
-    char *		tprDefaultTabStopsRevert;
-    char *		tprDefaultTabStopsApply;
+    const char *	tprDefaultTabStopsLabel;
+    const char *	tprDefaultTabStopsEvery;
+    const char *	tprDefaultTabStopsRevert;
+    const char *	tprDefaultTabStopsApply;
 
-    char *		tprPositions;
+    const char *	tprPositions;
 
-    char *		tprTabStop;
+    const char *	tprTabStop;
 
-    char *		tprPosition;
+    const char *	tprPosition;
 
-    char *		tprAlignment;
-    char *		tprAlignmentOptionTexts[DOCta_COUNT];
+    const char *	tprAlignment;
+    const char *	tprAlignmentOptionTexts[DOCta_COUNT];
 
-    char *		tprLeader;
-    char *		tprLeaderOptionTexts[DOCtl_COUNT];
+    const char *	tprLeader;
+    const char *	tprLeaderOptionTexts[DOCtl_COUNT];
 
-    char *		tprTooCloseMessage;
+    const char *	tprTooCloseMessage;
     } TabsPageResources;
 
 /************************************************************************/
@@ -49,7 +49,7 @@ typedef struct TabsTool
     AppInspector *		ttInspector;
     const TabsPageResources *	ttPageResources;
 
-    int				ttUnitType;
+    int				ttUnitInt;
 
     DocumentProperties		ttDocPropertiesSet;
     DocumentProperties		ttDocPropertiesChosen;
@@ -92,11 +92,14 @@ extern void tedFormatToolGetTabsResourceTable(	EditApplication *	ea,
 					TabsPageResources *		spr,
 					InspectorSubjectResources *	isr );
 
-extern void tedFormatToolRefreshTabsTool(
+extern void tedRefreshTabsTool(
 				TabsTool *			tt,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	bs,
+				const SelectionGeometry *	sg,
+				const SelectionDescription *	sd,
 				const DocumentProperties *	dp );
 
 extern void tedFormatFillTabsPage( TabsTool *			tt,
@@ -105,7 +108,7 @@ extern void tedFormatFillTabsPage( TabsTool *			tt,
 				APP_WIDGET			pageWidget,
 				const InspectorSubjectResources *	isr );
 
-extern void tedFormatFillTabsChoosers(	TabsTool *			tt );
+extern void tedTabsToolFillChoosers(	TabsTool *			tt );
 
 extern void tedFormatFinishTabsPage(	TabsTool *			tt );
 
