@@ -22,7 +22,7 @@
 /************************************************************************/
 
 void appColorChooserSetColor(	ColorChooser *		cc,
-				int			explicit,
+				int			colorExplicit,
 				const RGB8Color *	rgb8 )
     {
     int		changed= 0;
@@ -35,7 +35,7 @@ void appColorChooserSetColor(	ColorChooser *		cc,
 
     if  ( cc->ccColorExplicit )
 	{
-	if  ( explicit )
+	if  ( colorExplicit )
 	    {
 	    if  ( bmRGB8ColorsDiffer( &(cc->ccColorChosen), rgb8 ) )
 		{
@@ -49,7 +49,7 @@ void appColorChooserSetColor(	ColorChooser *		cc,
 	    }
 	}
     else{
-	if  ( explicit )
+	if  ( colorExplicit )
 	    {
 	    cc->ccColorExplicit= 1;
 	    cc->ccColorChosen= *rgb8;
@@ -61,7 +61,7 @@ void appColorChooserSetColor(	ColorChooser *		cc,
 	{
 	cc->ccInplaceColorAllocated= 0;
 
-	appExposeRectangle( &(cc->ccInplaceDrawingData), 0, 0, 0, 0 );
+	appExposeDrawnPulldownInplace( &(cc->ccPulldown) );
 	}
 
     return;
@@ -73,7 +73,7 @@ void appColorChooserUnset(	ColorChooser *	cc )
 	{
 	cc->ccColorSet= 0;
 
-	appExposeRectangle( &(cc->ccInplaceDrawingData), 0, 0, 0, 0 );
+	appExposeDrawnPulldownInplace( &(cc->ccPulldown) );
 	}
 
     return;
@@ -88,31 +88,39 @@ void appColorChooserUnset(	ColorChooser *	cc )
 static APP_EVENT_HANDLER_H( appColorChooserRedrawInplace, w, voidcc, exposeEvent )
     {
     ColorChooser *	cc= (ColorChooser *)voidcc;
-    AppDrawingData *	add= &(cc->ccInplaceDrawingData);
+    AppDrawingData *	addI= &(cc->ccPulldown.adpInplaceDrawingData);
 
     int			wide;
     int			high;
 
+    const int		ox= 0;
+    const int		oy= 0;
+    DocumentRectangle	drClip;
     DocumentRectangle	drI;
 
-    appDrawnPulldownDrawArrow( &wide, &high, w, add );
+    appCollectExposures( &drClip, addI, ox, oy, exposeEvent );
+
+    appDrawnPulldownDrawArrow( &drClip, &wide, &high, w, addI );
+
+    /* Why? */
+    appDrawNoClipping( addI );
 
     drI.drX0= 3; drI.drX1= wide- 3;
     drI.drY0= 3; drI.drY1= high- 3;
 
     if  ( ! cc->ccColorSet )
 	{
-	appDrawSetForegroundWhite( add );
-	appDrawFillRectangle( add, drI.drX0, drI.drY0,
+	appDrawSetForegroundWhite( addI );
+	appDrawFillRectangle( addI, drI.drX0, drI.drY0,
 			    drI.drX1- drI.drX0+ 1, drI.drY1- drI.drY0+ 1 );
 
-	appDrawSetForegroundBlack( add );
+	appDrawSetForegroundBlack( addI );
 
-	appDrawDrawRectangle( add, drI.drX0, drI.drY0,
+	appDrawDrawRectangle( addI, drI.drX0, drI.drY0,
 			    drI.drX1- drI.drX0+ 1, drI.drY1- drI.drY0+ 1 );
 
-	appDrawDrawLine( add, drI.drX0, drI.drY0, drI.drX1, drI.drY1 );
-	appDrawDrawLine( add, drI.drX0, drI.drY1, drI.drX1, drI.drY0 );
+	appDrawDrawLine( addI, drI.drX0, drI.drY0, drI.drX1, drI.drY1 );
+	appDrawDrawLine( addI, drI.drX0, drI.drY1, drI.drX1, drI.drY0 );
 
 	return;
 	}
@@ -122,17 +130,18 @@ static APP_EVENT_HANDLER_H( appColorChooserRedrawInplace, w, voidcc, exposeEvent
 	const ColorChooserResources *	ccr= cc->ccResources;
 	const int			row= 0;
 
-	appDrawSetFont( add, cc->ccTextFont );
+	appDrawSetFont( addI, cc->ccInplaceScreenFont );
 
-	appDrawSetForegroundWhite( add );
-	appDrawFillRectangle( add, drI.drX0, drI.drY0,
+	appDrawSetForegroundWhite( addI );
+	appDrawFillRectangle( addI, drI.drX0, drI.drY0,
 			    drI.drX1- drI.drX0+ 1, drI.drY1- drI.drY0+ 1 );
 
-	appDrawSetForegroundBlack( add );
+	appDrawSetForegroundBlack( addI );
 
-	appDrawDrawString( add, 
+	appDrawDrawString( addI, 
 		    cc->ccXShift+ 4, ( row+ 1 )* cc->ccStripHigh- 7,
-		    ccr->ccrAutomaticColor, strlen( ccr->ccrAutomaticColor ) );
+		    ccr->ccrAutomaticColor,
+		    strlen( ccr->ccrAutomaticColor ) );
 
 	return;
 	}
@@ -149,30 +158,122 @@ static APP_EVENT_HANDLER_H( appColorChooserRedrawInplace, w, voidcc, exposeEvent
 	}
 
     /**/
-    appDrawSetForegroundColor( add, &(cc->ccInplaceColor) );
+    appDrawSetForegroundColor( addI, &(cc->ccInplaceColor) );
 
-    appDrawFillRectangle( add, drI.drX0, drI.drY0,
+    appDrawFillRectangle( addI, drI.drX0, drI.drY0,
 			    drI.drX1- drI.drX0+ 1, drI.drY1- drI.drY0+ 1 );
 
     /**/
-    appDrawSetForegroundBlack( add );
+    appDrawSetForegroundBlack( addI );
 
-    appDrawDrawRectangle( add, drI.drX0, drI.drY0,
+    appDrawDrawRectangle( addI, drI.drX0, drI.drY0,
 			    drI.drX1- drI.drX0+ 1, drI.drY1- drI.drY0+ 1 );
 
     return;
     }
 
+/************************************************************************/
+
+static int appColorChooserGetColor(	ColorChooser *			cc,
+					ColorChooserPaletteColor **	pCcpc,
+					int				row,
+					int				col )
+    {
+    int				color;
+    ColorChooserPaletteColor *	ccpc;
+    const RGB8Color *		rgb8;
+
+    color= ( row- 1 )* cc->ccColumns+ col;
+    if  ( color < 0 || color >= cc->ccColorCount )
+	{ LLDEB(color,cc->ccColorCount); return 1;	}
+
+    ccpc= cc->ccColors+ color;
+    if  ( ccpc->ccpcStatus == CCstatusFREE )
+	{ /*LDEB(ccpc->ccpcStatus);*/ return 1;	}
+    rgb8= &(ccpc->ccpcRGB8Color);
+
+    if  ( ! ccpc->ccpcColorAllocated )
+	{
+	if  ( appColorRgb( &(ccpc->ccpcAllocatedColor),
+				    &(cc->ccPulldownColors),
+				    rgb8->rgb8Red,
+				    rgb8->rgb8Green,
+				    rgb8->rgb8Blue ) )
+	    { LDEB(1); return 1;	}
+
+	ccpc->ccpcColorAllocated= 1;
+	}
+
+    *pCcpc= ccpc;
+    return 0;
+    }
+
+/************************************************************************/
+
+static int appColorChooserAllocatePullDownDrawingData(
+					ColorChooser *			cc )
+    {
+    AppDrawingData *	addP= &(cc->ccPulldown.adpPulldownDrawingData);
+
+    if  ( appFinishDrawnPulldownPulldown( &(cc->ccPulldown) ) )
+	{ LDEB(1); return -1;	}
+
+    if  ( appAllocateColors( addP, &(cc->ccPulldownColors) ) )
+	{ LDEB(1);	}
+
+    appColorRgb( &(cc->ccPulldownBackgroundColor),
+				    &(cc->ccPulldownColors), 128, 128,128 );
+
+    cc->ccPulldownScreenFont= appGuiGetLabelFont( addP, cc->ccLabelWidget );
+
+    return 0;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Check that drawing resources for the pulldown are allocated.	*/
+/*									*/
+/************************************************************************/
+
+# ifdef USE_GTK
+static void appColorChooserAllocatePulldownResources( void * voidcc )
+    {
+    ColorChooser *	cc= (ColorChooser *)voidcc;
+    int			row;
+    int			col;
+
+    if  ( cc->ccPulldown.adpPulldownDrawingDataSet )
+	{ return;	}
+
+    if  ( appColorChooserAllocatePullDownDrawingData( cc ) )
+	{ LDEB(1); return;	}
+
+    for ( row= 1; row < cc->ccStrips- 1; row++ )
+	{
+	for ( col= 0; col < cc->ccColumns; col++ )
+	    {
+	    ColorChooserPaletteColor *	ccpc;
+
+	    if  ( appColorChooserGetColor( cc, &ccpc, row, col ) )
+		{ continue;	}
+
+	    }
+	}
+
+    return;
+    }
+# endif
+
 /************************************************************************/
 /*									*/
 /*  Redraw the 'Pulldown' part of a Color Chooser.			*/
 /*									*/
 /************************************************************************/
 
-static APP_EVENT_HANDLER_H( tedColorChooserRedrawPulldown, w, voidcc, exposeEvent )
+static APP_EVENT_HANDLER_H( appColorChooserRedrawPulldown, w, voidcc, exposeEvent )
     {
     ColorChooser *	cc= (ColorChooser *)voidcc;
-    AppDrawingData *	add= &(cc->ccPulldownDrawingData);
+    AppDrawingData *	addP= &(cc->ccPulldown.adpPulldownDrawingData);
 
     int			inplaceWide;
     int			inplaceHigh;
@@ -186,50 +287,43 @@ static APP_EVENT_HANDLER_H( tedColorChooserRedrawPulldown, w, voidcc, exposeEven
     int			row;
     int			col;
 
-    if  ( ! cc->ccPulldownDrawingDataSet )
-	{
-	appSetDrawingDataForWidget( cc->ccPulldown.adpPulldownDrawing,
-					1.0, &(cc->ccPulldownDrawingData) );
+    int			currCol= -1;
+    int			currRow= -1;
 
-	if  ( appAllocateColors( add, &(cc->ccPulldownColors) ) )
-	    { LDEB(1);	}
-
-	cc->ccPulldownDrawingDataSet= 1;
-
-	appColorRgb( &(cc->ccPulldownBackgroundColor),
-				    &(cc->ccPulldownColors), 128, 128,128 );
-	}
+    if  ( appColorChooserAllocatePullDownDrawingData( cc ) )
+	{ LDEB(1); return;	}
 
     appDrawGetSizeOfWidget( &pulldownWide, &pulldownHigh, w );
     appDrawGetSizeOfWidget( &inplaceWide, &inplaceHigh,
 					cc->ccPulldown.adpInplaceDrawing );
 
-    appCollectExposures( &drClip, add, ox, oy, exposeEvent );
+    appCollectExposures( &drClip, addP, ox, oy, exposeEvent );
 
-    appDrawSetForegroundColor( add, &(cc->ccPulldownBackgroundColor) );
+    appDrawSetForegroundColor( addP, &(cc->ccPulldownBackgroundColor) );
 
-    appDrawFillRectangle( add, drClip.drX0, drClip.drY0,
+    appDrawFillRectangle( addP, drClip.drX0, drClip.drY0,
 					    drClip.drX1- drClip.drX0+ 1,
 					    drClip.drY1- drClip.drY0+ 1 );
 
-    appDrawSetForegroundBlack( add );
-    appDrawDrawRectangle( add, 0, 0, pulldownWide- 1, pulldownHigh- 1 );
+    appDrawSetForegroundBlack( addP );
+    appDrawDrawRectangle( addP, 0, 0, pulldownWide- 1, pulldownHigh- 1 );
 
     /**/
-    if  ( ! cc->ccTextFont )
-	{ XDEB(cc->ccTextFont);	}
+    if  ( cc->ccPulldownScreenFont < 0 )
+	{ LDEB(cc->ccPulldownScreenFont);	}
     else{
 	const ColorChooserResources *	ccr= cc->ccResources;
 
-	appDrawSetFont( add, cc->ccTextFont );
+	appDrawSetFont( addP, cc->ccPulldownScreenFont );
 
 	row= 0;
-	appDrawDrawString( add, 
+	appDrawDrawString( addP, 
 		    cc->ccXShift+ 4, ( row+ 1 )* cc->ccStripHigh- 5,
-		    ccr->ccrAutomaticColor, strlen( ccr->ccrAutomaticColor ) );
+		    ccr->ccrAutomaticColor,
+		    strlen( ccr->ccrAutomaticColor ) );
 
 	row= cc->ccStrips- 1;
-	appDrawDrawString( add, 
+	appDrawDrawString( addP, 
 		    cc->ccXShift+ 4, ( row+ 1 )* cc->ccStripHigh- 8,
 		    ccr->ccrMoreColors, strlen( ccr->ccrMoreColors ) );
 	}
@@ -238,32 +332,42 @@ static APP_EVENT_HANDLER_H( tedColorChooserRedrawPulldown, w, voidcc, exposeEven
 	{
 	for ( col= 0; col < cc->ccColumns; col++ )
 	    {
-	    int				color;
 	    ColorChooserPaletteColor *	ccpc;
+	    const RGB8Color *		rgb8;
 
-	    color= ( row- 1 )* cc->ccColumns+ col;
-	    if  ( color < 0 || color >= cc->ccColorCount )
-		{ LLDEB(color,cc->ccColorCount); continue;	}
+	    if  ( appColorChooserGetColor( cc, &ccpc, row, col ) )
+		{ continue;	}
 
-	    ccpc= cc->ccColors+ color;
-	    if  ( ccpc->ccpcStatus == CCstatusFREE )
-		{ LDEB(ccpc->ccpcStatus); continue;	}
+	    appDrawSetForegroundColor( addP, &(ccpc->ccpcAllocatedColor) );
 
-	    if  ( ! ccpc->ccpcColorAllocated )
-		{
-		if  ( appColorRgb( &(ccpc->ccpcAllocatedColor),
-					    &(cc->ccPulldownColors),
-					    ccpc->ccpcRGB8Color.rgb8Red,
-					    ccpc->ccpcRGB8Color.rgb8Green,
-					    ccpc->ccpcRGB8Color.rgb8Blue ) )
-		    { LDEB(1); continue;	}
+	    appDrawFillRectangle( addP,
+				    cc->ccXShift+ col* cc->ccColumnWide+ 3,
+				    row* cc->ccStripHigh+ 3,
+				    cc->ccColumnWide- 6,
+				    cc->ccStripHigh- 6 );
 
-		ccpc->ccpcColorAllocated= 1;
-		}
+	    rgb8= &(ccpc->ccpcRGB8Color);
+	    if  ( cc->ccColorExplicit					&&
+		  cc->ccColorSet					&&
+		  rgb8->rgb8Red == cc->ccColorChosen.rgb8Red		&&
+		  rgb8->rgb8Green == cc->ccColorChosen.rgb8Green	&&
+		  rgb8->rgb8Blue == cc->ccColorChosen.rgb8Blue		)
+		{ currCol= col; currRow= row;	}
+	    }
+	}
 
-	    appDrawSetForegroundColor( add, &(ccpc->ccpcAllocatedColor) );
+    /**/
+    appDrawSetForegroundBlack( addP );
 
-	    appDrawFillRectangle( add,
+    appDrawSetLineAttributes( addP, 1,
+				LINEstyleSOLID, LINEcapBUTT, LINEjoinMITER, 
+				(const unsigned char *)0, 0 );
+
+    for ( row= 1; row < cc->ccStrips- 1; row++ )
+	{
+	for ( col= 0; col < cc->ccColumns; col++ )
+	    {
+	    appDrawDrawRectangle( addP,
 				    cc->ccXShift+ col* cc->ccColumnWide+ 3,
 				    row* cc->ccStripHigh+ 3,
 				    cc->ccColumnWide- 6,
@@ -271,19 +375,28 @@ static APP_EVENT_HANDLER_H( tedColorChooserRedrawPulldown, w, voidcc, exposeEven
 	    }
 	}
 
-    /**/
-    appDrawSetForegroundBlack( add );
-
-    for ( row= 1; row < cc->ccStrips- 1; row++ )
+    if  ( currRow >= 0 && currCol >= 0 )
 	{
-	for ( col= 0; col < cc->ccColumns; col++ )
-	    {
-	    appDrawDrawRectangle( add,
-				    cc->ccXShift+ col* cc->ccColumnWide+ 3,
-				    row* cc->ccStripHigh+ 3,
+	appDrawSetLineAttributes( addP, 3,
+				LINEstyleSOLID, LINEcapBUTT, LINEjoinMITER, 
+				(const unsigned char *)0, 0 );
+
+	appDrawDrawRectangle( addP, cc->ccXShift+ currCol* cc->ccColumnWide+ 3,
+				    currRow* cc->ccStripHigh+ 3,
 				    cc->ccColumnWide- 6,
 				    cc->ccStripHigh- 6 );
-	    }
+
+	appDrawSetForegroundWhite( addP );
+
+	appDrawSetLineAttributes( addP, 1,
+				LINEstyleSOLID, LINEcapBUTT, LINEjoinMITER, 
+				(const unsigned char *)0, 0 );
+
+	appDrawDrawRectangle( addP, cc->ccXShift+ currCol* cc->ccColumnWide+ 2,
+				    currRow* cc->ccStripHigh+ 2,
+				    cc->ccColumnWide- 4,
+				    cc->ccStripHigh- 4 );
+
 	}
 
     return;
@@ -331,7 +444,7 @@ static APP_EVENT_HANDLER_H( appColorChooserClickedPulldown, w, voidcc, mouseEven
 	if  ( cc->ccColorExplicit )
 	    {
 	    cc->ccColorExplicit= 0;
-	    appExposeRectangle( &(cc->ccInplaceDrawingData), 0, 0, 0, 0 );
+	    appExposeDrawnPulldownInplace( &(cc->ccPulldown) );
 	    }
 
 	if  ( cc->ccCallback )
@@ -375,9 +488,9 @@ static APP_EVENT_HANDLER_H( appColorChooserClickedPulldown, w, voidcc, mouseEven
 
     if  ( cc->ccCallback )
 	{
-	const int	explicit= 1;
+	const int	colorExplicit= 1;
 
-	appColorChooserSetColor( cc, explicit, &(ccpc->ccpcRGB8Color) );
+	appColorChooserSetColor( cc, colorExplicit, &(ccpc->ccpcRGB8Color) );
 
 	(*cc->ccCallback)( cc, cc->ccWhich, cc->ccTarget,
 				CHOICEccCOLOR, &(ccpc->ccpcRGB8Color) );
@@ -541,6 +654,16 @@ static int appColorChooserCompareIntensity(	const void *	vccpc1,
     ccpc1= (const ColorChooserPaletteColor *)vccpc1;
     ccpc2= (const ColorChooserPaletteColor *)vccpc2;
 
+    if  ( ccpc1->ccpcStatus == CCstatusFREE	&&
+	  ccpc2->ccpcStatus == CCstatusFREE	)
+	{ return  0;	}
+    if  ( ccpc1->ccpcStatus != CCstatusFREE	&&
+	  ccpc2->ccpcStatus == CCstatusFREE	)
+	{ return -1;	}
+    if  ( ccpc1->ccpcStatus == CCstatusFREE	&&
+	  ccpc2->ccpcStatus != CCstatusFREE	)
+	{ return  1;	}
+
     rgb1= &(ccpc1->ccpcRGB8Color);
     rgb2= &(ccpc2->ccpcRGB8Color);
 
@@ -570,6 +693,16 @@ static int appColorChooserCompareHue(		const void *	vccpc1,
     ccpc1= (const ColorChooserPaletteColor *)vccpc1;
     ccpc2= (const ColorChooserPaletteColor *)vccpc2;
 
+    if  ( ccpc1->ccpcStatus == CCstatusFREE	&&
+	  ccpc2->ccpcStatus == CCstatusFREE	)
+	{ return  0;	}
+    if  ( ccpc1->ccpcStatus != CCstatusFREE	&&
+	  ccpc2->ccpcStatus == CCstatusFREE	)
+	{ return -1;	}
+    if  ( ccpc1->ccpcStatus == CCstatusFREE	&&
+	  ccpc2->ccpcStatus != CCstatusFREE	)
+	{ return  1;	}
+
     rgb1= &(ccpc1->ccpcRGB8Color);
     rgb2= &(ccpc2->ccpcRGB8Color);
 
@@ -767,8 +900,7 @@ void appColorChooserSuggestPalette(	ColorChooser *		cc,
 
 void appCleanColorChooser(		ColorChooser *		cc )
     {
-    appCleanDrawingData( &(cc->ccInplaceDrawingData) );
-    appCleanDrawingData( &(cc->ccPulldownDrawingData) );
+    appCleanDrawnPulldown( &(cc->ccPulldown) );
 
     if  ( cc->ccColors )
 	{ free( cc->ccColors );	}
@@ -777,16 +909,17 @@ void appCleanColorChooser(		ColorChooser *		cc )
     }
 
 void appFinishColorChooser(	ColorChooser *		cc,
-				APP_FONT *		textFont )
+				APP_WIDGET		fontWidget )
     {
-    AppDrawingData *	add= &(cc->ccInplaceDrawingData);
+    AppDrawingData *	addI= &(cc->ccPulldown.adpInplaceDrawingData);
 
     int			wide;
     int			strips;
 
-    appSetDrawingDataForWidget( cc->ccPulldown.adpInplaceDrawing, 1.0, add );
+    if  ( appFinishDrawnPulldownInplace( &(cc->ccPulldown) ) )
+	{ LDEB(1);	}
 
-    if  ( appAllocateColors( add, &(cc->ccInplaceColors) ) )
+    if  ( appAllocateColors( addI, &(cc->ccInplaceColors) ) )
 	{ LDEB(1);	}
 
     appDrawGetSizeOfWidget( &wide, &(cc->ccStripHigh),
@@ -801,8 +934,10 @@ void appFinishColorChooser(	ColorChooser *		cc,
 
     appGuiSetDrawnPulldownStrips( &(cc->ccPulldown), cc->ccStrips );
 
+    cc->ccLabelWidget= fontWidget;
+
     /**/
-    cc->ccTextFont= textFont;
+    cc->ccInplaceScreenFont= appGuiGetLabelFont( addI, fontWidget );
 
     return;
     }
@@ -815,15 +950,14 @@ void appInitColorChooser(	ColorChooser *			cc )
 
     appInitDrawnPulldown( &(cc->ccPulldown) );
 
-    appInitDrawingData( &(cc->ccInplaceDrawingData) );
-    appInitDrawingData( &(cc->ccPulldownDrawingData) );
-    cc->ccPulldownDrawingDataSet= 0;
+    cc->ccLabelWidget= (APP_WIDGET)0;
 
     appInitColors( &(cc->ccInplaceColors) );
     appInitColors( &(cc->ccPulldownColors) );
 
     cc->ccInplaceColorAllocated= 0;
-    cc->ccTextFont= (APP_FONT *)0;
+    cc->ccInplaceScreenFont= -1;
+    cc->ccPulldownScreenFont= -1;
 
     bmInitRGB8Color( &(cc->ccColorChosen) );
     cc->ccColorExplicit= 0;
@@ -851,8 +985,13 @@ void appMakeColorChooserInRow(	ColorChooser *			cc,
 				int				which,
 				void *				through )
     {
-    const int	valueColumn= 1;
-    const int	valueColspan= 1;
+    const int			valueColumn= 1;
+    const int			valueColspan= 1;
+    AppDrawnPulldownPuldown	pullDown= (AppDrawnPulldownPuldown)0;
+
+#   ifdef USE_GTK
+    pullDown= appColorChooserAllocatePulldownResources;
+#   endif
 
     /**/
     cc->ccResources= ccr;
@@ -862,11 +1001,12 @@ void appMakeColorChooserInRow(	ColorChooser *			cc,
     cc->ccTarget= through;
 
     appMakeDrawnPulldownInRow( &(cc->ccPulldown),
-					    appColorChooserRedrawInplace,
-					    tedColorChooserRedrawPulldown,
-					    appColorChooserClickedPulldown,
-					    row, valueColumn, valueColspan,
-					    (void *)cc );
+				    appColorChooserRedrawInplace,
+				    appColorChooserRedrawPulldown,
+				    appColorChooserClickedPulldown,
+				    pullDown,
+				    row, valueColumn, valueColspan,
+				    (void *)cc );
 
     /**/
     cc->ccFilled= 1;
@@ -922,10 +1062,10 @@ void appColorChooserColorChosen(	PropertyMask *		isSetMask,
 					RGB8Color *		rgb8To,
 					int *			pExplicit,
 					const RGB8Color *	rgb8Set,
-					int			explicit,
+					int			colorExplicit,
 					int			which )
     {
-    if  ( explicit )
+    if  ( colorExplicit )
 	{
 	if  ( ! *pExplicit )
 	    {
