@@ -5,14 +5,14 @@
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
-#   include	<appGeoString.h>
+#   include	<geoString.h>
+
+#   include	<appDebugon.h>
 
 #   ifdef USE_GTK
 
 #   include	<gdk/gdkkeysyms.h>
 
-#   include	<appDebugon.h>
-
 /************************************************************************/
 /*									*/
 /*  Make a dialog with a vertical organisation.				*/
@@ -25,13 +25,15 @@ void appMakeVerticalDialog(	AppDialog *		ad,
 				APP_CLOSE_CALLBACK_T	closeCallback,
 				APP_DESTROY_CALLBACK_T	destroyCallback,
 				void *			through,
-				char *			widgetName )
+				const char *		widgetName )
     {
     APP_WIDGET		shell;
     APP_WIDGET		column;
 
+#   if  GTK_MAJOR_VERSION < 2
     const gboolean	homogeneous= FALSE;
     const gint		spacing= COLUMN_SPACING_GTK;
+#   endif
 
 #   if GTK_MAJOR_VERSION < 2
     shell= gtk_window_new( GTK_WINDOW_DIALOG );
@@ -146,7 +148,8 @@ void appGuiSetCancelButtonForDialog(	AppDialog *		ad,
 					keyCode, keyMask, GTK_ACCEL_VISIBLE );
     }
 
-void appGuiShowDialog(			AppDialog *		ad,
+void appGuiShowDialog(			EditApplication *	ea,
+					AppDialog *		ad,
 					APP_WIDGET		relative )
     {
     gint		x;
