@@ -1,12 +1,13 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
-#   include	<appDebugon.h>
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
-#   include	<appGeoString.h>
+#   include	<geoString.h>
+
+#   include	<appDebugon.h>
 
 #   ifdef USE_MOTIF
 
@@ -18,10 +19,19 @@
 
 /************************************************************************/
 /*									*/
-/*  Force keyboard focus to a modal dialog.				*/
+/*  1)  Force keyboard focus to a modal dialog.				*/
+/*									*/
+/*  2)  Remember te initial size of a shell as its minimum and its	*/
+/*	maximum size. This event handler can be used to prevent the	*/
+/*	user from resizing a dialog. E.G. because KDE does not listen	*/
+/*	to the window manager hints about this. On the other hand some	*/
+/*	window managers (mwm on vms!) cause this event handler to be	*/
+/*	called too early. That results in dialogs that are MUCH too	*/
+/*	small.								*/
 /*									*/
 /************************************************************************/
 
+/*  1  */
 static void appDialogSetFocus(		Widget			w,
 					XtPointer		voidqc,
 					XEvent *		event,
@@ -29,8 +39,7 @@ static void appDialogSetFocus(		Widget			w,
     {
     if  ( event->type == MapNotify )
 	{
-	XSetInputFocus( XtDisplay( w ), XtWindow( w ),
-					    RevertToNone, CurrentTime );
+	appGuiMotifSetFocusToWindow( w );
 
 	/*
 	XtRemoveEventHandler( w, StructureNotifyMask, False,
@@ -41,6 +50,9 @@ static void appDialogSetFocus(		Widget			w,
     *pRefused= 1; return;
     }
 
+#   define FIX_DIALOG_SIZE 0
+#   if FIX_DIALOG_SIZE
+/*  2  */
 static void appFixDialogSize(	Widget			w,
 				void *			through,
 				XEvent *		event,
@@ -64,6 +76,7 @@ static void appFixDialogSize(	Widget			w,
 
     return;
     }
+#   endif
 
 /************************************************************************/
 /*									*/
@@ -74,10 +87,10 @@ static void appFixDialogSize(	Widget			w,
 void appMakeVerticalDialog(	AppDialog *		ad,
 				Widget *		pPaned,
 				EditApplication *	ea,
-				APP_CLOSE_CALLBACK	closeCallback,
-				APP_DESTROY_CALLBACK	destroyCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
+				APP_DESTROY_CALLBACK_T	destroyCallback,
 				void *			through,
-				char *			widgetName )
+				const char *		widgetName )
     {
     Widget		shell;
     Widget		dialog;
@@ -104,18 +117,20 @@ void appMakeVerticalDialog(	AppDialog *		ad,
     XtSetArg( al[ac], XmNdeleteResponse,	XmDO_NOTHING ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
     XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
+    XtSetArg( al[ac], XmNwaitForWm,		False ); ac++; /* LessTif BUG */
+    XtSetArg( al[ac], XmNwmTimeout,		0 ); ac++; /* LessTif BUG */
     if  ( hints.flags & MWM_HINTS_FUNCTIONS )
 	{ XtSetArg( al[ac], XmNmwmFunctions,	hints.functions ); ac++; }
     if  ( hints.flags & MWM_HINTS_DECORATIONS )
 	{ XtSetArg( al[ac], XmNmwmDecorations,	hints.decorations ); ac++; }
 
     shell= XmCreateDialogShell( ea->eaToplevel.atTopWidget,
-							widgetName, al, ac );
+						(char *)widgetName, al, ac );
 
     if  ( closeCallback && ea->eaCloseAtom > 0 )
 	{
 	XmAddWMProtocolCallback( shell, ea->eaCloseAtom,
-						    closeCallback, through );
+						closeCallback, through );
 	}
 
     if  ( destroyCallback )
@@ -124,15 +139,18 @@ void appMakeVerticalDialog(	AppDialog *		ad,
 						destroyCallback, through );
 	}
 
+#   if FIX_DIALOG_SIZE
+    See Above
     XtAddEventHandler( shell, StructureNotifyMask, False,
 						appFixDialogSize, through );
+#   endif
 
     ac= 0;
     XtSetArg( al[ac], XmNmarginWidth,		0 ); ac++;
     XtSetArg( al[ac], XmNmarginHeight,		0 ); ac++;
     XtSetArg( al[ac], XmNallowResize,		True ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
-    /* Does not work due to motif bug.
+    /* Does not work due to motif bug:
     XtSetArg( al[ac], XmNdialogStyle,		XmDIALOG_APPLICATION_MODAL );
 									ac++;
     */
@@ -148,9 +166,9 @@ void appMakeVerticalDialog(	AppDialog *		ad,
     XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
     XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
     XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac], XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac], XmNmarginHeight,		5 ); ac++;
-    XtSetArg( al[ac], XmNspacing,		5 ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
     paned= XmCreatePanedWindow( dialog, WIDGET_NAME, al, ac );
 
     XtManageChild( paned );
@@ -175,7 +193,7 @@ void appGuiRunDialog(			AppDialog *		ad,
 			NULL );
 
     if  ( defaultButton )
-	{ XmProcessTraversal( defaultButton, XmTRAVERSE_CURRENT ); }
+	{ appGuiFocusToWidget( defaultButton ); }
 #   endif
 
     XtAddGrab( ad->adTopWidget, True, False );
@@ -256,7 +274,17 @@ static void appRelativeMapCallback(	Widget		w,
     XtRemoveCallback( w, XmNmapCallback, appRelativeMapCallback, voidRelative );
     }
 
-void appGuiShowDialog(			AppDialog *		ad,
+/************************************************************************/
+/*									*/
+/*  Show a shell widget. Because of the RIDICULOUS and ABSURD situation	*/
+/*  the you cannot rely on a shell window to be visible after we tell	*/
+/*  it to become visible, we handle a series of events in a loop until	*/
+/*  it becomes visible.							*/
+/*									*/
+/************************************************************************/
+
+void appGuiShowDialog(			EditApplication *	ea,
+					AppDialog *		ad,
 					Widget			relative )
     {
     if  ( relative )
@@ -266,9 +294,12 @@ void appGuiShowDialog(			AppDialog *		ad,
 	}
 
     XtManageChild( ad->adDialog );
+    XSync( XtDisplay( ad->adDialog ), False );
 
     if  ( relative )
 	{ appDialogRelative( relative, ad->adDialog );	}
+
+    appGuiMotifWaitForWindow( XtParent( ad->adDialog ), ea->eaContext );
     }
 
 void appGuiHideDialog(			AppDialog *		ad )
