@@ -16,108 +16,6 @@
 
 #   include	<appDebugon.h>
 
-/************************************************************************/
-/*									*/
-/*  Allow a user to change the selected rectangle.			*/
-/*									*/
-/*  Move the vertical hair on the Document.				*/
-/*									*/
-/*  NOTE: Exposures and Drawing go to the document widget, NOT the	*/
-/*  ruler.								*/
-/*									*/
-/************************************************************************/
-
-void appHorMetricRulerMoveHair(		void *		voided,
-					int		from,
-					int		to,
-					int		asUpper	)
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    AppDrawingData *		add= &(ed->edDrawingData);
-    int				ox= ed->edVisibleRect.drX0;
-
-    int				high;
-
-    high= ed->edVisibleRect.drY1- ed->edVisibleRect.drY0;
-
-    if  ( from >= 0 )
-	{
-	if  ( asUpper )
-	    { appExposeRectangle( add, from- ox, 0, 2, high );		}
-	else{ appExposeRectangle( add, from- ox- 2, 0, 2, high );	}
-	}
-
-    if  ( to >= 0 )
-	{
-	appDrawSetForegroundBlack( add );
-
-	if  ( asUpper )
-	    {
-	    appDrawFillRectangle( add, to- ox, 0, 1, high );
-	    appDrawSetForegroundWhite( add );
-	    appDrawFillRectangle( add, to- ox+ 1, 0, 1, high );
-	    }
-	else{
-	    appDrawFillRectangle( add, to- ox- 1, 0, 1, high );
-	    appDrawSetForegroundWhite( add );
-	    appDrawFillRectangle( add, to- ox- 2, 0, 1, high );
-	    }
-	}
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Allow a user to change the selected rectangle.			*/
-/*									*/
-/*  Move the horizontal hair on the Document.				*/
-/*									*/
-/*  NOTE: Exposures and Drawing go to the document widget, NOT the	*/
-/*  ruler.								*/
-/*									*/
-/************************************************************************/
-
-void appVertMetricRulerMoveHair(	void *		voided,
-					int		from,
-					int		to,
-					int		asUpper	)
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    AppDrawingData *		add= &(ed->edDrawingData);
-    int				oy= ed->edVisibleRect.drY0;
-
-    int				wide;
-
-    wide= ed->edVisibleRect.drX1- ed->edVisibleRect.drX0;
-
-    if  ( from >= 0 )
-	{
-	if  ( asUpper )
-	    { appExposeRectangle( add, 0, from- oy, wide, 2 );		}
-	else{ appExposeRectangle( add, 0, from- oy- 2, wide, 2 );	}
-	}
-
-    if  ( to >= 0 )
-	{
-	appDrawSetForegroundBlack( add );
-
-	if  ( asUpper )
-	    {
-	    appDrawFillRectangle( add, 0, to- oy, wide, 1 );
-	    appDrawSetForegroundWhite( add );
-	    appDrawFillRectangle( add, 0, to- oy+ 1, wide, 1 );
-	    }
-	else{
-	    appDrawFillRectangle( add, 0, to- oy- 1, wide, 1 );
-	    appDrawSetForegroundWhite( add );
-	    appDrawFillRectangle( add, 0, to- oy- 2, wide, 1 );
-	    }
-	}
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Draw a selection rectangle over the document. Strictly spoken this	*/
@@ -225,42 +123,103 @@ void appDrawRectangleSelection(	DocumentRectangle *	drClip,
 /*									*/
 /************************************************************************/
 
-void appRemoveRectangleSelection( DocumentRectangle *	drVisible,
-				DocumentRectangle *	drSelected,
-				AppDrawingData *	add )
+static void appMetricRulerExposeX0(
+				const DocumentRectangle *	drVisible,
+				int				x0,
+				AppDrawingData *		add )
     {
     int			ox= drVisible->drX0;
-    int			oy= drVisible->drY0;
-
-    int			wide= drVisible->drX1- drVisible->drX0;
     int			high= drVisible->drY1- drVisible->drY0;
 
+    DocumentRectangle	drExpose;
+
+    drExpose.drX0= x0- ox- 2;
+    drExpose.drX1= x0+ 1;
+    drExpose.drY0= 0;
+    drExpose.drY1= high- 1;
+
+    appExposeRectangle( add, &drExpose );
+    }
+
+static void appMetricRulerExposeY0(
+				const DocumentRectangle *	drVisible,
+				int				y0,
+				AppDrawingData *		add )
+    {
+    int			oy= drVisible->drY0;
+    int			wide= drVisible->drX1- drVisible->drX0;
+
+    DocumentRectangle	drExpose;
+
+    drExpose.drX0= 0;
+    drExpose.drX1= wide- 1;
+    drExpose.drY0= y0- oy- 2;
+    drExpose.drY1= y0+ 1;
+
+    appExposeRectangle( add, &drExpose );
+    }
+
+static void appMetricRulerExposeX1(
+				const DocumentRectangle *	drVisible,
+				int				x1,
+				AppDrawingData *		add )
+    {
+    int			ox= drVisible->drX0;
+    int			high= drVisible->drY1- drVisible->drY0;
+
+    DocumentRectangle	drExpose;
+
+    drExpose.drX0= x1- ox;
+    drExpose.drX1= x1+ 1;
+    drExpose.drY0= 0;
+    drExpose.drY1= high- 1;
+
+    appExposeRectangle( add, &drExpose );
+    }
+
+static void appMetricRulerExposeY1(
+				const DocumentRectangle *	drVisible,
+				int				y1,
+				AppDrawingData *		add )
+    {
+    int			oy= drVisible->drY0;
+    int			wide= drVisible->drX1- drVisible->drX0;
+
+    DocumentRectangle	drExpose;
+
+    drExpose.drX0= 0;
+    drExpose.drX1= wide- 1;
+    drExpose.drY0= y1- oy;
+    drExpose.drY1= y1+ 1;
+
+    appExposeRectangle( add, &drExpose );
+    }
+
+void appRemoveRectangleSelection( const DocumentRectangle *	drVisible,
+				DocumentRectangle *		drSelected,
+				AppDrawingData *		add )
+    {
     if  ( drSelected->drX0 >= 0 )
 	{
-	appExposeRectangle( add, drSelected->drX0- ox- 2, 0, 2, high );
-
+	appMetricRulerExposeX0( drVisible, drSelected->drX0, add );
 	drSelected->drX0= -1;
 	}
 
     if  ( drSelected->drX1 >= 0 )
 	{
-	appExposeRectangle( add,
-			drSelected->drX1- ox, 0, 2, high );
-
+	appMetricRulerExposeX1( drVisible, drSelected->drX1, add );
 	drSelected->drX1= -1;
 	}
 
     if  ( drSelected->drY0 >= 0 )
 	{
-	appExposeRectangle( add, 0, drSelected->drY0- oy- 2, wide, 2 );
-
+	appMetricRulerExposeY0( drVisible, drSelected->drY0, add );
 	drSelected->drY0= -1;
 	}
 
     if  ( drSelected->drY1 >= 0 )
 	{
-	appExposeRectangle( add, 0, drSelected->drY1- oy, wide, 2 );
-
+	appMetricRulerExposeY1( drVisible, drSelected->drY1, add );
 	drSelected->drY1= -1;
 	}
 
@@ -273,21 +232,16 @@ void appSetHorRectangleSelection( DocumentRectangle *	drVisible,
 				int			x1Screen,
 				AppDrawingData *	add )
     {
-    int			ox= drVisible->drX0;
-    int			high= drVisible->drY1- drVisible->drY0;
-
     if  ( drSelected->drX0 != x0Screen )
 	{
 	if  ( drSelected->drX0 >= 0 )
 	    {
-	    appExposeRectangle( add,
-		    drSelected->drX0- ox- 2, 0, 2, high );
+	    appMetricRulerExposeX0( drVisible, drSelected->drX0, add );
 	    }
 
 	if  ( x0Screen >= 0 )
 	    {
-	    appExposeRectangle( add,
-				x0Screen- ox- 2, 0, 2, high );
+	    appMetricRulerExposeX0( drVisible, x0Screen, add );
 	    }
 
 	drSelected->drX0= x0Screen;
@@ -297,13 +251,12 @@ void appSetHorRectangleSelection( DocumentRectangle *	drVisible,
 	{
 	if  ( drSelected->drX1 >= 0 )
 	    {
-	    appExposeRectangle( add,
-			drSelected->drX1- ox, 0, 2, high );
+	    appMetricRulerExposeX1( drVisible, drSelected->drX1, add );
 	    }
 
 	if  ( x1Screen >= 0 )
 	    {
-	    appExposeRectangle( add, x1Screen- ox, 0, 2, high );
+	    appMetricRulerExposeX1( drVisible, x1Screen, add );
 	    }
 
 	drSelected->drX1= x1Screen;
@@ -319,19 +272,16 @@ void appSetVertRectangleSelection( DocumentRectangle *	drVisible,
 				int			y1Screen,
 				AppDrawingData *	add )
     {
-    int			oy= drVisible->drY0;
-    int			wide= drVisible->drX1- drVisible->drX0;
-
     if  ( drSelected->drY0 != y0Screen )
 	{
 	if  ( drSelected->drY0 >= 0 )
 	    {
-	    appExposeRectangle( add, 0, drSelected->drY0- oy- 2, wide, 2 );
+	    appMetricRulerExposeY0( drVisible, drSelected->drY0, add );
 	    }
 
 	if  ( y0Screen >= 0 )
 	    {
-	    appExposeRectangle( add, 0, y0Screen- oy- 2, wide, 2 );
+	    appMetricRulerExposeY0( drVisible, y0Screen, add );
 	    }
 
 	drSelected->drY0= y0Screen;
@@ -341,12 +291,12 @@ void appSetVertRectangleSelection( DocumentRectangle *	drVisible,
 	{
 	if  ( drSelected->drY1 >= 0 )
 	    {
-	    appExposeRectangle( add, 0, drSelected->drY1- oy, wide, 2 );
+	    appMetricRulerExposeY1( drVisible, drSelected->drY1, add );
 	    }
 
 	if  ( y1Screen >= 0 )
 	    {
-	    appExposeRectangle( add, 0, y1Screen- oy, wide, 2 );
+	    appMetricRulerExposeY1( drVisible, y1Screen, add );
 	    }
 
 	drSelected->drY1= y1Screen;
@@ -369,7 +319,7 @@ void appDocSetMetricTopRuler(	EditDocument *		ed,
 				APP_EVENT_HANDLER_T	mouseDown )
     {
     EditApplication *		ea= ed->edApplication;
-    AppDrawingData *		add= &(ed->edDrawingData);
+    AppDrawingData *		add= &(ed->edDocumentWidget.dwDrawingData);
 
     ed->edTopRuler= appMakeMetricRuler(
 
@@ -377,8 +327,8 @@ void appDocSetMetricTopRuler(	EditDocument *		ed,
 		    add->addMagnifiedPixelsPerTwip,
 		    magnification,		/*  magnification	*/
 
-		    ed->edLeftRulerWidthPixels,	/*  minUnused		*/
-		    ed->edRightRulerWidthPixels,/*  maxUnused		*/
+		    ed->edLeftRulerWidePixels,	/*  minUnused		*/
+		    ed->edRightRulerWidePixels,/*  maxUnused		*/
 
 		    add->addDocRect.drX0,	/*  documentC0		*/
 		    add->addDocRect.drX1,	/*  documentC1		*/
@@ -397,6 +347,9 @@ void appDocSetMetricTopRuler(	EditDocument *		ed,
 
     appDrawSetButtonPressHandler( ed->edTopRulerWidget,
 						mouseDown, (void *)ed );
+#   ifdef USE_GTK
+    gtk_widget_add_events( ed->edTopRulerWidget, GDK_POINTER_MOTION_MASK );
+#   endif
 
     return;
     }
@@ -415,7 +368,7 @@ void appDocSetMetricLeftRuler(	EditDocument *		ed,
 				APP_EVENT_HANDLER_T	mouseDown )
     {
     EditApplication *		ea= ed->edApplication;
-    AppDrawingData *		add= &(ed->edDrawingData);
+    AppDrawingData *		add= &(ed->edDocumentWidget.dwDrawingData);
 
     ed->edLeftRuler= appMakeMetricRuler(
 		    leftRulerWidth,		/*  sizeAcross		*/
@@ -441,6 +394,9 @@ void appDocSetMetricLeftRuler(	EditDocument *		ed,
 
     appDrawSetButtonPressHandler( ed->edLeftRulerWidget,
 						    mouseDown, (void *)ed );
+#   ifdef USE_GTK
+    gtk_widget_add_events( ed->edTopRulerWidget, GDK_POINTER_MOTION_MASK );
+#   endif
 
     return;
     }
