@@ -13,9 +13,13 @@
 
 #   include	<appSystem.h>
 #   include	"appFrame.h"
+#   include	"guiWidgetScreen.h"
+#   include	"guiWidgetDrawingSurface.h"
 
 #   include	<appDebugon.h>
 
+#   define	LOG_REDRAWS	0
+
 /************************************************************************/
 /*									*/
 /*  Callback for the file menu.						*/
@@ -45,9 +49,7 @@ int appDocSaveDocumentByName(		EditDocument *		ed,
     return 0;
     }
 
-void appDocFileSave(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
+APP_MENU_CALLBACK_H( appDocFileSave, option, voided, e )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
@@ -60,7 +62,24 @@ void appDocFileSave(	APP_WIDGET	option,
     if  ( ! ed->edFilename		||
 	  ed->edFileReadOnly		||
 	  ed->edFormat < 0		)
-	{ appDocFileSaveAs( option, voided, (void *)0 ); return; }
+	{
+	int	saved= 0;
+
+#	ifdef USE_MOTIF
+	appDocFileSaveAs( option, voided, (void *)0 );
+	saved= 1;
+#	endif
+
+#	ifdef USE_GTK
+	appDocFileSaveAs( option, voided );
+	saved= 1;
+#	endif
+
+	if  ( ! saved )
+	    { LSDEB(saved,ed->edFilename);	}
+
+	return;
+	}
 
     if  ( appDocSaveDocumentByName( ed, option, interactive,
 					    ed->edFormat, ed->edFilename ) )
@@ -84,9 +103,7 @@ void appDocFileSave(	APP_WIDGET	option,
 /*									*/
 /************************************************************************/
 
-void appDocFileClose(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
+APP_MENU_CALLBACK_H( appDocFileClose, option, voided, e )
     {
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
@@ -95,6 +112,8 @@ void appDocFileClose(	APP_WIDGET	option,
 	{ appRunReallyCloseDialog( option, ed ); return; }
 
     appCloseDocument( ea, ed );
+
+    return;
     }
 
 APP_CLOSE_CALLBACK_H( appDocFileCloseCallback, w, voided )
@@ -108,19 +127,19 @@ APP_CLOSE_CALLBACK_H( appDocFileCloseCallback, w, voided )
     appCloseDocument( ea, ed );
     }
 
-void appDocFileNew(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data	 )
+APP_MENU_CALLBACK_H( appDocFileNew, option, voided, e )
     {
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
+    const char * const	filename= (const char *)0;
 
-    appAppFileNew( option, (void *)ea, call_data );
+    if  ( appNewDocument( ea, filename ) )
+	{ SDEB(filename); }
+
+    return;
     }
 
-void appDocFileQuit(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
+APP_MENU_CALLBACK_H( appDocFileQuit, option, voided, e )
     {
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
@@ -130,36 +149,6 @@ void appDocFileQuit(	APP_WIDGET	option,
     return;
     }
 
-void appDocFileMini(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-
-    appIconifyShellWidget( ed->edToplevel.atTopWidget );
-
-    return;
-    }
-
-void appDocFileHide(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data	 )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    EditApplication *		ea= ed->edApplication;
-
-    if  ( ed->edIsVisible )
-	{
-	appHideShellWidget( ed->edToplevel.atTopWidget );
-
-	ed->edIsVisible= 0;
-
-	appDocVisible( ea, ed, ed->edIsVisible );
-	}
-
-    return;
-    }
-
 void appReportSaveFailure(	EditApplication *	ea,
 				APP_WIDGET		option,
 				APP_WIDGET		relative,
@@ -184,16 +173,14 @@ void appReportSaveFailure(	EditApplication *	ea,
 /*									*/
 /************************************************************************/
 
-void appDocAbout(	APP_WIDGET	option,
-			void *		voided,
-			void *		call_data )
+APP_MENU_CALLBACK_H( appDocAbout, option, voided, e )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
 
     ea->eaMainVisibleAsAbout= 1;
 
-    appShowShellWidget( ea->eaToplevel.atTopWidget );
+    appShowShellWidget( ea, ea->eaToplevel.atTopWidget );
 
     return;
     }
@@ -207,7 +194,7 @@ void appDocAbout(	APP_WIDGET	option,
 void appMakeDocVisible(	EditApplication *	ea,
 			EditDocument *		ed )
     {
-    appShowShellWidget( ed->edToplevel.atTopWidget );
+    appShowShellWidget( ea, ed->edToplevel.atTopWidget );
 
     if  ( ! ed->edIsVisible )
 	{
@@ -219,7 +206,7 @@ void appMakeDocVisible(	EditApplication *	ea,
 
 void appDocToFront(	APP_WIDGET	option,
 			void *		voided,
-			void *		call_data )
+			void *		e )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
@@ -237,9 +224,7 @@ void appDocToFront(	APP_WIDGET	option,
 /*									*/
 /************************************************************************/
 
-void appDocEditCopy(	APP_WIDGET	option,
-			void *		voided,
-			void *		voidpbcs )
+APP_MENU_CALLBACK_H( appDocEditCopy, option, voided, e )
     {
     EditDocument *			ed= (EditDocument *)voided;
     EditApplication *			ea= ed->edApplication;
@@ -250,9 +235,7 @@ void appDocEditCopy(	APP_WIDGET	option,
     (*ea->eaDocCopy)( ed ); return;
     }
 
-void appDocEditCut(	APP_WIDGET	option,
-			void *		voided,
-			void *		voidpbcs )
+APP_MENU_CALLBACK_H( appDocEditCut, option, voided, e )
     {
     EditDocument *			ed= (EditDocument *)voided;
     EditApplication *			ea= ed->edApplication;
@@ -263,18 +246,14 @@ void appDocEditCut(	APP_WIDGET	option,
     (*ea->eaDocCut)( ed ); return;
     }
 
-void appDocEditPaste(	APP_WIDGET	option,
-			void *		voided,
-			void *		voidpbcs )
+APP_MENU_CALLBACK_H( appDocEditPaste, option, voided, e )
     {
     EditDocument *			ed= (EditDocument *)voided;
 
-    appDocAskForPaste( ed, "PRIMARY" );
+    appDocAskForPaste( ed, "CLIPBOARD" );
     }
 
-void appDocEditSelAll(	APP_WIDGET	option,
-			void *		voided,
-			void *		voidpbcs )
+APP_MENU_CALLBACK_H( appDocEditSelAll, option, voided, e )
     {
     EditDocument *			ed= (EditDocument *)voided;
     EditApplication *			ea= ed->edApplication;
@@ -285,22 +264,6 @@ void appDocEditSelAll(	APP_WIDGET	option,
     ea->eaDocSelAll( ed ); return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Close a a document.							*/
-/*									*/
-/************************************************************************/
-
-void appCloseDocument(	EditApplication *	ea,
-			EditDocument *		ed	)
-    {
-    appRemoveDocument( ea, ed );
-
-    appDestroyShellWidget( ed->edToplevel.atTopWidget );
-
-    return;
-    }
-
 int appSetDocumentFilename(	EditDocument *		ed,
 				const char *		filename )
     {
@@ -388,8 +351,24 @@ int appSetDocumentTitle(	EditDocument *		ed,
 /*									*/
 /************************************************************************/
 
-void appFreeDocument(		EditApplication *	ea,
-				EditDocument *		ed )
+static void appCleanDocumentWidget(	DocumentWidget *	dw )
+    {
+#   ifdef USE_MOTIF
+    if  ( dw->dwInputContext )
+	{ XDestroyIC( dw->dwInputContext ); }
+#   endif
+
+#   if defined(USE_GTK) && defined(GTK_TYPE_IM_CONTEXT)
+    if  ( dw->dwInputContext )
+	{ g_object_unref( dw->dwInputContext ); }
+#   endif
+
+    appCleanDrawingData( &(dw->dwDrawingData) );
+    appCleanColors( &(dw->dwColors) );
+    }
+
+static void appCleanDocument(		EditApplication *	ea,
+					EditDocument *		ed )
     {
     if  ( ed->edLeftRuler )
 	{ (*ea->eaFreeLeftRuler)( ed->edLeftRuler ); }
@@ -405,27 +384,44 @@ void appFreeDocument(		EditApplication *	ea,
     if  ( ed->edTitle )
 	{ free( ed->edTitle );	}
 
-    if  ( ed->edCheckpointFilename )
-	{
-	if  ( appRemoveFile( ed->edCheckpointFilename ) )
-	    { SDEB(ed->edCheckpointFilename);	}
+    appCleanDocumentWidget( &(ed->edDocumentWidget) );
+    if  ( ed->edScreenWidget )
+	{ guiFreeScreenWidget( ed->edScreenWidget ); }
+    if  ( ed->edDrawingSurface )
+	{ drawFreeDrawingSurface( ed->edDrawingSurface ); }
 
-	free( ed->edCheckpointFilename );
-	}
-
-#   ifdef USE_MOTIF
-    if  ( ed->edInputContext )
-	{ XDestroyIC( ed->edInputContext ); }
-#   endif
-
-    appCleanDrawingData( &(ed->edDrawingData) );
-    appCleanColors( &(ed->edColors) );
+    return;
+    }
 
+static void appFreeDocument(		EditApplication *	ea,
+					EditDocument *		ed )
+    {
+    appCleanDocument( ea, ed );
     free( ed );
 
     return;
     }
 
+static void appInitDocumentWidget(	DocumentWidget *	dw )
+    {
+    dw->dwWidget= (APP_WIDGET)0;
+
+#   ifdef USE_MOTIF
+    dw->dwInputContext= (XIC)0;
+#   endif
+
+#   ifdef USE_GTK
+#	ifdef GTK_TYPE_IM_CONTEXT
+	dw->dwInputContext= (GtkIMContext *)0;
+#	else
+	dw->dwInputContext= 0;
+#	endif
+#   endif
+
+    appInitColors( &(dw->dwColors) );
+    appInitDrawingData( &(dw->dwDrawingData) );
+    }
+
 void appInitEditDocument(	EditApplication *	ea,
 				EditDocument *		ed )
     {
@@ -433,7 +429,6 @@ void appInitEditDocument(	EditApplication *	ea,
 
     ed->edFilename= (char *)0;
     ed->edTitle= (char *)0;
-    ed->edCheckpointFilename= (char *)0;
     ed->edFormat= -1;
     ed->edFileReadOnly= 0;
 
@@ -452,7 +447,9 @@ void appInitEditDocument(	EditApplication *	ea,
     ed->edScrolledWindow= (APP_WIDGET)0;
 	ed->edVerticalScrollbar= (APP_WIDGET)0;
 	ed->edHorizontalScrollbar= (APP_WIDGET)0;
-	ed->edDocumentWidget= (APP_WIDGET)0;
+	appInitDocumentWidget( &(ed->edDocumentWidget) );
+	ed->edScreenWidget= (ScreenWidget)0;
+	ed->edDrawingSurface= (DrawingSurface)0;
 
 	ed->edLeftRulerWidget= (APP_WIDGET)0;
 	ed->edTopRulerWidget= (APP_WIDGET)0;
@@ -464,14 +461,10 @@ void appInitEditDocument(	EditApplication *	ea,
 	ed->edRightRuler= (void *)0;
 	ed->edBottomRuler= (void *)0;
 
-#   ifdef USE_MOTIF
-    ed->edInputContext= (XIC)0;
-#   endif
-
-    ed->edLeftRulerWidthPixels= 0;
-    ed->edTopRulerHeightPixels= 0;
-    ed->edRightRulerWidthPixels= 0;
-    ed->edBottomRulerHeightPixels= 0;
+    ed->edLeftRulerWidePixels= 0;
+    ed->edTopRulerHighPixels= 0;
+    ed->edRightRulerWidePixels= 0;
+    ed->edBottomRulerHighPixels= 0;
 
     ed->edHasBeenChanged= 0;
     ed->edIsReadonly= 0;
@@ -480,11 +473,10 @@ void appInitEditDocument(	EditApplication *	ea,
     ed->edShellExtraWidth= -1;
     ed->edShellExtraHeight= -1;
 
-    appInitColors( &(ed->edColors) );
-    appInitDrawingData( &(ed->edDrawingData) );
-
     ed->edMapped= 0;
+    ed->edNotYetDrawn= 1;
 
+    appInitSelectRectangle( &(ed->edSelectRectangle) );
     return;
     }
 
@@ -496,7 +488,8 @@ int appMakeDocumentWindow(	EditDocument **		pEd,
     {
     char			scratch[100];
 
-    EditDocument *		ed;
+    int				rval= 0;
+    EditDocument *		ed= (EditDocument *)0;
 
     if  ( ! title )
 	{
@@ -511,10 +504,13 @@ int appMakeDocumentWindow(	EditDocument **		pEd,
     /*  1  */
     ed= (EditDocument *)malloc( sizeof(EditDocument) );
     if  ( ! ed )
-	{ XDEB(ed); return -1;	}
+	{ XDEB(ed); rval= -1; goto ready;	}
 
     appInitEditDocument( ea, ed );
 
+    ea->eaNextDocumentId++; /* Never 0 */
+    ed->edDocumentId= ea->eaNextDocumentId;
+
     if  ( filename )
 	{ ed->edFilename= strdup( filename );	}
     else{ ed->edFilename= (char *)0;		}
@@ -526,19 +522,27 @@ int appMakeDocumentWindow(	EditDocument **		pEd,
 	{
 	ed->edPrivateData= (*ea->eaMakePrivateData)();
 	if  ( ! ed->edPrivateData )
-	    { XDEB(ed->edPrivateData); appFreeDocument( ea, ed ); return -1; }
+	    { XDEB(ed->edPrivateData); rval= -1; goto ready; }
 	}
 
     if  ( appFinishDocumentWindow( ed ) )
-	{ appFreeDocument( ea, ed ); return -1; }
+	{ LDEB(1); rval= -1; goto ready; }
 
-    *pEd= ed; return 0;
+    *pEd= ed; ed= (EditDocument *)0; /* steal */
+
+  ready:
+
+    if  ( ed )
+	{ appFreeDocument( ea, ed ); }
+
+    return rval;
     }
 
 int appSetupDocument(	EditApplication *	ea,
 			EditDocument *		ed )
     {
-    AppDrawingData *		add= &(ed->edDrawingData);
+    DocumentWidget *		dw= &(ed->edDocumentWidget);
+    AppDrawingData *		add= &(dw->dwDrawingData);
 
     double			screenPixelsPerMM;
     double			verPixPerMM;
@@ -550,17 +554,23 @@ int appSetupDocument(	EditApplication *	ea,
 
     appGetFactors( ea, &screenPixelsPerMM, &verPixPerMM, &xfac, &yfac );
 
-    appSetDrawingEnvironment( add, ea->eaMagnification, xfac,
-			screenPixelsPerMM,
-			ea->eaAfmDirectory,
-			ea->eaGhostscriptFontmap,
-			ea->eaGhostscriptFontToXmapping,
-			ea->eaToplevel.atTopWidget );
+    appSetScreenDrawingEnvironment( add, &(dw->dwColors),
+					    ea->eaMagnification, xfac,
+					    screenPixelsPerMM,
+					    &(ea->eaPostScriptFontList) );
+
+#   ifdef USE_MOTIF
+    {
+    Display * display= XtDisplay( dw->dwWidget );
+
+    add->addDisplay= display;
+    add->addScreen= DefaultScreen( display );
+    }
+#   endif
 
     add->addPageGapPixels= (int)( ea->eaPageGapMM* verPixPerMM );
 
-    if  ( (*ea->eaLayoutDocument)( ed->edPrivateData, ed->edFormat,
-					&(ed->edDrawingData),
+    if  ( (*ea->eaLayoutDocument)( ed->edPrivateData, ed->edFormat, add,
 					&(ea->eaDefaultDocumentGeometry) ) )
 	{ SDEB(ed->edFilename); return -1; }
 
@@ -575,7 +585,7 @@ int appSetupDocument(	EditApplication *	ea,
 	{ high=( 4* ea->eaScreenPixelsHigh )/ 5; }
 
 #   ifdef USE_MOTIF
-    XtVaSetValues( ed->edDocumentWidget,
+    XtVaSetValues( dw->dwWidget,
 			    XmNborderWidth,		0,
 			    XmNshadowThickness,		0,
 			    XmNwidth,			wide,
@@ -585,26 +595,26 @@ int appSetupDocument(	EditApplication *	ea,
 
 #   ifdef USE_GTK
     gtk_drawing_area_size( GTK_DRAWING_AREA( ed->edTopRulerWidget ),
-					ed->edLeftRulerWidthPixels+
+					ed->edLeftRulerWidePixels+
 					wide+
-					ed->edRightRulerWidthPixels,
-					ed->edTopRulerHeightPixels );
+					ed->edRightRulerWidePixels,
+					ed->edTopRulerHighPixels );
     gtk_drawing_area_size( GTK_DRAWING_AREA( ed->edLeftRulerWidget ),
-					ed->edLeftRulerWidthPixels, high );
+					ed->edLeftRulerWidePixels, high );
 
 
-    gtk_drawing_area_size( GTK_DRAWING_AREA( ed->edDocumentWidget ),
+    gtk_drawing_area_size( GTK_DRAWING_AREA( dw->dwWidget ),
 								wide, high );
 
     gtk_drawing_area_size( GTK_DRAWING_AREA( ed->edRightRulerWidget ),
-					ed->edRightRulerWidthPixels, high );
+					ed->edRightRulerWidePixels, high );
     gtk_drawing_area_size( GTK_DRAWING_AREA( ed->edBottomRulerWidget ),
-					ed->edLeftRulerWidthPixels+
+					ed->edLeftRulerWidePixels+
 					wide+
-					ed->edRightRulerWidthPixels,
-					ed->edBottomRulerHeightPixels );
+					ed->edRightRulerWidePixels,
+					ed->edBottomRulerHighPixels );
 
-    gtk_object_set_user_data( GTK_OBJECT( ed->edDocumentWidget ), (void *)ed );
+    gtk_object_set_user_data( GTK_OBJECT( dw->dwWidget ), (void *)ed );
 #   endif
 
     ed->edVisibleRect.drX0= add->addPaperRect.drX0;
@@ -620,6 +630,14 @@ int appSetupDocument(	EditApplication *	ea,
 	  (*ea->eaSetLeftRuler)( ed )	)
 	{ SDEB(ed->edFilename);	}
 
+    if  ( ea->eaSetRightRuler		&&
+	  (*ea->eaSetRightRuler)( ed )	)
+	{ SDEB(ed->edFilename);	}
+
+    if  ( ea->eaSetBottomRuler		&&
+	  (*ea->eaSetBottomRuler)( ed )	)
+	{ SDEB(ed->edFilename);	}
+
     appDocSetScrollbarValues( ed );
 
 #   ifdef USE_MOTIF
@@ -630,10 +648,23 @@ int appSetupDocument(	EditApplication *	ea,
     gtk_widget_realize( ed->edToplevel.atTopWidget );
 #   endif
 
-    if  ( ea->eaFinishDocumentSetup && (*ea->eaFinishDocumentSetup)( ea, ed ) )
+    if  ( appAllocateColors( add, &(dw->dwColors) ) )
+	{ LDEB(1); return -1;	}
+
+    ed->edScreenWidget= guiScreenWidgetForNativeWidget(
+					    ed->edDocumentWidget.dwWidget,
+					    &(ed->edDocumentWidget.dwColors) );
+    if  ( ! ed->edScreenWidget )
+	{ PDEB(ed->edScreenWidget); appFreeDocument( ea, ed ); return -1; }
+
+    ed->edDrawingSurface= guiMakeDrawingSurfaceForWidget( ed->edScreenWidget );
+    if  ( ! ed->edDrawingSurface )
+	{ PDEB(ed->edDrawingSurface); appFreeDocument( ea, ed ); return -1; }
+
+    if  ( ea->eaFinishDocumentSetup && (*ea->eaFinishDocumentSetup)( ed ) )
 	{ SDEB(ed->edFilename); return -1; }
 
-    appShowShellWidget( ed->edToplevel.atTopWidget );
+    appShowShellWidget( ea, ed->edToplevel.atTopWidget );
 
     ed->edIsVisible= 1;
     ed->edHasBeenChanged= 0;
@@ -643,6 +674,59 @@ int appSetupDocument(	EditApplication *	ea,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Close a a document.							*/
+/*									*/
+/************************************************************************/
+
+void appCloseDocument(	EditApplication *	ea,
+			EditDocument *		ed )
+    {
+    appRemoveDocument( ea, ed );
+
+    /* No! is done by the widget destroy callback
+    if  ( ed->edPrivateData )
+	{
+	(*ea->eaFreeDocument)( ed->edPrivateData, ed->edFormat,
+						    &(ed->edDrawingData) );
+	ed->edPrivateData= (void *)0;
+	}
+    */
+
+    appCleanDocument( ea, ed );
+    appDestroyShellWidget( ed->edToplevel.atTopWidget );
+
+    /* No! is done by the widget destroy callback that is called 
+       asynchronuously
+    free( ed );
+    */
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Destroy callback on a document widget.				*/
+/*									*/
+/************************************************************************/
+
+APP_DESTROY_CALLBACK_H( appDestroyEditDocument, w, voided )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    EditApplication *		ea= ed->edApplication;
+
+    if  ( ed->edPrivateData )
+	{
+	(*ea->eaFreeDocument)( ed->edPrivateData, ed->edFormat );
+	ed->edPrivateData= (void *)0;
+	}
+
+    free( ed );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Fill the menu of a document window.					*/
@@ -714,8 +798,7 @@ EditDocument * appOpenDocument(	EditApplication *	ea,
 
 	    resp= appQuestionRunSubjectOkCancelDialog( ea,
 					relative, option,
-					filename, afmr->afmrFileReadOnly,
-					(char *)0, (char *)0 );
+					filename, afmr->afmrFileReadOnly );
 
 	    if  ( resp != AQDrespOK )
 		{ return (EditDocument *)0;	}
@@ -875,46 +958,20 @@ int appFileConvert(	EditApplication *	ea,
 					ea->eaToplevel.atTopWidget, toName );
 		}
 
-	    (*ea->eaFreeDocument)( privateData, fromFormat,
-						    (AppDrawingData *)0 );
+	    (*ea->eaFreeDocument)( privateData, fromFormat );
 	    return -1;
 	    }
 
-	(*ea->eaFreeDocument)( privateData, fromFormat,
-						    (AppDrawingData *)0 );
+	(*ea->eaFreeDocument)( privateData, fromFormat );
 	return 0;
 	}
 
     SSLDEB(fromName,toName,toFormat);
-    (*ea->eaFreeDocument)( privateData, fromFormat, (AppDrawingData *)0 );
+    (*ea->eaFreeDocument)( privateData, fromFormat );
     return -1;
     }
 
 
-/************************************************************************/
-/*									*/
-/*  Destroy callback on a document widget.				*/
-/*									*/
-/************************************************************************/
-
-void appDestroyEditDocument(	APP_WIDGET		w,
-				void *			voided,
-				void *			callData )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    EditApplication *		ea= ed->edApplication;
-
-    if  ( ed->edPrivateData )
-	{
-	(*ea->eaFreeDocument)( ed->edPrivateData, ed->edFormat,
-						    &(ed->edDrawingData) );
-	}
-
-    appFreeDocument( ea, ed );
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Redraw a rectangle in the document widget.				*/
@@ -924,7 +981,7 @@ void appDestroyEditDocument(	APP_WIDGET		w,
 APP_EVENT_HANDLER_H( appDocExposeHandler, w, voided, exposeEvent )
     {
     EditDocument *		ed= (EditDocument *)voided;
-    AppDrawingData *		add= &(ed->edDrawingData);
+    AppDrawingData *		add= &(ed->edDocumentWidget.dwDrawingData);
     EditApplication *		ea= ed->edApplication;
 
     int				ox= ed->edVisibleRect.drX0;
@@ -932,11 +989,17 @@ APP_EVENT_HANDLER_H( appDocExposeHandler, w, voided, exposeEvent )
 
     DocumentRectangle		drClip;
 
+    if  ( ed->edNotYetDrawn				&&
+	  ea->eaDocumentFirstVisible			)
+	{ (*ea->eaDocumentFirstVisible)( ed );	}
+
+    ed->edNotYetDrawn= 0;
+
     /*  1  */
     appCollectExposures( &drClip, add, ox, oy, exposeEvent );
 
     /*  2,3,4  */
-    (*ea->eaDrawRectangle)( w, ed, &drClip, ox, oy );
+    (*ea->eaDrawRectangle)( ed, &drClip, ox, oy );
 
     appDrawNoClipping( add );
 
@@ -960,11 +1023,11 @@ void appDocExposeRectangle(	const EditDocument *		ed,
     DocumentRectangle		drExpose;
     DocumentRectangle		drScrolled;
 
-    const AppDrawingData *	add= &(ed->edDrawingData);
+    const AppDrawingData *	add= &(ed->edDocumentWidget.dwDrawingData);
     const DocumentRectangle *	drVisible= &(ed->edVisibleRect);
 
     if  ( ! drChanged )
-	{ appExposeRectangle( add, 0, 0, 0, 0 ); return; }
+	{ appExposeDrawingData( add ); return;	}
 
     drScrolled= *drChanged;
     drScrolled.drX0 -= scrolledX;
@@ -984,27 +1047,43 @@ void appDocExposeRectangle(	const EditDocument *		ed,
 	drScrolled.drX1= drVisible->drX1;
 	}
 
-    docUnionRectangle( &drScrolled, &drScrolled, drChanged );
+    geoUnionRectangle( &drScrolled, &drScrolled, drChanged );
 
     drExpose.drX0= drVisible->drX0;
     drExpose.drY0= drVisible->drY0;
     drExpose.drX1= drVisible->drX1;
     drExpose.drY1= drVisible->drY1;
 
-    if  ( docIntersectRectangle( &drExpose, &drExpose, &drScrolled ) )
+    if  ( geoIntersectRectangle( &drExpose, &drExpose, &drScrolled ) )
 	{
 	int	ox= drVisible->drX0;
 	int	oy= drVisible->drY0;
 
+	drExpose.drX0 -= ox;
+	drExpose.drX1 -= ox;
+	drExpose.drY0 -= oy;
+	drExpose.drY1 -= oy;
+
+	if  ( drExpose.drX0 < 0 )
+	    { drExpose.drX0=  0;	}
+	if  ( drExpose.drY0 < 0 )
+	    { drExpose.drY0=  0;	}
+
 #	if LOG_REDRAWS
 	docLogRectangle( "CLEAR!", &drExpose );
 #	endif
 
-	appExposeRectangle( add,
-				    drExpose.drX0- ox,
-				    drExpose.drY0- oy,
-				    drExpose.drX1- drExpose.drX0+ 1,
-				    drExpose.drY1- drExpose.drY0+ 1 );
+	if  ( drExpose.drX1 >= drExpose.drX0 && drExpose.drY1 >= drExpose.drY0 )
+	    { appExposeRectangle( add, &drExpose );	}
 	}
     }
 
+void appInitSelectRectangle(	SelectRectangle *	sr )
+    {
+    sr->srDirection= DOCselNONE;
+
+    geoInitRectangle( &(sr->srSelected) );
+    geoInitRectangle( &(sr->srLTM) );
+    geoInitRectangle( &(sr->srRBM) );
+    }
+
