@@ -99,6 +99,9 @@ static int appDocMakeMainWindow(	EditDocument *		ed,
     appSetCloseCallback( ed->edToplevel.atTopWidget, ea,
 					appDocFileCloseCallback, (void *)ed );
 
+    appSetDestroyCallback( ed->edToplevel.atTopWidget,
+					appDestroyEditDocument, (void *)ed );
+
     ed->edMainWindow= gtk_vbox_new( FALSE, spacing );
     gtk_container_add( GTK_CONTAINER( ed->edToplevel.atTopWidget ),
 							ed->edMainWindow );
@@ -236,21 +239,21 @@ int appMakeDocumentWidget(	EditApplication *	ea,
 
     gtk_widget_show( ed->edRightRulerWidget );
 
-    ed->edDocumentWidget= gtk_drawing_area_new();
+    ed->edDocumentWidget.dwWidget= gtk_drawing_area_new();
 
     gtk_table_attach( GTK_TABLE( form ),
-			ed->edDocumentWidget,
+			ed->edDocumentWidget.dwWidget,
 			1, 2,
 			1, 2,
 			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			0, 0 );
 
-    gtk_widget_show( ed->edDocumentWidget );
+    gtk_widget_show( ed->edDocumentWidget.dwWidget );
 
-    GTK_WIDGET_SET_FLAGS( ed->edDocumentWidget, GTK_CAN_FOCUS );
+    GTK_WIDGET_SET_FLAGS( ed->edDocumentWidget.dwWidget, GTK_CAN_FOCUS );
     gtk_window_set_focus( GTK_WINDOW( ed->edToplevel.atTopWidget ),
-						    ed->edDocumentWidget );
+					    ed->edDocumentWidget.dwWidget );
 
     return 0;
     }
@@ -283,6 +286,7 @@ static int appDocMakeToolbar(	EditDocument *		ed,
 int appFinishDocumentWindow(	EditDocument *		ed )
     {
     EditApplication *	ea= ed->edApplication;
+    DocumentWidget *	dw= &(ed->edDocumentWidget);
 
     /*  3  */
     if  ( appDocMakeMainWindow( ed, ea )		||
@@ -291,7 +295,7 @@ int appFinishDocumentWindow(	EditDocument *		ed )
 	  (*ea->eaMakeDocumentWidget)( ea, ed )		)
 	{ LDEB(1); return -1; }
 
-    gtk_signal_connect( GTK_OBJECT( ed->edDocumentWidget ),
+    gtk_signal_connect( GTK_OBJECT( dw->dwWidget ),
 				    "configure_event",
 				    (GtkSignalFunc)appDocConfigure,
 				    (void *)ed );
@@ -301,12 +305,17 @@ int appFinishDocumentWindow(	EditDocument *		ed )
 				    (GtkSignalFunc)appDocTopConfigure,
 				    (void *)ed );
 
-    gtk_signal_connect( GTK_OBJECT( ed->edDocumentWidget ),
+    gtk_signal_connect( GTK_OBJECT( dw->dwWidget ),
 				    "selection_clear_event",
 				    (GtkSignalFunc)appDocCopyPasteHandler,
 				    (void *)ed );
 
-    gtk_signal_connect( GTK_OBJECT( ed->edDocumentWidget ),
+    gtk_signal_connect( GTK_OBJECT( dw->dwWidget ),
+				    "selection_received",
+				    (GtkSignalFunc)appDocGotPasteReplyGtk,
+				    (void *)ed );
+
+    gtk_signal_connect( GTK_OBJECT( dw->dwWidget ),
 				    "selection_get",
 				    (GtkSignalFunc)appDocReplyToCopyRequest,
 				    (void *)ed );
@@ -331,7 +340,7 @@ void appSetShellConstraints(	EditDocument *		ed )
     gint		maxWidth;
     gint		maxHeight;
 
-    AppDrawingData *	add= &(ed->edDrawingData);
+    AppDrawingData *	add= &(ed->edDocumentWidget.dwDrawingData);
 
     appDrawGetSizeOfWidget( &wide, &high, ed->edToplevel.atTopWidget );
 
@@ -403,17 +412,16 @@ void appSetShellConstraints(	EditDocument *		ed )
 /*									*/
 /************************************************************************/
 
-int appFinishDocumentSetup(	EditApplication *	ea,
-				EditDocument *		ed )
+int appFinishDocumentSetup(	EditDocument *		ed )
     {
-    AppDrawingData *	add= &(ed->edDrawingData);
+    EditApplication *	ea= ed->edApplication;
+    DocumentWidget *	dw= &(ed->edDocumentWidget);
+    AppDrawingData *	add= &(dw->dwDrawingData);
 
-    ed->edInputContext= 0;
-
-    appSetDrawingDataForWidget( ed->edDocumentWidget,
+    appSetDrawingDataForWidget( dw->dwWidget, &(dw->dwColors),
 					    ea->eaMagnification, add );
 
-    appDrawSetRedrawHandler( ed->edDocumentWidget,
+    appDrawSetRedrawHandler( dw->dwWidget,
 					    appDocExposeHandler, (void *)ed );
 
     if  ( ea->eaObserveFocus )
@@ -423,23 +431,29 @@ int appFinishDocumentSetup(	EditApplication *	ea,
 	appGuiSetFocusChangeHandler( ed->edToplevel.atTopWidget,
 					    ea->eaObserveFocus, (void *)ed );
 #	else
-	appGuiSetFocusChangeHandler( ed->edDocumentWidget,
+	appGuiSetFocusChangeHandler( dw->dwWidget,
 					    ea->eaObserveFocus, (void *)ed );
 #	endif
 	}
 
     if  ( ea->eaDocumentMouseHandler )
 	{
-	appDrawSetButtonPressHandler( ed->edDocumentWidget,
+	appDrawSetButtonPressHandler( dw->dwWidget,
 				    ea->eaDocumentMouseHandler, (void *)ed );
+
+	gtk_widget_add_events( dw->dwWidget, GDK_POINTER_MOTION_MASK );
 	}
 
-    if  ( ea->eaDocumentKeyboardHandler )
+    if  ( ea->eaDocumentScrollHandler )
 	{
-	appDrawSetKeyboardHandler( ed->edDocumentWidget,
-				ea->eaDocumentKeyboardHandler, (void *)ed );
+	appDrawSetScrollHandler( dw->dwWidget,
+				    ea->eaDocumentScrollHandler, (void *)ed );
 	}
 
+    dw->dwOwner= (void *)ed;
+    dw->dwGotString= ea->eaDocGotString;
+    dw->dwGotKey= ea->eaDocGotKey;
+
     gtk_signal_connect( GTK_OBJECT( ed->edVerticalAdjustment ),
 			    "value_changed", 
 			    (GtkSignalFunc)appDocVerticalScrollbarCallback,
@@ -450,6 +464,8 @@ int appFinishDocumentSetup(	EditApplication *	ea,
 			    (GtkSignalFunc)appDocHorizontalScrollbarCallback,
 			    (void *)ed );
 
+    appDocumentSetInputContext( ea->eaInputMethod, dw );
+
     return 0;
     }
 
