@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdio.h>
@@ -14,6 +14,8 @@
 
 #   include	"appFrame.h"
 
+#   include	<appDebugon.h>
+
 #   ifdef USE_MOTIF
 
 #   include	<Xm/ToggleB.h>
@@ -25,8 +27,6 @@
 #   include	<Xm/PanedW.h>
 #   include	<Xm/ScrollBar.h>
 
-#   include	<appDebugon.h>
-
 /************************************************************************/
 /*									*/
 /*  Event handler to keep track of resize events for the document.	*/
@@ -89,8 +89,8 @@ static int appDocMakeMainWindow(	EditDocument *		ed,
     const char *	fullTitle;
     const char *	fullIconName;
 
-    int			maxWidth= ( 5* ea->eaScreenPixelsWide )/ 6;
-    int			maxHeight= ( 5* ea->eaScreenPixelsHigh )/ 6;
+    int			maxWidth= ( 4* ea->eaScreenPixelsWide )/ 5;
+    int			maxHeight= ( 4* ea->eaScreenPixelsHigh )/ 5;
 
     if  ( ea->eaDocumentIcon						&&
 	  appGetImagePixmap( ea, ea->eaDocumentIcon, &pixmap, &mask )	)
@@ -105,6 +105,8 @@ static int appDocMakeMainWindow(	EditDocument *		ed,
     XtSetArg( al[ac], XmNdeleteResponse,	XmDO_NOTHING ); ac++;
     XtSetArg( al[ac], XmNallowShellResize,	True ); ac++;
     XtSetArg( al[ac], XmNuseAsyncGeometry,	True ); ac++;
+    XtSetArg( al[ac], XmNwaitForWm,		False ); ac++; /* LessTif BUG */
+    XtSetArg( al[ac], XmNwmTimeout,		0 ); ac++; /* LessTif BUG */
     XtSetArg( al[ac], XmNmappedWhenManaged,	False ); ac++;
     XtSetArg( al[ac], XmNinput,			True ); ac++;
     XtSetArg( al[ac], XmNmaxWidth,		maxWidth ); ac++;
@@ -220,7 +222,7 @@ int appMakeDocumentWidget(	EditApplication *	ea,
     XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_FORM ); ac++;
     XtSetArg( al[ac],	XmNrightOffset,		0 ); ac++;
     XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_OPPOSITE_FORM ); ac++;
-    XtSetArg( al[ac],	XmNbottomOffset,	-ed->edTopRulerHeightPixels ); ac++;
+    XtSetArg( al[ac],	XmNbottomOffset,	-ed->edTopRulerHighPixels ); ac++;
 
     ed->edTopRulerWidget= XmCreateDrawingArea( form, WIDGET_NAME, al, ac );
 
@@ -231,7 +233,7 @@ int appMakeDocumentWidget(	EditApplication *	ea,
     XtSetArg( al[ac],	XmNleftAttachment,	XmATTACH_FORM ); ac++;
     XtSetArg( al[ac],	XmNleftOffset,		0 ); ac++;
     XtSetArg( al[ac],	XmNtopAttachment,	XmATTACH_OPPOSITE_FORM ); ac++;
-    XtSetArg( al[ac],	XmNtopOffset,		-ed->edBottomRulerHeightPixels ); ac++;
+    XtSetArg( al[ac],	XmNtopOffset,		-ed->edBottomRulerHighPixels ); ac++;
     XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_FORM ); ac++;
     XtSetArg( al[ac],	XmNrightOffset,		0 ); ac++;
     XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_FORM ); ac++;
@@ -252,7 +254,7 @@ int appMakeDocumentWidget(	EditApplication *	ea,
     XtSetArg( al[ac],	XmNtopOffset,		0 ); ac++;
 
     XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_OPPOSITE_FORM ); ac++;
-    XtSetArg( al[ac],	XmNrightOffset,		-ed->edLeftRulerWidthPixels ); ac++;
+    XtSetArg( al[ac],	XmNrightOffset,		-ed->edLeftRulerWidePixels ); ac++;
 
     XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_WIDGET ); ac++;
     XtSetArg( al[ac],	XmNbottomWidget,	ed->edBottomRulerWidget ); ac++;
@@ -265,7 +267,7 @@ int appMakeDocumentWidget(	EditApplication *	ea,
     XtSetArg( al[ac],	XmNforeground,		blackPixel ); ac++;
     XtSetArg( al[ac],	XmNtraversalOn,		False ); ac++;
     XtSetArg( al[ac],	XmNleftAttachment,	XmATTACH_OPPOSITE_FORM ); ac++;
-    XtSetArg( al[ac],	XmNleftOffset,		-ed->edRightRulerWidthPixels ); ac++;
+    XtSetArg( al[ac],	XmNleftOffset,		-ed->edRightRulerWidePixels ); ac++;
     XtSetArg( al[ac],	XmNtopAttachment,	XmATTACH_WIDGET ); ac++;
     XtSetArg( al[ac],	XmNtopWidget,		ed->edTopRulerWidget ); ac++;
     XtSetArg( al[ac],	XmNtopOffset,		0 ); ac++;
@@ -297,16 +299,17 @@ int appMakeDocumentWidget(	EditApplication *	ea,
     XtSetArg( al[ac],	XmNbottomWidget,	ed->edBottomRulerWidget ); ac++;
     XtSetArg( al[ac],	XmNbottomOffset,	0 ); ac++;
 
-    ed->edDocumentWidget= XmCreateDrawingArea( form, WIDGET_NAME, al, ac );
+    ed->edDocumentWidget.dwWidget=
+			    XmCreateDrawingArea( form, WIDGET_NAME, al, ac );
 
-    XtAddCallback( ed->edDocumentWidget,
+    XtAddCallback( ed->edDocumentWidget.dwWidget,
 		XmNdestroyCallback, appDestroyEditDocument, (void *)ed );
 
     XtManageChild( ed->edTopRulerWidget );
     XtManageChild( ed->edLeftRulerWidget );
     XtManageChild( ed->edRightRulerWidget );
     XtManageChild( ed->edBottomRulerWidget );
-    XtManageChild( ed->edDocumentWidget );
+    XtManageChild( ed->edDocumentWidget.dwWidget );
     XtManageChild( form );
 
     return 0;
@@ -348,19 +351,20 @@ int appFinishDocumentWindow(	EditDocument *		ed )
 	  (*ea->eaMakeDocumentWidget)( ea, ed )		)
 	{ LDEB(1); return -1; }
 
-    XtAddEventHandler( ed->edDocumentWidget, StructureNotifyMask, False,
+    XtAddEventHandler( ed->edDocumentWidget.dwWidget,
+					StructureNotifyMask, False,
 					appDocConfigure, (void *)ed );
     XtAddEventHandler( ed->edToplevel.atTopWidget, StructureNotifyMask, False,
 					appDocTopConfigure, (void *)ed );
 
     /*  4  */
-    XtAddEventHandler( ed->edDocumentWidget,
+    XtAddEventHandler( ed->edDocumentWidget.dwWidget,
 		    PropertyChangeMask, True, appDocCopyPasteHandler, ed );
 
     XtManageChild( ed->edVerticalScrollbar );
     XtManageChild( ed->edHorizontalScrollbar );
 
-    XtManageChild( ed->edDocumentWidget );
+    XtManageChild( ed->edDocumentWidget.dwWidget );
     XtManageChild( ed->edScrolledWindow );
 
     if  ( ed->edToolbar )
@@ -386,7 +390,7 @@ void appSetShellConstraints(	EditDocument *		ed )
     Arg			al[20];
     int			ac= 0;
 
-    AppDrawingData *	add= &(ed->edDrawingData);
+    AppDrawingData *	add= &(ed->edDocumentWidget.dwDrawingData);
 
     appDrawGetSizeOfWidget( &wide, &high, ed->edToplevel.atTopWidget );
 
@@ -395,15 +399,22 @@ void appSetShellConstraints(	EditDocument *		ed )
 	XtWidgetGeometry	preferred;
 
 	int			maxWidth;
+	int			minWidth;
 
 	maxWidth= add->addBackRect.drX1- add->addBackRect.drX0;
 	maxWidth += ed->edShellExtraWidth;
 
+	minWidth= 30;
+	minWidth += ed->edShellExtraWidth;
+	minWidth += ed->edLeftRulerWidePixels;
+	minWidth += ed->edRightRulerWidePixels;
+
 	XtQueryGeometry( ed->edMenuBar, (XtWidgetGeometry *)0, &preferred );
 	if  ( maxWidth < preferred.width+ preferred.border_width )
 	    { maxWidth=  preferred.width+ preferred.border_width;	}
 
 	XtSetArg( al[ac],	XmNmaxWidth,	maxWidth ); ac++;
+	XtSetArg( al[ac],	XmNminWidth,	minWidth ); ac++;
 
 	if  ( wide > maxWidth )
 	    {
@@ -415,7 +426,8 @@ void appSetShellConstraints(	EditDocument *		ed )
 
     if  ( ed->edShellExtraHeight >= 0 )
 	{
-	int	maxHeight;
+	int			maxHeight;
+	int			minHeight;
 
 	maxHeight= add->addBackRect.drY1- add->addBackRect.drY0;
 
@@ -424,7 +436,13 @@ void appSetShellConstraints(	EditDocument *		ed )
 
 	maxHeight += ed->edShellExtraHeight;
 
+	minHeight= 30;
+	minHeight += ed->edShellExtraHeight;
+	minHeight += ed->edTopRulerHighPixels;
+	minHeight += ed->edBottomRulerHighPixels;
+
 	XtSetArg( al[ac],	XmNmaxHeight,	maxHeight ); ac++;
+	XtSetArg( al[ac],	XmNminHeight,	minHeight ); ac++;
 
 	if  ( high > maxHeight )
 	    {
@@ -440,110 +458,48 @@ void appSetShellConstraints(	EditDocument *		ed )
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Finish the setup of the document after it is realised.		*/
-/*									*/
-/************************************************************************/
-
-static XIMStyle	appXimPreferences[]=
+int appFinishDocumentSetup(	EditDocument *		ed )
     {
-    XIMPreeditCallbacks,
-    XIMPreeditPosition,
-    XIMPreeditArea,
-    XIMPreeditNothing,
-    XIMStatusCallbacks,
-    XIMStatusArea,
-    XIMStatusNothing,
-    };
+    EditApplication *	ea= ed->edApplication;
+    AppDrawingData *	add= &(ed->edDocumentWidget.dwDrawingData);
+    DocumentWidget *	dw= &(ed->edDocumentWidget);
 
-int appFinishDocumentSetup(	EditApplication *	ea,
-				EditDocument *		ed )
-    {
-    AppDrawingData *	add= &(ed->edDrawingData);
-
-    appSetDrawingDataForWidget( ed->edDocumentWidget,
+    appSetDrawingDataForWidget( dw->dwWidget, &(dw->dwColors),
 					    ea->eaMagnification, add );
 
-    appDrawSetRedrawHandler( ed->edDocumentWidget,
+    appDrawSetRedrawHandler( dw->dwWidget,
 					appDocExposeHandler, (void *)ed );
 
     if  ( ea->eaObserveFocus )
 	{
-	appGuiSetFocusChangeHandler( ed->edDocumentWidget,
+	appGuiSetFocusChangeHandler( dw->dwWidget,
 					    ea->eaObserveFocus, (void *)ed );
 	}
 
     if  ( ea->eaDocumentMouseHandler )
 	{
-	appDrawSetButtonPressHandler( ed->edDocumentWidget,
+	appDrawSetButtonPressHandler( dw->dwWidget,
 				ea->eaDocumentMouseHandler, (void *)ed );
 	}
 
-    if  ( ea->eaDocumentKeyboardHandler )
+    if  ( ea->eaDocumentScrollHandler )
 	{
-	appDrawSetKeyboardHandler( ed->edDocumentWidget,
-			    ea->eaDocumentKeyboardHandler, (void *)ed );
+	appDrawSetScrollHandler( dw->dwWidget,
+				    ea->eaDocumentScrollHandler, (void *)ed );
 	}
 
+    dw->dwOwner= (void *)ed;
+    dw->dwGotString= ea->eaDocGotString;
+    dw->dwGotKey= ea->eaDocGotKey;
+
     if  ( ea->eaDocumentCursor )
 	{
-	XDefineCursor( XtDisplay( ed->edDocumentWidget ),
-		    XtWindow( ed->edDocumentWidget ), ea->eaDocumentCursor );
+	XDefineCursor( XtDisplay( dw->dwWidget ),
+				XtWindow( ed->edDocumentWidget.dwWidget ),
+				ea->eaDocumentCursor );
 	}
 
-    if  ( ea->eaInputMethod )
-	{
-	XIMStyles *	availableStyles;
-	XIMStyle	supportedStyles;
-	XIMStyle	likedStyle;
-	int		style;
-
-	XGetIMValues( ea->eaInputMethod,
-		XNQueryInputStyle, &availableStyles,
-		NULL );
-
-	supportedStyles=	XIMPreeditArea		|
-				XIMPreeditNothing	|
-				XIMPreeditNone		|
-				XIMStatusArea		|
-				XIMStatusNothing	|
-				XIMStatusNone		;
-
-	likedStyle= 0;
-	for ( style= 0; style < availableStyles->count_styles; style++ )
-	    {
-	    XIMStyle	foundStyle= availableStyles->supported_styles[style];
-	    unsigned	j;
-
-	    if  ( ( supportedStyles & foundStyle ) != foundStyle )
-		{ continue;	}
-
-	    if  ( ! likedStyle )
-		{ likedStyle= foundStyle; continue;	}
-
-	    for ( j= 0; j < sizeof(appXimPreferences)/sizeof(XIMStyle); j++ )
-		{
-		if  (   ( foundStyle & appXimPreferences[j] )	&&
-		      ! ( likedStyle & appXimPreferences[j] )	)
-		    { likedStyle= foundStyle; break;	}
-		}
-	    }
-
-	if  ( likedStyle )
-	    {
-	    ed->edInputContext= XCreateIC( ea->eaInputMethod,
-		XNClientWindow,		XtWindow( ed->edDocumentWidget ),
-		XNInputStyle,		likedStyle,
-		/*
-		XNPeeditAttributes,	attributeList,
-		XNStatusAttributes,	attributeList,
-		*/
-		NULL );
-	    }
-	else{ ed->edInputContext= (XIC)0;	}
-	}
-    else{ ed->edInputContext= (XIC)0;	}
+    appDocumentSetInputContext( ea->eaInputMethod, dw );
 
     return 0;
     }
