@@ -1,6 +1,6 @@
 /************************************************************************/
 /*									*/
-/*  Drawing primitives for X11/Motif.					*/
+/*  Drawing primitives for GTK.						*/
 /*									*/
 /************************************************************************/
 
@@ -8,15 +8,13 @@
 
 #   include	<stddef.h>
 #   include	<stdio.h>
-
-#   define	y0	math_y0
-#   define	y1	math_y1
+#   include	<limits.h>
 #   include	<math.h>
-#   undef	y0
-#   undef	y1
 
 #   include	<geo2DInteger.h>
 #   include	"appDraw.h"
+#   include	"appScreenFont.h"
+#   include	"guiPixmapImpl.h"
 
 #   include	<appDebugon.h>
 
@@ -87,7 +85,7 @@ void appCollectExposures(	DocumentRectangle *	drClip,
 			drMore.drY1- drMore.drY0+ 1 );
 #	endif
 
-	docUnionRectangle( drClip, drClip, &drMore );
+	geoUnionRectangle( drClip, drClip, &drMore );
 
 #	if LOG_EXPOSES
 	appDebug( "......: [%4d+%4d]x[%4d+%4d]\n",
@@ -108,100 +106,73 @@ void appCollectExposures(	DocumentRectangle *	drClip,
     return;
     }
 
-void appExposeRectangle(	const AppDrawingData *	add,
-				int			x,
-				int			y,
-				int			wide,
-				int			high )
+void appExposeDrawingData(	const AppDrawingData *	add )
     {
-    if  ( wide == 0 || high == 0 )
-	{
-	gdk_window_clear_area_e( add->addDrawable, x, y, wide, high );
-	}
-    else{
-#	if GTK_MAJOR_VERSION < 2
-	gdk_window_clear_area_e( add->addDrawable, x, y, wide, high );
-#	else
+    gdk_window_clear_area_e( add->addDrawable, 0, 0, 0, 0 );
+    return;
+    }
 
-	GdkRectangle	rect;
+void appExposeRectangle(	const AppDrawingData *		add,
+				const DocumentRectangle *	drExpose )
+    {
+    int		x= drExpose->drX0;
+    int		y= drExpose->drY0;
+    int		wide= drExpose->drX1- drExpose->drX0+ 1;
+    int		high= drExpose->drY1- drExpose->drY0+ 1;
 
-	rect.x= x;
-	rect.y= y;
-	rect.width= wide;
-	rect.height= high;
+#   if GTK_MAJOR_VERSION < 2
+    gdk_window_clear_area_e( add->addDrawable, x, y, wide, high );
+#   else
 
-	gdk_window_invalidate_rect( add->addDrawable, &rect, TRUE );
+    {
+    GdkRectangle	rect;
 
-	/*
-	gtk_widget_queue_draw_area ( add->addDrawable, x, y, wide, high ) ?
-	*/
+    rect.x= x;
+    rect.y= y;
+    rect.width= wide;
+    rect.height= high;
 
-#	endif
-	}
+    gdk_window_invalidate_rect( add->addDrawable, &rect, TRUE );
+    }
+
+    /*
+    gtk_widget_queue_draw_area ( add->addDrawable, x, y, wide, high ) ?
+    */
+
+#   endif
 
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Initialise drawing data.						*/
-/*									*/
-/************************************************************************/
-
-void appInitDrawingData(	AppDrawingData *	add )
-    {
-    add->addForScreenDrawing= 0;
-
-    docInitRectangle( &(add->addBackRect) );
-    docInitRectangle( &(add->addPaperRect) );
-    docInitRectangle( &(add->addDocRect) );
-
-    add->addBottomMarginPixels= 0;
-    add->addPageGapPixels= 0;
-    add->addPageStepPixels= 0;
-
-    add->addMagnification= 1.0;
-    add->addMagnifiedPixelsPerTwip= 0.0;
-    add->addScreenPixelsPerMM= 0.0;
-    add->addDrawable= (GdkWindow *)0;
-    add->addGc= (GdkGC *)0;
-    add->addCurrentFont= (GdkFont *)0;
-
-    appInitFontList( &(add->addPhysicalFontList) );
-    }
-
 void appCleanDrawingData(	AppDrawingData *	add )
     {
+    appDrawCleanScreenFontList( add );
+
+#   ifdef USE_XFT
+    appCleanDrawingDataXft( add );
+#   endif
+
     if  ( add->addGc )
 	{ gdk_gc_unref( add->addGc );	}
 
-    appCleanFontList( add, &(add->addPhysicalFontList) );
-
     return ;
     }
 
-void appSetDrawingEnvironment(
-			AppDrawingData *	add,
-			double			magnification,
-			double			xfac,
-			double			screenPixelsPerMM,
-			const char *		afmDirectory,
-			const char *		ghostscriptFontmap,
-			const char *		ghostscriptFontToXmapping,
-			APP_WIDGET		w )
+void appSetScreenDrawingEnvironment(
+			AppDrawingData *		add,
+			AppColors *			colors,
+			double				magnification,
+			double				xfac,
+			double				screenPixelsPerMM,
+			const PostScriptFontList *	psfl )
     {
-    if  ( w )
-	{ add->addForScreenDrawing= 1; }
-
+    add->addColors= colors;
     add->addMagnification= magnification;
     add->addMagnifiedPixelsPerTwip= xfac;
     add->addScreenPixelsPerMM= screenPixelsPerMM;
 
-    add->addPhysicalFontList.apflAfmDirectory= afmDirectory;
-    add->addPhysicalFontList.apflGhostscriptFontmap=
-					    ghostscriptFontmap;
-    add->addPhysicalFontList.apflGhostscriptFontToXmapping=
-					    ghostscriptFontToXmapping;
+    appDrawSetPostScriptFontList( add, psfl );
+    add->addForScreenDrawing= 1;
 
     return;
     }
@@ -210,20 +181,25 @@ void appCloneDrawingEnvironment(	AppDrawingData *	add,
 					const AppDrawingData *	parent_add,
 					double			magnification,
 					double			xfac,
-					APP_WINDOW		drawable )
+					APP_WINDOW		w )
     {
+    if  ( w )
+	{ add->addForScreenDrawing= 1;	}
+
     add->addMagnification= magnification;
     add->addMagnifiedPixelsPerTwip= xfac;
     add->addScreenPixelsPerMM= parent_add->addScreenPixelsPerMM;
+    add->addColors= parent_add->addColors;
 
-    add->addPhysicalFontList.apflAfmDirectory=
-		parent_add->addPhysicalFontList.apflAfmDirectory;
-    add->addPhysicalFontList.apflGhostscriptFontmap=
-		parent_add->addPhysicalFontList.apflGhostscriptFontmap;
-    add->addPhysicalFontList.apflGhostscriptFontToXmapping=
-		parent_add->addPhysicalFontList.apflGhostscriptFontToXmapping;
+    appDrawSetPostScriptFontList( add, parent_add->addPostScriptFontList );
 
-    add->addDrawable= drawable;
+    add->addDrawable= w;
+
+    add->addAvoidFontconfig= parent_add->addAvoidFontconfig;
+#   ifdef USE_XFT
+    if  ( ! add->addAvoidFontconfig )
+	{ appGtkXftDrawCreate( add );	}
+#   endif
 
     add->addColorMap= parent_add->addColorMap;
     add->addGc= gdk_gc_new( add->addDrawable );
@@ -237,6 +213,7 @@ void appCloneDrawingEnvironment(	AppDrawingData *	add,
     }
 
 int appSetDrawingDataForWidget(	APP_WIDGET		w,
+				AppColors *		colors,
 				double			magnification,
 				AppDrawingData *	add )
     {
@@ -253,10 +230,24 @@ int appSetDrawingDataForWidget(	APP_WIDGET		w,
     if  ( ! GTK_WIDGET_REALIZED( w ) )
 	{ gtk_widget_realize( w );	}
 
+    add->addForScreenDrawing= 1;
+    add->addColors= colors;
+
     add->addDrawable= w->window;
     add->addColorMap= gtk_widget_get_colormap( w );
     add->addGc= gdk_gc_new( add->addDrawable );
 
+#   ifdef USE_XFT
+    if  ( ! add->addAvoidFontconfig )
+	{
+	appGtkXftDrawCreate( add );
+#	if GTK_MAJOR_VERSION >= 2
+	if  ( add->addXftDrawable )
+	    { gtk_widget_set_double_buffered( w, FALSE ); }
+#	endif
+	}
+#   endif
+
     horPixPerMM= (double)gdk_screen_width()/ (double)gdk_screen_width_mm();
     verPixPerMM= (double)gdk_screen_height()/ (double)gdk_screen_height_mm();
 
@@ -278,7 +269,7 @@ int appSetDrawingDataForWidget(	APP_WIDGET		w,
     add->addTopColor= gs->bg[GTK_STATE_NORMAL];
     add->addBottomColor= gs->bg[GTK_STATE_NORMAL];
 
-# define sq(x) ((1.0*x)*(1.0*x))
+# define sq(x) ((1.0*(x))*(1.0*(x)))
 
     f1= 0;
     f1 += sq( add->addTopColor.red );
@@ -357,11 +348,18 @@ void appDrawSetBackgroundWhite(	AppDrawingData *	add )
 
 void appDrawSetForegroundColor(	AppDrawingData *	add,
 				APP_COLOR_RGB *		xc )
-    { gdk_gc_set_foreground( add->addGc, xc ); }
+    {
+    gdk_gc_set_foreground( add->addGc, xc );
+#   ifdef USE_XFT
+    appSolidXftColor( &(add->addXftCurrentColor), xc );
+#   endif
+    }
 
 void appDrawSetBackgroundColor(	AppDrawingData *	add,
 				APP_COLOR_RGB *		xc )
-    { gdk_gc_set_background( add->addGc, xc ); }
+    {
+    gdk_gc_set_background( add->addGc, xc );
+    }
 
 void appDrawFillRectangle(	AppDrawingData *	add,
 				int			x,
@@ -369,6 +367,15 @@ void appDrawFillRectangle(	AppDrawingData *	add,
 				int			wide,
 				int			high )
     {
+#   ifdef USE_XFT
+
+    if  ( ! appDrawFillRectangleXft( add->addXftDrawable,
+			&(add->addXftCurrentColor), &(add->addXftColorList),
+			x, y, wide, high ) )
+	{ return;	}
+
+#   endif
+
     gdk_draw_rectangle( add->addDrawable, add->addGc, TRUE, x, y, wide, high );
     }
 
@@ -382,33 +389,6 @@ void appDrawDrawRectangle(	AppDrawingData *	add,
 							    x, y, wide, high );
     }
 
-APP_FONT * appDrawOpenFont(	AppDrawingData *	add,
-				const char *		name )
-    {
-    return gdk_font_load( name );
-    }
-
-void appDrawSetFont(		AppDrawingData *	add,
-				APP_FONT *		xfs )
-    {
-    add->addCurrentFont= xfs;
-
-    gdk_gc_set_font( add->addGc, xfs );
-    }
-
-void appDrawDrawString(		AppDrawingData *	add,
-				int			x,
-				int			y,
-				const char *		s,
-				int			len )
-    {
-    if  ( ! add->addCurrentFont )
-	{ XDEB(add->addCurrentFont); return;	}
-
-    gdk_draw_text( add->addDrawable, add->addCurrentFont, add->addGc,
-							    x, y, s, len );
-    }
-
 void appDrawDrawLine(		AppDrawingData *	add,
 				int			x0,
 				int			y0,
@@ -431,43 +411,21 @@ void appDrawMoveArea(		AppDrawingData *	add,
 					    wide, high );
     }
 
-void appDrawFreeFont(		AppDrawingData *	add,
-				APP_FONT *		font )
-    {
-    if  ( add->addCurrentFont == font )
-	{ add->addCurrentFont= (APP_FONT *)0;	}
-
-    gdk_font_unref( font );
-
-    return;
-    }
-
-void appDrawTextExtents(	int *			pWidth,
-				int *			pFontAscent,
-				int *			pFontDescent,
-				const AppDrawingData *	add,
-				APP_FONT *		xfs,
-				const char *		s,
-				int			l )
-    {
-    if  ( l > 0 )
-	{ *pWidth= gdk_text_width( xfs, s, l );	}
-    else{ *pWidth= 0;				}
-
-    *pFontAscent= xfs->ascent;
-    *pFontDescent= xfs->descent;
-
-    return;
-    }
-
 void appDrawNoClipping(		AppDrawingData *	add )
     {
+#   ifdef USE_XFT
+    if  ( add->addXftDrawable )
+	{
+	XftDrawSetClip( add->addXftDrawable, 0 );
+	}
+#   endif
+
     gdk_gc_set_clip_rectangle( add->addGc, (GdkRectangle *)0 );
 
     return;
     }
 
-extern void appDrawSetClipRect(	AppDrawingData *		add,
+void appDrawSetClipRect(	AppDrawingData *		add,
 				const DocumentRectangle *	drClip )
     {
     GdkRectangle		gRect;
@@ -477,43 +435,61 @@ extern void appDrawSetClipRect(	AppDrawingData *		add,
     gRect.width= drClip->drX1- drClip->drX0+ 1;
     gRect.height= drClip->drY1- drClip->drY0+ 1;
 
+#   ifdef USE_XFT
+    if  ( add->addXftDrawable )
+	{
+	XRectangle		xRect;
+
+	xRect.x= drClip->drX0;
+	xRect.y= drClip->drY0;
+	xRect.width= drClip->drX1- drClip->drX0+ 1;
+	xRect.height= drClip->drY1- drClip->drY0+ 1;
+
+	XftDrawSetClipRectangles( add->addXftDrawable, 0, 0, &xRect, 1 );
+	}
+#   endif
+
     gdk_gc_set_clip_rectangle( add->addGc, &gRect );
 
     return;
     }
 
 void appDrawFillPolygon(		AppDrawingData *	add,
-					APP_POINT *		points,
+					const APP_POINT *	points,
 					int			count )
     {
-    gdk_draw_polygon( add->addDrawable, add->addGc, TRUE, points, count );
+    gdk_draw_polygon( add->addDrawable, add->addGc, TRUE,
+						(APP_POINT *)points, count );
     }
 
 void appDrawDrawLines(			AppDrawingData *	add,
-					APP_POINT *		points,
-					int			count )
+					const APP_POINT *	points,
+					int			count,
+					int			close )
     {
     int		i;
 
-    /*
-    XDrawLines( add->addDisplay, add->addDrawable, add->addGc,
-					points, count, CoordModeOrigin );
-    */
-
     count--;
 
-    for ( i= 0; i < count; points++, i++ )
+    for ( i= 0; i < count; i++ )
 	{
 	gdk_draw_line( add->addDrawable, add->addGc,
-						points[0].x, points[0].y,
-						points[1].x, points[1].y );
+					    points[i+0].x, points[i+0].y,
+					    points[i+1].x, points[i+1].y );
+	}
+
+    if  ( close )
+	{
+	gdk_draw_line( add->addDrawable, add->addGc,
+					    points[i].x, points[i].y,
+					    points[0].x, points[0].y );
 	}
 
     return;
     }
 
 void appDrawDrawPixmap(			AppDrawingData *	add,
-					APP_BITMAP_IMAGE	pixmap,
+					const ScreenPixmap	sp,
 					int			xSrc,
 					int			ySrc,
 					int			wide,
@@ -521,7 +497,7 @@ void appDrawDrawPixmap(			AppDrawingData *	add,
 					int			xDest,
 					int			yDest )
     {
-    gdk_draw_pixmap( add->addDrawable, add->addGc, pixmap,
+    gdk_draw_pixmap( add->addDrawable, add->addGc, sp->spBitmapImage,
 				    xSrc, ySrc, xDest, yDest, wide, high );
 
     return;
@@ -558,9 +534,20 @@ APP_BITMAP_IMAGE appMakePixmap(	AppDrawingData *	add,
 				int			wide,
 				int			high )
     {
-    int		depth= gdk_visual_get_system()->depth;
+    int			depth= gdk_visual_get_system()->depth;
+    APP_BITMAP_IMAGE	rval;
+    APP_COLOR_RGB	xc;
 
-    return gdk_pixmap_new( add->addDrawable, wide, high, depth );
+    rval= gdk_pixmap_new( add->addDrawable, wide, high, depth );
+
+    if  ( ! rval || ! gdk_color_white( add->addColorMap, &xc ) )
+	{ XDEB(rval);	}
+    else{
+	gdk_gc_set_foreground( add->addGc, &xc );
+	gdk_draw_rectangle( rval, add->addGc, TRUE, 0, 0, wide, high );
+	}
+
+    return rval;
     }
 
 void appDrawDrawSegments(	AppDrawingData *	add,
@@ -570,16 +557,26 @@ void appDrawDrawSegments(	AppDrawingData *	add,
     gdk_draw_segments( add->addDrawable, add->addGc, segments, count );
     }
 
+/************************************************************************/
+/*									*/
+/*  Draw an arc. Angles are in X11 style:				*/
+/*									*/
+/*  Unit:		1/64 degree.					*/
+/*  Origin:		At the right of the circle.			*/
+/*  Orientation:	Counterclockwise.				*/
+/*									*/
+/************************************************************************/
+
 void appDrawDrawArc(		AppDrawingData *	add,
 				int			x,
 				int			y,
 				int			wide,
 				int			high,
-				int			angle1,
-				int			angle2 )
+				int			alpha0,
+				int			alpha_step )
     {
     gdk_draw_arc( add->addDrawable, add->addGc, FALSE,
-					    x, y, wide, high, angle1, angle2 );
+					x, y, wide, high, alpha0, alpha_step );
     }
 
 void appDrawFillArc(		AppDrawingData *	add,
@@ -587,11 +584,69 @@ void appDrawFillArc(		AppDrawingData *	add,
 				int			y,
 				int			wide,
 				int			high,
-				int			angle1,
-				int			angle2 )
+				int			alpha0,
+				int			alpha_step )
     {
     gdk_draw_arc( add->addDrawable, add->addGc, TRUE,
-					    x, y, wide, high, angle1, angle2 );
+					x, y, wide, high, alpha0, alpha_step );
+    }
+
+void appDrawDrawArcs(		AppDrawingData *	add,
+				const APP_ARC *		arcs,
+				int			count )
+    {
+    int		i;
+
+    for ( i= 0; i < count; arcs++, i++ )
+	{
+	gdk_draw_arc( add->addDrawable, add->addGc, FALSE,
+					    arcs->x, arcs->y,
+					    arcs->width, arcs->height,
+					    arcs->angle1, arcs->angle2 );
+	}
+
+    return;
+    }
+
+void appDrawFillArcs(		AppDrawingData *	add,
+				const APP_ARC *		arcs,
+				int			count )
+    {
+    int		i;
+
+    for ( i= 0; i < count; arcs++, i++ )
+	{
+	gdk_draw_arc( add->addDrawable, add->addGc, TRUE,
+					    arcs->x, arcs->y,
+					    arcs->width, arcs->height,
+					    arcs->angle1, arcs->angle2 );
+	}
+
+    return;
+    }
+
+int appDrawSetLineAttributes(	AppDrawingData *	add,
+				int			lineWidth,
+				int			lineStyle,
+				int			capStyle,
+				int			joinStyle,
+				const unsigned char *	dashList,
+				int			dashCount )
+    {
+
+    gdk_gc_set_line_attributes( add->addGc,
+				lineWidth, lineStyle, capStyle, joinStyle );
+
+    if  ( lineStyle != LINEstyleSOLID )
+	{
+	if  ( ! dashList || dashCount == 0 )
+	    { XLDEB(dashList,dashCount);	}
+	else{
+	    gdk_gc_set_dashes( add->addGc, 0, (gint8 *)dashList, dashCount );
+	    }
+	}
+
+    return 0;
     }
 
 #   endif
