@@ -8,15 +8,12 @@
 
 #   include	<stddef.h>
 #   include	<stdio.h>
-
-#   define	y0	math_y0
-#   define	y1	math_y1
 #   include	<math.h>
-#   undef	y0
-#   undef	y1
 
 #   include	<geo2DInteger.h>
 #   include	"appDraw.h"
+#   include	"appScreenFont.h"
+#   include	"guiPixmapImpl.h"
 
 #   include	<appDebugon.h>
 
@@ -41,13 +38,13 @@ void appCollectExposures(	DocumentRectangle *	drClip,
     region= XCreateRegion();
     XtAddExposureToRegion( event, region );
 
-    drClip->drX0= ox+ event->xexpose.x;
-    drClip->drY0= oy+ event->xexpose.y;
-    drClip->drX1= ox+ event->xexpose.x+ event->xexpose.width;
-    drClip->drY1= oy+ event->xexpose.y+ event->xexpose.height;
+    drClip->drX0= ox+ (int)event->xexpose.x;
+    drClip->drY0= oy+ (int)event->xexpose.y;
+    drClip->drX1= ox+ (int)event->xexpose.x+ (int)event->xexpose.width;
+    drClip->drY1= oy+ (int)event->xexpose.y+ (int)event->xexpose.height;
 
 #   if LOG_EXPOSES
-    appDebug( "======: [%4d+%4d]x[%4d+%4d]\n",
+    appDebug( "E=====: [%4d+%4d]x[%4d+%4d]\n",
 		    drClip->drX0,
 		    drClip->drX1- drClip->drX0+ 1,
 		    drClip->drY0,
@@ -68,23 +65,23 @@ void appCollectExposures(	DocumentRectangle *	drClip,
 
 	XNextEvent( add->addDisplay, &nextEvent );
 	XtAddExposureToRegion( &nextEvent, region );
-	drMore.drX0= ox+ nextEvent.xexpose.x;
-	drMore.drY0= oy+ nextEvent.xexpose.y;
-	drMore.drX1= ox+ nextEvent.xexpose.x+ nextEvent.xexpose.width;
-	drMore.drY1= oy+ nextEvent.xexpose.y+ nextEvent.xexpose.height;
+	drMore.drX0= ox+ (int)nextEvent.xexpose.x;
+	drMore.drY0= oy+ (int)nextEvent.xexpose.y;
+	drMore.drX1= ox+ (int)nextEvent.xexpose.x+ (int)nextEvent.xexpose.width;
+	drMore.drY1= oy+ (int)nextEvent.xexpose.y+ (int)nextEvent.xexpose.height;
 
 #	if LOG_EXPOSES
-	appDebug( "++++++: [%4d+%4d]x[%4d+%4d]\n",
+	appDebug( "E+++++: [%4d+%4d]x[%4d+%4d]\n",
 			drMore.drX0,
 			drMore.drX1- drMore.drX0+ 1,
 			drMore.drY0,
 			drMore.drY1- drMore.drY0+ 1 );
 #	endif
 
-	docUnionRectangle( drClip, drClip, &drMore );
+	geoUnionRectangle( drClip, drClip, &drMore );
 
 #	if LOG_EXPOSES
-	appDebug( "......: [%4d+%4d]x[%4d+%4d]\n",
+	appDebug( "E.....: [%4d+%4d]x[%4d+%4d]\n",
 			drClip->drX0,
 			drClip->drX1- drClip->drX0+ 1,
 			drClip->drY0,
@@ -99,20 +96,26 @@ void appCollectExposures(	DocumentRectangle *	drClip,
     return;
     }
 
-void appExposeRectangle(	const AppDrawingData *	add,
-				int			x,
-				int			y,
-				int			wide,
-				int			high )
+void appExposeDrawingData(	const AppDrawingData *	add )
     {
+    if  ( ! add->addDrawable )
+	{ XDEB(add->addDrawable); return; }
+
+    XClearArea( add->addDisplay, add->addDrawable, 0, 0, 0, 0, True );
+    }
+
+void appExposeRectangle(	const AppDrawingData *		add,
+				const DocumentRectangle *	drExpose )
+    {
+    int		x= drExpose->drX0;
+    int		y= drExpose->drY0;
+    int		wide= drExpose->drX1- drExpose->drX0+ 1;
+    int		high= drExpose->drY1- drExpose->drY0+ 1;
+
     XEvent	xev;
 
-    if  ( wide == 0 || high == 0 )
-	{
-	XClearArea( add->addDisplay, add->addDrawable,
-						x, y, wide, high, True );
-	return;
-	}
+    if  ( ! add->addDrawable )
+	{ XDEB(add->addDrawable); return; }
 
     xev.type= Expose;
     xev.xexpose.serial= 0;
@@ -133,73 +136,66 @@ void appExposeRectangle(	const AppDrawingData *	add,
     }
 
 /************************************************************************/
-/*									*/
-/*  Initialise drawing data.						*/
-/*									*/
-/************************************************************************/
 
-void appInitDrawingData(	AppDrawingData *	add )
+#   ifdef USE_XFT
+
+static void appMotifXftDrawCreate(	AppDrawingData *	add )
     {
-    add->addForScreenDrawing= 0;
+    Display *		display= add->addDisplay;
+    int			screen= add->addScreen;
+    Colormap		cmap= DefaultColormap( display, screen );
+    Visual *		vis= DefaultVisual( display, screen );
 
-    docInitRectangle( &(add->addBackRect) );
-    docInitRectangle( &(add->addPaperRect) );
-    docInitRectangle( &(add->addDocRect) );
+    if  ( add->addXftDrawable )
+	{ return;				}
 
-    add->addBottomMarginPixels= 0;
-    add->addPageGapPixels= 0;
-    add->addPageStepPixels= 0;
+    add->addXftColorList.axclDisplay= display;
+    add->addXftColorList.axclVisual= vis;
+    add->addXftColorList.axclColormap= cmap;
 
-    add->addMagnification= 1.0;
-    add->addMagnifiedPixelsPerTwip= 0.0;
-    add->addScreenPixelsPerMM= 0.0;
-    add->addDisplay= (Display *)0;
-    add->addDrawable= None;
-    add->addGc= None;
-    add->addScreen= -1;
+    add->addXftDrawable= XftDrawCreate( add->addXftColorList.axclDisplay,
+					    add->addDrawable,
+					    add->addXftColorList.axclVisual,
+					    add->addXftColorList.axclColormap );
 
-    appInitFontList( &(add->addPhysicalFontList) );
+    if  ( ! add->addXftDrawable )
+	{ XDEB(add->addXftDrawable);	}
+
+    return;
     }
 
+#   endif
+
+
 void appCleanDrawingData(	AppDrawingData *	add )
     {
+    if  ( add->addDisplay )
+	{ appDrawCleanScreenFontList( add );	}
+
+#   ifdef USE_XFT
+    appCleanDrawingDataXft( add );
+#   endif
+
     if  ( add->addGc )
 	{ XFreeGC( add->addDisplay, add->addGc );	}
 
-    if  ( add->addDisplay )
-	{ appCleanFontList( add, &(add->addPhysicalFontList) ); }
-
     return ;
     }
 
-void appSetDrawingEnvironment(
-			AppDrawingData *	add,
-			double			magnification,
-			double			xfac,
-			double			screenPixelsPerMM,
-			const char *		afmDirectory,
-			const char *		ghostscriptFontmap,
-			const char *		ghostscriptFontToXmapping,
-			APP_WIDGET		w )
+void appSetScreenDrawingEnvironment(
+			AppDrawingData *		add,
+			AppColors *			colors,
+			double				magnification,
+			double				xfac,
+			double				screenPixelsPerMM,
+			const PostScriptFontList *	psfl )
     {
-    Display *	display= XtDisplay( w );
-
-    if  ( w )
-	{ add->addForScreenDrawing= 1; }
-
     add->addMagnification= magnification;
     add->addMagnifiedPixelsPerTwip= xfac;
     add->addScreenPixelsPerMM= screenPixelsPerMM;
 
-    add->addPhysicalFontList.apflAfmDirectory=
-					    afmDirectory;
-    add->addPhysicalFontList.apflGhostscriptFontmap=
-					    ghostscriptFontmap;
-    add->addPhysicalFontList.apflGhostscriptFontToXmapping=
-					    ghostscriptFontToXmapping;
-
-    add->addDisplay= display;
-    add->addScreen= DefaultScreen( display );
+    appDrawSetPostScriptFontList( add, psfl );
+    add->addForScreenDrawing= 1;
 
     return;
     }
@@ -210,21 +206,25 @@ void appCloneDrawingEnvironment(	AppDrawingData *	add,
 					double			xfac,
 					APP_WINDOW		drawable )
     {
+    if  ( drawable )
+	{ add->addForScreenDrawing= 1;	}
+
     add->addMagnification= magnification;
     add->addMagnifiedPixelsPerTwip= xfac;
     add->addScreenPixelsPerMM= parent_add->addScreenPixelsPerMM;
 
-    add->addPhysicalFontList.apflAfmDirectory=
-		parent_add->addPhysicalFontList.apflAfmDirectory;
-    add->addPhysicalFontList.apflGhostscriptFontmap=
-		parent_add->addPhysicalFontList.apflGhostscriptFontmap;
-    add->addPhysicalFontList.apflGhostscriptFontToXmapping=
-		parent_add->addPhysicalFontList.apflGhostscriptFontToXmapping;
+    appDrawSetPostScriptFontList( add, parent_add->addPostScriptFontList );
 
     add->addDisplay= parent_add->addDisplay;
     add->addScreen= parent_add->addScreen;
     add->addDrawable= drawable;
 
+    add->addAvoidFontconfig= parent_add->addAvoidFontconfig;
+#   ifdef USE_XFT
+    if  ( ! add->addAvoidFontconfig )
+	{ appMotifXftDrawCreate( add );	}
+#   endif
+
     if  ( add->addDisplay )
 	{
 	add->addGc= XCreateGC( add->addDisplay, add->addDrawable,
@@ -241,6 +241,7 @@ void appCloneDrawingEnvironment(	AppDrawingData *	add,
 
 
 int appSetDrawingDataForWidget(	APP_WIDGET		w,
+				AppColors *		colors,
 				double			magnification,
 				AppDrawingData *	add )
     {
@@ -251,11 +252,19 @@ int appSetDrawingDataForWidget(	APP_WIDGET		w,
 
     Colormap		cmap;
 
+    add->addForScreenDrawing= 1;
+    add->addColors= colors;
+
     add->addDisplay= XtDisplay( w );
     add->addScreen= DefaultScreen( add->addDisplay );
     add->addDrawable= XtWindow( w );
     add->addGc= XCreateGC( add->addDisplay, add->addDrawable, 0, NULL );
 
+#   ifdef USE_XFT
+    if  ( ! add->addAvoidFontconfig )
+	{ appMotifXftDrawCreate( add );	}
+#   endif
+
     horPixPerMM= ( (double)DisplayWidth( add->addDisplay, add->addScreen ) )/
 			    DisplayWidthMM( add->addDisplay, add->addScreen );
     verPixPerMM= ( (double)DisplayHeight( add->addDisplay, add->addScreen ) )/
@@ -306,14 +315,20 @@ int appDrawBlackColor(		AppDrawingData *		add,
 
 void appDrawSetForegroundBlack(	AppDrawingData *	add )
     {
-    XSetForeground( add->addDisplay, add->addGc,
-			    BlackPixel( add->addDisplay, add->addScreen ) );
+    XColor		xc;
+
+    appDrawBlackColor( add, &xc );
+
+    appDrawSetForegroundColor( add, &xc );
     }
 
 void appDrawSetForegroundWhite(	AppDrawingData *	add )
     {
-    XSetForeground( add->addDisplay, add->addGc,
-			    WhitePixel( add->addDisplay, add->addScreen ) );
+    XColor		xc;
+
+    appDrawWhiteColor( add, &xc );
+
+    appDrawSetForegroundColor( add, &xc );
     }
 
 void appDrawSetBackgroundWhite(	AppDrawingData *	add )
@@ -324,7 +339,13 @@ void appDrawSetBackgroundWhite(	AppDrawingData *	add )
 
 void appDrawSetForegroundColor(	AppDrawingData *	add,
 				XColor *		xc )
-    { XSetForeground( add->addDisplay, add->addGc, xc->pixel ); }
+    {
+    XSetForeground( add->addDisplay, add->addGc, xc->pixel );
+
+#   ifdef USE_XFT
+    appSolidXftColor( &(add->addXftCurrentColor), xc );
+#   endif
+    }
 
 void appDrawSetBackgroundColor(	AppDrawingData *	add,
 				XColor *		xc )
@@ -336,6 +357,15 @@ void appDrawFillRectangle(	AppDrawingData *	add,
 				int			wide,
 				int			high )
     {
+#   ifdef USE_XFT
+
+    if  ( ! appDrawFillRectangleXft( add->addXftDrawable,
+			&(add->addXftCurrentColor), &(add->addXftColorList),
+			x, y, wide, high ) )
+	{ return;	}
+
+#   endif
+
     XFillRectangle( add->addDisplay, add->addDrawable, add->addGc,
 							    x, y, wide, high );
     }
@@ -350,28 +380,6 @@ void appDrawDrawRectangle(	AppDrawingData *	add,
 							    x, y, wide, high );
     }
 
-APP_FONT * appDrawOpenFont(	AppDrawingData *	add,
-				const char *		name )
-    {
-    return XLoadQueryFont( add->addDisplay, name );
-    }
-
-void appDrawSetFont(		AppDrawingData *	add,
-				APP_FONT *		xfs )
-    {
-    XSetFont( add->addDisplay, add->addGc, xfs->fid );
-    }
-
-void appDrawDrawString(		AppDrawingData *	add,
-				int			x,
-				int			y,
-				const char *		s,
-				int			len )
-    {
-    XDrawString( add->addDisplay, add->addDrawable, add->addGc,
-							    x, y, s, len );
-    }
-
 void appDrawDrawLine(		AppDrawingData *	add,
 				int			x0,
 				int			y0,
@@ -393,31 +401,19 @@ void appDrawMoveArea(		AppDrawingData *	add,
 				    src_x, src_y, wide, high, dest_x, dest_y );
     }
 
-void appDrawFreeFont(		AppDrawingData *	add,
-				APP_FONT *		font )
-    {
-    XFreeFont( add->addDisplay, font );
-
-    return;
-    }
-
-void appDrawTextExtents(	int *			pWidth,
-				int *			pFontAscent,
-				int *			pFontDescent,
-				const AppDrawingData *	add,
-				APP_FONT *		xfs,
-				const char *		s,
-				int			l )
-    {
-    *pWidth= XTextWidth( xfs, s, l );
-    *pFontAscent= xfs->ascent;
-    *pFontDescent= xfs->descent;
-
-    return;
-    }
-
 void appDrawNoClipping(		AppDrawingData *	add )
     {
+#   if LOG_EXPOSES
+    appDebug( "C=====: OFF\n" );
+#   endif
+
+#   ifdef USE_XFT
+    if  ( add->addXftDrawable )
+	{
+	XftDrawSetClip( add->addXftDrawable, None );
+	}
+#   endif
+
     XSetClipMask( add->addDisplay, add->addGc, None );
 
     return;
@@ -433,6 +429,24 @@ extern void appDrawSetClipRect(	AppDrawingData *		add,
     xRect.width= drClip->drX1- drClip->drX0+ 1;
     xRect.height= drClip->drY1- drClip->drY0+ 1;
 
+    if  ( xRect.width < 1 || xRect.height < 1 )
+	{ LLDEB(xRect.width,xRect.height); return; }
+
+#   if LOG_EXPOSES
+    appDebug( "C=====: [%4d+%4d]x[%4d+%4d]\n",
+		    drClip->drX0,
+		    drClip->drX1- drClip->drX0+ 1,
+		    drClip->drY0,
+		    drClip->drY1- drClip->drY0+ 1 );
+#   endif
+
+#   ifdef USE_XFT
+    if  ( add->addXftDrawable )
+	{
+	XftDrawSetClipRectangles( add->addXftDrawable, 0, 0, &xRect, 1 );
+	}
+#   endif
+
     XSetClipRectangles( add->addDisplay, add->addGc,
 						0, 0, &xRect, 1, Unsorted );
 
@@ -440,38 +454,46 @@ extern void appDrawSetClipRect(	AppDrawingData *		add,
     }
 
 void appDrawFillPolygon(		AppDrawingData *	add,
-					APP_POINT *		points,
+					const APP_POINT *	points,
 					int			count )
     {
     XFillPolygon( add->addDisplay, add->addDrawable, add->addGc,
-				    points, count, Complex, CoordModeOrigin );
+			(APP_POINT *)points, count, Complex, CoordModeOrigin );
     }
 
 void appDrawDrawLines(			AppDrawingData *	add,
-					APP_POINT *		points,
-					int			count )
+					const APP_POINT *	points,
+					int			count,
+					int			close )
     {
     int		i;
 
     /*
     XDrawLines( add->addDisplay, add->addDrawable, add->addGc,
-					points, count, CoordModeOrigin );
+				(APP_POINT *)points, count, CoordModeOrigin );
     */
 
     count--;
 
-    for ( i= 0; i < count; points++, i++ )
+    for ( i= 0; i < count; i++ )
 	{
 	XDrawLine( add->addDisplay, add->addDrawable, add->addGc,
-						    points[0].x, points[0].y,
-						    points[1].x, points[1].y );
+					    points[i+0].x, points[i+0].y,
+					    points[i+1].x, points[i+1].y );
+	}
+
+    if  ( close )
+	{
+	XDrawLine( add->addDisplay, add->addDrawable, add->addGc,
+					    points[i].x, points[i].y,
+					    points[0].x, points[0].y );
 	}
 
     return;
     }
 
 void appDrawDrawPixmap(			AppDrawingData *	add,
-					APP_BITMAP_IMAGE	pixmap,
+					const ScreenPixmap	sp,
 					int			xSrc,
 					int			ySrc,
 					int			wide,
@@ -479,8 +501,20 @@ void appDrawDrawPixmap(			AppDrawingData *	add,
 					int			xDest,
 					int			yDest )
     {
-    XCopyArea( add->addDisplay, pixmap, add->addDrawable, add->addGc,
+    if  ( xSrc < 0 || ySrc < 0 )
+	{ LLDEB(xSrc,ySrc); return;	}
+
+    if  ( xDest < 0 )
+	{ xSrc -= xDest; wide += xDest; xDest= 0; }
+    if  ( yDest < 0 )
+	{ ySrc -= yDest; high += yDest; yDest= 0; }
+
+    if  ( wide > 0 && high > 0 )
+	{
+	XCopyArea( add->addDisplay, sp->spBitmapImage,
+				    add->addDrawable, add->addGc,
 				    xSrc, ySrc, wide, high, xDest, yDest );
+	}
 
     return;
     }
@@ -517,10 +551,20 @@ APP_BITMAP_IMAGE appMakePixmap(	AppDrawingData *	add,
 				int			wide,
 				int			high )
     {
-    int		depth= DefaultDepth( add->addDisplay, add->addScreen );
+    int			depth= DefaultDepth( add->addDisplay, add->addScreen );
+    APP_BITMAP_IMAGE	rval;
 
-    return XCreatePixmap( add->addDisplay, add->addDrawable,
+    rval= XCreatePixmap( add->addDisplay, add->addDrawable,
 						    wide, high, depth );
+    if  ( ! rval )
+	{ XDEB(rval);	}
+
+    XSetForeground( add->addDisplay, add->addGc,
+			    WhitePixel( add->addDisplay, add->addScreen ) );
+
+    XFillRectangle( add->addDisplay, rval, add->addGc, 0, 0, wide, high );
+
+    return rval;
     }
 
 void appDrawDrawSegments(	AppDrawingData *	add,
@@ -531,6 +575,16 @@ void appDrawDrawSegments(	AppDrawingData *	add,
 							    segments, count );
     }
 
+/************************************************************************/
+/*									*/
+/*  Draw an arc. Angles are in X11 style:				*/
+/*									*/
+/*  Unit:		1/64 degree.					*/
+/*  Origin:		At the right of the circle.			*/
+/*  Orientation:	Counterclockwise.				*/
+/*									*/
+/************************************************************************/
+
 void appDrawDrawArc(		AppDrawingData *	add,
 				int			x,
 				int			y,
@@ -555,4 +609,44 @@ void appDrawFillArc(		AppDrawingData *	add,
 					x, y, wide, high, alpha0, alpha_step );
     }
 
+void appDrawDrawArcs(		AppDrawingData *	add,
+				const APP_ARC *		arcs,
+				int			count )
+    {
+    XDrawArcs( add->addDisplay, add->addDrawable, add->addGc,
+						    (APP_ARC *)arcs, count );
+    }
+
+void appDrawFillArcs(		AppDrawingData *	add,
+				const APP_ARC *		arcs,
+				int			count )
+    {
+    XFillArcs( add->addDisplay, add->addDrawable, add->addGc,
+						    (APP_ARC *)arcs, count );
+    }
+
+int appDrawSetLineAttributes(	AppDrawingData *	add,
+				int			lineWidth,
+				int			lineStyle,
+				int			capStyle,
+				int			joinStyle,
+				const unsigned char *	dashList,
+				int			dashCount )
+    {
+    XSetLineAttributes( add->addDisplay, add->addGc,
+				lineWidth, lineStyle, capStyle, joinStyle );
+
+    if  ( lineStyle != LINEstyleSOLID )
+	{
+	if  ( ! dashList || dashCount == 0 )
+	    { XLDEB(dashList,dashCount);	}
+	else{
+	    XSetDashes( add->addDisplay, add->addGc, 0,
+						(char *)dashList, dashCount );
+	    }
+	}
+
+    return 0;
+    }
+
 #   endif
