@@ -14,10 +14,10 @@
 #   include	<appSystem.h>
 
 #   include	<appFrame.h>
-#   include	<appDebugon.h>
-
 #   include	"appFileChooser.h"
 
+#   include	<appDebugon.h>
+
 #   ifdef USE_MOTIF
 
 #   include	<Xm/Text.h>
@@ -36,27 +36,32 @@
 static void appFreeAci(		Widget		fileChooser,
 				XtPointer	voidaci,
 				XtPointer	voidcbs	 )
-    { free( voidaci );	}
+    {
+    AppChooserInformation *	aci= (AppChooserInformation *)voidaci;
+
+    if  ( aci->aciFilename )
+	{ free( aci->aciFilename );	}
+
+    free( aci );
+    }
 
 static void appInitAci(		AppChooserInformation *	aci )
     {
     appInitOptionmenu( &(aci->aciFilterOptionmenu) );
 
     aci->aciFilename= (char *)0;
+    aci->aciDocument= (EditDocument *)0;
     aci->aciThrough= (void *)0;
     aci->aciOpenDocument= (APP_OPEN_DOCUMENT)0;
+    aci->aciSaveDocument= (APP_SAVE_DOCUMENT)0;
     aci->aciExtensions= (AppFileExtension *)0;
     aci->aciExtensionCount= 0;
     aci->aciApplication= (EditApplication *)0;
     aci->aciOption= (Widget)0;
+    aci->aciRelativeTo= (Widget)0;
     aci->aciFormat= -1;
-    aci->aciExtension= (char *)0;
 
-    aci->aciIsDirecoryMessage= (char *)0;
-    aci->aciNotWritableMessage= (char *)0;
-    aci->aciNotReadableMessage= (char *)0;
-    aci->aciOverwriteMessage= (char *)0;
-    aci->aciNoSuchDirMessage= (char *)0;
+    aci->aciResources= (const AppFileChooserResources *)0;
     }
 
 /************************************************************************/
@@ -83,11 +88,14 @@ static void appChooserCancel(	APP_WIDGET	fileChooser,
 /************************************************************************/
 
 static int appChooserGetFilename(	void *			voidcbs,
-					AppChooserInformation *	aci )
+					AppChooserInformation *	aci,
+					const char *		extension )
     {
+    int					rval= 0;
+    int					ret;
     XmFileSelectionBoxCallbackStruct *	cbs;
 
-    char *				filename;
+    char *				filename= (char *)0;
 
     cbs= (XmFileSelectionBoxCallbackStruct *)voidcbs;
 
@@ -95,15 +103,26 @@ static int appChooserGetFilename(	void *			voidcbs,
 	{
 	LDEB(1);
 	appGuiBreakDialog( &(aci->aciDialog), ACIrespFAILURE );
-	return -1;
+	rval= -1; goto ready;
 	}
 
-    if  ( appChooserSaveFilename( aci, filename ) )
-	{ XtFree( filename ); return -1;	}
+    aci->aciRelativeTo= aci->aciDialog.adTopWidget;
+    ret= appChooserSaveFilename( &(aci->aciFilename), aci,
+						    filename, extension );
 
-    XtFree( filename );
+    if  ( ret < 0 )
+	{
+	appGuiBreakDialog( &(aci->aciDialog), ACIrespFAILURE );
+	rval= -1; goto ready;
+	}
+    if  ( ret > 0 )
+	{ rval= -1; goto ready;		}
 
-    return 0;
+  ready:
+    if  ( filename )
+	{ XtFree( filename );	}
+
+    return rval;
     }
 
 /************************************************************************/
@@ -113,26 +132,26 @@ static int appChooserGetFilename(	void *			voidcbs,
 /*									*/
 /************************************************************************/
 
-static void appChooserSetText(		Widget  text,
+static void appChooserSetText(		Widget  w,
 					char *	s )
     {
-    if  ( XmIsText( text ) )
+    if  ( XmIsText( w ) )
 	{
-	XmTextSetString( text, s );
-	XmTextSetInsertionPosition( text, strlen( s ) );
+	XmTextSetString( w, s );
+	XmTextSetInsertionPosition( w, strlen( s ) );
 
 	return;
 	}
 
-    if  ( XmIsTextField( text ) )
+    if  ( XmIsTextField( w ) )
 	{
-	XmTextFieldSetString( text, s );
-	XmTextFieldSetInsertionPosition( text, strlen( s ) );
+	XmTextFieldSetString( w, s );
+	XmTextFieldSetInsertionPosition( w, strlen( s ) );
 
 	return;
 	}
 
-    SDEB( (char *)text );
+    SDEB( s );
     }
 
 static void appChooserSetDirectory(	char *			filename,
@@ -158,8 +177,14 @@ static void appChooserSetDirectory(	char *			filename,
     return;
     }
 
-static void appChooserSetFileExtension(	char *	filename,
-					char *	extension,
+/************************************************************************/
+/*									*/
+/*  Change the extension of the file name.				*/
+/*									*/
+/************************************************************************/
+
+static void appChooserSetFileExtension(	char *		filename,
+					const char *	extension,
 					Widget	text )
     {
     const char *	ext;
@@ -190,23 +215,35 @@ static void appChooserSetFileExtension(	char *	filename,
 /*									*/
 /************************************************************************/
 
-static void appOpenChooserOk(	Widget		fileChooser,
+static void appOpenOkPushed(	Widget		fileChooser,
 				XtPointer	voidaci,
 				XtPointer	voidcbs	 )
     {
     AppChooserInformation *	aci= (AppChooserInformation *)voidaci;
     Widget			option= aci->aciOption;
+    const char *		extension= (const char *)0;
 
-    if  ( appChooserGetFilename( voidcbs, aci ) )
+    if  ( aci->aciExtensionCount > 0 )
+	{
+	if  ( aci->aciFormat >= 0 && aci->aciFormat < aci->aciExtensionCount )
+	    {
+	    const AppFileExtension *	afe= aci->aciExtensions+ aci->aciFormat;
+
+	    extension= afe->afeExtension;
+	    }
+	}
+
+    if  ( appChooserGetFilename( voidcbs, aci, extension ) )
 	{ return;	}
 
+    aci->aciRelativeTo= aci->aciDialog.adTopWidget;
     if  ( appFileChooserTestNameForOpen( aci, aci->aciFilename ) )
 	{
 	free( aci->aciFilename ); aci->aciFilename= (char *)0;
 	return;
 	}
 
-    if  ( (*aci->aciOpenDocument)( aci->aciThrough,
+    if  ( (*aci->aciOpenDocument)( aci->aciApplication, aci->aciThrough,
 					    aci->aciDialog.adTopWidget, option,
 					    aci->aciFilename ) )
 	{ free( aci->aciFilename ); aci->aciFilename= (char *)0; return; }
@@ -220,14 +257,14 @@ static void appOpenChooserOk(	Widget		fileChooser,
 
 /************************************************************************/
 /*									*/
-/*  The file selection box was toold to filter via the normal motif	*/
+/*  The file selection box was told to filter via the normal motif	*/
 /*  mechanisms: Adapt the OptionMenu with extensions to the filter.	*/
 /*									*/
 /************************************************************************/
 
 static void appFileFilterActivated(	Widget		w,
 					XtPointer	voidaci,
-					XtPointer	voidpbcs )
+					XtPointer	e )
     {
     AppChooserInformation *	aci= (AppChooserInformation *)voidaci;
 
@@ -298,7 +335,7 @@ static void appFileFilterActivated(	Widget		w,
 
 static void appFileFilterChosen(	Widget		w,
 					XtPointer	voidaci,
-					XtPointer	voidpbcs )
+					XtPointer	e )
     {
     AppChooserInformation *	aci= (AppChooserInformation *)voidaci;
     const AppFileExtension *	afe= aci->aciExtensions;
@@ -308,7 +345,7 @@ static void appFileFilterChosen(	Widget		w,
     int				previousExtension= aci->aciFormat;
     short			extensionChosen= -1;
 
-    char *			filter;
+    const char *		filter;
     XmString			filterString;
 
     char *			fileSelected;
@@ -316,14 +353,13 @@ static void appFileFilterChosen(	Widget		w,
     text= XmFileSelectionBoxGetChild( chooser, XmDIALOG_TEXT );
     fileSelected= appGetStringFromTextWidget( text );
 
-    XtVaGetValues( w,	XmNpositionIndex,	&extensionChosen,
-			NULL );
-
+    extensionChosen= appGuiGetOptionmenuItemIndex(
+					&(aci->aciFilterOptionmenu), w );
     if  ( extensionChosen < 0 )
 	{ LDEB(extensionChosen); return;	}
 
     filter= afe[extensionChosen].afeFilter;
-    filterString= XmStringCreateLocalized( filter );
+    filterString= XmStringCreateLocalized( (char *)filter );
 
     XtVaSetValues( chooser,
 			XmNpattern,		filterString,
@@ -341,7 +377,7 @@ static void appFileFilterChosen(	Widget		w,
 	      afe[extensionChosen].afeExtension				&&
 	      ! strcmp( extS, afe[previousExtension].afeExtension )	)
 	    {
-	    char *	ext= afe[extensionChosen].afeExtension;
+	    const char *	ext= afe[extensionChosen].afeExtension;
 
 	    appChooserSetFileExtension( fileSelected, ext, text );
 	    }
@@ -363,32 +399,14 @@ static void appFileFilterChosen(	Widget		w,
 /*									*/
 /************************************************************************/
 
-static void appMenuConfigure(	Widget		w,
-				void *		through,
-				XEvent *	event,
-				Boolean *	pRefused )
-    {
-    XConfigureEvent *		cevent= &(event->xconfigure);
-
-    if  ( cevent->type != ConfigureNotify )
-	{ return;	}
-
-    appOptionmenuSetWidthMotif( w, cevent->width );
-
-    *pRefused= 1;
-
-    return;
-    }
-
 static void appChooserMakeOptionmenu(	AppOptionmenu *	aom,
-					Widget		parentForm )
+					Widget		parent )
     {
-    Widget			labelGadget;
-    Widget			buttonGadget;
-
     Arg				al[20];
     int				ac= 0;
 
+    appInitOptionmenu( aom );
+
     ac= 0;
     XtSetArg( al[ac], XmNresizeWidth,		False ); ac++;
     XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
@@ -396,7 +414,7 @@ static void appChooserMakeOptionmenu(	AppOptionmenu *	aom,
     XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_FORM ); ac++;
     XtSetArg( al[ac], XmNbottomAttachment,	XmATTACH_FORM ); ac++;
 
-    aom->aomPulldown= XmCreatePulldownMenu( parentForm, WIDGET_NAME, al, ac );
+    aom->aomPulldown= XmCreatePulldownMenu( parent, WIDGET_NAME, al, ac );
 
     ac= 0;
     XtSetArg( al[ac], XmNsubMenuId,		aom->aomPulldown ); ac++;
@@ -413,19 +431,9 @@ static void appChooserMakeOptionmenu(	AppOptionmenu *	aom,
     XtSetArg( al[ac], XmNleftOffset,		5 ); ac++;
     XtSetArg( al[ac], XmNrightOffset,		5 ); ac++;
 
-    aom->aomInplace= XmCreateOptionMenu( parentForm, WIDGET_NAME, al, ac );
-    XtAddEventHandler( aom->aomInplace, StructureNotifyMask, False,
-					appMenuConfigure, (void *)0 );
+    aom->aomInplace= XmCreateOptionMenu( parent, WIDGET_NAME, al, ac );
 
-    labelGadget= XmOptionLabelGadget( aom->aomInplace );
-    buttonGadget= XmOptionButtonGadget( aom->aomInplace );
-
-    XtUnmanageChild( labelGadget );
-    XtManageChild( buttonGadget );
-
-    XtVaSetValues( buttonGadget,
-			XmNalignment,		XmALIGNMENT_BEGINNING,
-			NULL );
+    appFinishOptionmenuMotif( aom->aomInplace, aom->aomPulldown );
 
     XtVaSetValues( aom->aomPulldown,
 			    XmNresizeWidth,	False,
@@ -461,35 +469,16 @@ static void appFileFillFilter(	EditApplication *	ea,
 				AppChooserInformation *	aci,
 				AppFileExtension *	extensions,
 				int			extensionCount,
-				char *			defaultFilter )
+				const char *		defaultFilter )
     {
-    AppConfigurableResource *	acr;
-    AppConfigurableResource *	acri;
     AppFileExtension *		afe;
 
     int				i;
 
     int				selected= -1;
-    char *			child0Filter= (char *)0;
+    const char *		child0Filter= (char *)0;
 
-    acr= (AppConfigurableResource *)
-		    malloc( extensionCount* sizeof(AppConfigurableResource) );
-    if  ( ! acr )
-	{ LXDEB(extensionCount,acr); return;	}
-
-    afe= extensions;
-    acri= acr;
-    for ( i= 0; i < extensionCount; afe++, acri++, i++ )
-	{
-	APP_SET_RESOURCE( acri, afe->afeId,
-			i* sizeof(AppFileExtension)+
-				    offsetof(AppFileExtension,afeDescription),
-			afe->afeDescription );
-	}
-
-    appGuiGetResourceValues( ea, extensions, acr, extensionCount );
-
-    free( acr );
+    appFileFilterGetDescriptions( ea, extensions, extensionCount );
 
     appEmptyOptionmenu( &(aci->aciFilterOptionmenu) );
 
@@ -526,7 +515,7 @@ static void appFileFillFilter(	EditApplication *	ea,
 
 	if  ( child0Filter )
 	    {
-	    filterString= XmStringCreateLocalized( child0Filter );
+	    filterString= XmStringCreateLocalized( (char *)child0Filter );
 
 	    XtVaSetValues( aci->aciDialog.adDialog,
 					XmNpattern,	filterString,
@@ -549,6 +538,7 @@ static void appFileFillFilter(	EditApplication *	ea,
 static int appMakeFileChooser( AppChooserInformation **	pAci,
 				Widget				relative,
 				EditApplication *		ea,
+				int				withFilter,
 				const char *			defaultFilter,
 				Widget				option )
     {
@@ -624,15 +614,21 @@ static int appMakeFileChooser( AppChooserInformation **	pAci,
 			NULL );
 
     /*  Not necessary
-    XtAddCallback( text, XmNactivateCallback,
+    if  ( withFilter )
+	{
+	XtAddCallback( text, XmNactivateCallback,
 				    appFileFilterActivated, (void *)aci );
+	}
     */
 
     filterButton= XmFileSelectionBoxGetChild( aci->aciDialog.adDialog,
 						    XmDIALOG_APPLY_BUTTON );
 
-    XtAddCallback( filterButton, XmNactivateCallback,
+    if  ( withFilter )
+	{
+	XtAddCallback( filterButton, XmNactivateCallback,
 				    appFileFilterActivated, (void *)aci );
+	}
 
     text= XmFileSelectionBoxGetChild( aci->aciDialog.adDialog, XmDIALOG_TEXT );
     XtVaSetValues( text,
@@ -672,7 +668,7 @@ static void appRunFileChooser(	EditApplication *		ea,
     {
     aci->aciFilename= (char *)0;
 
-    appGuiShowDialog( &(aci->aciDialog), relative );
+    appGuiShowDialog( ea, &(aci->aciDialog), relative );
 
     /*  1  */
     if  ( aci->aciFilterOptionmenu.aomInplace )
@@ -695,39 +691,42 @@ static void appRunFileChooser(	EditApplication *		ea,
 /*									*/
 /************************************************************************/
 
-void appRunOpenChooser( Widget			option,
-			Widget			relative,
-			int			extensionCount,
-			AppFileExtension *	extensions,
-			char *			defaultFilter,
-			void *			through,
-			APP_OPEN_DOCUMENT	openDocument,
-			EditApplication *	ea )
+void appRunOpenChooser( Widget				option,
+			Widget				relative,
+			int				extensionCount,
+			AppFileExtension *		extensions,
+			const char *			defaultFilter,
+			APP_OPEN_DOCUMENT		openDocument,
+			EditApplication *		ea,
+			void *				through )
     {
     AppChooserInformation *	aci= (AppChooserInformation *)0;
+    const int			withFilter= extensionCount > 0;
 
-    if  ( appMakeFileChooser( &aci, relative, ea, defaultFilter, option ) )
+    if  ( appMakeFileChooser( &aci, relative, ea,
+					withFilter, defaultFilter, option ) )
 	{ LDEB(1); return;	}
 
     XtAddCallback( aci->aciDialog.adDialog,
-			XmNokCallback, appOpenChooserOk, (XtPointer)aci );
+			XmNokCallback, appOpenOkPushed, (XtPointer)aci );
 
-    if  ( extensionCount > 0 )
+    if  ( withFilter )
 	{
 	appFileMakeFilter( aci );
 
-	appGuiShowDialog( &(aci->aciDialog), relative );
+	appGuiShowDialog( ea, &(aci->aciDialog), relative );
 
 	appFileFillFilter( ea, aci,
 				extensions, extensionCount, defaultFilter );
 	}
 
     aci->aciOption= option;
+    aci->aciRelativeTo= relative;
     aci->aciThrough= through;
     aci->aciOpenDocument= openDocument;
     aci->aciFilename= (char *)0;
 
-    if  ( extensionCount > 0 )
+    if  ( withFilter )
 	{
 	WidgetList		children;
 	Cardinal		childCount= 0;
@@ -743,9 +742,7 @@ void appRunOpenChooser( Widget			option,
 	afe= extensions;
 	for ( i= 0; i < childCount; afe++, i++ )
 	    {
-	    if  ( ! ( afe->afeUseFlags & APPFILE_CAN_OPEN )	||
-		  ( afe->afeUseFlags & APPFILE_IS_BASIC_TYPE )	||
-		  ( afe->afeUseFlags & APPFILE_HIDE_OPEN )	)
+	    if  ( ! FILE_CHOOSER_CAN_OPEN( afe ) )
 		{ XtUnmanageChild( children[i] ); continue;	}
 
 	    XtManageChild( children[i] );
@@ -759,35 +756,33 @@ void appRunOpenChooser( Widget			option,
     return;
     }
 
-static int appChooserOpenDocument(	void *		through,
-					Widget		relative,
-					Widget		option,
-					const char *	filename )
+static int appChooserOpenDocument(	EditApplication *	ea,
+					void *			through,
+					Widget			relative,
+					Widget			option,
+					const char *		filename )
     {
     const int	read_only= 0;
 
-    if  ( ! appOpenDocument( (EditApplication *)through,
-				    relative, option, read_only, filename ) )
+    if  ( ! appOpenDocument( ea, relative, option, read_only, filename ) )
 	{ return -1;	}
 
     return 0;
     }
 
-void appAppFileOpen(	APP_WIDGET	option,
-			void *		voidea,
-			void *		call_data )
+APP_MENU_CALLBACK_H( appAppFileOpen, option, voidea, e )
     {
     EditApplication *	ea= (EditApplication *)voidea;
 
     appRunOpenChooser( option, ea->eaToplevel.atTopWidget,
 			ea->eaFileExtensionCount, ea->eaFileExtensions,
 			ea->eaDefaultFileFilter,
-			(void *)ea, appChooserOpenDocument, ea );
+			appChooserOpenDocument, ea, (void *)ea );
     }
 
 void appDocFileOpen(	APP_WIDGET	option,
 			void *		voided,
-			void *		call_data )
+			void *		e )
     {
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
@@ -795,25 +790,34 @@ void appDocFileOpen(	APP_WIDGET	option,
     appRunOpenChooser( option, ed->edToplevel.atTopWidget,
 			ea->eaFileExtensionCount, ea->eaFileExtensions,
 			ea->eaDefaultFileFilter,
-			(void *)ea, appChooserOpenDocument, ea );
+			appChooserOpenDocument, ea, (void *)ea );
     }
 
-static void appSaveOkPushed(		Widget		fileChooser,
-					XtPointer	voidaci,
-					XtPointer	voidcbs	 )
+static void appFileOkPushed(		AppChooserInformation *	aci,
+					XtPointer		voidcbs,
+					const char *		extension )
     {
-    AppChooserInformation *		aci= (AppChooserInformation *)voidaci;
-    int					response;
+    int			response;
 
-    if  ( appChooserGetFilename( voidcbs, aci ) )
+    if  ( appChooserGetFilename( voidcbs, aci, extension ) )
 	{ return;	}
 
-    response= appFileChooserTestNameForSave( aci, aci->aciFilename );
+    aci->aciRelativeTo= aci->aciDialog.adTopWidget;
+    response= appFileChooserTestNameForWrite( aci, aci->aciFilename );
 
     switch( response )
 	{
 	case ACIrespSAVE:
-	    appGuiBreakDialog( &(aci->aciDialog), ACIrespSAVE );
+	    if  ( (*aci->aciSaveDocument)( aci->aciDocument, aci->aciThrough, 
+			    aci->aciDialog.adTopWidget, aci->aciOption,
+			    aci->aciFormat, aci->aciFilename ) )
+		{
+		SDEB(aci->aciFilename);
+		free( aci->aciFilename ); aci->aciFilename= (char *)0;
+		}
+	    else{
+		appGuiBreakDialog( &(aci->aciDialog), ACIrespSAVE );
+		}
 	    return;
 
 	case ACIrespCANCEL:
@@ -834,6 +838,36 @@ static void appSaveOkPushed(		Widget		fileChooser,
     return;
     }
 
+static void appSaveOkPushed(		Widget		fileChooser,
+					XtPointer	voidaci,
+					XtPointer	voidcbs	 )
+    {
+    AppChooserInformation *		aci= (AppChooserInformation *)voidaci;
+    const char *			extension= (const char *)0;
+
+    if  ( aci->aciExtensionCount > 0 )
+	{
+	if  ( aci->aciFormat >= 0 && aci->aciFormat < aci->aciExtensionCount )
+	    {
+	    const AppFileExtension *	afe= aci->aciExtensions+ aci->aciFormat;
+
+	    extension= afe->afeExtension;
+	    }
+	}
+
+    appFileOkPushed( aci, voidcbs, extension );
+    }
+
+static void appPrintOkPushed(		Widget		fileChooser,
+					XtPointer	voidaci,
+					XtPointer	voidcbs	 )
+    {
+    AppChooserInformation *		aci= (AppChooserInformation *)voidaci;
+    const char *			extension= "ps";
+
+    appFileOkPushed( aci, voidcbs, extension );
+    }
+
 /************************************************************************/
 /*									*/
 /*  Run Filechooser for save.						*/
@@ -875,7 +909,7 @@ static int appSaveChooserFilterExtensions( EditApplication *	ea,
 	XmString	filterString;
 
 	filterString= XmStringCreateLocalized(
-				ea->eaFileExtensions[choice].afeFilter );
+			    (char *)ea->eaFileExtensions[choice].afeFilter );
 
 	appSetOptionmenu( &(aci->aciFilterOptionmenu), choice );
 
@@ -895,7 +929,7 @@ static int appSaveChooserFilterExtensions( EditApplication *	ea,
 	    if  ( choice == ed->edFormat )
 		{ appChooserSetText( text, ed->edFilename ); }
 	    else{
-		char *	ext= ea->eaFileExtensions[choice].afeExtension;
+		const char *	ext= ea->eaFileExtensions[choice].afeExtension;
 
 		if  ( ext )
 		    {
@@ -911,36 +945,42 @@ static int appSaveChooserFilterExtensions( EditApplication *	ea,
     return 0;
     }
 
-int appRunSaveChooser(		APP_WIDGET		option,
-				APP_WIDGET		relative,
-				EditApplication *	ea,
-				const EditDocument *	ed,
-				unsigned int		useFlags,
-				int *			pFormat,
-				char **			pFilename )
+int appRunSaveChooser(	APP_WIDGET		option,
+			APP_WIDGET		relative,
+			unsigned int		useFlags,
+			APP_SAVE_DOCUMENT	saveDocument,
+			EditDocument *		ed,
+			void *			through )
     {
+    EditApplication *		ea= ed->edApplication;
     AppChooserInformation *	aci= (AppChooserInformation *)0;
+    const int			withFilter= ea->eaFileExtensionCount > 0;
     int				response;
 
-    if  ( appMakeFileChooser( &aci, relative, ea,
+    if  ( appMakeFileChooser( &aci, relative, ea, withFilter,
 					    ea->eaDefaultFileFilter, option ) )
 	{ return ACIrespFAILURE;	}
 
     XtAddCallback( aci->aciDialog.adDialog,
 			    XmNokCallback, appSaveOkPushed, (XtPointer)aci );
 
-    if  ( ea->eaFileExtensionCount > 0 )
+    if  ( withFilter )
 	{
 	appFileMakeFilter( aci );
 
-	appGuiShowDialog( &(aci->aciDialog), relative );
+	appGuiShowDialog( ea, &(aci->aciDialog), relative );
 
 	appFileFillFilter( ea, aci,
-			    ea->eaFileExtensions, ea->eaFileExtensionCount,
-			    ea->eaDefaultFileFilter );
+		ea->eaFileExtensions,
+		ea->eaFileExtensionCount,
+		ea->eaDefaultFileFilter );
 	}
 
     aci->aciOption= option;
+    aci->aciRelativeTo= relative;
+    aci->aciDocument= ed;
+    aci->aciThrough= through;
+    aci->aciSaveDocument= saveDocument;
 
     if  ( ed->edFilename )
 	{ appChooserSetDirectory( ed->edFilename, aci ); }
@@ -951,13 +991,7 @@ int appRunSaveChooser(		APP_WIDGET		option,
 
     appRunFileChooser( ea, aci, relative );
 
-    response= aci->aciDialog.adResponse;
-
-    if  ( response == ACIrespSAVE )
-	{
-	*pFilename= aci->aciFilename;
-	*pFormat= aci->aciFormat;
-	}
+    response= aci->aciDialog.adResponse;;
 
     XtDestroyWidget( aci->aciDialog.adTopWidget );
 
@@ -970,24 +1004,29 @@ int appRunSaveChooser(		APP_WIDGET		option,
 /*									*/
 /************************************************************************/
 
-int appRunPrintToFileChooser(		Widget			option,
-					Widget			panel,
-					EditApplication *	ea,
-					const EditDocument *	ed,
-					char **			pFilename )
+int appRunPrintToFileChooser(
+			APP_WIDGET		option,
+			APP_WIDGET		relative,
+			APP_SAVE_DOCUMENT	printDocument,
+			EditDocument *		ed,
+			void *			through )
     {
+    EditApplication *		ea= ed->edApplication;
     AppChooserInformation *	aci= (AppChooserInformation *)0;
+    const int			withFilter= 0;
     int				response;
 
-    if  ( appMakeFileChooser( &aci, panel, ea, "*.ps", option ) )
+    if  ( appMakeFileChooser( &aci, relative, ea, withFilter, "*.ps", option ) )
 	{ LDEB(1); return ACIrespFAILURE;	}
 
     XtAddCallback( aci->aciDialog.adDialog,
-				XmNokCallback, appSaveOkPushed, (void *)aci );
+				XmNokCallback, appPrintOkPushed, (void *)aci );
 
-    aci->aciExtension= "ps";
     aci->aciOption= option;
-    aci->aciFilename= (char *)0;
+    aci->aciRelativeTo= relative;
+    aci->aciDocument= ed;
+    aci->aciThrough= through;
+    aci->aciSaveDocument= printDocument;
 
     if  ( ed->edFilename )
 	{
@@ -1000,16 +1039,36 @@ int appRunPrintToFileChooser(		Widget			option,
 	appChooserSetFileExtension( ed->edFilename, "ps", text );
 	}
 
-    appRunFileChooser( ea, aci, panel );
+    appRunFileChooser( ea, aci, relative );
 
     response= aci->aciDialog.adResponse;
 
-    if  ( response == ACIrespSAVE )
-	{ *pFilename= aci->aciFilename; }
-
     XtDestroyWidget( aci->aciDialog.adTopWidget );
 
     return response;
     }
 
+int appFileChooserConfirmOverWrite(	const AppChooserInformation *	aci,
+					const char *		filename )
+    {
+    int			rcc;
+
+    rcc= appQuestionRunSubjectYesNoCancelDialog(
+			aci->aciApplication, aci->aciRelativeTo, aci->aciOption,
+			filename, aci->aciResources->acrOverwriteMessage );
+
+    switch( rcc )
+	{
+	case AQDrespYES:
+	    return ACIrespSAVE;
+	case AQDrespNO:
+	    return ACIrespNONE;
+	default:
+	    LDEB(rcc);
+	    /*FALLTHROUGH*/
+	case AQDrespCANCEL:
+	    return ACIrespCANCEL;
+	}
+    }
+
 #   endif
