@@ -3,13 +3,14 @@
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<string.h>
+
+#   include	"appFrame.h"
+#   include	<geoString.h>
+
 #   include	<appDebugon.h>
 
 #   ifdef USE_GTK
 
-#   include	"appFrame.h"
-#   include	<appGeoString.h>
-
 /************************************************************************/
 /*									*/
 /*  Retrieve configurable resource values from the GUI environment.	*/
@@ -19,7 +20,8 @@
 /*									*/
 /************************************************************************/
 
-void appGuiGetResourceValues(	EditApplication *		ea,
+void appGuiGetResourceValues(	int *				pGotResources,
+				EditApplication *		ea,
 				void *				pValues,
 				AppConfigurableResource *	acrList,
 				int				acrCount )
@@ -28,6 +30,12 @@ void appGuiGetResourceValues(	EditApplication *		ea,
     char *			values= (char *)pValues;
     int				i;
 
+    if  ( *pGotResources )
+	{ LDEB(*pGotResources); return;	}
+
+    if  ( ! *pGotResources )
+	{ appSetResourceDefaults( ea, acrList, acrCount );	}
+
     acr= acrList;
     for ( i= 0; i < acrCount; acr++, i++ )
 	{
@@ -37,6 +45,7 @@ void appGuiGetResourceValues(	EditApplication *		ea,
     if  ( ea->eaToplevel.atTopWidget )
 	{ appGuiGetResourceValuesGtkX( ea, pValues, acrList, acrCount ); }
 
+    *pGotResources= 1;
     return;
     }
 
@@ -193,6 +202,15 @@ void appGuiEnableWidget(	APP_WIDGET		w,
     gtk_widget_set_sensitive( w, on_off != 0 );
     }
 
+
+void appGuiSetWidgetVisible(	APP_WIDGET		w,
+				int			on_off )
+    {
+    if  ( on_off )
+	{ gtk_widget_show( w );	}
+    else{ gtk_widget_hide( w );	}
+    }
+
 /************************************************************************/
 /*									*/
 /*  Use the text of an option and the name of the application as a	*/
@@ -224,21 +242,28 @@ void appSetShellTitle(	APP_WIDGET		shell,
     free( title );
     }
 
-void appGuiSetShellTitle(		APP_WIDGET		shell,
-					const char *		fullTitle )
+void appGuiSetShellTitle(	APP_WIDGET		shell,
+				const char *		fullTitle )
     {
     gtk_window_set_title( GTK_WINDOW( shell ), fullTitle );
 
     return;
     }
 
-void appGuiSetIconTitle(		APP_WIDGET		shell,
-					const char *		fullIconName )
+void appGuiSetIconTitle(	APP_WIDGET		shell,
+				const char *		fullIconName )
     {
     gdk_window_set_icon_name( shell->window, fullIconName );
     return;
     }
 
+void appGuiLowerShellWidget(	APP_WIDGET	shell )
+    {
+    gdk_window_lower( shell->window );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Change the label/state of a toggle button.				*/
@@ -263,22 +288,23 @@ int appGuiGetToggleState(	APP_WIDGET		toggle )
     return gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) ) != 0;
     }
 
-int appGuiGetToggleStateFromCallbackGtk( GtkToggleButton *	toggle )
+int appGuiGetToggleStateFromCallbackGtk( GtkWidget *	toggle )
     {
-    return gtk_toggle_button_get_active( toggle ) != 0;
+    return gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) ) != 0;
     }
 
 APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
 				const char *		text,
 				APP_TOGGLE_CALLBACK_T	callback,
 				void *			through,
-				int			position )
+				int			col,
+				int			colspan )
     {
-    GtkWidget *	toggle= gtk_check_button_new_with_label( text );
+    GtkWidget *	toggle= gtk_check_button_new_with_label( (char *)text );
 
     gtk_table_attach( GTK_TABLE( row ),
 			toggle,
-			position, position+ 1,
+			col, col+ colspan,
 			0, 1,
 			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
@@ -292,59 +318,6 @@ APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
     return toggle;
     }
 
-/************************************************************************/
-/*									*/
-/*  Make a row in a column: Rows are implemented as..			*/
-/*									*/
-/************************************************************************/
-
-APP_WIDGET appMakeRowInColumn(	APP_WIDGET	column,
-				int		columnCount,
-				int		heightResizable )
-    {
-    gboolean	expand= FALSE;
-    gboolean	fill= TRUE;
-
-    GtkWidget *	row= gtk_table_new( 1, columnCount, TRUE );
-
-    if  ( heightResizable )
-	{ expand= TRUE;	}
-
-    gtk_box_pack_start( GTK_BOX( column ), row, expand, fill, 0 );
-
-    gtk_widget_show( row );
-
-    return row;
-    }
-
-void appMakeButtonInRow(	APP_WIDGET *		pButton,
-				APP_WIDGET		row,
-				const char *		text,
-				APP_BUTTON_CALLBACK_T	callback,
-				void *			through,
-				int			position,
-				int			showAsDefault )
-    {
-    GtkWidget *		button= gtk_button_new_with_label( text );
-
-    gtk_table_attach( GTK_TABLE( row ),
-			button,
-			position, position+ 1,
-			0, 1,
-			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			ROW_XPADDING_GTK, ROW_YPADDING_GTK );
-
-    gtk_widget_show( button );
-
-    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
-					(GtkSignalFunc)callback, through );
-
-    *pButton= button;
-
-    return;
-    }
-
 void appGuiSetScrollbarValues(		APP_WIDGET	scrollbar,
 					int		value,
 					int		sliderSize )
@@ -375,16 +348,11 @@ void appGuiGetScrollbarValues(		int *		pValue,
     return;
     }
 
-void appGuiChangeButtonText(	APP_WIDGET	button,
-				const char *	label )
-    {
-    if  ( ! appGuiGtkSetChildLabel( button, label ) )
-	{ SDEB(label);	}
-    }
-
-void appShowShellWidget(		APP_WIDGET		shell )
+void appShowShellWidget(		EditApplication *	ea,
+					APP_WIDGET		shell )
     {
     gtk_widget_show( shell );
+    gtk_window_present( GTK_WINDOW( shell ) );
     gdk_window_raise( shell->window );
     }
 
@@ -418,26 +386,6 @@ void appGuiInsertSeparatorInColumn(	APP_WIDGET *	pSeparator,
     return;
     }
 
-void appMakeColumnInRow(	APP_WIDGET *	pColumn,
-				APP_WIDGET	row,
-				int		position,
-				int		colspan )
-    {
-    GtkWidget *		column= gtk_vbox_new( FALSE, COLUMN_SPACING_GTK );
-
-    gtk_table_attach( GTK_TABLE( row ),
-			column,
-			position, position+ colspan,
-			0, 1,
-			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			ROW_XPADDING_GTK, ROW_YPADDING_GTK );
-
-    gtk_widget_show( column );
-
-    *pColumn= column;
-    }
-
 void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 				    APP_WIDGET			column,
 				    int				wide,
@@ -566,6 +514,19 @@ void appMakeColumnFrameInColumn(	APP_WIDGET *		pFrame,
     *pFrame= frame; *pPaned= paned; return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Set the title of a frame.						*/
+/*									*/
+/************************************************************************/
+
+int appGuiSetFrameTitle(		APP_WIDGET		frame,
+					const char *		title )
+    {
+    gtk_frame_set_label( GTK_FRAME( frame ), (const char *)title );
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Insert a frame in a column. The frame is to be a row itself.	*/
@@ -589,8 +550,7 @@ void appMakeRowFrameInColumn(	APP_WIDGET *		pFrame,
     *pFrame= frame; *pRow= row; return;
     }
 
-int appGuiGetScrollbarValueFromCallback( APP_WIDGET		scrollbar,
-					void *			voidcbs )
+int appGuiGetScrollbarValueFromCallbackGtk( APP_WIDGET		scrollbar )
     {
     GtkAdjustment *	adjustment= (GtkAdjustment *)scrollbar;
 
@@ -630,23 +590,10 @@ typedef struct DragLoop
     guint			dlUpId;
     guint			dlDownId;
     guint			dlMoveId;
+
+    GtkWidget *			dlWidget;
     } DragLoop;
 
-static int appDragTick(	void *		vdl )
-    {
-    DragLoop *		dl= (DragLoop *)vdl;
-
-    (*dl->dlTimerHandler)( dl->dlThrough );
-
-    if  ( dl->dlTimerHandler && dl->dlTimerInterval > 0 )
-	{
-	dl->dlIntervalId= gtk_timeout_add( dl->dlTimerInterval,
-						    appDragTick, (void *)dl );
-	}
-
-    return 0;
-    }
-
 static void appDragMouseUp(		APP_WIDGET		w,
 					APP_EVENT *		event,
 					void *			vdl )
@@ -659,7 +606,7 @@ static void appDragMouseUp(		APP_WIDGET		w,
 	dl->dlIntervalId= 0;
 	}
 
-    if  ( dl->dlUpHandler )
+    if  ( event && dl->dlUpHandler )
 	{ (*dl->dlUpHandler)( w, event, dl->dlThrough );	}
 
     gtk_main_quit();
@@ -671,10 +618,46 @@ static void appDragMouseMove(		APP_WIDGET		w,
     {
     DragLoop *		dl= (DragLoop *)vdl;
 
+    if  ( ! ( event->motion.state & GDK_BUTTON1_MASK ) )
+	{
+	/* really happens: XXDEB(event->motion.state,GDK_BUTTON1_MASK); */
+	appDragMouseUp( w, event, vdl );
+	return;
+	}
+
     if  ( dl->dlMoveHandler )
 	{ (*dl->dlMoveHandler)( w, event, dl->dlThrough );	}
     }
 
+static int appDragTick(	void *		vdl )
+    {
+    DragLoop *		dl= (DragLoop *)vdl;
+
+    gint		winX;
+    gint		winY;
+    GdkModifierType	mask= 0;
+
+    gdk_window_get_pointer( dl->dlWidget->window, &winX, &winY, &mask );
+
+    if  ( ! ( mask & GDK_BUTTON1_MASK ) )
+	{
+	/* really happens: XXDEB(mask,GDK_BUTTON1_MASK); */
+	appDragMouseUp( dl->dlWidget, (APP_EVENT *)0, vdl );
+	return 0;
+	}
+
+    if  ( dl->dlTimerHandler )
+	{ (*dl->dlTimerHandler)( dl->dlThrough );	}
+
+    if  ( dl->dlTimerInterval > 0 )
+	{
+	dl->dlIntervalId= gtk_timeout_add( dl->dlTimerInterval,
+						    appDragTick, (void *)dl );
+	}
+
+    return 0;
+    }
+
 void appRunDragLoop(	APP_WIDGET		w,
 			EditApplication *	ea,
 			APP_EVENT *		downEvent,
@@ -693,6 +676,10 @@ void appRunDragLoop(	APP_WIDGET		w,
     dl.dlThrough= through;
 
     dl.dlIntervalId= 0;
+    dl.dlWidget= w;
+
+    if  ( dl.dlTimerInterval == 0 )
+	{ dl.dlTimerInterval= 200;	}
 
     gtk_widget_add_events( w,
 			GDK_BUTTON_RELEASE_MASK | GDK_BUTTON1_MOTION_MASK );
@@ -710,7 +697,7 @@ void appRunDragLoop(	APP_WIDGET		w,
 					    (GtkSignalFunc)appDragMouseMove,
 					    (void *)&dl );
 
-    if  ( dl.dlTimerHandler && dl.dlTimerInterval > 0 )
+    if  ( dl.dlTimerInterval > 0 )
 	{
 	dl.dlIntervalId= gtk_timeout_add( dl.dlTimerInterval,
 						appDragTick, (void *)&dl );
@@ -739,33 +726,6 @@ void appRunDragLoop(	APP_WIDGET		w,
     return;
     }
 
-void appGuiGetStringFromKeyboardEvent(	APP_INPUT_CONTEXT	ic,
-					APP_WIDGET		w,
-					APP_EVENT *		event,
-					int *			pGotString,
-					int *			pGotKey,
-					unsigned int *		pState,
-					char *			buf,
-					int			capacity,
-					APP_KEY_VALUE *		pKey )
-    {
-    GdkEventKey *	keyEvent= &(event->key);
-    int			got;
-
-    got= keyEvent->length;
-    if  ( got > capacity )
-	{ got=  capacity;	}
-
-    strncpy( buf, keyEvent->string, got );
-
-    *pGotString= got;
-    *pGotKey= 1;
-    *pState= keyEvent->state;
-    *pKey= keyEvent->keyval;
-
-    return;
-    }
-
 void appGuiSetFocusChangeHandler(	APP_WIDGET		w,
 					APP_EVENT_HANDLER_T	handler,
 					void *			through )
@@ -786,8 +746,14 @@ void appSetDestroyCallback(	APP_WIDGET		shell,
     {
     if  ( destroyCallback )
 	{
+	/*
 	gtk_signal_connect( GTK_OBJECT( shell ), "destroy_event",
 				    (GtkSignalFunc)destroyCallback, through );
+	gtk_signal_connect( GTK_OBJECT( shell ), "destroy-event",
+				    (GtkSignalFunc)destroyCallback, through );
+	*/
+	gtk_signal_connect( GTK_OBJECT( shell ), "destroy",
+				    (GtkSignalFunc)destroyCallback, through );
 	}
     }
 
@@ -863,4 +829,9 @@ int appGuiGtkGetChildLabel(	char **			pLabel,
     return rval;
     }
 
+void appGuiFocusToWidget(	APP_WIDGET		w )
+    {
+    gtk_widget_grab_focus( w );
+    }
+
 #   endif
