@@ -2,11 +2,12 @@
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
-#   include	<appDebugon.h>
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
-#   include	<appGeoString.h>
+#   include	<geoString.h>
+
+#   include	<appDebugon.h>
 
 /************************************************************************/
 /*									*/
@@ -47,8 +48,8 @@ void appMakeToggleAndTextRow(	APP_WIDGET *		pRow,
 				APP_WIDGET *		pToggle,
 				APP_WIDGET *		pText,
 				APP_WIDGET		column,
-				char *			labelText,
-				APP_TOGGLE_CALLBACK	callback,
+				const char *		labelText,
+				APP_TOGGLE_CALLBACK_T	callback,
 				void *			through,
 				int			textColumns,
 				int			textEnabled )
@@ -65,10 +66,9 @@ void appMakeToggleAndTextRow(	APP_WIDGET *		pRow,
 
     row= appMakeRowInColumn( column, columnCount, heightResizable );
 
-    toggle= appMakeToggleInRow( row, labelText, callback, through, 0 );
+    toggle= appMakeToggleInRow( row, labelText, callback, through, 0, 1 );
     appMakeTextInRow( &text, row, textColumn, textColspan,
 						    textColumns, textEnabled );
 
     *pRow= row; *pToggle= toggle; *pText= text; return;
     }
-
