@@ -2,7 +2,7 @@
 /*  Scan, main module.							*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdio.h>
@@ -13,32 +13,6 @@
 
 #   include	<appDebugon.h>
 
-/************************************************************************/
-/*									*/
-/*  Clean an image structure.						*/
-/*									*/
-/************************************************************************/
-
-void appCleanBitmapImage(	AppBitmapImage *	abi )
-    {
-    if  ( abi->abiBuffer )
-	{
-	free( abi->abiBuffer );
-	abi->abiBuffer= (unsigned char *)0;
-	}
-
-    bmCleanDescription( &abi->abiBitmap );
-    bmInitDescription( &abi->abiBitmap );
-    }
-
-void appInitBitmapImage(	AppBitmapImage *	abi )
-    {
-    bmInitDescription( &abi->abiBitmap );
-    abi->abiBuffer= (unsigned char *)0;
-    abi->abiFormat= -1;
-    abi->abiFactor= 10;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Make the list of bitmap FileExtensions that can be opened/saved.	*/
@@ -48,12 +22,12 @@ void appInitBitmapImage(	AppBitmapImage *	abi )
 int appImgMakeFileExtensions(	AppFileExtension **	pAfeList,
 				int *			pAfeCount )
     {
-    int			i;
-    AppFileExtension *	afe;
-    int			afeCount;
+    int				i;
+    AppFileExtension *		afe;
+    int				afeCount;
 
-    BitmapFileFormat *	bff;
-    BitmapFileType **	pbft;
+    const BitmapFileFormat *	bff;
+    BitmapFileType **		pbft;
 
     afeCount=  bmNumberOfFileTypes+ bmNumberOfFileFormats;
     afe= (AppFileExtension *)malloc( afeCount * sizeof(AppFileExtension) );
