@@ -4,27 +4,55 @@
 #   include	<stdio.h>
 
 #   include	"appFrame.h"
-#   include	"appSystem.h"
-#   include	<appGeoString.h>
-
-#   ifdef USE_GTK
 
 #   include	<appDebugon.h>
 
+#   ifdef USE_GTK
+
 /************************************************************************/
 /*									*/
 /*  List specific functionality.					*/
 /*									*/
 /************************************************************************/
 
+typedef struct ListAction
+    {
+    APP_WIDGET			laList;
+    APP_BUTTON_CALLBACK_T	laActionCallback;
+    void *			laThrough;
+    } ListAction;
+
+static const char	APP_ListAction[]= "ListAction";
+
+static void appGuiGtkFreeListAction( void * voidla )
+    { free( voidla );	}
+
+static gboolean appGuiGtkHandlePotentialListAction(
+					GtkWidget *		item,
+					GdkEventButton *	e,
+					gpointer		voidla )
+    {
+    ListAction *	la= (ListAction *)voidla;
+
+    if  ( la && e->type == GDK_2BUTTON_PRESS )
+	{
+	(*la->laActionCallback)( la->laList, la->laThrough );
+	return TRUE;
+	}
+
+    return FALSE;
+    }
+
 void appGuiMakeListInColumn(	APP_WIDGET *		pList,
 				APP_WIDGET		column,
 				int			visibleItems,
 				APP_LIST_CALLBACK_T	callback,
+				APP_BUTTON_CALLBACK_T	actionCallback,
 				void *			through )
     {
     APP_WIDGET		list;
     APP_WIDGET		scrolled_window;
+    GtkAdjustment *	vadjust;
 
     gboolean		expand= TRUE;
     gboolean		fill= TRUE;
@@ -34,14 +62,19 @@ void appGuiMakeListInColumn(	APP_WIDGET *		pList,
 
     gtk_box_pack_start( GTK_BOX( column ), scrolled_window, expand, fill, 0 );
 
-    gtk_widget_show( scrolled_window );
-
     gtk_scrolled_window_set_placement( GTK_SCROLLED_WINDOW( scrolled_window ),
 						    GTK_CORNER_BOTTOM_LEFT );
     gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolled_window ),
 						    GTK_POLICY_NEVER,
 						    GTK_POLICY_AUTOMATIC );
 
+    vadjust= gtk_scrolled_window_get_vadjustment(
+				    GTK_SCROLLED_WINDOW( scrolled_window ) );
+    gtk_container_set_focus_vadjustment(
+				    GTK_CONTAINER( scrolled_window ), vadjust );
+
+    gtk_widget_show( scrolled_window );
+
     if  ( visibleItems > 0 )
 	{
 	GtkStyle *	gs= gtk_widget_get_style( scrolled_window );
@@ -53,8 +86,13 @@ void appGuiMakeListInColumn(	APP_WIDGET *		pList,
 	gf= gtk_style_get_font( gs );
 #	endif
 
+#	if GTK_MAJOR_VERSION < 2
 	gtk_widget_set_usize( scrolled_window, -1, ( 110* visibleItems*
 					( gf->ascent+ gf->descent ) )/ 100 );
+#	else
+	gtk_widget_set_size_request( scrolled_window, 0, ( 110* visibleItems*
+					( gf->ascent+ gf->descent ) )/ 100 );
+#	endif
 	}
 
     list= gtk_list_new();
@@ -70,15 +108,61 @@ void appGuiMakeListInColumn(	APP_WIDGET *		pList,
 
     if  ( callback )
 	{
-	gtk_signal_connect( GTK_OBJECT( list ), "select_child",
+	gtk_signal_connect( GTK_OBJECT( list ), "select-child",
 					    (GtkSignalFunc)callback, through );
 	}
 
+    if  ( actionCallback )
+	{
+	ListAction *	la= malloc( sizeof(ListAction) );
+
+	la->laList= list;
+	la->laThrough= through;
+	la->laActionCallback= actionCallback;
+
+	if  ( la )
+	    {
+	    gtk_object_set_data_full( GTK_OBJECT(list), APP_ListAction, la,
+						    appGuiGtkFreeListAction );
+	    }
+	}
+
     gtk_widget_show( list );
 
     *pList= list; return;
     }
 
+static GtkWidget * appGuiGetListChild(		APP_WIDGET	list,
+						int *		pCount,
+						int		position )
+    {
+    GList *		glf;
+    GList *		gl;
+
+    int		n= 0;
+    GtkWidget *	item= (GtkWidget *)0;
+
+    gl= glf= gtk_container_children( GTK_CONTAINER( list ) );
+
+    while( gl )
+	{
+	if  ( n == position )
+	    {
+	    item= (GtkWidget *)gl->data;
+	    if  ( ! pCount )
+		{ break;	}
+	    }
+	n++; gl= gl->next;
+	}
+
+    if  ( glf )
+	{ g_list_free( glf );	}
+
+    if  ( pCount )
+	{ *pCount= n;	}
+    return item;
+    }
+
 void appGuiEmptyListWidget(	APP_WIDGET		list )
     {
     gtk_list_unselect_all( GTK_LIST( list ) );
@@ -87,18 +171,62 @@ void appGuiEmptyListWidget(	APP_WIDGET		list )
     return;
     }
 
-void appGuiAddValueToListWidget(	APP_WIDGET	list,
+void appGuiDeleteValueFromListWidget(	APP_WIDGET	list,
+					int		pos )
+    {
+    gtk_list_clear_items( GTK_LIST( list ), pos, pos+ 1 );
+    }
+
+void appGuiReplaceValueInListWidget(	APP_WIDGET	list,
+					int		pos,
 					const char *	value )
     {
-    GList *	glist= (GList *)0;
-    GtkWidget *	item= gtk_list_item_new_with_label( value );
+    GtkWidget *	item= appGuiGetListChild( list, (int *)0, pos );
+
+    if  ( item )
+	{
+#	if GTK_MAJOR_VERSION >= 2
+	GtkWidget *	labelw= gtk_bin_get_child( GTK_BIN( item ) );
+#	else
+	GtkWidget *	labelw= GTK_BIN( item )->child;
+#	endif
+
+	gtk_label_set_text( GTK_LABEL( labelw ), value );
+	}
+
+    return;
+    }
+
+void appGuiAddValueToListWidget(	APP_WIDGET		list,
+					int			pos,
+					const char *		value )
+    {
+    GList *		glist= (GList *)0;
+    GtkWidget *		item= gtk_list_item_new_with_label( value );
+    ListAction *	la;
 
     glist= g_list_append( glist, item );
 
-    gtk_list_append_items( GTK_LIST( list ), glist );
+    if  ( pos < 0 )
+	{
+	gtk_list_append_items( GTK_LIST( list ), glist );
+	}
+    else{
+	gtk_list_insert_items( GTK_LIST( list ), glist, pos );
+	}
 
     gtk_widget_show( item );
 
+#   if GTK_MAJOR_VERSION >= 2
+    la= (ListAction *)gtk_object_get_data( GTK_OBJECT( list ), APP_ListAction );
+    if  ( la )
+	{
+	gtk_signal_connect( GTK_OBJECT(item),
+		     "button-press-event",
+		      G_CALLBACK( appGuiGtkHandlePotentialListAction ), la );
+	}
+#   endif
+
     /*  No:
     g_list_free( glist );
     */
@@ -118,33 +246,11 @@ void appGuiSelectPositionInListWidget(		APP_WIDGET	list,
     if  ( ! scrolled_window )
 	{ XDEB(scrolled_window); return;	}
     else{
-	GtkAdjustment *	ga;
+	int		count= 1;
+	GtkWidget *	item= appGuiGetListChild( list, &count, position );
 
-	GList *		glf;
-	GList *		gl;
-
-	int		childCount= 0;
-	GtkWidget *	selectedItem= (GtkWidget *)0;
-
-	ga= gtk_scrolled_window_get_vadjustment(
-				    GTK_SCROLLED_WINDOW( scrolled_window ) );
-
-	gl= glf= gtk_container_children( GTK_CONTAINER( list ) );
-
-	while( gl )
-	    {
-	    if  ( childCount == position )
-		{ selectedItem= (GtkWidget *)gl->data; }
-	    childCount++; gl= gl->next;
-	    }
-
-	if  ( glf )
-	    { g_list_free( glf );	}
-
-	if  ( ! selectedItem )
-	    { return;	}
-
-	gtk_container_set_focus_child( GTK_CONTAINER( list ), selectedItem );
+	gtk_container_set_focus_child( GTK_CONTAINER( list ), item );
+	gtk_container_set_focus_child( GTK_CONTAINER( scrolled_window ), item );
 	}
 
     return;
@@ -157,7 +263,7 @@ void appGuiRemoveSelectionFromListWidget(	APP_WIDGET	list )
     return;
     }
 
-int appGuiGetPositionFromListCallback(		APP_LIST	list,
+int appGuiGetPositionFromListCallback(		APP_WIDGET	list,
 						void *		voiditem )
     {
     GtkWidget *	item= (GtkWidget *)voiditem;
@@ -166,7 +272,7 @@ int appGuiGetPositionFromListCallback(		APP_LIST	list,
     return gtk_list_child_position( GTK_LIST( list ), item );
     }
 
-char * appGuiGetStringFromListCallback(	APP_LIST		list,
+char * appGuiGetStringFromListCallback(	APP_WIDGET		list,
 					APP_LIST_CHOICE *	item )
     {
     static char	slabel[]= "<< ? >>";
@@ -179,6 +285,6 @@ char * appGuiGetStringFromListCallback(	APP_LIST		list,
     }
 
 void appFreeStringFromListCallback(	char *		s )
-    { /* first implement appGuiGetStringFromListCallback(); */ return;	}
+    { return;	}
 
 #   endif
