@@ -18,46 +18,9 @@
 #   include	<appFrame.h>
 #   include	"appUnit.h"
 #   include	"appPaper.h"
-#   include	"appGeoString.h"
+#   include	"geoString.h"
 #   include	<appDebugon.h>
 
-/************************************************************************/
-/*									*/
-/*  Hide the application window if shown as an 'about' window.		*/
-/*									*/
-/************************************************************************/
-
-void appAppFileHide(	APP_WIDGET	option,
-			void *		voidea,
-			void *		call_data )
-    {
-    EditApplication *		ea= (EditApplication *)voidea;
-
-    ea->eaMainVisibleAsAbout= 0;
-
-    if  ( ea->eaVisibleDocumentCount > 0 )
-	{ appHideShellWidget( ea->eaToplevel.atTopWidget ); }
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Minimise the application.						*/
-/*									*/
-/************************************************************************/
-
-void appAppFileMini(	APP_WIDGET	option,
-			void *		voidea,
-			void *		call_data )
-    {
-    EditApplication *		ea= (EditApplication *)voidea;
-
-    appIconifyShellWidget( ea->eaToplevel.atTopWidget );
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Callback that is invoked when the application is closed through	*/
@@ -176,9 +139,7 @@ void appQuitApplication(	APP_WIDGET		option,
     return;
     }
 
-void appAppFileQuit(	APP_WIDGET		option,
-			void *			voidea,
-			void *			call_data )
+APP_MENU_CALLBACK_H( appAppFileQuit, option, voidea, e )
     {
     EditApplication *	ea= (EditApplication *)voidea;
 
@@ -191,9 +152,7 @@ void appAppFileQuit(	APP_WIDGET		option,
 /*									*/
 /************************************************************************/
 
-void appAppFileNew(	APP_WIDGET		option,
-			void *			voidea,
-			void *			call_data )
+APP_MENU_CALLBACK_H( appAppFileNew, option, voidea, e )
     {
     EditApplication *		ea= (EditApplication *)voidea;
     const char *		filename= (const char *)0;
@@ -278,7 +237,7 @@ void appDocVisible(	EditApplication *	ea,
     else{ from= ea->eaVisibleDocumentCount--;	}
 
     if  ( ea->eaVisibleDocumentCount == 0 )
-	{ appShowShellWidget( ea->eaToplevel.atTopWidget ); }
+	{ appShowShellWidget( ea, ea->eaToplevel.atTopWidget ); }
     else{
 	if  ( ! ea->eaMainVisibleAsAbout )
 	    { appHideShellWidget( ea->eaToplevel.atTopWidget );	}
@@ -368,7 +327,7 @@ static void appAppSetWindowsOption(	APP_WIDGET		menu,
     ami.amiItemText= label;
     ami.amiKey= (char *)0;
     ami.amiKeyText= (char *)0;
-    ami.amiCallback= (APP_MENU_CALLBACK)appDocToFront;
+    ami.amiCallback= (APP_MENU_CALLBACK_T)appDocToFront;
 
     windowsOption= appSetToggleMenuItem( menu, &(ed->edToplevel),
 							&ami, (void *)ed );
@@ -438,11 +397,7 @@ static AppConfigurableResource	APP_ApplicationResourceTable[]=
 		"72pt" ),
     APP_RESOURCE( "magnification",
 		offsetof(EditApplication,eaMagnificationString),
-		"1.2" ),
-
-    APP_RESOURCE( "supportXvCopyPaste",
-		offsetof(EditApplication,eaSupportXvCopyPasteString),
-		(char *)0 ),
+		"120%" ),
 
     APP_RESOURCE( "hideSaveToOption",
 		offsetof(EditApplication,eaHideSaveToOptionString),
@@ -451,22 +406,46 @@ static AppConfigurableResource	APP_ApplicationResourceTable[]=
     APP_RESOURCE( "usePostScriptFilters",
 		offsetof(EditApplication,eaUsePostScriptFiltersString),
 		(char *)0 ),
-
     APP_RESOURCE( "usePostScriptIndexedImages",
 		offsetof(EditApplication,eaUsePostScriptIndexedImagesString),
 		(char *)0 ),
+    APP_RESOURCE( "sevenBitsPostScript",
+		offsetof(EditApplication,ea7BitsPostScriptString),
+		(char *)0 ),
+
+    APP_RESOURCE( "skipEmptyPages",
+		offsetof(EditApplication,eaSkipEmptyPagesString),
+		(char *)0 ),
+    APP_RESOURCE( "skipBlankPages",
+		offsetof(EditApplication,eaSkipBlankPagesString),
+		(char *)0 ),
+    APP_RESOURCE( "omitHeadersOnEmptyPages",
+		offsetof(EditApplication,eaOmitHeadersOnEmptyPagesString),
+		(char *)0 ),
+    APP_RESOURCE( "customPsSetupFilename",
+		offsetof(EditApplication,eaCustomPsSetupFilename),
+		(char *)0 ),
+
+    APP_RESOURCE( "avoidFontconfig",
+		offsetof(EditApplication,eaAvoidFontconfigString),
+		(char *)0 ),
+    APP_RESOURCE( "preferBase35Fonts",
+		offsetof(EditApplication,eaPreferBase35FontsString),
+		(char *)0 ),
+    APP_RESOURCE( "embedFonts",
+		offsetof(EditApplication,eaEmbedFontsString),
+		"1" ),
 
     APP_RESOURCE( "author",
 		offsetof(EditApplication,eaAuthor),
 		(char *)0 ),
 
-    APP_RESOURCE( "pageNumberFormat",
-		offsetof(EditApplication,eaPageNumberFormat),
-		"Page %d" ),
-
     APP_RESOURCE( "afmDirectory",
 		offsetof(EditApplication,eaAfmDirectory),
 		AFMDIR ),
+    APP_RESOURCE( "fontDirectory",
+		offsetof(EditApplication,eaFontDirectory),
+		(char *)0 ),
     APP_RESOURCE( "ghostscriptFontmap",
 		offsetof(EditApplication,eaGhostscriptFontmap),
 		(char *)0 ),
@@ -476,15 +455,18 @@ static AppConfigurableResource	APP_ApplicationResourceTable[]=
     APP_RESOURCE( "defaultFont",
 		offsetof(EditApplication,eaDefaultFont),
 		"Helvetica,,,10" ),
-    APP_RESOURCE( "faxCommand",
-		offsetof(EditApplication,eaFaxCommand),
-		(char *)0 ),
     APP_RESOURCE( "customPrintCommand",
 		offsetof(EditApplication,eaCustomPrintCommand),
 		(char *)0 ),
     APP_RESOURCE( "customPrinterName",
 		offsetof(EditApplication,eaCustomPrinterName),
 		(char *)0 ),
+    APP_RESOURCE( "customPrintCommand2",
+		offsetof(EditApplication,eaCustomPrintCommand2),
+		(char *)0 ),
+    APP_RESOURCE( "customPrinterName2",
+		offsetof(EditApplication,eaCustomPrinterName2),
+		(char *)0 ),
 
     APP_RESOURCE( "rulerFont",
 		offsetof(EditApplication,eaRulerFont),
@@ -511,10 +493,21 @@ static void appInitializeGeometrySettings(	EditApplication *	ea )
     ea->eaMagnification= 1.2;
     if  ( ea->eaMagnificationString )
 	{
-	ea->eaMagnification= atof( ea->eaMagnificationString );
-	if  ( ea->eaMagnification < 0.1 )
+	double	mag= 0.0;
+	char	c;
+	int	res;
+
+	res= sscanf( ea->eaMagnificationString, "%lg%c", &mag, &c );
+	if  ( res == 1 )
+	    { ea->eaMagnification= mag;		}
+	if  ( res == 2 && c == '%' )
+	    { ea->eaMagnification= mag/100.0;	}
+
+	if  ( mag < 0.1				||
+	      ea->eaMagnification < 0.1		||
+	      ea->eaMagnification > 10.0	)
 	    {
-	    SFDEB(ea->eaMagnificationString,ea->eaMagnification);
+	    SFDEB(ea->eaMagnificationString,mag);
 	    ea->eaMagnification= 1.2;
 	    }
 	}
@@ -527,37 +520,32 @@ static void appInitializeGeometrySettings(	EditApplication *	ea )
 
     if  ( ea->eaLeftMarginString )
 	{
-	if  ( appGeoLengthFromString( ea->eaLeftMarginString, ea->eaUnitInt,
+	if  ( geoLengthFromString( ea->eaLeftMarginString, ea->eaUnitInt,
 			&ea->eaDefaultDocumentGeometry.dgLeftMarginTwips ) )
 	    { SDEB(ea->eaLeftMarginString);	}
 	}
 
     if  ( ea->eaRightMarginString )
 	{
-	if  ( appGeoLengthFromString( ea->eaRightMarginString, ea->eaUnitInt,
+	if  ( geoLengthFromString( ea->eaRightMarginString, ea->eaUnitInt,
 			&ea->eaDefaultDocumentGeometry.dgRightMarginTwips ) )
 	    { SDEB(ea->eaRightMarginString);	}
 	}
 
     if  ( ea->eaTopMarginString )
 	{
-	if  ( appGeoLengthFromString( ea->eaTopMarginString, ea->eaUnitInt,
+	if  ( geoLengthFromString( ea->eaTopMarginString, ea->eaUnitInt,
 			&ea->eaDefaultDocumentGeometry.dgTopMarginTwips ) )
 	    { SDEB(ea->eaTopMarginString);	}
 	}
 
     if  ( ea->eaBottomMarginString )
 	{
-	if  ( appGeoLengthFromString( ea->eaBottomMarginString, ea->eaUnitInt,
+	if  ( geoLengthFromString( ea->eaBottomMarginString, ea->eaUnitInt,
 			&ea->eaDefaultDocumentGeometry.dgBottomMarginTwips ) )
 	    { SDEB(ea->eaBottomMarginString);	}
 	}
 
-    ea->eaSupportXvCopyPaste= 0;
-    if  ( ea->eaSupportXvCopyPasteString			&&
-	  ! strcmp( ea->eaSupportXvCopyPasteString, "1" )	)
-	{ ea->eaSupportXvCopyPaste= 1;	}
-
     ea->eaHideSaveToOption= 0;
     if  ( ea->eaHideSaveToOptionString			&&
 	  ! strcmp( ea->eaHideSaveToOptionString, "1" )	)
@@ -573,6 +561,50 @@ static void appInitializeGeometrySettings(	EditApplication *	ea )
 	  ! strcmp( ea->eaUsePostScriptIndexedImagesString, "1" )	)
 	{ ea->eaUsePostScriptIndexedImages= 1;	}
 
+    ea->ea7BitsPostScript= 0;
+    if  ( ea->ea7BitsPostScriptString			&&
+	  ! strcmp( ea->ea7BitsPostScriptString, "1" )	)
+	{ ea->ea7BitsPostScript= 1;	}
+
+    ea->eaSkipEmptyPages= 0;
+    if  ( ea->eaSkipEmptyPagesString			&&
+	  ! strcmp( ea->eaSkipEmptyPagesString, "1" )	)
+	{ ea->eaSkipEmptyPages= 1;	}
+
+    ea->eaSkipBlankPages= 0;
+    if  ( ea->eaSkipBlankPagesString			&&
+	  ! strcmp( ea->eaSkipBlankPagesString, "1" )	)
+	{ ea->eaSkipBlankPages= 1;	}
+
+    ea->eaOmitHeadersOnEmptyPages= 0;
+    if  ( ea->eaOmitHeadersOnEmptyPagesString			&&
+	  ! strcmp( ea->eaOmitHeadersOnEmptyPagesString, "1" )	)
+	{ ea->eaOmitHeadersOnEmptyPages= 1;	}
+
+    ea->eaAvoidFontconfig= 0;
+    if  ( ea->eaAvoidFontconfigString			&&
+	  ! strcmp( ea->eaAvoidFontconfigString, "1" )	)
+	{ ea->eaAvoidFontconfig= 1;	}
+
+    ea->eaPreferBase35Fonts= 0;
+    if  ( ea->eaPreferBase35FontsString			&&
+	  ! strcmp( ea->eaPreferBase35FontsString, "1" )	)
+	{ ea->eaPreferBase35Fonts= 1;	}
+
+    /*  3  */
+    if  ( ea->eaEmbedFontsString		&&
+	  ea->eaEmbedFonts == 0			)
+	{
+	if  ( ! strcmp( ea->eaEmbedFontsString, "0" ) )
+	    { ea->eaEmbedFonts= -1;	}
+	if  ( ! strcmp( ea->eaEmbedFontsString, "1" ) )
+	    { ea->eaEmbedFonts=  1;	}
+
+	if  ( ea->eaEmbedFonts == 0 )
+	    { SDEB(ea->eaEmbedFontsString);	}
+	}
+
+
     if  ( ea->eaToplevel.atTopWidget )
 	{
 	double			horPixPerMM;
@@ -586,6 +618,52 @@ static void appInitializeGeometrySettings(	EditApplication *	ea )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Resolve application resources.					*/
+/*									*/
+/*  9)  Theoretically a program could have more than one application	*/
+/*	object. This has never beem tested. The use of a single		*/
+/*	table and different flags to check for reuse here is one of the	*/
+/*	things to fix.							*/
+/*									*/
+/************************************************************************/
+
+void appGetApplicationResourceValues(	EditApplication *	ea )
+    {
+    /*  b  */
+    if  ( ! ea->eaGotResourceTable )
+	{
+	appGuiGetResourceValues( &(ea->eaGotResourceTable), ea,
+					    ea->eaResourceData,
+					    ea->eaResourceTable,
+					    ea->eaResourceCount );
+	}
+
+    if  ( ! ea->eaGotFileMessageResourceTable )
+	{
+	appGuiGetResourceValues( &(ea->eaGotFileMessageResourceTable), ea,
+					&(ea->eaFileMessageResources),
+					ea->eaFileMessageResourceTable,
+					ea->eaFileMessageResourceCount );
+	}
+
+    /*  9  */
+    if  ( ! ea->eaGotApplicationResources )
+	{
+	appGuiGetResourceValues( &(ea->eaGotApplicationResources), ea,
+				(void *)ea,
+				APP_ApplicationResourceTable,
+				sizeof(APP_ApplicationResourceTable)/
+				sizeof(AppConfigurableResource) );
+	}
+
+    appInitializeGeometrySettings( ea );
+
+    return;
+    }
+
+
 /************************************************************************/
 /*									*/
 /*  Fill the application main widget.					*/
@@ -666,8 +744,31 @@ static int appFinishApplicationWindow(	EditApplication *	ea )
 
     if  ( ea->eaNameAndVersion )
 	{
-	appMakeLabelInColumn( &label, ea->eaMainWindow,
-						    ea->eaNameAndVersion );
+	char *		ident= (char *)0;
+
+
+	if  ( ea->eaPlatformCompiled )
+	    {
+	    ident= malloc( strlen( ea->eaNameAndVersion )+ 3+
+					strlen( ea->eaPlatformCompiled )+ 1 );
+	    if  ( ! ident )
+		{ XDEB(ident); }
+	    else{
+		sprintf( ident, "%s - %s",
+				ea->eaNameAndVersion, ea->eaPlatformCompiled );
+		}
+	    }
+	else{
+	    ident= strdup( ea->eaNameAndVersion );
+	    }
+
+	if  ( ! ident )
+	    { XDEB(ident);	}
+	else{
+	    appMakeLabelInColumn( &label, ea->eaMainWindow, ident );
+
+	    free( ident );
+	    }
 	}
 
     if  ( ea->eaReference )
@@ -698,154 +799,297 @@ static int appFinishApplicationWindow(	EditApplication *	ea )
     }
 
 /************************************************************************/
-/*									*/
-/*  Resolve application resources.					*/
-/*									*/
-/************************************************************************/
 
-void appGetApplicationResourceValues(	EditApplication *	ea )
+static int app_version(		EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
     {
-    /*  b  */
-    appGuiGetResourceValues( ea, ea->eaResourceData,
-				ea->eaResourceTable, ea->eaResourceCount );
-
-    appGuiGetResourceValues( ea, &(ea->eaFileMessageResources),
-				ea->eaFileMessageResourceTable,
-				ea->eaFileMessageResourceCount );
-
-    appGuiGetResourceValues( ea, (void *)ea,
-				APP_ApplicationResourceTable,
-				sizeof(APP_ApplicationResourceTable)/
-				sizeof(AppConfigurableResource) );
-
-    appInitializeGeometrySettings( ea );
-
-    return;
+    printf( "%s\n", ea->eaNameAndVersion );
+    return 0;
     }
 
+static int app_platform(	EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    printf( "%s\n", ea->eaPlatformCompiled );
+    return 0;
+    }
+
+static int app_build(		EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    printf( "%s\n", ea->eaHostDateCompiled );
+    return 0;
+    }
+
+static int app_fullVersion(	EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    printf( "%s %s %s %s\n",
+		    ea->eaNameAndVersion,
+		    ea->eaOptionalComponents,
+		    ea->eaPlatformCompiled,
+		    ea->eaHostDateCompiled );
+
+    return 0;
+    }
+
+static int app_saveTo(		EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 2 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appFileConvert( ea, argv[0], argv[1] ) )
+	{ SSSDEB(call,argv[0],argv[0]); return -1; }
+
+    return 2;
+    }
+
+static int app_printToFile(	EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 2 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appPrintToFile( ea, argv[0], argv[1], (const char *)0 ) )
+	{ SSSDEB(call,argv[0],argv[1]); return -1; }
+
+    return 2;
+    }
+
+static int app_printToFilePaper( EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 3 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appPrintToFile( ea, argv[0], argv[1], argv[2] ) )
+	{ SSSDEB(call,argv[0],argv[2]); return -1; }
+
+    return 3;
+    }
+
+static int app_print( 		EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 1 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appPrintToPrinter( ea, argv[0], (const char *)0, (const char *)0 ) )
+	{ SSDEB(call,argv[0]); return -1; }
+
+    return 1;
+    }
+
+static int app_printPaper( 	EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 2 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appPrintToPrinter( ea, argv[0], (const char *)0, argv[1] ) )
+	{ SSDEB(call,argv[0]); return -1; }
+
+    return 2;
+    }
+
+static int app_printToPrinter( 	EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 2 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appPrintToPrinter( ea, argv[0], argv[1], (const char *)0 ) )
+	{ SSDEB(call,argv[0]); return -1; }
+
+    return 2;
+    }
+
+static int app_printToPrinterPaper(
+				EditApplication *		ea,
+				const char *			prog,
+				const char *			call,
+				int				argc,
+				char **				argv )
+    {
+    if  ( argc < 3 )
+	{ SLDEB(call,argc); return -1;	}
+
+    if  ( appPrintToPrinter( ea, argv[0], argv[1], argv[2] ) )
+	{ SSDEB(call,argv[0]); return -1; }
+
+    return 3;
+    }
+
+static const SpecialCall AppSpecialCalls[]=
+{
+    { "version",		app_version,			},
+    { "platform",		app_platform,			},
+    { "build",			app_build,			},
+    { "fullVersion",		app_fullVersion,		},
+    { "saveTo",			app_saveTo,			},
+    { "printToFile",		app_printToFile,		},
+    { "printToFilePaper",	app_printToFilePaper,		},
+    { "print",			app_print,			},
+    { "printPaper",		app_printPaper,			},
+    { "printToPrinter",		app_printToPrinter,		},
+    { "printToPrinterPaper",	app_printToPrinterPaper,	},
+};
+
+static const int AppSpecialCallCount= sizeof(AppSpecialCalls)/sizeof(SpecialCall);
+
 /************************************************************************/
 /*									*/
 /*  Handle Special Calls.						*/
 /*									*/
 /*  1)  It is not necessary to initialize TedApplication to convert	*/
 /*	files. Intercept conversion calls to prevent the program from	*/
-/*	becoming interactive. NOTE that the calls differfrom the calls	*/
+/*	becoming interactive. NOTE that the calls differ from the calls	*/
 /*	that initialize the application: They begin with -- instead of	*/
 /*	++ . To override configurable options, the ++Something calls	*/
 /*	should be used.							*/
 /*									*/
 /************************************************************************/
 
-static int appMainHandleSpecialCall(	EditApplication *	ea,
+static const SpecialCall * appGetSpecialCall(
+				const SpecialCall *	calls,
+				int			callCount,
+				const char *		call )
+    {
+    int		c;
+
+    for ( c= 0; c < callCount; calls++, c++ )
+	{
+	if  ( ! strcmp( call, calls->scCall ) )
+	    { return calls;	}
+	}
+
+    return (SpecialCall *)0;
+    }
+
+static int appMainHandleSpecialCalls(	EditApplication *	ea,
 					const char *		prefix,
 					int			getResources,
+					const char *		prog,
 					int			argc,
 					char *			argv[] )
     {
     int			prefixLength= strlen( prefix );
-    const char *	command;
+    int			done= 0;
 
-    if  ( argc < 2 )
+    if  ( argc < 1 )
 	{ return 0;	}
 
-    if  ( strncmp( argv[1], prefix, prefixLength ) )
-	{ return 0;	}
-
-    command= argv[1]+ prefixLength;
-
-    /*  1  */
-    if  ( argc == 4				&&
-	  ! strcmp( command, "saveTo" )	)
+    while( done < argc )
 	{
+	const char *		call;
+	const SpecialCall *	sc;
+	int			args;
+	int			off;
+
+	if  ( strncmp( argv[done], prefix, prefixLength ) )
+	    { break;	}
+
+	call= argv[done]+ prefixLength;
+	sc= appGetSpecialCall( ea->eaSpecialCalls,
+					    ea->eaSpecialCallCount, call );
+	if  ( ! sc )
+	    {
+	    sc= appGetSpecialCall( AppSpecialCalls,
+					    AppSpecialCallCount, call );
+	    }
+	if  ( ! sc )
+	    { break;	}
+
 	if  ( getResources )
 	    { appGetApplicationResourceValues( ea );	}
 
-	if  ( appFileConvert( ea, argv[2], argv[3] ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return -1; }
-
-	return 4;
+	off= done+ 1;
+	args= (*sc->scExecuteCall)( ea, prog, call, argc- off, argv+ off );
+	if  ( args < 0 )
+	    { SLDEB(argv[1+done],args); return -1;	}
+	done += 1+ args;
 	}
 
-    /**  print to file **/
+    return done;
+    }
 
-    if  ( argc == 4				&&
-	  ! strcmp( command, "printToFile" )	)
+/************************************************************************/
+/*									*/
+/*  Use the contents of /etc/papersize as the default papersize.	*/
+/*									*/
+/************************************************************************/
+
+static void appDefaultPapersize(	EditApplication *	ea )
+    {
+    const char *	sizeFile= "/etc/papersize";
+
+    if  ( ! appTestFileExists( sizeFile ) )
 	{
-	if  ( getResources )
-	    { appGetApplicationResourceValues( ea );	}
+	FILE *	f;
 
-	if  ( appPrintToFile( ea, argv[2], argv[3], (const char *)0 ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return -1; }
+	f= fopen( sizeFile, "r" );
+	if  ( ! f )
+	    { SXDEB(sizeFile,f);	}
+	else{
+	    char	buf[150];
 
-	return 4;
+	    while( fgets( buf, sizeof(buf)-1, f ) )
+		{
+		int		l;
+
+		buf[sizeof(buf)-1]= '\0';
+		l= strlen( buf );
+		if  ( l > 0 && buf[l-1] == '\n' )
+		    { buf[l-1]= '\0'; l--; }
+
+		if  ( l > 0 && buf[0] == '#' )
+		    { continue;	}
+
+		if  ( l > 0 )
+		    {
+		    if  ( appSetSystemProperty( ea, "paper", buf ) )
+			{ SDEB(buf);	}
+
+		    break;
+		    }
+		}
+
+	    fclose( f );
+	    }
 	}
 
-    if  ( argc == 5					&&
-	  ! strcmp( command, "printToFilePaper" )	)
-	{
-	if  ( getResources )
-	    { appGetApplicationResourceValues( ea );	}
-
-	if  ( appPrintToFile( ea, argv[2], argv[3], argv[4] ) )
-	    { SSSDEB(argv[1],argv[2],argv[3]); return -1; }
-
-	return 5;
-	}
-
-    /**  print to printer **/
-
-    if  ( argc == 3			&&
-	  ! strcmp( command, "print" )	)
-	{
-	if  ( getResources )
-	    { appGetApplicationResourceValues( ea );	}
-
-	if  ( appPrintToPrinter( ea, argv[2],
-					(const char *)0, (const char *)0 ) )
-	    { SSDEB(argv[1],argv[2]); return -1; }
-
-	return 3;
-	}
-
-    if  ( argc == 4				&&
-	  ! strcmp( command, "printPaper" )	)
-	{
-	if  ( getResources )
-	    { appGetApplicationResourceValues( ea );	}
-
-	if  ( appPrintToPrinter( ea, argv[2], (const char *)0, argv[3] ) )
-	    { SSDEB(argv[1],argv[2]); return -1; }
-
-	return 4;
-	}
-
-    if  ( argc == 4				&&
-	  ! strcmp( command, "printToPrinter" )	)
-	{
-	if  ( getResources )
-	    { appGetApplicationResourceValues( ea );	}
-
-	if  ( appPrintToPrinter( ea, argv[2], argv[3], (const char *)0 ) )
-	    { SSDEB(argv[1],argv[2]); return -1; }
-
-	return 4;
-	}
-
-    if  ( argc == 5					&&
-	  ! strcmp( command, "printToPrinterPaper" )	)
-	{
-	if  ( getResources )
-	    { appGetApplicationResourceValues( ea );	}
-
-	if  ( appPrintToPrinter( ea, argv[2], argv[3], argv[4] ) )
-	    { SSDEB(argv[1],argv[2]); return -1; }
-
-	return 5;
-	}
-
-    /**  ----- **/
-
-    return 0;
+    return;
     }
 
 /************************************************************************/
@@ -856,6 +1100,8 @@ static int appMainHandleSpecialCall(	EditApplication *	ea,
 /*  2)  Retrieve resource values and derive some application wide	*/
 /*	settings from them.						*/
 /*  3)  Do not die too easily.						*/
+/*	And try to be immune for shells that make you sensitive for	*/
+/*	signals that an X11 application does not want to know about.	*/
 /*									*/
 /************************************************************************/
 
@@ -863,34 +1109,62 @@ int appMain(	EditApplication *	ea,
 		int			argc,
 		char *			argv[] )
     {
+    int			argTo;
     int			arg;
     int			res;
+    int			didSpecial= 0;
+    char *		prog;
 
     setlocale( LC_ALL, "" );
 
-    res= appMainHandleSpecialCall( ea, "--", 1, argc, argv );
+    appDefaultPapersize( ea );
+
+    res= appReadUserProperties( ea );
+    if  ( res )
+	{ LDEB(res); return 1;	}
+
+    res= appReadSystemProperties( ea );
+    if  ( res )
+	{ LDEB(res); return 1;	}
+
+    argTo= 1;
+    for ( arg= 1; arg < argc; arg++ )
+	{
+	if  ( arg+ 3 <= argc				&&
+	      ! strcmp( argv[arg], "--setProperty" )	)
+	    {
+	    if  ( appSetUserProperty( ea, argv[arg+ 1], argv[arg+ 2] ) )
+		{
+		SSDEB(argv[arg+ 1], argv[arg+ 2]);
+		return 1;
+		}
+
+	    arg += 2;
+	    continue;
+	    }
+	argv[argTo++]= argv[arg];
+	}
+    argc= argTo;
+
+    prog= argv[0];
+    res= appMainHandleSpecialCalls( ea, "--", 1, prog, argc- 1, argv+ 1 );
     if  ( res < 0 )
 	{ LDEB(res); return 1;	}
     if  ( res > 0 )
-	{ return 0;		}
+	{ didSpecial= 1; argc -= res; argv += res; argv[0]= prog;	}
+
+    if  ( argc <= 1 && didSpecial )
+	{ return 0;	}
 
     /*  1  */
     if  ( appGuiInitApplication( ea, &argc, &argv ) )
 	{ LDEB(1); return 1;	}
 
-    appInitDocumentGeometry( &(ea->eaDefaultDocumentGeometry) );
+    utilInitDocumentGeometry( &(ea->eaDefaultDocumentGeometry) );
 
     /*  b  */
     appGetApplicationResourceValues( ea );
 
-    if  ( ea->eaPrintDestinationCount == 0				&&
-	  utilPrinterGetPrinters( &(ea->eaPrintDestinationCount),
-				    &(ea->eaDefaultPrintDestination),
-				    &(ea->eaPrintDestinations),
-				    ea->eaCustomPrintCommand,
-				    ea->eaCustomPrinterName )		)
-	{ LDEB(1); 	}
-
     if  ( appFinishApplicationWindow( ea ) )
 	{ LDEB(1); return -1;	}
 
@@ -904,15 +1178,21 @@ int appMain(	EditApplication *	ea,
 	}
 #   endif
 
-    res= appMainHandleSpecialCall( ea, "++", 0, argc, argv );
+    res= appMainHandleSpecialCalls( ea, "++", 0, prog, argc- 1, argv+ 1 );
     if  ( res < 0 )
 	{ LDEB(res); return 1;	}
     if  ( res > 0 )
-	{ return 0;		}
+	{ didSpecial= 1; argc -= res; argv += res; argv[0]= prog;	}
+
+    if  ( argc <= 1 && didSpecial )
+	{ return 0;	}
 
     /*  3  */
     (void) signal( SIGHUP, SIG_IGN );
     (void) signal( SIGINT, SIG_IGN );
+#   ifdef SIGWINCH
+    (void) signal( SIGWINCH, SIG_IGN );
+#   endif
 
     for ( arg= 1; arg < argc; arg++ )
 	{
@@ -922,76 +1202,17 @@ int appMain(	EditApplication *	ea,
 
 	EditDocument *			ed;
 
-	if  ( ! strcmp( argv[arg], "++Upd" ) )
-	    {
-	    if  ( arg+ 3 >= argc )
-		{ LLDEB(arg,argc); arg= argc- 1; continue;	}
-
-	    ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget, (APP_WIDGET)0,
-						read_only, argv[arg+ 1] );
-
-	    if  ( ed )
-		{
-		appSetDocumentTitle( ed, argv[arg+ 2] );
-		appSetDocumentFilename( ed, argv[arg+ 3] );
-
-		appDocumentChanged( ed, 1 );
-		}
-
-	    if  ( appRemoveFile( argv[arg+ 1] ) )
-		{ SDEB(argv[arg+ 1]);	}
-
-	    arg += 3; continue;
-	    }
-
-	if  ( ! strcmp( argv[arg], "++New" ) )
-	    {
-	    if  ( arg+ 2 >= argc )
-		{ LLDEB(arg,argc); arg= argc- 1; continue;	}
-
-	    ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget, (APP_WIDGET)0,
-						read_only, argv[arg+ 1] );
-
-	    if  ( ed )
-		{
-		appSetDocumentTitle( ed, argv[arg+ 2] );
-		appSetDocumentFilename( ed, (char *)0 );
-
-		appDocumentChanged( ed, 1 );
-		}
-
-	    if  ( appRemoveFile( argv[arg+ 1] ) )
-		{ SDEB(argv[arg+ 1]);	}
-
-	    arg += 2; continue;
-	    }
-
-	if  ( ! strcmp( argv[arg], "++Old" ) )
-	    {
-	    if  ( arg+ 1 >= argc )
-		{ LLDEB(arg,argc); arg= argc- 1; continue;	}
-
-	    if  ( appRemoveFile( argv[arg+ 1] ) )
-		{ SDEB(argv[arg+ 1]);	}
-
-	    arg += 1; continue;
-	    }
-
-	if  ( ! strcmp( argv[arg], "++Exit" ) )
-	    { continue;	}
-
 	if  ( appTestFileExists( argv[arg] ) )
 	    {
 	    if  ( ea->eaCreateNewFromCommand )
 		{
-		int	res;
+		int	ynRes;
 
-		res= appQuestionRunSubjectYesNoDialog( ea,
+		ynRes= appQuestionRunSubjectYesNoDialog( ea,
 				    ea->eaToplevel.atTopWidget, (APP_WIDGET)0,
-				    argv[arg], afmr->afmrMakeItQuestion,
-		    (char *)0, (char *)0 );
+				    argv[arg], afmr->afmrMakeItQuestion );
 
-		if  ( res == AQDrespYES )
+		if  ( ynRes == AQDrespYES )
 		    {
 		    if  ( appNewDocument( ea, argv[arg] ) )
 			{ SDEB(argv[arg]);	}
@@ -1005,9 +1226,10 @@ int appMain(	EditApplication *	ea,
 	    }
 	else{
 	    const char * const	fileRelativeTo= (const char *)0;
+	    int			relativeIsFile= 1;
 
-	    if  ( appAbsoluteName( absolute, 1000,
-					argv[arg], fileRelativeTo ) < 0 )
+	    if  ( appAbsoluteName( absolute, sizeof(absolute)- 1,
+			    argv[arg], relativeIsFile, fileRelativeTo ) < 0 )
 		{
 		SDEB(argv[arg]);
 		ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget,
@@ -1022,7 +1244,7 @@ int appMain(	EditApplication *	ea,
 
     if  ( ea->eaVisibleDocumentCount > 0 )
 	{ appHideShellWidget( ea->eaToplevel.atTopWidget );	}
-    else{ appShowShellWidget( ea->eaToplevel.atTopWidget );	}
+    else{ appShowShellWidget( ea, ea->eaToplevel.atTopWidget );	}
 
 #   ifdef USE_MOTIF
     XtAppMainLoop( ea->eaContext );
@@ -1032,5 +1254,9 @@ int appMain(	EditApplication *	ea,
     gtk_main();
 #   endif
 
+#   ifdef USE_QT
+    appRunMainLoop( ea );
+#   endif
+
     return 0;
     }
