@@ -15,8 +15,8 @@
 #   include	<utilPropMask.h>
 
 #   include	<appUnit.h>
-#   include	<appGeo.h>
-#   include	<appGeoString.h>
+#   include	<geo2DInteger.h>
+#   include	<geoString.h>
 
 typedef struct AppMarginTool
     {
@@ -58,10 +58,11 @@ extern int appMarginToolGetMargins(	PropertyMask *		pUpdMask,
 
 extern int appMarginToolCheckMargins(	const DocumentGeometry * dg );
 
-extern void appMakeMarginToolWidgets(	APP_WIDGET		parent,
-					const MarginToolResources * mtr,
-					AppMarginTool *		amt,
-					APP_TEXT_ACTIVATE_CALLBACK changed,
-					void *			through );
+extern void appMakeMarginToolWidgets(
+				APP_WIDGET			parent,
+				const MarginToolResources *	mtr,
+				AppMarginTool *			amt,
+				APP_TXACTIVATE_CALLBACK_T	gotValue,
+				void *				through );
 
 #   endif /*  APP_MARGIN_TOOL_H  */
