@@ -120,7 +120,7 @@ APP_WIDGET appSetMenuItem(	APP_WIDGET		menu,
     if  ( ami->amiCallback )
 	{
 	gtk_signal_connect( GTK_OBJECT( rval ),
-				    "activate", ami->amiCallback, target );
+			"activate", (GtkSignalFunc)ami->amiCallback, target );
 	}
 
     gtk_menu_append( GTK_MENU( menu ), rval );
@@ -164,7 +164,7 @@ APP_WIDGET appSetToggleMenuItem(	APP_WIDGET		menu,
 	    }
 	}
 
-    gtk_check_menu_item_set_state( GTK_CHECK_MENU_ITEM( rval ),
+    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( rval ),
 					ami->amiItemType == ITEMtyTOGGLE_ON );
     gtk_check_menu_item_set_show_toggle( GTK_CHECK_MENU_ITEM( rval ),
 					TRUE );
@@ -175,7 +175,7 @@ APP_WIDGET appSetToggleMenuItem(	APP_WIDGET		menu,
     if  ( ami->amiCallback )
 	{
 	gtk_signal_connect( GTK_OBJECT( rval ),
-				    "toggled", ami->amiCallback, target );
+			"toggled", (GtkSignalFunc)ami->amiCallback, target );
 	}
 
     if  ( target )
@@ -197,16 +197,21 @@ APP_WIDGET appMakeMenuInParent(	APP_WIDGET *		pButton,
 
     button= gtk_menu_item_new_with_label( itemText );
     gtk_menu_bar_append( GTK_MENU_BAR( menuBar ), button );
-    gtk_widget_show( button );
 
     gtk_menu_item_set_submenu( GTK_MENU_ITEM( button ), menu );
 
     if  ( isHelp )
 	{
+#	if GTK_MAJOR_VERSION < 2
 	gtk_menu_item_set_placement( GTK_MENU_ITEM( button ),
 						    GTK_DIRECTION_RIGHT );
+#	else
+	gtk_menu_item_set_right_justified( GTK_MENU_ITEM( button ), TRUE );
+#	endif
 	}
 
+    gtk_widget_show( button );
+
     *pButton= button;
 
     return menu;
@@ -215,7 +220,7 @@ APP_WIDGET appMakeMenuInParent(	APP_WIDGET *		pButton,
 void appGuiSetToggleItemState(	APP_WIDGET		toggle,
 				int			set )
     {
-    gtk_check_menu_item_set_state( GTK_CHECK_MENU_ITEM( toggle ), set != 0 );
+    gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( toggle ), set != 0 );
     }
 
 void appGuiSetToggleItemLabel(	APP_WIDGET		toggle,
@@ -226,8 +231,7 @@ void appGuiSetToggleItemLabel(	APP_WIDGET		toggle,
     return;
     }
 
-int appGuiGetMenuToggleStateFromCallback( APP_WIDGET		toggle,
-					void *			voidcbs )
+int appGuiGetMenuToggleStateFromCallbackGtk( APP_WIDGET		toggle )
     {
     GtkCheckMenuItem *	item= GTK_CHECK_MENU_ITEM( toggle );
 
@@ -257,5 +261,44 @@ char *	appGetTextFromMenuOption(	APP_WIDGET		w )
 void appFreeTextFromMenuOption(	char *		s )
     { return;	}
 
-#   endif
+/************************************************************************/
+/*									*/
+/*  Hide a menu option.							*/
+/*									*/
+/************************************************************************/
 
+void appHideMenuOption(		APP_WIDGET		w )
+    {
+    gtk_widget_hide( w );
+    }
+
+/************************************************************************/
+/*									*/
+/*  The implementation and usage are very inefficient. We do not care	*/
+/*  as the routine is rarely called.					*/
+/*									*/
+/************************************************************************/
+
+void appHideMenuChild(		APP_WIDGET		menu,
+				int			n )
+    {
+    GList *	glf;
+    GList *	gl;
+
+    gl= glf= gtk_container_children( GTK_CONTAINER( menu ) );
+
+    while( gl && n > 0 )
+	{ gl= gl->next; n--; }
+
+    if  ( gl )
+	{
+	GtkWidget *	child= (GtkWidget *)gl->data;
+
+	gtk_widget_hide( child );
+	}
+
+    if  ( glf )
+	{ g_list_free( glf );	}
+    }
+
+#   endif
