@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
@@ -35,17 +35,14 @@ typedef struct MetricRuler
 typedef struct MetricRulerDrag
     {
     int *		mrdValuePointer;
-    int *		mrdChangedPointer;
     MetricRuler *	mrdMr;
     APP_WIDGET		mrdWidget;
     int			mrdOc;
-    void *		mrdThrough;
-    DragHair		mrdDragHair;
-    int			mrdAsUpper;
+    EditDocument *	mrdDocument;
     } MetricRulerDrag;
 
 void * appMakeMetricRuler(	int			sizeAcross,
-				double			pixPerMM,
+				double			magnifiedPixelsPerTwip,
 				double			magnification,
 
 				int			minUnused,
@@ -74,7 +71,7 @@ void * appMakeMetricRuler(	int			sizeAcross,
 
     rd->rdUnit= whatUnit;
     rd->rdFontName= fontName;
-    appRulerCalculateIncrements( rd, pixPerMM, magnification );
+    appRulerCalculateIncrements( rd, magnifiedPixelsPerTwip, magnification );
 
     rd->rdMinUsed= minUnused;
     rd->rdMaxUsed= 0;
@@ -120,7 +117,6 @@ static void appDrawHorizontalRuler(	APP_WIDGET		w,
     RulerData *		rd= &(mr->mrRulerData);
     int			sizeAcross= rd->rdSizeAcross;
 
-    double		item;
     int			tick;
     double		units;
 
@@ -163,7 +159,7 @@ static void appDrawHorizontalRuler(	APP_WIDGET		w,
 	drSel.drY0= 0;
 	drSel.drY1= sizeAcross;
 
-	if  ( docIntersectRectangle( &drSel, &drSel, drClip ) )
+	if  ( geoIntersectRectangle( &drSel, &drSel, drClip ) )
 	    {
 	    appDrawSetForegroundColor( add, &(add->addBottomColor) );
 
@@ -188,49 +184,47 @@ static void appDrawHorizontalRuler(	APP_WIDGET		w,
 	appDrawFillRectangle( add, x0, sizeAcross- 1, x1- x0, 1 );
 	}
 
-    item= 0; units= 0; tick= 0;
-    while( item < drClip->drX1 )
+    units= 0; tick= 0;
+    for (;;)
 	{
-	if  ( item- ox <  rd->rdMinUsed	||
-	      item- ox >= rd->rdMaxUsed	)
-	    {
-	    tick++;
-	    units += rd->rdUnitsPerTick;
-	    item += rd->rdTickDistancePixels;
-	    continue;
-	    }
+	int	twips= units* rd->rdTwipsPerUnit;
+	int	iitem= rd->rdMagnifiedPixelsPerTwip* twips;
 
-	if  ( item != 0 && tick % rd->rdTicksPerText == 0 )
+	if  ( iitem > drClip->drX1 )
+	    { break;	}
+
+	if  ( iitem- ox <  rd->rdMinUsed	||
+	      iitem- ox >= rd->rdMaxUsed	)
+	    { units += rd->rdUnitsPerTick; tick++; continue; }
+
+	if  ( iitem != 0 && tick % rd->rdTicksPerText == 0 )
 	    {
-	    char	scratch[30];
-	    int		len;
-	    int		wide;
+	    char		scratch[30];
+	    int			len;
+	    int			wide;
 
 	    appRulerTagText( scratch, &wide, &len, rd, units );
 
-	    if  ( item- ox- wide/2 >= rd->rdMinUsed	&&
-		  item- ox+ wide/2 <  rd->rdMaxUsed	)
+	    if  ( iitem- ox- wide/2 >= rd->rdMinUsed	&&
+		  iitem- ox+ wide/2 <  rd->rdMaxUsed	)
 		{
 		appDrawDrawString( add,
-			    item- ox- wide/2, fontBottom, scratch, len );
+			    iitem- ox- wide/2, fontBottom, scratch, len );
 
 		appDrawFillRectangle( add,
-			item- ox, fontBottom+ 2, 1, sizeAcross- fontBottom- 2 );
+		    iitem- ox, fontBottom+ 2, 1, sizeAcross- fontBottom- 2 );
 		}
 	    else{
-		appDrawFillRectangle( add, item- ox, tagY, 1, tagH );
+		appDrawFillRectangle( add, iitem- ox, tagY, 1, tagH );
 		}
 	    }
 	else{
 	    if  ( tick % rd->rdTicksPerTag == 0 )
-		{ appDrawFillRectangle( add, item- ox, tagY, 1, tagH );	}
-	    else{ appDrawFillRectangle( add, item- ox, tickY, 1, tickH ); }
+		{ appDrawFillRectangle( add, iitem- ox, tagY, 1,  tagH );  }
+	    else{ appDrawFillRectangle( add, iitem- ox, tickY, 1, tickH ); }
 	    }
 
-	tick++;
-	units += rd->rdUnitsPerTick;
-	item += rd->rdTickDistancePixels;
-	continue;
+	units += rd->rdUnitsPerTick; tick++; continue;
 	}
 
     /* Already done:
@@ -291,7 +285,6 @@ static void appDrawVerticalRuler(	APP_WIDGET		w,
     RulerData *		rd= &(mr->mrRulerData);
     int			sizeAcross= rd->rdSizeAcross;
 
-    double		item;
     int			tick;
     double		units;
 
@@ -337,7 +330,7 @@ static void appDrawVerticalRuler(	APP_WIDGET		w,
 	if  ( mr->mrV1Pixels >= 0 )
 	    { drSel.drY1= mr->mrV1Pixels;	}
 
-	if  ( docIntersectRectangle( &drSel, &drSel, drClip ) )
+	if  ( geoIntersectRectangle( &drSel, &drSel, drClip ) )
 	    {
 	    appDrawSetForegroundColor( add, &(add->addBottomColor) );
 
@@ -362,60 +355,61 @@ static void appDrawVerticalRuler(	APP_WIDGET		w,
 	appDrawFillRectangle( add, sizeAcross- 1, y0, 1, y1- y0 );
 	}
 
-    item= 0; units= 0; tick= 0;
-    while( item < drClip->drY1 )
+    units= 0; tick= 0;
+    for (;;)
 	{
-	if  ( item- oy <  rd->rdMinUsed	||
-	      item- oy >= rd->rdMaxUsed	)
-	    {
-	    tick++;
-	    units += rd->rdUnitsPerTick;
-	    item += rd->rdTickDistancePixels;
-	    continue;
-	    }
+	int	twips= units* rd->rdTwipsPerUnit;
+	int	iitem= rd->rdMagnifiedPixelsPerTwip* twips;
 
-	if  ( item != 0 && tick % rd->rdTicksPerText == 0 )
-	    {
-	    char	scratch[30];
-	    int		len;
-	    int		wide;
+	if  ( iitem > drClip->drY1 )
+	    { break;	}
 
-	    int		ascent;
-	    int		descent;
+	if  ( iitem- oy <  rd->rdMinUsed	||
+	      iitem- oy >= rd->rdMaxUsed	)
+	    { units += rd->rdUnitsPerTick; tick++; continue; }
+
+	if  ( iitem != 0 && tick % rd->rdTicksPerText == 0 )
+	    {
+	    char		scratch[30];
+	    int			len;
+	    int			wide;
+
+	    int			ascent;
+	    int			descent;
 
 	    appRulerTagText( scratch, &wide, &len, rd, units );
 
 	    ascent= FONT_HEIGHT(sizeAcross)/ 2;
 	    descent= FONT_HEIGHT(sizeAcross)/ 2;
 
-	    if  ( item- oy- descent >= rd->rdMinUsed	&&
-		  item- oy+ ascent  <  rd->rdMaxUsed	)
+	    if  ( iitem- oy- descent >= rd->rdMinUsed	&&
+		  iitem- oy+ ascent  <  rd->rdMaxUsed	)
 		{
+SDEB((char *)scratch);
+		/*
 		appRulerDrawVerticalString( rd,
-			    fontBottom, item+ wide/2- oy, scratch, len );
+			    fontBottom, iitem+ wide/2- oy, scratch, len );
+		*/
 
 		appDrawFillRectangle( add,
-				fontBottom+ 2, item- oy,
+				fontBottom+ 2, iitem- oy,
 				sizeAcross- fontBottom- 2, 1 );
 		}
 	    else{
-		appDrawFillRectangle( add, tagX, item- oy, tagW, 1 );
+		appDrawFillRectangle( add, tagX, iitem- oy, tagW, 1 );
 		}
 	    }
 	else{
 	    if  ( tick % rd->rdTicksPerTag == 0 )
 		{
-		appDrawFillRectangle( add, tagX, item- oy, tagW, 1 );
+		appDrawFillRectangle( add, tagX, iitem- oy, tagW, 1 );
 		}
 	    else{
-		appDrawFillRectangle( add, tickX, item- oy, tickW, 1 );
+		appDrawFillRectangle( add, tickX, iitem- oy, tickW, 1 );
 		}
 	    }
 
-	tick++;
-	units += rd->rdUnitsPerTick;
-	item += rd->rdTickDistancePixels;
-	continue;
+	units += rd->rdUnitsPerTick; tick++; continue;
 	}
 
     /* Already done:
@@ -462,7 +456,7 @@ static void appDrawVerticalRuler(	APP_WIDGET		w,
     return;
     }
 
-APP_EVENT_HANDLER( appRedrawHorizontalMetricRuler, w, voidmr, event )
+APP_EVENT_HANDLER_H( appRedrawHorizontalMetricRuler, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -486,7 +480,7 @@ APP_EVENT_HANDLER( appRedrawHorizontalMetricRuler, w, voidmr, event )
     appDrawHorizontalRuler( w, mr, &drClip, ox );
     }
 
-APP_EVENT_HANDLER( appRedrawVerticalMetricRuler, w, voidmr, event )
+APP_EVENT_HANDLER_H( appRedrawVerticalMetricRuler, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -510,7 +504,7 @@ APP_EVENT_HANDLER( appRedrawVerticalMetricRuler, w, voidmr, event )
     appDrawVerticalRuler( w, mr, &drClip, oy );
     }
 
-APP_EVENT_HANDLER( appHorizontalRulerConfigure, w, voidmr, event )
+APP_EVENT_HANDLER_H( appHorizontalRulerConfigure, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -526,7 +520,7 @@ APP_EVENT_HANDLER( appHorizontalRulerConfigure, w, voidmr, event )
     return;
     }
 
-APP_EVENT_HANDLER( appVerticalRulerConfigure, w, voidmr, event )
+APP_EVENT_HANDLER_H( appVerticalRulerConfigure, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -606,15 +600,160 @@ void appSetMetricRulerRange(	void *			voidmr,
 #   endif
 	{
 	if  ( ! rd->rdDrawingDataAllocated				&&
-	      appSetDrawingDataForWidget( w, mr->mrMagnification, add )	)
+	      appSetDrawingDataForWidget( w, (AppColors *)0, mr->mrMagnification, add )	)
 	    { LDEB(1); return;	}
 
-	appExposeRectangle( add, 0, 0, 0, 0 );
+	appExposeDrawingData( add );
 	}
 
     return;
     }
 
+/************************************************************************/
+
+void appMetricRulerDrawHair(	EditDocument *		ed )
+    {
+    const SelectRectangle *	sr= &(ed->edSelectRectangle);
+    DocumentRectangle		drWhite;
+    DocumentRectangle		drBlack;
+    int				ox= ed->edVisibleRect.drX0;
+    int				oy= ed->edVisibleRect.drY0;
+    AppDrawingData *		add= &(ed->edDocumentWidget.dwDrawingData);
+
+    drWhite= ed->edVisibleRect;
+    drBlack= ed->edVisibleRect;
+
+    switch( sr->srDirection )
+	{
+	case DOCselW:
+	    drWhite.drX0= sr->srSelected.drX0+ sr->srLTM.drX0;
+	    drWhite.drX1= sr->srSelected.drX0;
+	    drBlack.drX0= sr->srSelected.drX0;
+	    drBlack.drX1= sr->srSelected.drX0+ sr->srLTM.drX1;
+	    break;
+
+	case DOCselN:
+	    drWhite.drY0= sr->srSelected.drY0+ sr->srLTM.drY0;
+	    drWhite.drY1= sr->srSelected.drY0;
+	    drBlack.drY0= sr->srSelected.drY0;
+	    drBlack.drY1= sr->srSelected.drY0+ sr->srLTM.drY1;
+	    break;
+
+	case DOCselE:
+	    drBlack.drX0= sr->srSelected.drX1+ sr->srRBM.drX0;
+	    drBlack.drX1= sr->srSelected.drX1;
+	    drWhite.drX0= sr->srSelected.drX1;
+	    drWhite.drX1= sr->srSelected.drX1+ sr->srRBM.drX1;
+	    break;
+
+	case DOCselS:
+	    drBlack.drY0= sr->srSelected.drY1+ sr->srRBM.drY0;
+	    drBlack.drY1= sr->srSelected.drY1;
+	    drWhite.drY0= sr->srSelected.drY1;
+	    drWhite.drY1= sr->srSelected.drY1+ sr->srRBM.drY1;
+	    break;
+
+	default:
+	    LDEB(sr->srDirection);
+	    return;
+	}
+
+    appDrawSetForegroundBlack( add );
+    appDrawFillRectangle( add, drBlack.drX0- ox, drBlack.drY0- oy,
+					drBlack.drX1- drBlack.drX0+ 1,
+					drBlack.drY1- drBlack.drY0+ 1 );
+    appDrawSetForegroundWhite( add );
+    appDrawFillRectangle( add, drWhite.drX0- ox, drWhite.drY0- oy,
+					drWhite.drX1- drWhite.drX0+ 1,
+					drWhite.drY1- drWhite.drY0+ 1 );
+
+    return;
+    }
+
+
+/************************************************************************/
+
+void appMetricRulerExposeValue(	EditDocument *		ed )
+    {
+    const SelectRectangle *	sr= &(ed->edSelectRectangle);
+    DocumentRectangle		drExpose;
+    int				ox= ed->edVisibleRect.drX0;
+    int				oy= ed->edVisibleRect.drY0;
+    AppDrawingData *		add= &(ed->edDocumentWidget.dwDrawingData);
+
+    drExpose= ed->edVisibleRect;
+
+    switch( sr->srDirection )
+	{
+	case DOCselW:
+	    drExpose.drX0= sr->srSelected.drX0+ sr->srLTM.drX0;
+	    drExpose.drX1= sr->srSelected.drX0+ sr->srLTM.drX1;
+	    break;
+
+	case DOCselN:
+	    drExpose.drY0= sr->srSelected.drY0+ sr->srLTM.drY0;
+	    drExpose.drY1= sr->srSelected.drY0+ sr->srLTM.drY1;
+	    break;
+
+	case DOCselE:
+	    drExpose.drX0= sr->srSelected.drX1+ sr->srRBM.drX0;
+	    drExpose.drX1= sr->srSelected.drX1+ sr->srRBM.drX1;
+	    break;
+
+	case DOCselS:
+	    drExpose.drY0= sr->srSelected.drY1+ sr->srRBM.drY0;
+	    drExpose.drY1= sr->srSelected.drY1+ sr->srRBM.drY1;
+	    break;
+
+	default:
+	    LDEB(sr->srDirection);
+	    return;
+	}
+
+    drExpose.drX0 -= ox;
+    drExpose.drX1 -= ox;
+    drExpose.drY0 -= oy;
+    drExpose.drY1 -= oy;
+    appExposeRectangle( add, &drExpose );
+
+    return;
+    }
+
+void appMetricRulerChangeValue(	EditDocument *		ed,
+				int			newValue )
+    {
+    SelectRectangle *	sr= &(ed->edSelectRectangle);
+
+    appMetricRulerExposeValue( ed );
+
+    switch( sr->srDirection )
+	{
+	case DOCselW:
+	    sr->srSelected.drX0= newValue;
+	    break;
+
+	case DOCselN:
+	    sr->srSelected.drY0= newValue;
+	    break;
+
+	case DOCselE:
+	    sr->srSelected.drX1= newValue;
+	    break;
+
+	case DOCselS:
+	    sr->srSelected.drY1= newValue;
+	    break;
+
+	default:
+	    LDEB(sr->srDirection);
+	    return;
+	}
+
+    appMetricRulerExposeValue( ed );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Let the user drag one of the controls on the top ruler.		*/
@@ -638,39 +777,50 @@ static void appHorizontalRulerSetValue(	MetricRulerDrag *	mrd,
     {
     MetricRuler *		mr= mrd->mrdMr;
     RulerData *			rd= &(mr->mrRulerData);
+    AppDrawingData *		add= &(rd->rdDrawingData);
+    int				ox= rd->rdVisibleC0- rd->rdMinUsed;
+    const int			oy= 0;
+
     int				oldValue= *(mrd->mrdValuePointer);
 
-    DocumentRectangle		drClip;
+    DocumentRectangle		drExpose;
 
     if  ( newValue == oldValue )
 	{ return;	}
 
-    *(mrd->mrdValuePointer)= newValue;
-    *(mrd->mrdChangedPointer)= 1;
+    if  ( mrd->mrdValuePointer )
+	{ *(mrd->mrdValuePointer)= newValue;	}
 
     if  ( oldValue < newValue )
 	{
-	drClip.drX0= oldValue- rd->rdSizeAcross;
-	drClip.drX1= newValue+ rd->rdSizeAcross;
-	drClip.drY0= 0;
-	drClip.drY1= rd->rdSizeAcross;
+	drExpose.drX0= oldValue- rd->rdSizeAcross;
+	drExpose.drX1= newValue+ rd->rdSizeAcross;
+	drExpose.drY0= 0;
+	drExpose.drY1= rd->rdSizeAcross;
 	}
     else{
-	drClip.drX0= newValue- rd->rdSizeAcross;
-	drClip.drX1= oldValue+ rd->rdSizeAcross;
-	drClip.drY0= 0;
-	drClip.drY1= rd->rdSizeAcross;
+	drExpose.drX0= newValue- rd->rdSizeAcross;
+	drExpose.drX1= oldValue+ rd->rdSizeAcross;
+	drExpose.drY0= 0;
+	drExpose.drY1= rd->rdSizeAcross;
 	}
 
-    appDrawHorizontalRuler( mrd->mrdWidget, mr, &drClip, mrd->mrdOc );
+    /*
+    appDrawHorizontalRuler( mrd->mrdWidget, mr, &drExpose, mrd->mrdOc );
+    */
+    drExpose.drX0 -= ox;
+    drExpose.drX1 -= ox;
+    drExpose.drY0 -= oy;
+    drExpose.drY1 -= oy;
+    appExposeRectangle( add, &drExpose );
 
     /*  5  */
-    (*mrd->mrdDragHair)( mrd->mrdThrough, oldValue, newValue, mrd->mrdAsUpper );
+    appMetricRulerChangeValue( mrd->mrdDocument, newValue );
 
     return;
     }
 
-static APP_EVENT_HANDLER( appHorizontalRulerMouseMove, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appHorizontalRulerMouseMove, w, vmrd, event )
     {
     MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
 
@@ -685,7 +835,7 @@ static APP_EVENT_HANDLER( appHorizontalRulerMouseMove, w, vmrd, event )
     return;
     }
 
-static APP_EVENT_HANDLER( appHorizontalRulerMouseUp, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appHorizontalRulerMouseUp, w, vmrd, event )
     {
     MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
 
@@ -695,15 +845,15 @@ static APP_EVENT_HANDLER( appHorizontalRulerMouseUp, w, vmrd, event )
     int				button;
     int				upDown;
     int				seq;
+    unsigned int		keyState;
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-				&x, &y, &button, &upDown, &seq, w, event ) )
+			&x, &y, &button, &upDown, &seq, &keyState, w, event ) )
 	{ return;	}
 
     appHorizontalRulerSetValue( mrd, x+ mrd->mrdOc );
+    appMetricRulerExposeValue( mrd->mrdDocument );
 
-    (*mrd->mrdDragHair)( mrd->mrdThrough,
-			    *(mrd->mrdValuePointer), -1, mrd->mrdAsUpper );
     return;
     }
 
@@ -714,30 +864,61 @@ static void appHorizontalRulerDragItem(	int *			pValue,
 					APP_EVENT *		downEvent,
 					MetricRuler *		mr,
 					int			ox,
-					void *			voided,
-					DragHair		dragHair,
-					int			asUpper )
+					EditDocument *		ed,
+					int			direction )
     {
+    SelectRectangle *		sr= &(ed->edSelectRectangle);
     MetricRulerDrag		mrd;
+    int				startValue= *pValue;
+    int				oldDirection= sr->srDirection;
 
     mrd.mrdValuePointer= pValue;
-    mrd.mrdChangedPointer= pChanged;
     mrd.mrdMr= mr;
     mrd.mrdWidget= w;
     mrd.mrdOc= ox;
-    mrd.mrdThrough= voided;
-    mrd.mrdDragHair= dragHair;
-    mrd.mrdAsUpper= asUpper;
+    mrd.mrdDocument= ed;
+
+    switch( direction )
+	{
+	case DOCselW:
+	    sr->srSelected.drX0= *pValue;
+	    break;
+
+	case DOCselE:
+	    sr->srSelected.drX1= *pValue;
+	    break;
+
+	default:
+	    LDEB(sr->srDirection);
+	    return;
+	}
 
     /*  1  */
-    (*mrd.mrdDragHair)( mrd.mrdThrough,
-				-1, *(mrd.mrdValuePointer), mrd.mrdAsUpper );
+    sr->srDirection= direction;
+
+    sr->srLTM.drX0= -1;
+    sr->srLTM.drX1=  1;
+    sr->srLTM.drY0= -1;
+    sr->srLTM.drY1=  1;
+
+    sr->srRBM.drX0= -1;
+    sr->srRBM.drX1=  1;
+    sr->srRBM.drY0= -1;
+    sr->srRBM.drY1=  1;
+
+    appMetricRulerExposeValue( ed );
 
     appRunDragLoop( w, ea, downEvent,
 				appHorizontalRulerMouseUp,
 				appHorizontalRulerMouseMove,
 				0, (APP_TIMER_CALLBACK)0,
 				(void *)&mrd );
+
+    if  ( pChanged && *pValue != startValue )
+	{ *pChanged= 1;	}
+
+    sr->srDirection= oldDirection;
+    return;
     }
 
 /************************************************************************/
@@ -755,8 +936,7 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 					EditApplication *	ea,
 					APP_EVENT *		downEvent,
 					void *			voidmr,
-					void *			voided,
-					DragHair		dragHair )
+					EditDocument *		ed )
     {
     MetricRuler  *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -774,22 +954,21 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
     int			button;
     int			upDown;
     int			seq;
+    unsigned int	keyState;
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-			    &x, &y, &button, &upDown, &seq, w, downEvent ) )
+		    &x, &y, &button, &upDown, &seq, &keyState, w, downEvent ) )
 	{ return;	}
 
     if  ( upDown < 1 || button != 1 || seq > 1 )
 	{ return;	}
 
-    if  ( mr->mrV0Pixels >= 0				&&
-	  x+ ox <  mr->mrV0Pixels + xp			&&
+    if  ( mr->mrV0Pixels >= 0					&&
+	  x+ ox <  mr->mrV0Pixels + xp				&&
 	  x+ ox >= mr->mrV0Pixels- HAIR_THICK( sizeAcross ) -xp	)
 	{
-	asUpper= 0;
-
 	appHorizontalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselW );
 	goto ready;
 	}
 
@@ -800,7 +979,7 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	asUpper= 1;
 
 	appHorizontalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselE );
 
 	goto ready;
 	}
@@ -818,10 +997,8 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	    mr->mrV0Pixels= x+ ox;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, x+ ox, asUpper );
-
 	    appHorizontalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				w, ea, downEvent, mr, ox, ed, DOCselW );
 
 	    goto ready;
 	    }
@@ -830,10 +1007,9 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	    mr->mrV1Pixels= x+ ox;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, x+ ox, asUpper );
 
 	    appHorizontalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselE );
 
 	    goto ready;
 	    }
@@ -849,10 +1025,9 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	    mr->mrV1Pixels= x+ ox;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, x+ ox, asUpper );
 
 	    appHorizontalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselE );
 
 	    goto ready;
 	    }
@@ -861,10 +1036,9 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	    mr->mrV0Pixels= x+ ox;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, x+ ox, asUpper );
 
 	    appHorizontalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselE );
 
 	    goto ready;
 	    }
@@ -876,20 +1050,16 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	mr->mrV0Pixels= x+ ox;
 
 	changed= 1;
-	(*dragHair)( voided, -1, x+ ox, asUpper );
-
 	appHorizontalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselW );
 	}
     else{
 	asUpper= 1;
 	mr->mrV1Pixels= x+ ox;
 
 	changed= 1;
-	(*dragHair)( voided, -1, x+ ox, asUpper );
-
 	appHorizontalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, ox, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, ox, ed, DOCselE );
 	}
 
   ready:
@@ -899,23 +1069,16 @@ void     appHorizontalRulerTrackMouse(	int *			pX0pixels,
 	{
 	int		swap;
 
-	changed= 1;
-
-	(*dragHair)( voided, mr->mrV1Pixels, -1, 1 );
-	(*dragHair)( voided, mr->mrV0Pixels, -1, 0 );
-
 	swap= mr->mrV1Pixels;
 	mr->mrV1Pixels= mr->mrV0Pixels;
 	mr->mrV0Pixels= swap;
-
-	(*dragHair)( voided, -1, mr->mrV1Pixels, 1 );
-	(*dragHair)( voided, -1, mr->mrV0Pixels, 0 );
+	changed= 1;
 	}
 
     if  ( changed )
 	{
 	*pChanged= 1;
-	appExposeRectangle( add, 0, 0, 0, 0 );
+	appExposeDrawingData( add );
 	}
 
     *pX0pixels= mr->mrV0Pixels;
@@ -947,9 +1110,13 @@ static void appVerticalRulerSetValue(	MetricRulerDrag *	mrd,
     {
     MetricRuler *		mr= mrd->mrdMr;
     RulerData *			rd= &(mr->mrRulerData);
+    AppDrawingData *		add= &(rd->rdDrawingData);
+    const int			ox= 0;
+    int				oy= rd->rdVisibleC0- rd->rdMinUsed;
+
     int				oldValue= *(mrd->mrdValuePointer);
 
-    DocumentRectangle		drClip;
+    DocumentRectangle		drExpose;
 
     if  ( newValue < rd->rdDocumentC0 )
 	{ newValue= rd->rdDocumentC0;	}
@@ -960,31 +1127,38 @@ static void appVerticalRulerSetValue(	MetricRulerDrag *	mrd,
 	{ return;	}
 
     *(mrd->mrdValuePointer)= newValue;
-    *(mrd->mrdChangedPointer)= 1;
 
     if  ( oldValue < newValue )
 	{
-	drClip.drX0= 0;
-	drClip.drX1= rd->rdSizeAcross;
-	drClip.drY0= oldValue- rd->rdSizeAcross;
-	drClip.drY1= newValue+ rd->rdSizeAcross;
+	drExpose.drX0= 0;
+	drExpose.drX1= rd->rdSizeAcross;
+	drExpose.drY0= oldValue- rd->rdSizeAcross;
+	drExpose.drY1= newValue+ rd->rdSizeAcross;
 	}
     else{
-	drClip.drX0= 0;
-	drClip.drX1= rd->rdSizeAcross;
-	drClip.drY0= newValue- rd->rdSizeAcross;
-	drClip.drY1= oldValue+ rd->rdSizeAcross;
+	drExpose.drX0= 0;
+	drExpose.drX1= rd->rdSizeAcross;
+	drExpose.drY0= newValue- rd->rdSizeAcross;
+	drExpose.drY1= oldValue+ rd->rdSizeAcross;
 	}
 
-    appDrawVerticalRuler( mrd->mrdWidget, mr, &drClip, mrd->mrdOc );
+    /*
+    appDrawVerticalRuler( mrd->mrdWidget, mr, &drExpose, mrd->mrdOc );
+    */
+    drExpose.drX0 -= ox;
+    drExpose.drX1 -= ox;
+    drExpose.drY0 -= oy;
+    drExpose.drY1 -= oy;
+    appExposeRectangle( add, &drExpose );
 
     /*  5  */
-    (*mrd->mrdDragHair)( mrd->mrdThrough, oldValue, newValue, mrd->mrdAsUpper );
+    appMetricRulerChangeValue( mrd->mrdDocument, newValue );
+
 
     return;
     }
 
-static APP_EVENT_HANDLER( appVerticalRulerMouseMove, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appVerticalRulerMouseMove, w, vmrd, event )
     {
     MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
 
@@ -999,25 +1173,24 @@ static APP_EVENT_HANDLER( appVerticalRulerMouseMove, w, vmrd, event )
     return;
     }
 
-static APP_EVENT_HANDLER( appVerticalRulerMouseUp, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appVerticalRulerMouseUp, w, vmrd, event )
     {
-    MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
+    MetricRulerDrag *	mrd= (MetricRulerDrag *)vmrd;
 
-    int				x;
-    int				y;
+    int			x;
+    int			y;
 
-    int				button;
-    int				upDown;
-    int				seq;
+    int			button;
+    int			upDown;
+    int			seq;
+    unsigned int	keyState;
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-				&x, &y, &button, &upDown, &seq, w, event ) )
+			&x, &y, &button, &upDown, &seq, &keyState, w, event ) )
 	{ return;	}
 
     appVerticalRulerSetValue( mrd, y+ mrd->mrdOc );
-
-    (*mrd->mrdDragHair)( mrd->mrdThrough,
-			    *(mrd->mrdValuePointer), -1, mrd->mrdAsUpper );
+    appMetricRulerExposeValue( mrd->mrdDocument );
 
     return;
     }
@@ -1029,30 +1202,60 @@ static void appVerticalRulerDragItem(	int *			pValue,
 					APP_EVENT *		downEvent,
 					MetricRuler *		mr,
 					int			oy,
-					void *			voided,
-					DragHair		dragHair,
-					int			asUpper )
+					EditDocument *		ed,
+					int			direction )
     {
+    SelectRectangle *		sr= &(ed->edSelectRectangle);
     MetricRulerDrag		mrd;
+    int				startValue= *pValue;
+    int				oldDirection= sr->srDirection;
 
     mrd.mrdValuePointer= pValue;
-    mrd.mrdChangedPointer= pChanged;
     mrd.mrdMr= mr;
     mrd.mrdWidget= w;
     mrd.mrdOc= oy;
-    mrd.mrdThrough= voided;
-    mrd.mrdDragHair= dragHair;
-    mrd.mrdAsUpper= asUpper;
+    mrd.mrdDocument= ed;
+
+    switch( direction )
+	{
+	case DOCselN:
+	    sr->srSelected.drY0= *pValue;
+	    break;
+
+	case DOCselS:
+	    sr->srSelected.drY1= *pValue;
+	    break;
+
+	default:
+	    LDEB(sr->srDirection);
+	    return;
+	}
 
     /*  1  */
-    (*mrd.mrdDragHair)( mrd.mrdThrough,
-				-1, *(mrd.mrdValuePointer), mrd.mrdAsUpper );
+    sr->srDirection= direction;
+
+    sr->srLTM.drX0= -1;
+    sr->srLTM.drX1=  1;
+    sr->srLTM.drY0= -1;
+    sr->srLTM.drY1=  1;
+
+    sr->srRBM.drX0= -1;
+    sr->srRBM.drX1=  1;
+    sr->srRBM.drY0= -1;
+    sr->srRBM.drY1=  1;
+
+    appMetricRulerExposeValue( ed );
 
     appRunDragLoop( w, ea, downEvent,
 				appVerticalRulerMouseUp,
 				appVerticalRulerMouseMove,
 				0, (APP_TIMER_CALLBACK)0,
 				(void *)&mrd );
+    if  ( pChanged && *pValue != startValue )
+	{ *pChanged= 1;	}
+
+    sr->srDirection= oldDirection;
+    return;
     }
 
 /************************************************************************/
@@ -1070,8 +1273,7 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 					EditApplication *	ea,
 					APP_EVENT *		downEvent,
 					void *			voidmr,
-					void *			voided,
-					DragHair		dragHair )
+					EditDocument *		ed )
     {
     MetricRuler  *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -1089,9 +1291,10 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
     int			button;
     int			upDown;
     int			seq;
+    unsigned int	keyState;
 
     if  ( appGetCoordinatesFromMouseButtonEvent(
-			    &x, &y, &button, &upDown, &seq, w, downEvent ) )
+		    &x, &y, &button, &upDown, &seq, &keyState, w, downEvent ) )
 	{ return;	}
 
     if  ( upDown < 1 || button != 1 || seq > 1 )
@@ -1104,7 +1307,7 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	asUpper= 0;
 
 	appVerticalRulerDragItem( &(mr->mrV0Pixels), &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, oy, ed, DOCselN );
 
 	goto ready;
 	}
@@ -1116,7 +1319,7 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	asUpper= 1;
 
 	appVerticalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				w, ea, downEvent, mr, oy, ed, DOCselS );
 
 	goto ready;
 	}
@@ -1134,10 +1337,8 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	    mr->mrV0Pixels= y+ oy;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, y+ oy, asUpper );
-
 	    appVerticalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, oy, ed, DOCselN );
 
 	    goto ready;
 	    }
@@ -1146,10 +1347,8 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	    mr->mrV1Pixels= y+ oy;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, y+ oy, asUpper );
-
 	    appVerticalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, oy, ed, DOCselS );
 
 	    goto ready;
 	    }
@@ -1165,10 +1364,8 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	    mr->mrV1Pixels= y+ oy;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, y+ oy, asUpper );
-
 	    appVerticalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				w, ea, downEvent, mr, oy, ed, DOCselS );
 
 	    goto ready;
 	    }
@@ -1177,10 +1374,8 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	    mr->mrV0Pixels= y+ oy;
 
 	    changed= 1;
-	    (*dragHair)( voided, -1, y+ oy, asUpper );
-
 	    appVerticalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, oy, ed, DOCselN );
 
 	    goto ready;
 	    }
@@ -1192,10 +1387,8 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	mr->mrV0Pixels= y+ oy;
 
 	changed= 1;
-	(*dragHair)( voided, -1, y+ oy, asUpper );
-
 	appVerticalRulerDragItem( &mr->mrV0Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, oy, ed, DOCselN );
 
 	goto ready;
 	}
@@ -1204,10 +1397,8 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	mr->mrV1Pixels= y+ oy;
 
 	changed= 1;
-	(*dragHair)( voided, -1, y+ oy, asUpper );
-
 	appVerticalRulerDragItem( &mr->mrV1Pixels, &changed,
-		    w, ea, downEvent, mr, oy, voided, dragHair, asUpper );
+				    w, ea, downEvent, mr, oy, ed, DOCselS );
 
 	goto ready;
 	}
@@ -1219,23 +1410,16 @@ void     appVerticalRulerTrackMouse(	int *			pY0pixels,
 	{
 	int		swap;
 
-	changed= 1;
-
-	(*dragHair)( voided, mr->mrV1Pixels, -1, 1 );
-	(*dragHair)( voided, mr->mrV0Pixels, -1, 0 );
-
 	swap= mr->mrV1Pixels;
 	mr->mrV1Pixels= mr->mrV0Pixels;
 	mr->mrV0Pixels= swap;
-
-	(*dragHair)( voided, -1, mr->mrV1Pixels, 1 );
-	(*dragHair)( voided, -1, mr->mrV0Pixels, 0 );
+	changed= 1;
 	}
 
     if  ( changed )
 	{
 	*pChanged= 1;
-	appExposeRectangle( add, 0, 0, 0, 0 );
+	appExposeDrawingData( add );
 	}
 
     *pY0pixels= mr->mrV0Pixels;
@@ -1268,21 +1452,21 @@ void appSetMetricRulerValues(	APP_WIDGET	w,
 	{ mr->mrV1Pixels= v1; changed= 1; }
 
     if  ( changed )
-	{ appExposeRectangle( add, 0, 0, 0, 0 );	}
+	{ appExposeDrawingData( add );	}
     }
 
 void appSetMetricRulerMagnification(	APP_WIDGET	w,
 					void *		voidmr,
-					double		pixPerMM,
+					double		magnifiedPixelsPerTwip,
 					double		magnification )
     {
     MetricRuler  *		mr= (MetricRuler *)voidmr;
     RulerData *			rd= &(mr->mrRulerData);
     AppDrawingData *		add= &(rd->rdDrawingData);
 
-    appRulerCalculateIncrements( rd, pixPerMM, magnification );
+    appRulerCalculateIncrements( rd, magnifiedPixelsPerTwip, magnification );
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
@@ -1302,7 +1486,7 @@ void appAdaptMetricRuler(	APP_WIDGET	w,
 	{
 	rd->rdRulerC1= rulerC1;
 
-	appExposeRectangle( add, 0, 0, 0, 0 );
+	appExposeDrawingData( add );
 	}
     }
 
