@@ -11,7 +11,6 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"appFrame.h"
@@ -33,26 +32,22 @@ static void appPageLayoutRefreshPageLayoutPage(	PageLayoutTool *	plt )
     AppDrawingData *		add= &(plt->pltDrawingData);
     DocumentGeometry *		dg= &(plt->pltGeometryChosen);
 
-    char			scratch[50];
-
     appPaperChooserAdaptToGeometry( &(plt->pltPaperChooser), dg );
 
     appMarginToolShowMargins( &(plt->pltMarginTool), plt->pltUnitType, dg );
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     if  ( plt->pltHeaderPositionText )
 	{
-	appGeoLengthToString( scratch,
+	appLengthToTextWidget( plt->pltHeaderPositionText,
 				dg->dgHeaderPositionTwips, ea->eaUnitInt );
-	appStringToTextWidget( plt->pltHeaderPositionText, scratch );
 	}
 
     if  ( plt->pltFooterPositionText )
 	{
-	appGeoLengthToString( scratch,
+	appLengthToTextWidget( plt->pltFooterPositionText,
 				dg->dgFooterPositionTwips, ea->eaUnitInt );
-	appStringToTextWidget( plt->pltFooterPositionText, scratch );
 	}
 
     return;
@@ -70,7 +65,7 @@ void appPageLayoutPageRefresh(	PageLayoutTool *		plt,
     PropertyMask		setMask;
     PropertyMask		doneMask;
 
-    PROPmaskCLEAR( &setMask );
+    utilPropMaskClear( &setMask );
     utilPropMaskFill( &setMask, DGprop_COUNT );
 
     if  ( plt->pltManageSelection )
@@ -78,21 +73,21 @@ void appPageLayoutPageRefresh(	PageLayoutTool *		plt,
     else{ dgSelection= dgDoc;	}
 
     /**/
-    PROPmaskCLEAR( &doneMask );
+    utilPropMaskClear( &doneMask );
     utilUpdDocumentGeometry( &(plt->pltGeometryChosen), dgSelection,
 						    &doneMask, &setMask );
     if  ( ! utilPropMaskIsEmpty( &doneMask ) )
 	{ changed= 1;	}
 
     /**/
-    PROPmaskCLEAR( &doneMask );
+    utilPropMaskClear( &doneMask );
     utilUpdDocumentGeometry( &(plt->pltGeometrySetSelection), dgSelection,
 						    &doneMask, &setMask );
     if  ( ! utilPropMaskIsEmpty( &doneMask ) )
 	{ changed= 1;	}
 
     /**/
-    PROPmaskCLEAR( &doneMask );
+    utilPropMaskClear( &doneMask );
     utilUpdDocumentGeometry( &(plt->pltGeometrySetDocument), dgDoc,
 						    &doneMask, &setMask );
     if  ( ! utilPropMaskIsEmpty( &doneMask ) )
@@ -116,6 +111,7 @@ static void appPageLayoutChangeLayout(	PageLayoutTool *	plt,
 					int			wholeDocument )
     {
     EditApplication *		ea= plt->pltApplication;
+    EditDocument *		ed= ea->eaCurrentDocument;
 
     int				changed;
 
@@ -131,10 +127,13 @@ static void appPageLayoutChangeLayout(	PageLayoutTool *	plt,
     const int			maxValue= INT_MAX;
     const int			adaptToMax= 0;
 
-    PROPmaskCLEAR( &sizeUpdMask );
-    PROPmaskCLEAR( &margUpdMask );
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
 
-    PROPmaskCLEAR( &setMask );
+    utilPropMaskClear( &sizeUpdMask );
+    utilPropMaskClear( &margUpdMask );
+
+    utilPropMaskClear( &setMask );
     utilPropMaskFill( &setMask, DGprop_COUNT );
 
     utilInitDocumentGeometry( &dgNew );
@@ -145,7 +144,7 @@ static void appPageLayoutChangeLayout(	PageLayoutTool *	plt,
 								    &dgNew ) )
 	{ LDEB(1); return;	}
 
-    PROPmaskCLEAR( &margChgMask );
+    utilPropMaskClear( &margChgMask );
     utilPropMaskFill( &margChgMask, DGprop_COUNT );
 
     if  ( appMarginToolGetMargins( &margUpdMask, &margChgMask, plt->pltUnitType,
@@ -174,7 +173,7 @@ static void appPageLayoutChangeLayout(	PageLayoutTool *	plt,
 	PROPmaskUNSET( &setMask, DGpropFOOTER_POSITION );
 	}
 
-    (*ea->eaSetPageLayout)( ea, &dgNew, &setMask, wholeDocument );
+    (*ea->eaSetPageLayout)( ed, &dgNew, &setMask, wholeDocument );
 
     return;
     }
@@ -217,10 +216,7 @@ static void appSectHeaderFooterPosChanged(	PageLayoutTool *	plt,
     if  ( ! appGetLengthFromTextWidget( w, &width, &changed, ea->eaUnitInt,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
 	{
-	char	scratch[50];
-
-	appGeoLengthToString( scratch, width, ea->eaUnitInt );
-	appStringToTextWidget( w, scratch );
+	appLengthToTextWidget( w, width, ea->eaUnitInt );
 	}
     }
 
@@ -259,8 +255,8 @@ static APP_TXACTIVATE_CALLBACK_H( appPaperMarginChanged, w, voidplt )
 
     DocumentGeometry		dg;
 
-    PROPmaskCLEAR( &dgUpdMask );
-    PROPmaskCLEAR( &dgChgMask );
+    utilPropMaskClear( &dgUpdMask );
+    utilPropMaskClear( &dgChgMask );
 
     dg= *dgSect;
 
@@ -281,7 +277,7 @@ static APP_TXACTIVATE_CALLBACK_H( appPaperMarginChanged, w, voidplt )
     dgSect->dgRightMarginTwips= dg.dgRightMarginTwips;
     dgSect->dgBottomMarginTwips= dg.dgBottomMarginTwips;
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
@@ -346,7 +342,7 @@ static void appPageLayoutToolPaperRectChanged(
     plt->pltGeometryChosen.dgPageWideTwips= dg->dgPageWideTwips;
     plt->pltGeometryChosen.dgPageHighTwips= dg->dgPageHighTwips;
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
@@ -521,7 +517,7 @@ void appPageLayoutPageFinishPage(	PageLayoutTool *		plt,
 
     appPaperChooserRetreshMenuWidth( &(plt->pltPaperChooser) );
 
-    if  ( appSetDrawingDataForWidget( plt->pltPageDrawing,
+    if  ( appSetDrawingDataForWidget( plt->pltPageDrawing, (AppColors *)0,
 						    magnification, add ) )
 	{ LDEB(1);				}
     else{ plt->pltDrawingDataAllocated= 1;	}
@@ -529,6 +525,20 @@ void appPageLayoutPageFinishPage(	PageLayoutTool *		plt,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean the page layout tool.						*/
+/*									*/
+/************************************************************************/
+
+void appCleanPageLayoutTool(	PageLayoutTool *	plt )
+    {
+    appCleanPaperChooser( &(plt->pltPaperChooser) );
+    appCleanDrawingData( &(plt->pltDrawingData) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get page layout tool resources.					*/
