@@ -15,7 +15,7 @@
 
 #   include	<appFrame.h>
 #   include	"appUnit.h"
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 #   include	"appPaper.h"
 #   include	"appPaperChooser.h"
 #   include	"appMarginTool.h"
@@ -161,7 +161,7 @@ static void appPageToolPaperRectChanged(
     apt->aptGeometryChosen.dgPageWideTwips= dg->dgPageWideTwips;
     apt->aptGeometryChosen.dgPageHighTwips= dg->dgPageHighTwips;
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
@@ -197,7 +197,7 @@ static APP_BUTTON_CALLBACK_H( appPageToolRevertPushed, w, voidapt )
     appMarginToolShowMargins( &(apt->aptMarginTool),
 				apt->aptUnitType, &(apt->aptGeometryChosen) );
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
@@ -216,6 +216,7 @@ static APP_BUTTON_CALLBACK_H( appPageToolSetPushed, w, voidapt )
     {
     AppPageTool *	apt= (AppPageTool *)voidapt;
     EditApplication *	ea= apt->aptApplication;
+    EditDocument *	ed= ea->eaCurrentDocument;
 
     DocumentGeometry	dg;
 
@@ -227,17 +228,20 @@ static APP_BUTTON_CALLBACK_H( appPageToolSetPushed, w, voidapt )
     PropertyMask	margChgMask;
     PropertyMask	sizeChgMask;
 
-    PROPmaskCLEAR( &margUpdMask );
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    utilPropMaskClear( &margUpdMask );
     utilPropMaskFill( &margUpdMask, DGprop_COUNT );
 
-    PROPmaskCLEAR( &setMask );
+    utilPropMaskClear( &setMask );
     utilPropMaskFill( &setMask, DGprop_COUNT );
     PROPmaskUNSET( &setMask, DGpropHEADER_POSITION );
     PROPmaskUNSET( &setMask, DGpropFOOTER_POSITION );
 
-    PROPmaskCLEAR( &margChgMask );
+    utilPropMaskClear( &margChgMask );
     /*
-    PROPmaskCLEAR( &sizeChgMask );
+    utilPropMaskClear( &sizeChgMask );
     */
 
     dg= apt->aptGeometrySet;
@@ -254,7 +258,7 @@ static APP_BUTTON_CALLBACK_H( appPageToolSetPushed, w, voidapt )
 
     appPageToolSetProperties( (void *)apt, &dg );
 
-    (*ea->eaSetPageLayout)( ea, &dg, &setMask, wholeDocument );
+    (*ea->eaSetPageLayout)( ed, &dg, &setMask, wholeDocument );
 
     return;
     }
@@ -298,9 +302,9 @@ static APP_TXACTIVATE_CALLBACK_H( appPageToolMarginChanged, w, voidapt )
 
     DocumentGeometry		dg;
 
-    PROPmaskCLEAR( &dgChgMask );
+    utilPropMaskClear( &dgChgMask );
 
-    PROPmaskCLEAR( &dgUpdMask );
+    utilPropMaskClear( &dgUpdMask );
     utilPropMaskFill( &dgUpdMask, DGprop_COUNT );
 
     dg= apt->aptGeometryChosen;
@@ -320,7 +324,7 @@ static APP_TXACTIVATE_CALLBACK_H( appPageToolMarginChanged, w, voidapt )
     apt->aptGeometryChosen.dgRightMarginTwips= dg.dgRightMarginTwips;
     apt->aptGeometryChosen.dgBottomMarginTwips= dg.dgBottomMarginTwips;
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
@@ -364,14 +368,15 @@ static APP_WIDGET appPageToolMakeButtonRow( APP_WIDGET		parent,
 
     const int	heightResizable= 0;
     const int	buttonIsDefault= 0;
+    const int	colspan= 1;
 
     row= appMakeRowInColumn( parent, 2, heightResizable );
 
     appMakeButtonInRow( &revertButton, row, aptr->aptrRevert,
-		    appPageToolRevertPushed, (void *)apt, 0, buttonIsDefault );
+	    appPageToolRevertPushed, (void *)apt, 0, colspan, buttonIsDefault );
 
     appMakeButtonInRow( &setButton, row, aptr->aptrSet,
-		    appPageToolSetPushed, (void *)apt, 1, buttonIsDefault );
+	    appPageToolSetPushed, (void *)apt, 1, colspan, buttonIsDefault );
 
     return row;
     }
@@ -464,12 +469,12 @@ void * appMakePageTool(	EditApplication *	ea,
 
     appPaperChooserFillMenu( &(apt->aptPaperChooser), aptr.aptrCustom );
 
-    appShowShellWidget( apt->aptTopWidget );
+    appShowShellWidget( ea, apt->aptTopWidget );
 
     appPaperChooserRetreshMenuWidth( &(apt->aptPaperChooser) );
 
-    if  ( appSetDrawingDataForWidget( apt->aptPageDrawing, magnification,
-						    &(apt->aptDrawingData) ) )
+    if  ( appSetDrawingDataForWidget( apt->aptPageDrawing, (AppColors *)0,
+				    magnification, &(apt->aptDrawingData) ) )
 	{ LDEB(1);				}
     else{ apt->aptDrawingDataAllocated= 1;	}
 
@@ -486,7 +491,7 @@ void appShowPageTool(		EditApplication *	ea )
     {
     AppPageTool *		apt= (AppPageTool *)ea->eaPageTool;
 
-    appShowShellWidget( apt->aptTopWidget );
+    appShowShellWidget( ea, apt->aptTopWidget );
     }
 
 /************************************************************************/
@@ -495,7 +500,6 @@ void appShowPageTool(		EditApplication *	ea )
 /*									*/
 /************************************************************************/
 
-
 void appPageToolSetProperties(	void *				voidapt,
 				const DocumentGeometry *	dg )
     {
@@ -510,7 +514,7 @@ void appPageToolSetProperties(	void *				voidapt,
     appMarginToolShowMargins( &(apt->aptMarginTool),
 				apt->aptUnitType, &(apt->aptGeometryChosen) );
 
-    appExposeRectangle( add, 0, 0, 0, 0 );
+    appExposeDrawingData( add );
 
     return;
     }
