@@ -2,6 +2,8 @@
 /*									*/
 /*  Printing related functionality.					*/
 /*									*/
+/*  TODO: Look at http://www.pwg.org/standards.html 			*/
+/*									*/
 /************************************************************************/
 
 #   include	"appFrameConfig.h"
@@ -13,6 +15,7 @@
 
 #   include	<locale.h>
 #   include	<unistd.h>
+#   include	<errno.h>
 
 #   include	<appSystem.h>
 #   include	<sioPipe.h>
@@ -23,6 +26,9 @@
 #   include	<appPaper.h>
 #   include	"appFrame.h"
 
+#   include	"appPrintJob.h"
+#   include	<psNup.h>
+
 #   include	<appDebugon.h>
 
 #   ifdef HAVE_MKSTEMP
@@ -44,10 +50,7 @@
 /************************************************************************/
 
 int appCallPrintFunction(	SimpleOutputStream *		sos,
-				const PrintJob *		pj,
-				const PrintGeometry *		pg,
-				int				firstPage,
-				int				lastPage )
+				const PrintJob *		pj )
     {
     EditApplication *	ea= pj->pjApplication;
     int			rval= 0;
@@ -55,7 +58,7 @@ int appCallPrintFunction(	SimpleOutputStream *		sos,
     /*  1  */
     setlocale( LC_NUMERIC, "C" );
 
-    if  ( (*ea->eaPrintDocument)( sos, pj, pg, firstPage, lastPage ) )
+    if  ( (*ea->eaPrintDocument)( sos, pj, pj->pjPrintGeometry ) )
 	{ LDEB(1); rval= -1;	}
 
     /*  1  */
@@ -115,7 +118,9 @@ static int appPrintMakeTempfile(	int *			pFd,
     {
     int				fd;
 
-    tmpnam( scratchName );
+    if  ( ! tmpnam( scratchName ) )
+	{ LDEB(1); return -1;	}
+
     fd= -1;
 
     *pFd= fd;
@@ -183,10 +188,7 @@ static char * appDocBuildTmpfileCommand(	const char *	command,
     }
 
 int appPrintDocument(	int				printer,
-			const PrintJob *		pj,
-			const PrintGeometry *		pg,
-			int				firstPage,
-			int				lastPage )
+			const PrintJob *		pj )
     {
     EditApplication *		ea= pj->pjApplication;
     PrintDestination *		pd= ea->eaPrintDestinations+ printer;
@@ -205,7 +207,7 @@ int appPrintDocument(	int				printer,
 	    if  ( ! sos )
 		{ SXDEB(pd->pdCommand,sos); return -1;	}
 
-	    appCallPrintFunction( sos, pj, pg, firstPage, lastPage );
+	    appCallPrintFunction( sos, pj );
 
 	    if  ( sioOutClose( sos ) )
 		{ SDEB(pd->pdCommand); return -1;	}
@@ -236,7 +238,7 @@ int appPrintDocument(	int				printer,
 		return -1;
 		}
 
-	    appCallPrintFunction( sos, pj, pg, firstPage, lastPage );
+	    appCallPrintFunction( sos, pj );
 
 	    if  ( appPrintCloseTempfile( fd, sos ) )
 		{
@@ -245,7 +247,8 @@ int appPrintDocument(	int				printer,
 		return -1;
 		}
 
-	    system( command );
+	    if  ( system( command ) )
+		{ SLDEB(command,errno);	}
 	    free( command );
 	    return 0;
 
@@ -256,65 +259,6 @@ int appPrintDocument(	int				printer,
 	}
     }
 
-static char * appDocBuildFaxCommand(		const char *	faxCommand,
-						int		commandLen,
-						const char *	fileName,
-						const char *	faxNumber,
-						char *		title,
-						int		fileCount,
-						int		faxCount,
-						int		titleCount )
-    {
-    char *		command;
-    const char *	s;
-    char *		to;
-
-    int			fileLen;
-    int			faxLen;
-    int			titleLen;
-
-    if  ( ! title )
-	{ title= "";	}
-
-    fileLen= strlen( fileName );
-    faxLen= strlen( faxNumber );
-    titleLen= strlen( title );
-
-    to= command= (char *)malloc( commandLen+
-					fileCount* fileLen+
-					faxCount* faxLen+
-					titleCount* titleLen+ 1 );
-    if  ( ! command )
-	{ XDEB(command); return (char *)0;	}
-
-    s= faxCommand;
-    while( *s )
-	{
-	if  ( s[0] == '%' && s[1] == 'f' )
-	    {
-	    strcpy( to, fileName ); to += fileLen;
-	    s += 2; continue;
-	    }
-
-	if  ( s[0] == '%' && s[1] == 'n' )
-	    {
-	    strcpy( to, faxNumber ); to += faxLen;
-	    s += 2; continue;
-	    }
-
-	if  ( s[0] == '%' && s[1] == 't' )
-	    {
-	    strcpy( to, title ); to += titleLen;
-	    s += 2; continue;
-	    }
-
-	*(to++)= *(s++);
-	}
-    *to= '\0';
-
-    return command;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Fill a PrintJob for an EditDocument.				*/
@@ -322,132 +266,38 @@ static char * appDocBuildFaxCommand(		const char *	faxCommand,
 /************************************************************************/
 
 void appPrintJobForEditDocument(	PrintJob *		pj,
-					EditDocument *		ed )
+					EditDocument *		ed,
+					const PrintGeometry *	pg )
     {
     EditApplication *	ea= ed->edApplication;
 
     pj->pjPrivateData= ed->edPrivateData;
     pj->pjFormat= ed->edFormat;
-    pj->pjUsePostScriptFilters= ea->eaUsePostScriptFilters;
-    pj->pjUsePostScriptIndexedImages= ea->eaUsePostScriptIndexedImages;
-    pj->pjDrawingData= &(ed->edDrawingData);
+    pj->pjDrawingData= &(ed->edDocumentWidget.dwDrawingData);
     pj->pjApplication= ea;
     pj->pjTitle= ed->edTitle;
+    pj->pjPrintGeometry= pg;
 
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Fax a document.							*/
-/*									*/
-/************************************************************************/
-
-int appFaxDocument(	EditDocument *			ed,
-			const char *			faxNumber,
-			const PrintGeometry *		pg,
-			int				firstPage,
-			int				lastPage )
+void appApplicationSettingsToPrintGeometry(
+					PrintGeometry *		pg,
+					EditApplication *	ea )
     {
-    EditApplication *		ea= ed->edApplication;
-    int				fileCount= 0;
-    int				faxCount= 0;
-    int				titleCount= 0;
+    pg->pgUsePostScriptFilters= ea->eaUsePostScriptFilters;
+    pg->pgUsePostScriptIndexedImages= ea->eaUsePostScriptIndexedImages;
+    pg->pgEmbedFonts= ea->eaEmbedFonts > 0;
+    pg->pg7Bits= ea->ea7BitsPostScript;
 
-    int				commandLen;
+    pg->pgSkipEmptyPages= ea->eaSkipEmptyPages;
+    pg->pgSkipBlankPages= ea->eaSkipBlankPages;
+    pg->pgOmitHeadersOnEmptyPages= ea->eaOmitHeadersOnEmptyPages;
 
-    char			scratchName[L_tmpnam+1];
+    if  ( ea->eaCustomPsSetupFilename )
+	{ pg->pgCustomPsSetupFilename= strdup( ea->eaCustomPsSetupFilename ); }
 
-    char *			command;
-
-    int				fd= -1;
-    SimpleOutputStream *	sos= (SimpleOutputStream *)0;
-
-    PrintJob			pj;
-
-    if  ( ! ea->eaFaxCommand )
-	{ XDEB(ea->eaFaxCommand); return -1;	}
-
-    appPrintJobForEditDocument( &pj, ed );
-
-    {
-    const char * s= ea->eaFaxCommand;
-
-    while( *s )
-	{
-	if  ( s[0] == '%' && s[1] == 'f' )
-	    { fileCount++; s += 2; continue;	}
-
-	if  ( s[0] == '%' && s[1] == 'n' )
-	    { faxCount++; s += 2; continue;	}
-
-	if  ( s[0] == '%' && s[1] == 't' )
-	    { titleCount++; s += 2; continue;	}
-
-	s++;
-	}
-
-    commandLen= s- ea->eaFaxCommand;
-    }
-
-    if  ( faxCount == 0 )
-	{ LDEB(faxCount); return -1;	}
-
-    if  ( fileCount > 0 )
-	{
-	if  ( appPrintMakeTempfile( &fd, scratchName ) )
-	    { LDEB(1); return -1;	}
-
-	command= appDocBuildFaxCommand( ea->eaFaxCommand, commandLen,
-					scratchName, faxNumber, ed->edTitle,
-					fileCount, faxCount, titleCount );
-
-	if  ( ! command )
-	    {
-	    LXDEB(commandLen,command);
-	    appPrintCloseTempfile( fd, sos );
-	    return -1;
-	    }
-
-	if  ( appPrintOpenTempfile( &sos, fd, scratchName ) )
-	    {
-	    SLDEB(scratchName,fd);
-	    appPrintCloseTempfile( fd, sos );
-	    free( command );
-	    return -1;
-	    }
-
-	appCallPrintFunction( sos, &pj, pg, firstPage, lastPage );
-
-	if  ( appPrintCloseTempfile( fd, sos ) )
-	    {
-	    SLDEB(scratchName,fd);
-	    free( command );
-	    return -1;
-	    }
-
-	system( command );
-	}
-    else{
-	scratchName[0]= '\0';
-
-	command= appDocBuildFaxCommand( ea->eaFaxCommand, commandLen,
-					scratchName, faxNumber, ed->edTitle,
-					fileCount, faxCount, titleCount );
-
-	sos= sioOutPipeOpen( command );
-	if  ( ! sos )
-	    { SXDEB(command,sos); free( command ); return -1;	}
-
-	appCallPrintFunction( sos, &pj, pg, firstPage, lastPage );
-
-	if  ( sioOutClose( sos ) )
-	    { SDEB(command); return -1;	}
-	}
-
-    free( command );
-
-    return 0;
+    return;
     }
 
 /************************************************************************/
@@ -457,6 +307,7 @@ int appFaxDocument(	EditDocument *			ed,
 /************************************************************************/
 
 static int appPrintJobForCommand(	PrintJob *		pj,
+					PrintGeometry *		pg,
 					AppDrawingData *	add,
 					EditApplication *	ea,
 					const char *		fromName )
@@ -476,35 +327,28 @@ static int appPrintJobForCommand(	PrintJob *		pj,
     if  ( (*ea->eaOpenDocument)( ea, pj->pjPrivateData, &(pj->pjFormat),
 					    ea->eaToplevel.atTopWidget,
 					    (APP_WIDGET)0, fromName ) )
-	{ SDEB(fromName); return -1; }
+	{ SDEB((char *)fromName); return -1; }
 
     if  ( ea->eaToplevel.atTopWidget )
 	{
 	appGetFactors( ea, &screenPixelsPerMM, &verPixPerMM, &xfac, &yfac );
 
-	appSetDrawingEnvironment( add, ea->eaMagnification, xfac,
+	appSetScreenDrawingEnvironment( add,
+			    (AppColors *)0,
+			    ea->eaMagnification, xfac,
 			    screenPixelsPerMM,
-			    ea->eaAfmDirectory,
-			    ea->eaGhostscriptFontmap,
-			    ea->eaGhostscriptFontToXmapping,
-			    ea->eaToplevel.atTopWidget );
+			    &(ea->eaPostScriptFontList) );
 
 	add->addPageGapPixels= (int)( ea->eaPageGapMM* verPixPerMM );
 	}
     else{
-	add->addPhysicalFontList.apflAfmDirectory= ea->eaAfmDirectory;
-	add->addPhysicalFontList.apflGhostscriptFontmap=
-					    ea->eaGhostscriptFontmap;
-	add->addPhysicalFontList.apflGhostscriptFontToXmapping=
-					    ea->eaGhostscriptFontToXmapping;
+	appDrawSetPostScriptFontList( add, &(ea->eaPostScriptFontList) );
 	}
 
     pj->pjDrawingData= add;
     pj->pjApplication= ea;
     pj->pjTitle= fromName;
-
-    pj->pjUsePostScriptFilters= ea->eaUsePostScriptFilters;
-    pj->pjUsePostScriptIndexedImages= ea->eaUsePostScriptIndexedImages;
+    pj->pjPrintGeometry= pg;
 
     return 0;
     }
@@ -519,7 +363,7 @@ static void appPrintFinishCommandRun(	EditApplication *	ea,
 					PrintJob *		pj,
 					AppDrawingData *	add )
     {
-    (*ea->eaFreeDocument)( pj->pjPrivateData, pj->pjFormat, add );
+    (*ea->eaFreeDocument)( pj->pjPrivateData, pj->pjFormat );
 
     appCleanDrawingData( add );
 
@@ -547,9 +391,12 @@ static int appPrintStartCommandRun(	EditApplication *	ea,
 	    { SDEB(paperString); return -1;	}
 	}
 
-    if  ( appPrintJobForCommand( pj, add, ea, fromName ) )
+    if  ( appPrintJobForCommand( pj, pg, add, ea, fromName ) )
 	{ SDEB(fromName); return -1;	}
 
+    if  ( ea->eaSuggestNup )
+	{ (*ea->eaSuggestNup)( pg, pj->pjPrivateData );	}
+
     if  ( (*ea->eaLayoutDocument)( pj->pjPrivateData, pj->pjFormat, add,
 						    &(pg->pgSheetGeometry) ) )
 	{
@@ -580,14 +427,15 @@ int appPrintToFile(	EditApplication *	ea,
     PrintGeometry		pg;
     PrintJob			pj;
 
-    utilInitPrintGeometry( &pg );
+    psInitPrintGeometry( &pg );
+    appApplicationSettingsToPrintGeometry( &pg, ea );
     appInitDrawingData( &add );
 
     if  ( ! strcmp( fromName, toName ) )
-	{ SSDEB(fromName,toName); return -1;	}
+	{ SSDEB(fromName,toName); rval= -1; goto ready;	}
 
     if  ( appPrintStartCommandRun( ea, &pj, &pg, &add, paperString, fromName ) )
-	{ SDEB(fromName); return -1;	}
+	{ SDEB(fromName); rval= -1; goto ready;	}
 
     if  ( ! strcmp( toName, "-" ) )
 	{ sos= sioOutStdoutOpen();		}
@@ -597,21 +445,18 @@ int appPrintToFile(	EditApplication *	ea,
 	{
 	SXDEB(toName,sos);
 	appPrintFinishCommandRun( ea, &pj, &add );
-	return -1;
+	rval= -1; goto ready;
 	}
 
-    {
-    const int	firstPage= -1;
-    const int	lastPage= -1;
-
-    if  ( appCallPrintFunction( sos, &pj, &pg, firstPage, lastPage ) )
+    if  ( appCallPrintFunction( sos, &pj ) )
 	{ SDEB(fromName); rval= -1;	}
-    }
 
     sioOutClose( sos );
 
     appPrintFinishCommandRun( ea, &pj, &add );
 
+  ready:
+    psCleanPrintGeometry( &pg );
     return rval;
     }
 
@@ -634,16 +479,13 @@ int appPrintToPrinter(	EditApplication *	ea,
     PrintGeometry	pg;
     PrintJob		pj;
 
-    utilInitPrintGeometry( &pg );
+    psInitPrintGeometry( &pg );
+    appApplicationSettingsToPrintGeometry( &pg, ea );
     appInitDrawingData( &add );
 
-    if  ( ea->eaPrintDestinationCount == 0				&&
-	  utilPrinterGetPrinters( &(ea->eaPrintDestinationCount),
-				    &(ea->eaDefaultPrintDestination),
-				    &(ea->eaPrintDestinations),
-				    ea->eaCustomPrintCommand,
-				    ea->eaCustomPrinterName )		)
-	{ LDEB(1); return -1;	}
+    if  ( ! ea->eaPrintDestinationsCollected	&&
+	  appGetPrintDestinations( ea )		)
+	{ LDEB(1); rval= -1; goto ready;	}
 
     printer= ea->eaDefaultPrintDestination;
 
@@ -655,27 +497,27 @@ int appPrintToPrinter(	EditApplication *	ea,
 	pd= ea->eaPrintDestinations;
 	for ( i= 0; i < ea->eaPrintDestinationCount; pd++, i++ )
 	    {
-	    if  ( ! strcmp( fromName, pd->pdPrinterName ) )
+	    if  ( ! strcmp( toName, pd->pdPrinterName ) )
 		{ printer= i; break;	}
 	    }
+
+	if  ( i >= ea->eaPrintDestinationCount )
+	    { SDEB(toName); rval= -1; goto ready;	}
 	}
 
     if  ( printer < 0 )
-	{ LDEB(printer); return -1;	}
+	{ LDEB(printer); rval= -1; goto ready;	}
 
     if  ( appPrintStartCommandRun( ea, &pj, &pg, &add, paperString, fromName ) )
-	{ SDEB(fromName); return -1;	}
+	{ SDEB(fromName); rval= -1; goto ready;	}
 
-    {
-    const int	firstPage= -1;
-    const int	lastPage= -1;
-
-    if  ( appPrintDocument( printer, &pj, &pg, firstPage, lastPage ) )
+    if  ( appPrintDocument( printer, &pj ) )
 	{ SDEB(fromName); rval= -1;	}
-    }
 
     appPrintFinishCommandRun( ea, &pj, &add );
 
+  ready:
+    psCleanPrintGeometry( &pg );
     return rval;
     }
 
