@@ -1,12 +1,14 @@
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
+#   include	<string.h>
+
 #   include	<appDebugon.h>
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
-#   include	<appGeoString.h>
+#   include	<geoString.h>
 
 /************************************************************************/
 /*									*/
@@ -29,7 +31,7 @@ int appGetLengthFromTextWidget(		APP_WIDGET	w,
     char *	s;
 
     s= appGetStringFromTextWidget( w );
-    if  ( appGeoLengthFromString( s, defaultUnit, &value )	||
+    if  ( geoLengthFromString( s, defaultUnit, &value )	||
 	  ( ! adaptToMin && value < minValue )			||
 	  ( ! adaptToMax && value > maxValue )			)
 	{
@@ -38,7 +40,7 @@ int appGetLengthFromTextWidget(		APP_WIDGET	w,
 	appRefuseTextValue( w ); return -1;
 	}
 
-    appGeoLengthToString( scratch, *pValue, defaultUnit );
+    geoLengthToString( scratch, *pValue, defaultUnit );
     if  ( ! strcmp( scratch, s ) )
 	{ value= *pValue;	}
 
@@ -149,6 +151,18 @@ int appGetDoubleFromTextWidget(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
+void appLengthToTextWidget(		APP_WIDGET	w,
+					int		l,
+					int		unit )
+    {
+    char	scratch[30];
+
+    geoLengthToString( scratch, l, unit );
+    appStringToTextWidget( w, scratch );
+
+    return;
+    }
+
 void appIntegerToTextWidget(		APP_WIDGET	w,
 					int		n )
     {
