@@ -15,7 +15,7 @@
 #   include	<utilPropMask.h>
 #   include	<utilTextAttribute.h>
 
-#   include	<docExpandedTextAttribute.h>
+#   include	"docExpandedTextAttribute.h"
 
 /************************************************************************/
 /*									*/
@@ -25,10 +25,13 @@
 
 void docInitExpandedTextAttribute(	ExpandedTextAttribute *	eta )
     {
-    eta->etaFontFamilyName= (char *)0;
-    eta->etaFontEncoding= 0;
+    eta->etaFontNumber= -1;
 
     eta->etaFontSizeHalfPoints= 0;
+    eta->etaTextStyleNumber= 0;
+
+    eta->etaBorderNumber= 0;
+    eta->etaShadingNumber= 0;
 
     bmInitRGB8Color( &(eta->etaTextColor) );
     eta->etaTextColorExplicit= 0;
@@ -52,123 +55,9 @@ void docInitExpandedTextAttribute(	ExpandedTextAttribute *	eta )
 
 void docCleanExpandedTextAttribute(	ExpandedTextAttribute *	eta )
     {
-    if  ( eta->etaFontFamilyName )
-	{ free( eta->etaFontFamilyName );	}
-
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Set the family name of an ExpandedTextAttribute.			*/
-/*									*/
-/************************************************************************/
-
-int docExpandedTextAttributeSetFamilyName(
-			    ExpandedTextAttribute *		etaTo,
-			    int *				pChanged,
-			    const char *			familyName )
-    {
-    int		changed= 0;
-
-    if  ( familyName )
-	{
-	if  ( ! etaTo->etaFontFamilyName				||
-	      strcmp( familyName,
-					etaTo->etaFontFamilyName )	)
-	    {
-	    char *	fresh= strdup( familyName );
-
-	    if  ( ! fresh )
-		{ XDEB(fresh); return -1;	}
-	    else{
-		if  ( etaTo->etaFontFamilyName )
-		    { free( etaTo->etaFontFamilyName );	}
-
-		etaTo->etaFontFamilyName= fresh;
-		changed= 1;
-		}
-	    }
-	}
-    else{
-	if  ( etaTo->etaFontFamilyName )
-	    {
-	    free( etaTo->etaFontFamilyName );
-	    etaTo->etaFontFamilyName= (char *)0;
-	    changed= 1;
-	    }
-	}
-
-    *pChanged= changed;
-    return 0;
-    }
-
-void docExpandedTextAttributeSetFace(
-			    ExpandedTextAttribute *		etaTo,
-			    int *				pChanged,
-			    const AppFontTypeface *		aft )
-    {
-    int		changed= 0;
-
-    if  ( etaTo->etaFontIsBold != aft->aftIsBold )
-	{
-	etaTo->etaFontIsBold= aft->aftIsBold;
-	changed= 1;
-	}
-
-    if  ( etaTo->etaFontIsSlanted != aft->aftIsSlanted )
-	{
-	etaTo->etaFontIsSlanted= aft->aftIsSlanted;
-	changed= 1;
-	}
-
-    *pChanged= changed;
-    return;
-    }
-
-int docExpandedTextAttributeSetFamilyAndEncoding(
-			    ExpandedTextAttribute *		etaTo,
-			    int *				pChanged,
-			    const AppFontFamily *		aff,
-			    int					encoding )
-    {
-    int		changed= 0;
-
-    if  ( aff )
-	{
-	if  ( docExpandedTextAttributeSetFamilyName( etaTo, &changed,
-						    aff->affFontFamilyName ) )
-	    { LDEB(1); return -1;	}
-
-	if  ( encoding < 0					||
-	      encoding >= ENCODINGps_COUNT			||
-	      ! aff->affSupportedCharsets[encoding].scSupported	)
-	    { encoding= aff->affDefaultEncoding;	}
-
-	/*  avoid inconsistencies  */
-	if  ( encoding < 0					||
-	      encoding >= ENCODINGps_COUNT			||
-	      ! aff->affSupportedCharsets[encoding].scSupported	)
-	    { LDEB(encoding); encoding= 0;	}
-
-	if  ( etaTo->etaFontEncoding != encoding )
-	    {
-	    etaTo->etaFontEncoding= encoding;
-	    changed= 1;
-	    }
-	}
-    else{
-	if  ( docExpandedTextAttributeSetFamilyName( etaTo, &changed,
-							(const char *)0 ) )
-	    { LDEB(1); return -1;	}
-
-	etaTo->etaFontEncoding= 0;
-	}
-
-    *pChanged= changed;
-    return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Copy an expanded text attribute.					*/
@@ -179,15 +68,13 @@ int docCopyExpandedTextAttribute(
 				ExpandedTextAttribute *		etaTo,
 				const ExpandedTextAttribute *	etaFrom )
     {
-    int		changed= 0;
-
-    if  ( docExpandedTextAttributeSetFamilyName( etaTo, &changed,
-					    etaFrom->etaFontFamilyName ) )
-	{ LDEB(1);	}
-
-    etaTo->etaFontEncoding= etaFrom->etaFontEncoding;
+    etaTo->etaFontNumber= etaFrom->etaFontNumber;
 
     etaTo->etaFontSizeHalfPoints= etaFrom->etaFontSizeHalfPoints;
+    etaTo->etaTextStyleNumber= etaFrom->etaTextStyleNumber;
+
+    etaTo->etaBorderNumber= etaFrom->etaBorderNumber;
+    etaTo->etaShadingNumber= etaFrom->etaShadingNumber;
 
     if  ( etaTo->etaTextColorExplicit )
 	{
@@ -236,20 +123,12 @@ void docUpdateExpandedTextAttribute(
     {
     PropertyMask	doneMask= *pDoneMask;
 
-    if  ( PROPmaskISSET( setMask, TApropFONTFAMILY ) )
+    if  ( PROPmaskISSET( setMask, TApropDOC_FONT_NUMBER ) )
 	{
-	int		changed= 0;
-
-	if  ( docExpandedTextAttributeSetFamilyName( etaTo, &changed,
-						etaFrom->etaFontFamilyName ) )
-	    { LDEB(1);	}
-	if  ( changed )
-	    { PROPmaskADD( &doneMask, TApropFONTFAMILY );	}
-
-	if  ( etaTo->etaFontEncoding != etaFrom->etaFontEncoding )
+	if  ( etaTo->etaFontNumber != etaFrom->etaFontNumber )
 	    {
-	    etaTo->etaFontEncoding= etaFrom->etaFontEncoding;
-	    PROPmaskADD( &doneMask, TApropFONTFAMILY );
+	    etaTo->etaFontNumber= etaFrom->etaFontNumber;
+	    PROPmaskADD( &doneMask, TApropDOC_FONT_NUMBER );
 	    }
 	}
 
@@ -260,6 +139,27 @@ void docUpdateExpandedTextAttribute(
 	PROPmaskADD( &doneMask, TApropFONTSIZE );
 	}
 
+    if  ( PROPmaskISSET( setMask, TApropTEXT_STYLE )			&&
+	  etaTo->etaTextStyleNumber != etaFrom->etaTextStyleNumber )
+	{
+	etaTo->etaTextStyleNumber= etaFrom->etaTextStyleNumber;
+	PROPmaskADD( &doneMask, TApropTEXT_STYLE );
+	}
+
+    if  ( PROPmaskISSET( setMask, TApropBORDER )			&&
+	  etaTo->etaBorderNumber != etaFrom->etaBorderNumber )
+	{
+	etaTo->etaBorderNumber= etaFrom->etaBorderNumber;
+	PROPmaskADD( &doneMask, TApropBORDER );
+	}
+
+    if  ( PROPmaskISSET( setMask, TApropSHADING )			&&
+	  etaTo->etaShadingNumber != etaFrom->etaShadingNumber )
+	{
+	etaTo->etaShadingNumber= etaFrom->etaShadingNumber;
+	PROPmaskADD( &doneMask, TApropSHADING );
+	}
+
     if  ( PROPmaskISSET( setMask, TApropTEXT_COLOR ) )
 	{
 	if  ( etaTo->etaTextColorExplicit )
@@ -358,41 +258,11 @@ void docExpandTextAttribute(	PropertyMask *			pDoneMask,
     {
     PropertyMask	doneMask= *pDoneMask;
 
-    if  ( PROPmaskISSET( setMask, TApropFONTFAMILY ) )
+    if  ( PROPmaskISSET( setMask, TApropDOC_FONT_NUMBER )	&&
+	  etaTo->etaFontNumber != taFrom->taFontNumber		)
 	{
-	if  ( taFrom->taFontNumber >= 0			&&
-	      taFrom->taFontNumber < dfl->dflCount	)
-	    {
-	    const DocumentFont *	df= dfl->dflFonts+ taFrom->taFontNumber;
-
-	    int				changed= 0;
-
-	    if  ( docExpandedTextAttributeSetFamilyName( etaTo, &changed,
-								df->dfName ) )
-		{ LDEB(1);	}
-	    if  ( changed )
-		{ PROPmaskADD( &doneMask, TApropFONTFAMILY );	}
-
-
-	    if  ( etaTo->etaFontEncoding != df->dfEncodingUsed )
-		{
-		etaTo->etaFontEncoding= df->dfEncodingUsed;
-		PROPmaskADD( &doneMask, TApropFONTFAMILY );
-		}
-	    }
-	else{
-	    int				changed= 0;
-
-	    LLDEB(taFrom->taFontNumber,dfl->dflCount);
-
-	    if  ( docExpandedTextAttributeSetFamilyName( etaTo, &changed,
-							    (const char *)0 ) )
-		{ LDEB(1);	}
-	    if  ( changed )
-		{ PROPmaskADD( &doneMask, TApropFONTFAMILY );	}
-
-	    etaTo->etaFontEncoding= 0;
-	    }
+	etaTo->etaFontNumber= taFrom->taFontNumber;
+	PROPmaskADD( &doneMask, TApropDOC_FONT_NUMBER );
 	}
 
     if  ( PROPmaskISSET( setMask, TApropFONTSIZE )			&&
@@ -402,6 +272,27 @@ void docExpandTextAttribute(	PropertyMask *			pDoneMask,
 	PROPmaskADD( &doneMask, TApropFONTSIZE );
 	}
 
+    if  ( PROPmaskISSET( setMask, TApropTEXT_STYLE )		&&
+	  etaTo->etaTextStyleNumber != taFrom->taTextStyleNumber )
+	{
+	etaTo->etaTextStyleNumber= taFrom->taTextStyleNumber;
+	PROPmaskADD( &doneMask, TApropTEXT_STYLE );
+	}
+
+    if  ( PROPmaskISSET( setMask, TApropBORDER )		&&
+	  etaTo->etaBorderNumber != taFrom->taBorderNumber )
+	{
+	etaTo->etaBorderNumber= taFrom->taBorderNumber;
+	PROPmaskADD( &doneMask, TApropBORDER );
+	}
+
+    if  ( PROPmaskISSET( setMask, TApropSHADING )		&&
+	  etaTo->etaShadingNumber != taFrom->taShadingNumber )
+	{
+	etaTo->etaShadingNumber= taFrom->taShadingNumber;
+	PROPmaskADD( &doneMask, TApropSHADING );
+	}
+
     if  ( PROPmaskISSET( setMask, TApropTEXT_COLOR ) )
 	{
 	if  ( taFrom->taTextColorNumber == 0 )
@@ -492,32 +383,16 @@ void docIndirectTextAttribute(	PropertyMask *			pDoneMask,
 				TextAttribute *			taTo,
 				const ExpandedTextAttribute *	etaFrom,
 				const PropertyMask *		setMask,
-				DocumentFontList *		dfl,
 				RGB8Color **			pColors,
 				int *				pColorCount )
     {
     PropertyMask	doneMask= *pDoneMask;
 
-    if  ( PROPmaskISSET( setMask, TApropFONTFAMILY ) )
+    if  ( PROPmaskISSET( setMask, TApropDOC_FONT_NUMBER )	&&
+	  taTo->taFontNumber != etaFrom->etaFontNumber		)
 	{
-	if  ( etaFrom->etaFontFamilyName )
-	    {
-	    int		fontNumber;
-
-	    fontNumber= docGetFontByName( dfl, 
-				    etaFrom->etaFontFamilyName,
-						etaFrom->etaFontEncoding );
-	    if  ( fontNumber < 0 )
-		{ LDEB(fontNumber);	}
-	    else{
-		if  ( taTo->taFontNumber != fontNumber )
-		    {
-		    taTo->taFontNumber= fontNumber;
-		    PROPmaskADD( &doneMask, TApropFONTFAMILY );
-		    }
-		}
-	    }
-	else{ XDEB(etaFrom->etaFontFamilyName);	}
+	taTo->taFontNumber= etaFrom->etaFontNumber;
+	PROPmaskADD( &doneMask, TApropDOC_FONT_NUMBER );
 	}
 
     if  ( PROPmaskISSET( setMask, TApropFONTSIZE )			&&
@@ -527,6 +402,27 @@ void docIndirectTextAttribute(	PropertyMask *			pDoneMask,
 	PROPmaskADD( &doneMask, TApropFONTSIZE );
 	}
 
+    if  ( PROPmaskISSET( setMask, TApropTEXT_STYLE )			&&
+	  taTo->taTextStyleNumber != etaFrom->etaTextStyleNumber )
+	{
+	taTo->taTextStyleNumber= etaFrom->etaTextStyleNumber;
+	PROPmaskADD( &doneMask, TApropTEXT_STYLE );
+	}
+
+    if  ( PROPmaskISSET( setMask, TApropBORDER )			&&
+	  taTo->taBorderNumber != etaFrom->etaBorderNumber )
+	{
+	taTo->taBorderNumber= etaFrom->etaBorderNumber;
+	PROPmaskADD( &doneMask, TApropBORDER );
+	}
+
+    if  ( PROPmaskISSET( setMask, TApropSHADING )			&&
+	  taTo->taShadingNumber != etaFrom->etaShadingNumber )
+	{
+	taTo->taShadingNumber= etaFrom->etaShadingNumber;
+	PROPmaskADD( &doneMask, TApropSHADING );
+	}
+
     if  ( PROPmaskISSET( setMask, TApropTEXT_COLOR ) )
 	{
 	const int		maxColors= 256;
@@ -612,6 +508,7 @@ void docIndirectTextAttribute(	PropertyMask *			pDoneMask,
 void docExpandedAttributeToString(
 			char *				target,
 			const PropertyMask *		updMask,
+			const DocumentFontList *	dfl,
 			const ExpandedTextAttribute *	etaNew )
     {
     char		sizeStr[40];
@@ -627,8 +524,14 @@ void docExpandedAttributeToString(
 
     strcpy( sizeStr, star );
 
-    if  ( PROPmaskISSET( updMask, TApropFONTFAMILY ) )
-	{ familyName= etaNew->etaFontFamilyName;	}
+    if  ( PROPmaskISSET( updMask, TApropDOC_FONT_NUMBER )	&&
+	  etaNew->etaFontNumber >= 0				)
+	{
+	DocumentFont *	df= docFontListGetFontByNumber( dfl,
+						    etaNew->etaFontNumber );
+	if  ( df )
+	    { familyName= df->dfName;	}
+	}
 
     if  ( ! familyName )
 	{ familyName= star;	}
@@ -682,9 +585,10 @@ void docExpandedAttributeToString(
 /************************************************************************/
 
 int docExpandedAttributeFromString(	
-				const char *			attributeString,
-				PropertyMask *			pSetMask,
-				ExpandedTextAttribute *		pEtaSet )
+			PropertyMask *			pSetMask,
+			ExpandedTextAttribute *		pEtaSet,
+			DocumentFontList *		dfl,
+			const char *			attributeString )
     {
     PropertyMask		setMask;
     ExpandedTextAttribute	etaSet;
@@ -693,7 +597,7 @@ int docExpandedAttributeFromString(
     int				rval= 0;
 
     docInitExpandedTextAttribute( &etaSet );
-    PROPmaskCLEAR( &setMask );
+    utilPropMaskClear( &setMask );
 
     /*  1  */
     if  ( *s == '*' )
@@ -701,7 +605,7 @@ int docExpandedAttributeFromString(
     else{
 	const char *	familyName= s;
 	char *		tmp;
-	int		changed= 0;
+	int		fontNumber;
 
 	familyName= s;
 
@@ -714,12 +618,15 @@ int docExpandedAttributeFromString(
 
 	strncpy( tmp, familyName, s- familyName )[s- familyName]= '\0';
 
-	if  ( docExpandedTextAttributeSetFamilyName( &etaSet, &changed, tmp ) )
-	    { LDEB(1); rval= -1; goto ready;	}
+
+	fontNumber= docGetFontByName( dfl, tmp );
+	if  ( fontNumber < 0 )
+	    { SLDEB(tmp,fontNumber); free( tmp ); return -1; }
 
 	free( tmp );
 
-	PROPmaskADD( &setMask, TApropFONTFAMILY );
+	etaSet.etaFontNumber= fontNumber;
+	PROPmaskADD( &setMask, TApropDOC_FONT_NUMBER );
 	}
 
     if  ( *s )
