@@ -1,3 +1,7 @@
+
+#   ifndef	APP_CGI_IN_H
+#   define	APP_CGI_IN_H	1
+
 #   include	<appTagval.h>
 
 /************************************************************************/
@@ -8,11 +12,12 @@
 
 typedef struct CGIRequest
     {
-    TaggedValueList *	cgirHeaderValues;
-    TaggedValueList *	cgirEnvironmentValues;
-    TaggedValueList *	cgirStdinValues;
-    TaggedValueList *	cgirQueryValues;
+    void *		cgirHeaderValues;
+    void *		cgirEnvironmentValues;
+    void *		cgirQueryValues;
+    void *		cgirCookies;
     const char *	cgirRequestMethod;
+    int			cgirStdinUsed;
     } CGIRequest;
 
 /************************************************************************/
@@ -24,3 +29,5 @@ typedef struct CGIRequest
 extern CGIRequest *	appCgiInGetRequest( void );
 
 extern void appCgiInFreeRequest( CGIRequest * cgir );
+
+#   endif	/*  APP_CGI_IN_H  */
