@@ -9,15 +9,15 @@
 #   include	<string.h>
 
 #   include	"appPaper.h"
-#   include	"appGeoString.h"
+#   include	"geoString.h"
 #   include	<appDebugon.h>
 
 typedef struct PaperSize
     {
-    char *	psMnemonic;
-    char *	psLabel;
-    int		psWideTwips;
-    int		psHighTwips;
+    const char *	psMnemonic;
+    const char *	psLabel;
+    int			psWideTwips;
+    int			psHighTwips;
     } PaperSize;
 
 static PaperSize UtilPaperSizes[]=
@@ -28,6 +28,9 @@ static PaperSize UtilPaperSizes[]=
     { "letter",		"Letter: 8 1/2 x 11 \"",	12240,	15840,	},
     { "legal",		"Legal: 8 1/2 x 14 \"",		12240,	20720,	},
     { "executive",	"Executive: 7 1/4 x 10 1/2 \"",	10440,	15120,	},
+#   if 0
+    { "a3",		"A3: 297 x 420 mm",		16834,	23818,	},
+#   endif
 };
 
 /************************************************************************/
@@ -136,7 +139,7 @@ int appPaperFormatFromString(	int *		pFormat,
 	    { SDEB(paperString); return -1;	}
 	}
     else{
-	if  ( appGeoRectangleFromString( paperString, unitType,
+	if  ( geoRectangleFromString( paperString, unitType,
 							    &width, &height ) )
 	    { SDEB(paperString); return -1;	}
 	}
