@@ -2,13 +2,12 @@
 /*									*/
 /*  Consult the operating system about certain things.			*/
 /*									*/
-/*  0)  mkdir() is not declared on linux, suppress warning.		*/
 /*  1)  Note that we use shell commands to consult the file system.	*/
 /*	this is the easiest way.					*/
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appUtilConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -33,11 +32,6 @@
 #	include	<libc.h>
 #   endif
 
-    /*  0  */
-#   ifdef __linux__
-    extern int mkdir( const char * name, mode_t mode );
-#   endif
-
 #   define	USE_STAT	1
 #   define	USE_ACCESS	1
 #   define	USE_GETCWD	1
@@ -243,6 +237,42 @@ int appMakeDirectory(	const char *	dir )
     return 0;
     }
 
+int appMakeDirectories(	const char *	dir )
+    {
+    if  ( mkdir( dir, 0777 ) )
+	{
+	if  ( errno == ENOENT )
+	    {
+	    char *	scratch= strdup( dir );
+	    char *	slash;
+	    int		rval= 0;;
+
+	    if  ( ! scratch )
+		{ XDEB(scratch); return -1;	}
+
+	    slash= strrchr( scratch, '/' );
+	    if  ( ! slash )
+		{ SXDEB(dir,slash);	}
+	    else{
+		*slash= '\0';
+
+		if  ( appMakeDirectories( scratch ) )
+		    { SDEB(scratch); rval= -1;	}
+		else{
+		    if  ( mkdir( dir, 0777 ) )
+			{ SSDEB(dir,strerror(errno)); rval= -1;	}
+		    }
+		}
+
+	    free( scratch );
+	    return rval;
+	    }
+	else{ SSDEB(dir,strerror(errno)); return -1;	}
+	}
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Make a string that is as unique as possible.			*/
@@ -251,21 +281,23 @@ int appMakeDirectory(	const char *	dir )
 
 long appGetTimestamp( void )
     {
-    long	now;
+    static time_t	now;
 
-    now= time( (long *)0 ); sleep( 1 );
+    if  ( now == 0 )
+	{ now= time( (time_t *)0 );	}
+    else{ now++;			}
 
     return now;
     }
 
 int appMakeUniqueString(	char *		target,
-				int		maxlen )
+				unsigned int	maxlen )
     {
     int				pid= getpid();
-    long			now= time( (long *)0 );
-    int				needed;
+    time_t			now= time( (time_t *)0 );
+    unsigned int		needed;
 
-    static long			prevNow;
+    static time_t		prevNow;
     static unsigned long	count;
 
 #   if HAVE_UNAME
@@ -280,20 +312,22 @@ int appMakeUniqueString(	char *		target,
 	{ needed= 26; count++;	}
     else{ needed= 17; count= 0;	}
 
+    prevNow= now;
+
     if  ( maxlen < needed )
 	{ LLDEB(maxlen,needed); return -1;	}
 
     if  ( needed == 26 )
 	{
 	sprintf( target, "%08lx.%08lx.%08lx",
-				(unsigned long)pid,
 				(unsigned long)now,
+				(unsigned long)pid,
 				(unsigned long)count );
 	}
     else{
 	sprintf( target, "%08lx.%08lx",
-				(unsigned long)pid,
-				(unsigned long)now );
+				(unsigned long)now,
+				(unsigned long)pid );
 	}
 
     maxlen -= needed; target += needed;
@@ -338,43 +372,6 @@ int appMakeUniqueString(	char *		target,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Translate a file name to an absolute name.				*/
-/*									*/
-/************************************************************************/
-
-int appAbsoluteName(	char *		absolute,
-			int		len,
-			const char *	filename )
-    {
-    int		rellen;
-    int		abslen;
-
-    rellen= strlen( filename );
-
-    if  ( filename[0] == '/' )
-	{
-	if  ( rellen > len )
-	    { SLDEB(filename,len); return -1;	}
-
-	strcpy( absolute, filename ); return rellen;
-	}
-
-    if  ( rellen > len- 2 )
-	{ SLDEB(filename,len); return -1;	}
-
-    abslen= appCurrentDirectory( absolute, len- rellen- 2 );
-    if  ( abslen < 1 )
-	{ LSDEB(abslen,filename); return -1;	}
-
-    absolute[abslen]= '/';
-
-    strcpy( absolute+ abslen+ 1, filename );
-
-    return abslen+ 1+ rellen;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Remove a file.							*/
@@ -398,6 +395,7 @@ int appRemoveFile(	const char *	filename )
 #   define	FILEL		1000
 
 #if USE_OPENDIR
+
 int appForAllFiles(	const char *		dir,
 			const char *		ext,
 			void *			through,
@@ -406,21 +404,45 @@ int appForAllFiles(	const char *		dir,
     int			rval= 0;
     DIR *		d= opendir( dir );
     struct dirent *	de;
+    char *		scratch= (char *)0;
+    const int		dirlen= strlen( dir );
 
     if  ( ! d )
-	{ SSDEB(dir,strerror(errno)); return -1;	}
+	{ SSDEB(dir,strerror(errno)); rval= -1; goto ready;	}
 
     de= readdir( d );
     while( de )
 	{
-	char *	dot= strrchr( de->d_name, '.' );
+	const int	delen= strlen( de->d_name );
+	char *		dot= strrchr( de->d_name, '.' );
 
-	if  ( dot					&&
-	      ! strcmp( dot+ 1, ext )			)
+#	ifdef __VMS
+	char *	semicolon= strrchr( de->d_name, ';' );
+
+	if  ( semicolon )
+	    { *semicolon= '\0';	}
+#	endif
+
+	if  ( ! strcmp( de->d_name, "." ) )
+	    { de= readdir( d ); continue;	}
+	if  ( ! strcmp( de->d_name, ".." ) )
+	    { de= readdir( d ); continue;	}
+
+	if  ( ! ext					||
+	      ( dot				&&
+	      ! strcmp( dot+ 1, ext )		)	)
 	    {
-	    char	scratch[FILEL+1];
+	    char *	tmp= realloc( scratch, dirlen+ 1+ delen+ 1 );
 
+	    if  ( ! tmp )
+		{ LLXDEB(dirlen,delen,tmp); rval= -1; goto ready;	}
+	    scratch= tmp;
+
+#	    ifdef __VMS
+	    sprintf( scratch, "%s%s",  dir, de->d_name );
+#	    else
 	    sprintf( scratch, "%s/%s", dir, de->d_name );
+#	    endif
 
 	    if  ( (*callback)( scratch, through )	)
 		{ SDEB(scratch); rval= -1; break;	}
@@ -429,7 +451,12 @@ int appForAllFiles(	const char *		dir,
 	de= readdir( d );
 	}
 
-    if  ( closedir( d ) )
+  ready:
+
+    if  ( scratch )
+	{ free(scratch);	}
+
+    if  ( d && closedir( d ) )
 	{ SSDEB(dir,strerror(errno)); rval= -1;	}
 
     return rval;
@@ -442,14 +469,27 @@ int appForAllFiles(	const char *		dir,
 			FILE_CALLBACK		callback )
     {
     int			rval= 0;
+    const char *	format;
     char		scratch[FILEL+1];
-    char *		format= "( ls '%s'/*.'%s' ) 2>/dev/null";
     FILE *		names;
 
-    sprintf( scratch, format, dir, ext );
-    names= popen( scratch, "r" );
-    if  ( ! names )
-	{ SXDEB(scratch,names); return -1;	}
+    if  ( ext )
+	{
+	format= "( ls -d '%s'/*.'%s' '%s'/.*.'%s' ) 2>/dev/null";
+
+	sprintf( scratch, format, dir, ext, dir, ext );
+	names= popen( scratch, "r" );
+	if  ( ! names )
+	    { SXDEB(scratch,names); return -1;	}
+	}
+    else{
+	format= "( ls -d '%s'/* '%s'/.*  ) 2>/dev/null";
+
+	sprintf( scratch, format, dir, dir );
+	names= popen( scratch, "r" );
+	if  ( ! names )
+	    { SXDEB(scratch,names); return -1;	}
+	}
 
     while( fgets( scratch, FILEL, names ) )
 	{
@@ -459,6 +499,11 @@ int appForAllFiles(	const char *		dir,
 	    { scratch[len -1]= '\0';		}
 	else{ SLDEB(scratch,len); continue;	}
 
+	if  ( ! strcmp( scratch, "." ) )
+	    { continue;	}
+	if  ( ! strcmp( scratch, ".." ) )
+	    { continue;	}
+
 	if  ( (*callback)( scratch, through ) )
 	    { SDEB(scratch); rval= -1; break;	}
 	}
@@ -561,7 +606,7 @@ int appOpenSocket(	const char *		hostName,
 int appListenSocket(		const char *	portName,
 				unsigned int *	pPort,
 				void *		through,
-				APP_COMPLAIN	complain	)
+				APP_COMPLAIN	complain )
     {
     struct sockaddr_in	si;
     unsigned int	sinlen= sizeof( struct sockaddr_in );
@@ -572,7 +617,7 @@ int appListenSocket(		const char *	portName,
     memset( &si, 0, sinlen );
 
     c= 0;
-    if  ( sscanf( portName, "%d%c", &port, &c ) == 1 && c == 0 )
+    if  ( sscanf( portName, "%u%c", &port, &c ) == 1 && c == 0 )
 	{ si.sin_port= htons( port ); }
     else{
 	struct servent *	serv= getservbyname( portName, "tcp" );
