@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appUtilConfig.h"
 
 #   include	<string.h>
 
@@ -78,70 +78,6 @@ double appUnitToTwips(	double	units,
 	}
     }
 
-/************************************************************************/
-/*									*/
-/*  Convert a number to roman notation.					*/
-/*									*/
-/************************************************************************/
-
-typedef struct RomanPiece
-    {
-    int			rpValue;
-    const char *	rpString;
-    int			rpStrlen;
-    } RomanPiece;
-
-static const RomanPiece	APP_RomanPieces[]=
-    {
-	{	1000,	"m",	1	},
-	{	900,	"cm",	2	},
-	{	500,	"d",	1	},
-	{	400,	"cd",	2	},
-	{	100,	"c",	1	},
-	{	 90,	"xc",	2	},
-	{	 50,	"l",	1	},
-	{	 40,	"xl",	2	},
-	{	 10,	"x",	1	},
-	{	 9,	"ix",	2	},
-	{	 5,	"v",	1	},
-	{	 4,	"iv",	2	},
-	{	 1,	"i",	1	}
-    };
-
-int appRomanString(	char *		to,
-			int		maxlen,
-			int		n,
-			int		upper )
-    {
-    int			i;
-    const RomanPiece *	rp;
-
-    if  ( n <= 0 || n > 3500 )
-	{ LDEB(n); return -1;	}
-
-    rp= APP_RomanPieces;
-    for ( i= 0; i < sizeof(APP_RomanPieces)/sizeof(RomanPiece); rp++, i++ )
-	{
-	while( n >= rp->rpValue )
-	    {
-	    if  ( rp->rpStrlen > maxlen )
-		{ LLDEB(rp->rpStrlen,maxlen); return -1;	}
-
-	    strcpy( to, rp->rpString );
-	    if  ( upper )
-		{
-		while( *to )
-		    { *to= toupper( *to ); to++;	}
-		}
-	    else{ to += rp->rpStrlen;	}
-
-	    maxlen -= rp->rpStrlen; n -= rp->rpValue;
-	    }
-	}
-
-    return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Format a date according to an 'MS-Word' datetime picture		*/
@@ -211,7 +147,6 @@ int appWordFormatDate(			char *			to,
 		    default:
 			LDEB(count); return -1;
 		    }
-		continue;
 
 	    case 'd': case 'D':
 		count= 1;
@@ -254,7 +189,6 @@ int appWordFormatDate(			char *			to,
 		    default:
 			LDEB(count); return -1;
 		    }
-		continue;
 
 	    case 'y': case 'Y':
 		count= 1;
@@ -283,7 +217,6 @@ int appWordFormatDate(			char *			to,
 		    default:
 			LDEB(count); return -1;
 		    }
-		continue;
 
 	    case 'H':
 		count= 1;
@@ -311,7 +244,6 @@ int appWordFormatDate(			char *			to,
 		    default:
 			LDEB(count); return -1;
 		    }
-		continue;
 
 	    case 'h':
 		count= 1;
@@ -339,7 +271,6 @@ int appWordFormatDate(			char *			to,
 		    default:
 			LDEB(count); return -1;
 		    }
-		continue;
 
 	    case 'm':
 		count= 1;
@@ -367,7 +298,6 @@ int appWordFormatDate(			char *			to,
 		    default:
 			LDEB(count); return -1;
 		    }
-		continue;
 
 	    case 'A': case 'a':
 		if  ( ! strcmp( from, "AM/PM" )	||
