@@ -36,7 +36,7 @@ static int sioInHexClose( void *	voids )
 
 static int sioInHexReadBytes(	void *		voidsis,
 				unsigned char *	buffer,
-				int		count )
+				unsigned int	count )
     {
     SimpleInputStream *	sis= (SimpleInputStream *)voidsis;
     int			done= 0;
@@ -45,10 +45,10 @@ static int sioInHexReadBytes(	void *		voidsis,
 	{
 	int		c;
 
-	c= sioInGetCharacter( sis );
+	c= sioInGetByte( sis );
 
 	while( c == ' ' || c == '\r' || c == '\n' )
-	    { c= sioInGetCharacter( sis );	}
+	    { c= sioInGetByte( sis );	}
 
 	if  ( c == EOF )
 	    { return done;	}
@@ -58,12 +58,12 @@ static int sioInHexReadBytes(	void *		voidsis,
 	    else{ buffer[0]= SioHexIndices[c];			}
 	    }
 
-	c= sioInGetCharacter( sis );
+	c= sioInGetByte( sis );
 
 	buffer[0] *= 16;
 
 	while( c == ' ' || c == '\r' || c == '\n' )
-	    { c= sioInGetCharacter( sis );	}
+	    { c= sioInGetByte( sis );	}
 
 	if  ( c == EOF )
 	    { LDEB(c); return -1;			}
@@ -85,7 +85,7 @@ SimpleInputStream * sioInHexOpen(	SimpleInputStream *	sisHex )
 
     if  ( SioHexIndices[0] == 0 )
 	{
-	int	i;
+	unsigned int	i;
 
 	for ( i= 0; i < sizeof(SioHexIndices); i++ )
 	    { SioHexIndices[i]= 0xff;	}
@@ -98,8 +98,7 @@ SimpleInputStream * sioInHexOpen(	SimpleInputStream *	sisHex )
 	    { SioHexIndices[SioHexDigitsL[i]]= i; i++;	}
 	}
 
-    sis= sioInOpen( (void *)sisHex,
-			    sioInHexReadBytes, (SIOinSEEK)0, sioInHexClose );
+    sis= sioInOpen( (void *)sisHex, sioInHexReadBytes, sioInHexClose );
 
     if  ( ! sis )
 	{ XDEB(sis); return (SimpleInputStream *)0; }
@@ -117,6 +116,7 @@ typedef struct HexOutputStream
     {
     SimpleOutputStream *	hosSosHex;
     int				hosWide;
+    int				hosLastNl;
     int				hosColumn;
     } HexOutputStream;
 
@@ -125,9 +125,9 @@ static int sioOutHexClose( void *	voidhos )
     HexOutputStream *		hos= (HexOutputStream *)voidhos;
     SimpleOutputStream *	sos= hos->hosSosHex;
 
-    if  ( hos->hosWide > 0 && hos->hosColumn > 0 )
+    if  ( hos->hosWide > 0 && hos->hosLastNl && hos->hosColumn > 0 )
 	{
-	sioOutPutCharacter( '\n', sos );
+	sioOutPutByte( '\n', sos );
 	hos->hosColumn= 0;
 	}
 
@@ -148,12 +148,12 @@ static int sioOutHexWriteBytes(	void *			voidhos,
 	{
 	if  ( hos->hosWide > 0 && hos->hosColumn >= hos->hosWide )
 	    {
-	    sioOutPutCharacter( '\n', sos );
+	    sioOutPutByte( '\n', sos );
 	    hos->hosColumn= 0;
 	    }
 
-	sioOutPutCharacter( SioHexDigitsL[ ( (*buffer) >> 4 ) & 0x0f ], sos );
-	sioOutPutCharacter( SioHexDigitsL[ ( (*buffer) >> 0 ) & 0x0f ], sos );
+	sioOutPutByte( SioHexDigitsL[ ( (*buffer) >> 4 ) & 0x0f ], sos );
+	sioOutPutByte( SioHexDigitsL[ ( (*buffer) >> 0 ) & 0x0f ], sos );
 
 	if  ( hos->hosWide > 0 )
 	    { hos->hosColumn += 2;	}
@@ -164,23 +164,10 @@ static int sioOutHexWriteBytes(	void *			voidhos,
     return count;
     }
 
-static int sioHexSeek(		void *			voidhos,
-				long			pos )
-    {
-    HexOutputStream *		hos= (HexOutputStream *)voidhos;
-
-    if  ( hos->hosWide > 0 )
-	{ LDEB(hos->hosWide); return -1;	}
-
-    if  ( sioOutSeek( hos->hosSosHex, 2* pos ) )
-	{ LDEB(0); return -1;	}
-
-    return 0;
-    }
-
 SimpleOutputStream * sioOutHexOpenFolded(
 				    SimpleOutputStream *	sosHex,
-				    int				wide )
+				    int				wide,
+				    int				lastNl )
     {
     HexOutputStream *		hos= malloc( sizeof(HexOutputStream) );
     SimpleOutputStream *	sos;
@@ -190,10 +177,10 @@ SimpleOutputStream * sioOutHexOpenFolded(
 
     hos->hosSosHex= sosHex;
     hos->hosWide= wide;
+    hos->hosLastNl= lastNl;
     hos->hosColumn= 0;
 
-    sos= sioOutOpen( (void *)hos, sioOutHexWriteBytes,
-						sioHexSeek, sioOutHexClose );
+    sos= sioOutOpen( (void *)hos, sioOutHexWriteBytes, sioOutHexClose );
 
     if  ( ! sos )
 	{ XDEB(sos); free( hos ); return (SimpleOutputStream *)0; }
@@ -203,6 +190,6 @@ SimpleOutputStream * sioOutHexOpenFolded(
 
 SimpleOutputStream * sioOutHexOpen(	SimpleOutputStream *	sosHex )
     {
-    return sioOutHexOpenFolded( sosHex, 0 );
+    return sioOutHexOpenFolded( sosHex, 0, 0 );
     }
 
