@@ -4,9 +4,10 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appUtilConfig.h"
 
 #   include	<stdlib.h>
+#   include	<string.h>
 
 #   include	<sioMemory.h>
 #   include	<appDebugon.h>
@@ -22,7 +23,7 @@ static int sioMemoryClose(	void *	voidsmp )
 
 static int sioInMemoryReadBytes(	void *		voidsmp,
 					unsigned char *	buffer,
-					int		count )
+					unsigned int	count )
     {
     SioMemoryPrivate *	smp= (SioMemoryPrivate *)voidsmp;
     MemoryBuffer *	mb= smp->smpBuffer;
@@ -39,17 +40,8 @@ static int sioInMemoryReadBytes(	void *		voidsmp,
     return count;
     }
 
-static int sioMemorySeek(	void *	voidsmp,
-				long	pos )
-    {
-    SioMemoryPrivate *	smp= (SioMemoryPrivate *)voidsmp;
-
-    smp->smpPosition= pos;
-
-    return 0;
-    }
-
-SimpleInputStream * sioInMemoryOpen(	const MemoryBuffer *	mb )
+SimpleInputStream * sioInMemoryOpenFrom(	const MemoryBuffer *	mb,
+						int			from )
     {
     SimpleInputStream *	sis;
     SioMemoryPrivate *	smp;
@@ -59,10 +51,9 @@ SimpleInputStream * sioInMemoryOpen(	const MemoryBuffer *	mb )
 	{ XDEB(smp); return (SimpleInputStream *)0;	}
 
     smp->smpBuffer= (MemoryBuffer *)mb;
-    smp->smpPosition= 0;
+    smp->smpPosition= from;
 
-    sis= sioInOpen( (void *)smp,
-		    sioInMemoryReadBytes, sioMemorySeek, sioMemoryClose );
+    sis= sioInOpen( (void *)smp, sioInMemoryReadBytes, sioMemoryClose );
 
     if  ( ! sis )
 	{ XDEB(sis); free( smp ); return (SimpleInputStream *)0; }
@@ -70,6 +61,11 @@ SimpleInputStream * sioInMemoryOpen(	const MemoryBuffer *	mb )
     return sis;
     }
 
+SimpleInputStream * sioInMemoryOpen(	const MemoryBuffer *	mb )
+    {
+    return sioInMemoryOpenFrom( mb, 0 );
+    }
+
 static int sioOutMemoryWriteBytes(	void *			voidsmp,
 					const unsigned char *	buffer,
 					int			count )
@@ -97,25 +93,30 @@ static int sioOutMemoryWriteBytes(	void *			voidsmp,
     return count;
     }
 
-SimpleOutputStream * sioOutMemoryOpen(	MemoryBuffer *	mb )
+SimpleOutputStream * sioOutMemoryOpenTo(	MemoryBuffer *	mb,
+						int		to )
     {
     SimpleOutputStream *	sos;
     SioMemoryPrivate *		smp;
 
+    if  ( mb->mbSize < to )
+	{ LLDEB(mb->mbSize,to); return (SimpleOutputStream *)0;	}
+
     smp= (SioMemoryPrivate *)malloc( sizeof(SioMemoryPrivate) );
     if  ( ! smp )
 	{ XDEB(smp); return (SimpleOutputStream *)0;	}
 
     smp->smpBuffer= mb;
-    smp->smpPosition= 0;
-
-    sos= sioOutOpen( (void *)smp, sioOutMemoryWriteBytes,
-					    sioMemorySeek, sioMemoryClose );
+    smp->smpPosition= to;
 
+    sos= sioOutOpen( (void *)smp, sioOutMemoryWriteBytes, sioMemoryClose );
     if  ( ! sos )
 	{ XDEB(sos); free( smp ); return (SimpleOutputStream *)0; }
 
-    mb->mbSize= 0;
+    mb->mbSize= to;
 
     return sos;
     }
+
+SimpleOutputStream * sioOutMemoryOpen(	MemoryBuffer *	mb )
+    { return sioOutMemoryOpenTo( mb, 0 );	}
