@@ -1,6 +1,9 @@
-#   include	"config.h"
+#   include	"appUtilConfig.h"
 
 #   include	<sioSmtp.h>
+#   include	<sioMemory.h>
+#   include	<sioBase64.h>
+#   include	<utilMemoryBuffer.h>
 
 #   include	<stdlib.h>
 #   include	<stdarg.h>
@@ -17,6 +20,8 @@
 #   include	<sys/utsname.h>
 #   include	<netdb.h>
 
+#   include	<appDebugon.h>
+
 #   ifdef	NeXT
 #	include	<libc.h>
 
@@ -26,7 +31,11 @@
 #	endif
 #   endif
 
-#   include	<appDebugon.h>
+#   ifdef __VMS
+#	define	HAVE_PW_GECOS	0
+#   else
+#	define	HAVE_PW_GECOS	1
+#   endif
 
 /************************************************************************/
 /*									*/
@@ -60,9 +69,6 @@ static int sioOutSmtpWriteBytes( void *			voidmc,
 				const unsigned char *	buf,
 				int			len );
 
-static int sioOutSmtpSeek(	void *			voidmc,
-				long			pos );
-
 static int sioOutSmtpClose(	void *			voidmc );
 
 static int mmReadCRLF(	int			fd,
@@ -106,13 +112,62 @@ static void sioOutSmtpAbortMessage(	MailContext *		mc,
     }
 
 
+/************************************************************************/
+/*									*/
+/*  respond to a challenge from the mail server.			*/
+/*									*/
+/************************************************************************/
+
+static int sioOutSmtpChallengeResponse(	MailContext *		mc,
+					const char *		response,
+					int			responseLen,
+					void *			through,
+					APP_COMPLAIN		complain )
+    {
+    int				rval= 0;
+
+    SimpleOutputStream *	sosmb= (SimpleOutputStream *)0;
+    SimpleOutputStream *	sosb64= (SimpleOutputStream *)0;
+    MemoryBuffer		mb64;
+
+    utilInitMemoryBuffer( &mb64 );
+
+    sosmb= sioOutMemoryOpen( &mb64 );
+    if  ( ! sosmb )
+	{ XDEB(sosmb); rval= -1; goto ready; }
+    sosb64= sioOutBase64Open( sosmb );
+    if  ( ! sosb64 )
+	{ XDEB(sosb64); rval= -1; goto ready; }
+
+    sioOutWriteBytes( sosb64, (const unsigned char *)response, responseLen );
+
+    sioOutClose( sosb64 ); sosb64= (SimpleOutputStream *)0;
+    sioOutClose( sosmb ); sosmb= (SimpleOutputStream *)0;
+
+    if  ( mmWriteCRLF( mc->mcFd, (char *)mb64.mbBytes, through, complain ) )
+	{ LDEB(1); rval= -1;	}
+
+  ready:
+    if  ( sosb64 )
+	{ sioOutClose( sosb64 );	}
+    if  ( sosmb )
+	{ sioOutClose( sosmb );		}
+
+    utilCleanMemoryBuffer( &mb64 );
+
+    return rval;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Open a connection to an SMTP server and send the mail headers	*/
 /*									*/
 /************************************************************************/
 
-SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
+SimpleOutputStream * sioOutSmtpOpen(	const char *	mailHost,
+					const char *	mailPort,
+					const char *	user,
+					const char *	pass,
 					const char *	from,
 					const char *	to,
 					const char *	cc,
@@ -128,6 +183,11 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
     const char *		start;
     const char *		nodename= (const char *)0;
 
+    int				haveEsmtp= 1;
+    int				haveAuth= 0;
+    int				haveAuthLogin= 0;
+    int				haveAuthPlain= 0;
+
     SimpleOutputStream *	sos;
 
 #   if HAVE_UNAME
@@ -167,8 +227,13 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
     /********************************************************************/
     /*  Open connection							*/
     /********************************************************************/
-    mc->mcFd= appOpenSocket( ( relay && relay[0])?relay:"mailhost",
-						"smtp", through, complain );
+
+    if  ( ! mailHost || ! mailHost[0] )
+	{ mailHost= "mailhost";	}
+    if  ( ! mailPort || ! mailPort[0] )
+	{ mailPort= "smtp";	}
+
+    mc->mcFd= appOpenSocket( mailHost, mailPort, through, complain );
     if  ( mc->mcFd < 0 )
 	{ LDEB(mc->mcFd); goto mmError;	}
 
@@ -204,22 +269,136 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
     if  ( ! nodename )
 	{ XDEB(nodename); goto mmError;	}
 
-    sprintf( MMscratch, "HELO %s", nodename );
+    sprintf( MMscratch, "EHLO %s", nodename );
     if  ( mmWriteCRLF( mc->mcFd, MMscratch, through, complain ) )
 	{ LDEB(1); goto mmError;				}
 
     do  {
+	char *	head;
+	char *	tail;
+
 	if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
 	    { LDEB(1); goto mmError;				}
 
 	if  ( strncmp( MMscratch, "250", 3 ) )
 	    {
-	    SDEB(MMscratch);
-	    (void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
-	    goto mmError;
+	    if  ( strncmp( MMscratch, "500", 3 )	&&
+		  strncmp( MMscratch, "502", 3 )	&&
+		  strncmp( MMscratch, "554", 3 )	)
+		{
+		SDEB(MMscratch);
+		(void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
+		goto mmError;
+		}
+	    else{ haveEsmtp= 0;	}
 	    }
+
+	head= tail= MMscratch+ 4;
+	while( *tail && ! isspace( *tail ) )
+	    {
+	    if  ( islower( *tail ) )
+		{ *tail= toupper( *tail );	}
+	    tail++;
+	    }
+
+	if  ( tail- head == 4 && ! strncmp( head, "AUTH", 4 ) )
+	    {
+	    haveAuth= 1;
+
+	    while( *tail && isspace( *tail ) )
+		{ tail++;	}
+	    head= tail;
+	    while( *head )
+		{
+		while( *tail && ! isspace( *tail ) )
+		    {
+		    if  ( islower( *tail ) )
+			{ *tail= toupper( *tail );	}
+		    tail++;
+		    }
+
+		if  ( tail- head == 5 && ! strncmp( head, "LOGIN", 5 ) )
+		    { haveAuthLogin= 1;	}
+		if  ( tail- head == 5 && ! strncmp( head, "PLAIN", 5 ) )
+		    { haveAuthPlain= 1;	}
+
+		while( *tail && isspace( *tail ) )
+		    { tail++;	}
+		head= tail;
+		}
+	    }
+
 	} while( MMscratch[3] != ' ' );
 
+    if  ( ! haveEsmtp )
+	{
+	sprintf( MMscratch, "HELO %s", nodename );
+	if  ( mmWriteCRLF( mc->mcFd, MMscratch, through, complain ) )
+	    { LDEB(1); goto mmError;				}
+
+	do  {
+	    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
+		{ LDEB(1); goto mmError;				}
+
+	    if  ( strncmp( MMscratch, "250", 3 ) )
+		{
+		SDEB(MMscratch);
+		(void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
+		goto mmError;
+		}
+	    } while( MMscratch[3] != ' ' );
+	}
+
+    /********************************************************************/
+    /*  Login into the server.						*/
+    /********************************************************************/
+    if  ( user && pass && haveAuthLogin )
+	{
+	sprintf( MMscratch, "AUTH LOGIN" );
+	if  ( mmWriteCRLF( mc->mcFd, MMscratch, through, complain ) )
+	    { LDEB(1); goto mmError;				}
+
+	for (;;)
+	    {
+	    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
+		{ LDEB(1); goto mmError;				}
+
+	    if  ( ! strncmp( MMscratch, "235", 3 ) )
+		{ break;	}
+
+	    if  ( strncmp( MMscratch, "334", 3 ) )
+		{
+		SDEB(MMscratch);
+		(void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
+		goto mmAbort;
+		}
+
+	    /*  Username: */
+	    if  ( ! strcmp( MMscratch+ 4, "VXNlcm5hbWU6" ) )
+		{
+		if  ( sioOutSmtpChallengeResponse( mc,
+						user, strlen( user ),
+						through, complain ) )
+		    { LDEB(1);  goto mmAbort;	}
+
+		continue;
+		}
+
+	    /*  Password: */
+	    if  ( ! strcmp( MMscratch+ 4, "UGFzc3dvcmQ6" ) )
+		{
+		if  ( sioOutSmtpChallengeResponse( mc,
+						pass, strlen( pass ),
+						through, complain ) )
+		    { LDEB(1);  goto mmAbort;	}
+
+		continue;
+		}
+
+	    SDEB(MMscratch); goto mmAbort;
+	    }
+	}
+
     /********************************************************************/
     /*  Say sender							*/
     /********************************************************************/
@@ -290,15 +469,19 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
 	
     if  ( mimeBoundary )
 	{
+	const char *	mimeVersion= "MIME-Version";
+
 	const char *	name= "Content-Type";
 	int		len= strlen( name );
 	const char *	name2= " boundary=\"";
 
-	if  ( mmWrite( mc->mcFd, name, len, through, complain )		||
+	if  ( mmWriteHeader( mc->mcFd, mimeVersion, "1.0",
+						through, complain )	||
+	      mmWrite( mc->mcFd, name, len, through, complain )		||
 	      mmWrite( mc->mcFd, ": ", 2, through, complain )		||
 	      mmWrite( mc->mcFd, typeSlashSubtype,
-				strlen( typeSlashSubtype ),
-				through, complain )			||
+					strlen( typeSlashSubtype ),
+					through, complain )		||
 	      mmWriteCRLF( mc->mcFd, ";", through, complain )		)
 	    { LDEB(1); goto mmAbort;	}
 
@@ -319,8 +502,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
     if  ( mmWriteCRLF( mc->mcFd, "\n", through, complain )		)
 	{ LDEB(1); goto mmAbort;	}
 
-    sos= sioOutOpen( (void *)mc, sioOutSmtpWriteBytes,
-					    sioOutSmtpSeek, sioOutSmtpClose );
+    sos= sioOutOpen( (void *)mc, sioOutSmtpWriteBytes, sioOutSmtpClose );
 
     if  ( ! sos )
 	{ XDEB(sos); goto mmAbort;	}
@@ -331,6 +513,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
     sioOutSmtpAbortMessage( mc, through, complain );
 
   mmError:
+
     if  ( mc )
 	{
 	if  ( mc->mcFd != -1 )
@@ -341,16 +524,6 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *	relay,
     return (SimpleOutputStream *)0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Refuse to seek.							*/
-/*									*/
-/************************************************************************/
-
-static int sioOutSmtpSeek(	void *	voidmc,
-				long	pos )
-    { LDEB(pos); return -1;	}
-
 /************************************************************************/
 /*									*/
 /*  Send the next part of the mail body					*/
@@ -370,8 +543,8 @@ static int sioOutSmtpWriteBytes(	void *			voidmc,
     while( todo > 0 )
 	{
 	past= (const unsigned char *)memchr( start, '\n', todo );
-	if  ( ! past )
-	    { todo-= past- start;				}
+	if  ( past )
+	    { todo -= past- start;				}
 	else{ past= start+ todo; todo= 0;			}
 
 	if  ( past > start )
@@ -525,7 +698,7 @@ static int mmReadCRLF(	int			fd,
 	*buf= *smallbuf;
 
 	if  ( nbytes >= 2 && *buf == 10 && *(buf-1) == 13 )
-	    { *(buf-1)= 0; return 0;				}
+	    { *(buf-1)= 0; return 0;	}
 
 	buf++;
 	}
@@ -710,8 +883,10 @@ char *	sioSmtpGuessMailAddress( void )
 
     l= 0;
     l += strlen( pwd->pw_name );
+#   if HAVE_PW_GECOS
     if  ( pwd->pw_gecos && *pwd->pw_gecos )
 	{ l += strlen( pwd->pw_gecos )+ 3;	}
+#   endif
 
     if  ( domainname )
 	{
@@ -726,6 +901,7 @@ char *	sioSmtpGuessMailAddress( void )
 	{ XDEB(rval); return (char *)0;	}
 
     to= rval; *to= '\0';
+#   if HAVE_PW_GECOS
     if  ( pwd->pw_gecos && *pwd->pw_gecos )
 	{
 	char *	s;
@@ -753,6 +929,7 @@ char *	sioSmtpGuessMailAddress( void )
 	    }
 
 	}
+#   endif
     strcpy( to, pwd->pw_name ); to += strlen( to );
     strcpy( to, "@" ); to += strlen( to );
     if  ( domainname )
