@@ -7,6 +7,8 @@
 #   ifndef	UTIL_PROP_MASK_H
 #   define	UTIL_PROP_MASK_H
 
+#   include	<string.h>
+
 #   define	PROPmaskSIZE		8
 #   define	PROPmaskMAXPROPS	(8*(PROPmaskSIZE))
 
@@ -15,7 +17,7 @@ typedef struct PropertyMask
     unsigned char	pmBits[PROPmaskSIZE];
     } PropertyMask;
 
-#   define	PROPmaskCLEAR( pm ) \
+#   define	utilPropMaskClear( pm ) \
 			memset( (pm)->pmBits, 0, PROPmaskSIZE )
 
 #   define	PROPmaskADD( pm, v ) \
@@ -27,10 +29,6 @@ typedef struct PropertyMask
 #   define	PROPmaskISSET( pm, v ) \
 			( (pm)->pmBits[(v)/8] & (1<<((v)%8)) ) != 0
 
-#   define	PROPmaskFILL( pm, c )	utilPropMaskFill( pm, c )
-
-#   define	PROPmaskISEMPTY( pm )		utilPropMaskIsEmpty( pm )
-
 extern void utilPropMaskFill(		PropertyMask *		pm,
 					int			c );
 
