@@ -14,26 +14,33 @@ typedef struct TextAttribute
     short int		taFontNumber;
     short int		taFontSizeHalfPoints;
     short int		taTextColorNumber;
+    short int		taTextStyleNumber;
 
-    unsigned int	taFontIsBold:1;
-    unsigned int	taFontIsSlanted:1;
-    unsigned int	taTextIsUnderlined:1;
+    short int		taBorderNumber;
+    short int		taShadingNumber;
 
-    unsigned int	taSmallCaps:1;
-    unsigned int	taHasStrikethrough:1;
-    unsigned int	taCapitals:1;
+    unsigned char	taFontIsBold;
+    unsigned char	taFontIsSlanted;
+    unsigned char	taTextIsUnderlined;
+#   define			DOCunderlineNONE	0
+#   define			DOCunderlineSINGLE	1
+#   define			DOCunderlineDOUBLE	2
 
-    unsigned int	taSuperSub:2;
-#   define		DOCfontREGULAR		0
-#   define		DOCfontSUPERSCRIPT	1
-#   define		DOCfontSUBSCRIPT	2
+    unsigned char	taSmallCaps;
+    unsigned char	taHasStrikethrough;
+    unsigned char	taCapitals;
+
+    unsigned char	taSuperSub;
+#   define			DOCfontREGULAR		0
+#   define			DOCfontSUPERSCRIPT	1
+#   define			DOCfontSUBSCRIPT	2
     } TextAttribute;
 
 typedef enum TextProperty
     {
     TAprop_NONE= -1,
 
-    TApropFONTFAMILY= 0,
+    TApropDOC_FONT_NUMBER= 0,
     TApropFONTSIZE,
     TApropFONTBOLD,
     TApropFONTSLANTED,
@@ -43,6 +50,10 @@ typedef enum TextProperty
     TApropCAPITALS,
     TApropSTRIKETHROUGH,
     TApropTEXT_COLOR,
+    TApropTEXT_STYLE,
+
+    TApropBORDER,
+    TApropSHADING,
 
     TAprop_COUNT
     } TextProperty;
@@ -63,9 +74,13 @@ typedef enum TextProperty
 		(ta1)->taTextIsUnderlined == (ta2)->taTextIsUnderlined && \
 		(ta1)->taHasStrikethrough == (ta2)->taHasStrikethrough && \
 		(ta1)->taCapitals == (ta2)->taCapitals && \
-		(ta1)->taTextColorNumber == (ta2)->taTextColorNumber )
+		(ta1)->taTextColorNumber == (ta2)->taTextColorNumber && \
+		(ta1)->taBorderNumber == (ta2)->taBorderNumber && \
+		(ta1)->taShadingNumber == (ta2)->taShadingNumber )
 
-#   define	SUPERSUB_SIZE( sz )	( 7*(sz)/10 )
+#define FACE_INDEX( isS, isB ) ( 2*( (isS) != 0 )+ ( (isB) != 0 ) )
+#define FACE_BOLD( idx ) ( (idx) % 2 != 0 )
+#define FACE_SLANTED( idx ) ( ((idx)/2) % 2 != 0 )
 
 /************************************************************************/
 /*									*/
@@ -93,4 +108,11 @@ extern void utilUpdateTextAttribute(	PropertyMask *		pChangeMask,
 					const TextAttribute *	taFrom,
 					const PropertyMask *	updMask );
 
+extern int utilSetTextProperty(		TextAttribute *		ta,
+					int			prop,
+					int			arg );
+
+extern int utilGetTextProperty(		const TextAttribute *	ta,
+					int			prop );
+
 #   endif	/*  UTIL_TEXT_ATTRIBUTE_H	*/
