@@ -65,7 +65,9 @@ typedef	struct	TreeNode
 
 
 /************************************************************************/
+/*									*/
 /*  Avoid unnecessary typing						*/
+/*									*/
 /************************************************************************/
 
 #   define	sl_val	sl_ptr.pt_val
@@ -181,11 +183,11 @@ static TreeNode * treeGetLeaf(	TreeNode *	tn,
 
 /************************************************************************/
 /*									*/
-/*  suppose no is a leaf, return next leaf				*/
+/*  suppose tn is a leaf, return next leaf				*/
 /*									*/
 /************************************************************************/
 
-static TreeNode *	treeGetNextLeaf(	TreeNode *	tn )
+static TreeNode * treeGetNextLeaf(	TreeNode *	tn )
     {
     TreeNode *		dn;
 
@@ -217,11 +219,11 @@ static TreeNode *	treeGetNextLeaf(	TreeNode *	tn )
 
 /************************************************************************/
 /*									*/
-/*  suppose no is a leaf, return previous leaf				*/
+/*  suppose tn is a leaf, return previous leaf				*/
 /*									*/
 /************************************************************************/
 
-static TreeNode *	treeGetPrevLeaf(	TreeNode *	tn )
+static TreeNode * treeGetPrevLeaf(	TreeNode *	tn )
     {
     TreeNode *		dn;
 
@@ -265,6 +267,7 @@ static TreeNode *	treeGetPrevLeaf(	TreeNode *	tn )
 /************************************************************************/
 
 int utilTreeStoreValue(	void *		tree,
+			void **		pPrevVal,
 			const char **	pStoredKey,
 			const char *	key,
 			void *		val )
@@ -278,6 +281,7 @@ int utilTreeStoreValue(	void *		tree,
     int			where;
     int			nwkey;
 
+    void *		prevVal= (void *)0;
     char *		savedKey= (char *)0;
     char *		childKey;
 
@@ -308,7 +312,11 @@ int utilTreeStoreValue(	void *		tree,
 	tm->tr_root= no;
 	tm->tr_curr= no;
 	tm->trCurrentDeleted= 0;
-	*pStoredKey= tm->tr_lfcu= savedKey;
+	tm->tr_lfcu= savedKey;
+	if  ( pStoredKey )
+	    { *pStoredKey= tm->tr_lfcu;	}
+	if  ( pPrevVal )
+	    { *pPrevVal= prevVal;	}
 
 	return 0;
 	}
@@ -415,10 +423,16 @@ int utilTreeStoreValue(	void *		tree,
 
 	case 1:
 	    /*  in L	*/
+	    if  ( ! nwkey )
+		{ prevVal= no->no_L.sl_val;	}
 	    no->no_L.sl_val= val;
 	    tm->tr_curr= no;
 	    tm->trCurrentDeleted= 0;
-	    *pStoredKey= tm->tr_lfcu= no->no_L.nsKey;
+	    tm->tr_lfcu= no->no_L.nsKey;
+	    if  ( pStoredKey )
+		{ *pStoredKey= tm->tr_lfcu;	}
+	    if  ( pPrevVal )
+		{ *pPrevVal= prevVal;	}
 	    break;
 
 	case 2:
@@ -437,10 +451,15 @@ int utilTreeStoreValue(	void *		tree,
 	    /*  in M	*/
 	    if  ( nwkey )
 		{ no->no_M.nsKey= savedKey;	}
+	    else{ prevVal= no->no_M.sl_val;	}
 	    no->no_M.sl_val= val;
 	    tm->tr_curr= no;
 	    tm->trCurrentDeleted= 0;
-	    *pStoredKey= tm->tr_lfcu= no->no_M.nsKey;
+	    tm->tr_lfcu= no->no_M.nsKey;
+	    if  ( pStoredKey )
+		{ *pStoredKey= tm->tr_lfcu;	}
+	    if  ( pPrevVal )
+		{ *pPrevVal= prevVal;	}
 	    break;
 
 	case 4:
@@ -451,17 +470,26 @@ int utilTreeStoreValue(	void *		tree,
 	    no->no_R.nsKey= (char *)0;
 	    tm->tr_curr= nw;
 	    tm->trCurrentDeleted= 0;
-	    *pStoredKey= tm->tr_lfcu= savedKey;
+	    tm->tr_lfcu= savedKey;
+	    if  ( pStoredKey )
+		{ *pStoredKey= tm->tr_lfcu;	}
+	    if  ( pPrevVal )
+		{ *pPrevVal= prevVal;	}
 	    break;
 
 	case 5:
 	    /*  in R	*/
 	    if  ( nwkey )
 		{ no->no_R.nsKey= savedKey;	}
+	    else{ prevVal= no->no_R.sl_val;	}
 	    no->no_R.sl_val= val;
 	    tm->tr_curr= no;
 	    tm->trCurrentDeleted= 0;
-	    *pStoredKey= tm->tr_lfcu= no->no_R.nsKey;
+	    tm->tr_lfcu= no->no_R.nsKey;
+	    if  ( pStoredKey )
+		{ *pStoredKey= tm->tr_lfcu;	}
+	    if  ( pPrevVal )
+		{ *pPrevVal= prevVal;	}
 	    break;
 
 	case 6:
@@ -472,7 +500,11 @@ int utilTreeStoreValue(	void *		tree,
 	    no->no_R.nsKey= (char *)0;
 	    tm->tr_curr= nw;
 	    tm->trCurrentDeleted= 0;
-	    *pStoredKey= tm->tr_lfcu= savedKey;
+	    tm->tr_lfcu= savedKey;
+	    if  ( pStoredKey )
+		{ *pStoredKey= tm->tr_lfcu;	}
+	    if  ( pPrevVal )
+		{ *pPrevVal= prevVal;	}
 	    break;
 
 	default:
@@ -739,7 +771,7 @@ void * utilTreeGetGE(	void *		tree,
 /*									*/
 /************************************************************************/
 
-void * utilTreeGetEQ(		void *		tree,
+void * utilTreeGetEQ(		const void *	tree,
 				const char **	pStoredKey,
 				const char *	key )
     {
@@ -749,53 +781,113 @@ void * utilTreeGetEQ(		void *		tree,
     int			cmp;
 
     if  ( ! tree )
-	{ *pStoredKey= (const char *)0; return (void *)0;	}
+	{
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
     no= tm->tr_root;
     if  ( ! no )
-	{ *pStoredKey= (const char *)0; return (void *)0;	}
+	{
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
 
     if  ( tm->trStatusFlag == TRsCLOSE )
 	{
 	LDEB(tm->trStatusFlag);
-	*pStoredKey= (const char *)0; return (void *)0;
+
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
 	}
 
     /*  get leaf with key	*/
     no= treeGetLeaf( no, key );
     if  ( ! no )
-	{ XDEB(no); *pStoredKey= (const char *)0; return (void *)0; }
+	{
+	XDEB(no);
+
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
 
     cmp= strcmp( key, no->no_L.nsKey );
-    if  ( cmp <  0 )			{ return (void *)0;		}
+    if  ( cmp <  0 )
+	{
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
+
     if  ( cmp == 0 )
 	{
 	tm->tr_curr= no;
 	tm->trCurrentDeleted= 0;
-	*pStoredKey= tm->tr_lfcu= no->no_L.nsKey;
+
+	if  ( pStoredKey )
+	    { *pStoredKey= tm->tr_lfcu= no->no_L.nsKey;	}
+
 	return no->no_L.sl_val;
 	}
 
-    if  ( ! no->no_M.nsKey )		{ return (void *)0;		}
+    if  ( ! no->no_M.nsKey )
+	{
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
+
     cmp= strcmp( key, no->no_M.nsKey );
-    if  ( cmp <  0 )			{ return (void *)0;		}
+    if  ( cmp <  0 )
+	{
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
+
     if  ( cmp == 0 )
 	{
 	tm->tr_curr= no;
 	tm->trCurrentDeleted= 0;
-	*pStoredKey= tm->tr_lfcu= no->no_M.nsKey;
+
+	if  ( pStoredKey )
+	    { *pStoredKey= tm->tr_lfcu= no->no_M.nsKey;	}
+
 	return no->no_M.sl_val;
 	}
 
-    if  ( ! no->no_R.nsKey )	{ return (void *)0;		}
+    if  ( ! no->no_R.nsKey )
+	{
+	if  ( pStoredKey )
+	    { *pStoredKey= (const char *)0;	}
+
+	return (void *)0;
+	}
+
     cmp= strcmp( key, no->no_R.nsKey );
     if  ( cmp == 0 )
 	{
 	tm->tr_curr= no;
 	tm->trCurrentDeleted= 0;
-	*pStoredKey= tm->tr_lfcu= no->no_R.nsKey;
+
+	if  ( pStoredKey )
+	    { *pStoredKey= tm->tr_lfcu= no->no_R.nsKey;	}
+
 	return no->no_R.sl_val;
 	}
 
+    if  ( pStoredKey )
+	{ *pStoredKey= (const char *)0;	}
     return (void *)0;
     }
 
