@@ -1,5 +1,7 @@
 /************************************************************************/
+/*									*/
 /*  Description of a bitmap.						*/
+/*									*/
 /************************************************************************/
 
 #   ifndef	BITMAP_H
@@ -7,7 +9,8 @@
 
 #   include	<stdio.h>
 #   include	<sioGeneral.h>
-#   include	<bmcolor.h>
+#   include	<utilColor.h>
+#   include	<geo2DInteger.h>
 
 /************************************************************************/
 /*									*/
@@ -15,12 +18,17 @@
 /*									*/
 /************************************************************************/
 
-#   define	BMcoBLACKWHITE		0
-#   define	BMcoWHITEBLACK		1
-#   define	BMcoRGB			2
-#   define	BMcoRGB8PALETTE		3
+typedef enum BitmapColorEncoding
+    {
+    BMcoBLACKWHITE= 0,
+    BMcoWHITEBLACK,
+    BMcoRGB,
+    BMcoRGB8PALETTE,
 
-#   define	BMcoILLEGALVALUE	4
+    BMco_COUNT
+    } BitmapColorEncoding;
+
+#   define	BMcoILLEGALVALUE	BMco_COUNT
 
 extern const char *	bmcoIntToString( int colorEncodingInt );
 extern int		bmcoStringToInt( const char * colorEncodingString );
@@ -32,14 +40,18 @@ extern const char *	bmcoStrings[];
 /*									*/
 /************************************************************************/
 
-#   define	BMunM			0
-#   define	BMunINCH		1
-#   define	BMunPOINT		2
-#   define	BMunPIXEL		3
+typedef enum BitmapResolution
+    {
+    BMunM= 0,
+    BMunINCH,
+    BMunPOINT,
+    BMunPIXEL,
 
-#   define	BMunILLEGALVALUE	4
+    BMun_COUNT
+    } BitmapResolution;
+
+#   define	BMunILLEGALVALUE	BMun_COUNT
 
-#   define	x_POINTS_PER_CM		28.3465
 #   define	POINTS_PER_M		2834.65
 #   define	TWIPS_PER_M		(20*POINTS_PER_M)
 
@@ -113,9 +125,21 @@ typedef struct	BitmapDescription
 	} bdPaletteUninterpreted;
 
 #   define	bdRGB8Palette	bdPaletteUninterpreted.bdpRGB8Palette
-
     } BitmapDescription;
 
+/************************************************************************/
+/*									*/
+/*  Used to store and manage a bitmap image.				*/
+/*									*/
+/************************************************************************/
+
+typedef struct AppBitmapImage
+    {
+    BitmapDescription	abiBitmap;
+    unsigned char *	abiBuffer;
+    int			abiFormat;
+    } AppBitmapImage;
+
 /************************************************************************/
 /*									*/
 /*  Description of a bitmap file format.				*/
@@ -127,47 +151,30 @@ typedef struct	BitmapFileType
     int (*bftWrite)(	const char *			filename,
 			const unsigned char *		buffer,
 			const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor );
+			int				privateFormat );
 
     int (*bftCanWrite)( const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor );
+			int				privateFormat );
 
-    int (*bftRead)(	const char *		filename,
-			unsigned char **	pBuffer,
-			BitmapDescription *	bd,
-			int *			pPrivateFormat,
-			double *		pCompressionFactor	);
+    int (*bftRead)(	const char *			filename,
+			unsigned char **		pBuffer,
+			BitmapDescription *		bd,
+			int *				pPrivateFormat );
 
-    char *	bftFileExtension;
-    char *	bftFileFilter;
-    char *	bftTypeId;
-    char *	bftTypeDescription;
+    const char *	bftFileExtension;
+    const char *	bftFileFilter;
+    const char *	bftTypeId;
+    const char *	bftTypeDescription;
     } BitmapFileType;
 
 typedef struct	BitmapFileFormat
     {
-    char *		bffDescription;
-    char *		bffId;
+    const char *	bffDescription;
+    const char *	bffId;
     int			bffPrivate;
     BitmapFileType *	bffFileType;
     } BitmapFileFormat;
 
-/************************************************************************/
-/*									*/
-/*  Description of selection from a bitmap.				*/
-/*									*/
-/************************************************************************/
-
-typedef struct	BitmapSelection
-    {
-    int		bsX0;
-    int		bsY0;
-    int		bsPixelsWide;
-    int		bsPixelsHigh;
-    } BitmapSelection;
-
 /************************************************************************/
 /*									*/
 /*  Context for writing images to PostScript.				*/
@@ -193,6 +200,10 @@ extern int bmPsOpenBitmapPrinter(
 
 extern void bmCloseBitmapPrinter(	BitmapPrinter *		bp );
 
+extern void bmStartEpsFile(	SimpleOutputStream *		sos,
+				const char *			filename,
+				const BitmapDescription *	bd );
+
 /************************************************************************/
 /*									*/
 /*  Catalog of available of a bitmap file formats.			*/
@@ -214,22 +225,29 @@ extern const int	bmNumberOfFileTypes;
 extern int bmWriteGifFile(	const char *			filename,
 				const unsigned char *		buffer,
 				const BitmapDescription *	bd,
-				int				privateFormat,
-				double				factor );
+				int				privateFormat );
 
 extern int bmCanWriteGifFile(	const BitmapDescription *	bd,
-				int				privateFormat,
-				double				factor );
+				int				privateFormat );
 
 extern int bmWriteJpegFile(	const char *			filename,
 				const unsigned char *		buffer,
 				const BitmapDescription *	bd,
-				int				privateFormat,
-				double				factor );
+				int				privateFormat );
 
 extern int bmCanWriteJpegFile(	const BitmapDescription *	bd,
-				int				privateFormat,
-				double				factor );
+				int				privateFormat );
+
+/************************************************************************/
+
+extern int bmWriteRtfFile(	const char *			filename,
+				const unsigned char *		buffer,
+				const BitmapDescription *	bd,
+				int				privateFormat );
+
+extern int bmCanWriteRtfFile(	const BitmapDescription *	bd,
+				int				privateFormat );
+
 
 /************************************************************************/
 /*									*/
@@ -243,10 +261,7 @@ extern int bmPsPrintBitmap(	SimpleOutputStream *		sos,
 				double				yscale,
 				int				ox,
 				int				oy,
-				int				x0,
-				int				y0,
-				int				wide,
-				int				high,
+				const DocumentRectangle *	drSrc,
 				int				useFilters,
 				int				indexedImages,
 				const BitmapDescription *	bd,
@@ -258,10 +273,8 @@ extern int bmPsPrintBitmapImage( SimpleOutputStream *		sos,
 				double				yscale,
 				int				ox,
 				int				oy,
-				int				x0,
-				int				y0,
-				int				wide,
-				int				high,
+				const DocumentRectangle *	drSrc,
+				int				onWhite,
 				int				useFilters,
 				int				indexedImages,
 				const BitmapDescription *	bd,
@@ -270,16 +283,14 @@ extern int bmPsPrintBitmapImage( SimpleOutputStream *		sos,
 extern void bmPsWriteImageInstructions(
 				SimpleOutputStream *		sos,
 				const BitmapDescription *	bd,
+				int				onWhite,
 				int				wide,
 				int				high,
 				const char *			source,
 				int				indexedImages );
 
 extern int bmPsWriteBitmapData(	BitmapPrinter *			bp,
-				int				x0,
-				int				y0,
-				int				wide,
-				int				high,
+				const DocumentRectangle *	drSrc,
 				const BitmapDescription *	bd,
 				const unsigned char *		inputBuffer );
 
@@ -287,14 +298,37 @@ extern int bmPsRowStringSize(	const BitmapDescription *	bd,
 				int				pixelsWide,
 				int				indexedImages );
 
+extern int bmWmfWriteWmf(	const BitmapDescription *	bd,
+				const unsigned char *		buffer,
+				SimpleOutputStream *		sos );
+
+extern int bmRtfWriteWmfRtf(	const BitmapDescription *	bd,
+				const unsigned char *		buffer,
+				SimpleOutputStream *		sos );
+
+extern int bmRtfWriteRtf(	const BitmapDescription *	bd,
+				const unsigned char *		buffer,
+				SimpleOutputStream *		sos );
+
 extern int bmPngWritePng(	const BitmapDescription *	bd,
 				const unsigned char *		buffer,
 				SimpleOutputStream *		sos );
 
+extern int bmRtfWritePngRtf(	const BitmapDescription *	bd,
+				const unsigned char *		buffer,
+				SimpleOutputStream *		sos );
+
+extern int bmCanWritePngFile(	const BitmapDescription *	bd,
+				int				privateFormat );
+
 extern int bmJpegWriteJfif(	const BitmapDescription *	bd,
 				const unsigned char *		buffer,
 				SimpleOutputStream *		sos );
 
+extern int bmRtfWriteJfifRtf(	const BitmapDescription *	bd,
+				const unsigned char *		buffer,
+				SimpleOutputStream *		sos );
+
 extern int bmXvWritePaste(	const BitmapDescription *	bd,
 				const unsigned char *		buffer,
 				SimpleOutputStream *		sos );
@@ -323,18 +357,13 @@ extern int bmGifReadGif(	BitmapDescription *	bd,
 				unsigned char **	pBuffer,
 				SimpleInputStream *	sis );
 
-extern int bmBmpSaveDib(	const BitmapDescription *	bd,
-				const unsigned char *		buffer,
-				int				bytesWritten,
-				void *				voidsos );
-
 typedef int (*bmReadBitmap)(	BitmapDescription *		pBd,
 				unsigned char **		pBuffer,
 				SimpleInputStream *		sis );
 
 typedef int (*bmWriteBitmap)(	const BitmapDescription *	bd,
 				const unsigned char *		buffer,
-				SimpleOutputStream *		sis );
+				SimpleOutputStream *		sos );
 
 /************************************************************************/
 /*  Routines.								*/
@@ -350,27 +379,21 @@ extern int bmCopyDescription(	BitmapDescription *		bdOut,
 extern int bmWrite(	const char *			filename,
 			const unsigned char *		buffer,
 			const BitmapDescription *	bd,
-			int				fileFormat,
-			double				compressionFactor );
+			int				fileFormat );
 
 extern int bmCanWrite(	const BitmapDescription *	bd,
-			int				fileFormat,
-			double				compressionFactor );
+			int				fileFormat );
 
 extern int bmRead(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
-			int *			pFileFormat,
-			double *		pCompressionFactor );
+			int *			pFileFormat );
 
 extern int bmSelect(	BitmapDescription *		bdOut,
 			unsigned char **		pBufOut,
 			const BitmapDescription *	bdIn,
 			const unsigned char *		bufIn,
-			int				x0,
-			int				y0,
-			int				wide,
-			int				high );
+			const DocumentRectangle *	drSrc );
 
 extern int bmComponents( void ***		pComponents,
 			int *			pCount,
@@ -379,19 +402,19 @@ extern int bmComponents( void ***		pComponents,
 
 extern int bmComponentBitmap( unsigned char **		buffer,
 			BitmapDescription *		bdout,
-			BitmapSelection *		bs,
+			DocumentRectangle *		dr,
 			const BitmapDescription *	bdin,
-			const void *			component	);
+			const void *			component );
 
 extern int bmGroupBitmap( unsigned char **		buffer,
 			BitmapDescription *		bdout,
-			BitmapSelection *		bs,
+			DocumentRectangle *		dr,
 			const BitmapDescription *	bdin,
 			const void *			vbc	);
 
 int bmgGroupBitmap(	unsigned char **		pBuffer,
 			BitmapDescription *		bdout,
-			BitmapSelection *		bs,
+			DocumentRectangle *		dr,
 			const BitmapDescription *	bdin,
 			const void *			vcg );
 
@@ -422,7 +445,7 @@ int bmExpandRGB8Palette(	unsigned char *		to,
 /*									*/
 /************************************************************************/
 
-typedef void (*BM_PIX_FUN)(	void *				through,
+typedef int (*BM_PIX_FUN)(	void *				through,
 				int				row,
 				int				col );
 
@@ -456,7 +479,7 @@ extern int bmHorizontalFlip(	BitmapDescription *		bdOut,
 				const unsigned char *		bufIn,
 				int				ignoredInt );
 
-extern int bmUpsideDown(	BitmapDescription *		bdOut,
+extern int bmRotate180(		BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
 				unsigned char **		pBufOut,
 				const unsigned char *		bufIn,
@@ -522,7 +545,8 @@ extern int bmAverage(		BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
 				unsigned char **		pBufOut,
 				const unsigned char *		bufIn,
-				int				pixels );
+				int				xpixels,
+				int				ypixels );
 
 extern int bmExpandPaletteImage( BitmapDescription *		bdOut,
 				const BitmapDescription *	bdIn,
@@ -530,16 +554,6 @@ extern int bmExpandPaletteImage( BitmapDescription *		bdOut,
 				const unsigned char *		bufIn,
 				int				ignoredInt );
 
-extern int bmFillImage(		ColorAllocator *		ca,
-				int				bitmapUnit,
-				int				swapBitmapBytes,
-				int				swapBitmapBits,
-				int				dither,
-				unsigned char *			bufferOut,
-				const unsigned char *		bufferIn,
-				const BitmapDescription *	bdOut,
-				const BitmapDescription *	bdIn );
-
 extern int bmTransparentImage(	BitmapDescription *		bdOut,
 				unsigned char **		pBufOut,
 				int				colorEncoding,
@@ -572,6 +586,13 @@ extern void bmImageSizeTwips(
 			int *				pImageHighTwips,
 			const BitmapDescription *	bd );
 
+extern void bmRectangleSizeTwips(
+			int *				pRectangleWideTwips,
+			int *				pRectangleHighTwips,
+			const BitmapDescription *	bd,
+			int				pixelsWide,
+			int				pixelsHigh );
+
 extern int bmCalculateSizes(	BitmapDescription *	bd );
 
 extern int bmToGrayscale(
@@ -635,11 +656,11 @@ extern void bmDraw1BitImage(	const BitmapDescription *	bdTo,
 				unsigned char *			bufTo,
 				const BitmapDescription *	bdIn,
 				const unsigned char *		bufIn,
-				unsigned char			invertMask,
+				unsigned char			invertMaskIn,
 				int				rowTo0,
 				int				colTo0 );
 
-extern void bmForAll1Pixels(	const BitmapDescription *	bdIn,
+extern int bmForAll1Pixels(	const BitmapDescription *	bdIn,
 				const unsigned char *		bufIn,
 				unsigned char			invertMaskIn,
 				void *				through,
@@ -651,51 +672,99 @@ extern int bmFindLineSegments(	const BitmapDescription *	bdIn,
 				void *				through,
 				BM_SEG_FUN			segFun );
 
-extern int bmTextureMap(	ColorAllocator *		ca,
-				int				swapBitmapUnit,
-				int				swapBitmapBytes,
-				int				swapBitmapBits,
-				int				dither,
-				unsigned char *			bufferOut,
-				const unsigned char *		bufferIn,
-				const BitmapDescription *	bdOut,
-				const BitmapDescription *	bdIn,
-				double				E_x,
-				double				E_y,
-				double				E_z,
-				double				P_x,
-				double				P_y,
-				double				P_z,
-				double				N_x,
-				double				N_y,
-				double				N_z,
-				double				M_x,
-				double				M_y,
-				double				M_z );
+extern int bmCountLinePixels(	const unsigned char *		buffer,
+				const BitmapDescription *	bd,
+				int				x0,
+				int				y0,
+				int				x1,
+				int				y1 );
 
-extern int bmTextureMapInverse(	ColorAllocator *		ca,
-				int				swapBitmapUnit,
-				int				swapBitmapBytes,
-				int				swapBitmapBits,
-				int				dither,
-				unsigned char *			bufferOut,
-				const unsigned char *		bufferIn,
-				const BitmapDescription *	bdOut,
-				const BitmapDescription *	bdIn,
-				double				E_x,
-				double				E_y,
-				double				E_z,
-				double				P_x,
-				double				P_y,
-				double				P_z,
-				double				N_x,
-				double				N_y,
-				double				N_z,
-				double				M_x,
-				double				M_y,
-				double				M_z );
+extern int bmDrawLine(		unsigned char *			buffer,
+				const BitmapDescription *	bd,
+				int				x0,
+				int				y0,
+				int				x1,
+				int				y1,
+				int				wide );
 
-extern int bmSetColorAllocatorForImage(	ColorAllocator *		ca,
-					const BitmapDescription *	bd );
+extern int bmDrawBox(		unsigned char *			buffer,
+				const BitmapDescription *	bd,
+				int				x0,
+				int				y0,
+				int				x1,
+				int				y1,
+				int				wide );
 
+extern int bmSuggestFormat(	const char *			filename,
+				int				suggestedFormat,
+				const BitmapDescription *	bd );
+
+extern int bmSetSolidWhite(	unsigned char *			buffer,
+				BitmapDescription *		bd );
+
+extern int bmSetSolidBlack(	unsigned char *			buffer,
+				BitmapDescription *		bd );
+
+extern int bmCopyArea(		int				up,
+				int				vp,
+				unsigned char *			buffer1,
+				const unsigned char *		buffer2,
+				const BitmapDescription *	bd1,
+				const BitmapDescription *	bd2 );
+
+extern int bmPaintArea(		int				up,
+				int				vp,
+				unsigned char *			buffer1,
+				const unsigned char *		buffer2,
+				const BitmapDescription *	bd1,
+				const BitmapDescription *	bd2 );
+
+extern int bmEpsTestJpegEmbeddable(
+			unsigned int *			pPixelsWide,
+			unsigned int *			pPixelsHigh,
+			unsigned int *			pComponentCount,
+			unsigned int *			pBitsPerCompunent,
+			SimpleInputStream *		sis );
+
+extern int bmPsPrintJpegImage(
+			SimpleOutputStream *		sos,
+			double				xscale,
+			double				yscale,
+			int				componentCount,
+			int				ox,
+			int				oy,
+			int				pixelsWide,
+			int				pixelsHigh,
+			int				bitsPerComponent,
+			SimpleInputStream *		sis );
+
+extern int bmGetAlphaMask(
+			BitmapDescription *		bdOut,
+			const BitmapDescription *	bdIn,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufIn );
+
+extern int bmRemoveAlpha( BitmapDescription *		bdOut,
+			const BitmapDescription *	bdIn,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufIn );
+
+extern int bmSetAlphaMask(
+			BitmapDescription *		bdOut,
+			const BitmapDescription *	bdImage,
+			const BitmapDescription *	bdAlpha,
+			unsigned char **		pBufOut,
+			const unsigned char *		bufImage,
+			const unsigned char *		bufAlpha );
+
+extern void bmInitBitmapImage(	AppBitmapImage *	abi );
+extern void bmCleanBitmapImage(	AppBitmapImage *	abi );
+
+extern int bmMacPictGetImage(	AppBitmapImage *	abi,
+				DocumentRectangle *	drSource,
+				DocumentRectangle *	drDest,
+				SimpleInputStream *	sis,
+				int			packed,
+				int			direct,
+				int			clipRegion );
 #   endif
