@@ -1,8 +1,9 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	"bmintern.h"
 #   include	<string.h>
 #   include	<tiffio.h>
+#   include	<appSystem.h>
 #   include	<appDebugon.h>
 
 static BitmapFileType	bmTiffFile=
@@ -104,6 +105,28 @@ static BitmapFileType	bmEpsFile=
     "Encapsulated Postscript ( *.eps )",
     };
 
+static BitmapFileType	bmPdfFile=
+    {
+    bmWritePdfFile,
+    bmCanWritePdfFile,
+    NULL, /* bmReadPdfFile, */
+    "pdf",
+    "*.pdf",
+    "pdfFile",
+    "Acrobat PDF ( *.pdf )",
+    };
+
+static BitmapFileType	bmRtfFile=
+    {
+    bmWriteRtfFile,
+    bmCanWriteRtfFile,
+    NULL, /* bmReadRtfFile, */
+    "rtf",
+    "*.rtf",
+    "rtfFile",
+    "Rich Text Format ( *.rtf )",
+    };
+
 static BitmapFileType	bmXbmFile=
     {
     bmWriteXbmFile,
@@ -115,6 +138,17 @@ static BitmapFileType	bmXbmFile=
     "X-Windows bitmap ( *.xbm )",
     };
 
+static BitmapFileType	bmWmfFile=
+    {
+    bmWriteWmfFile,
+    bmCanWriteWmfFile,
+    NULL, /* bmReadWmfFile, */
+    "wmf",
+    "*.wmf",
+    "wmfFile",
+    "Windows Meta File ( *.wmf )",
+    };
+
 static BitmapFileType	bmPngFile=
     {
     bmWritePngFile,
@@ -181,6 +215,18 @@ static BitmapFileType	bmPpmFile=
     "Portable Pixmap ( *.ppm )",
     };
 
+static BitmapFileType	bmWbmpFile=
+    {
+    bmWriteWbmpFile,
+    bmCanWriteWbmpFile,
+    bmReadWbmpFile,
+    "wbmp",
+    "*.wbmp",
+    "wbmpFile",
+    "WAP Wireless Bitmap Format ( *.wbmp )",
+    };
+
+# if 0
 static BitmapFileType	bmSgiFile=
     {
     bmWriteSgiFile,
@@ -191,29 +237,58 @@ static BitmapFileType	bmSgiFile=
     "sgiFile",
     "SGI Image File ( *.sgi )",
     };
+# endif
 
 BitmapFileType * bmFileTypes[]=
     {
-    &bmTiffFile,
-    &bmTifFile,
-    &bmBmpFile,
-    &bmIcoFile,
-    &bmXwdFile,
-    &bmGifFile,
-    &bmEpsFile,
+    &bmPngFile,
     &bmJpegFile,
     &bmJpgFile,
+    &bmBmpFile,
+    &bmGifFile,
+    &bmXwdFile,
+    &bmEpsFile,
+    &bmPdfFile,
+    &bmRtfFile,
     &bmXbmFile,
-    &bmPngFile,
+    &bmIcoFile,
+    &bmTiffFile,
+    &bmTifFile,
     &bmPcxFile,
     &bmXpmFile,
     &bmPgmFile,
     &bmPbmFile,
     &bmPpmFile,
+    &bmWbmpFile,
+    &bmWmfFile,
     };
 
 BitmapFileFormat	bmFileFormats[]=
     {
+	{ "PNG  Portable Network Graphics", "png1File",
+			    0,				&bmPngFile },
+	{ "JPG  Independent JPEG Group (.jpg)", "jpg1File",
+			    1,				&bmJpgFile },
+	{ "JPEG  Independent JPEG Group (.jpeg)", "jpeg1File",
+			    1,				&bmJpegFile },
+	{ "BMP  Microsoft Windows 3.x bitmap", "bmp3File",
+			    40,				&bmBmpFile },
+	{ "GIF  Compuserve GIF", "gif87File",
+			    87,				&bmGifFile },
+	{ "XWD  X/11 Window dump", "xwd11File",
+			    11,				&bmXwdFile },
+	{ "XWD  X/10 Window dump", "xwd10File",
+			    10,				&bmXwdFile },
+	{ "EPS  Encapsulated Postscript", "eps1File",
+			    1,				&bmEpsFile },
+	{ "EPS  Encapsulated Postscript (Level 2)", "eps2File",
+			    2,				&bmEpsFile },
+	{ "PDF  Acrobat Portable Document Format", "pdf2File",
+			    12,				&bmPdfFile },
+	{ "XBM  X/11 Bitmap File", "xbm11File",
+			    11,				&bmXbmFile },
+	{ "ICO  Microsoft 3.x Windows Icon", "ico3File",
+			    40,				&bmIcoFile },
 	{ "TIFF  No compression", "tiffPlainFile",
 			    COMPRESSION_NONE,		&bmTiffFile },
 	{ "TIFF  CCITT run length", "tiffCcittRleFile",
@@ -234,26 +309,6 @@ BitmapFileFormat	bmFileFormats[]=
 			    COMPRESSION_THUNDERSCAN,	&bmTiffFile },
 	{ "TIFF  JPEG", "tiffJpegFile",
 			    COMPRESSION_JPEG,		&bmTiffFile },
-	{ "BMP  Microsoft Windows 3.x bitmap", "bmp3File",
-			    40,				&bmBmpFile },
-	{ "XWD  X/11 Window dump", "xwd11File",
-			    11,				&bmXwdFile },
-	{ "XWD  X/10 Window dump", "xwd10File",
-			    10,				&bmXwdFile },
-	{ "GIF  Compuserve GIF", "gif87File",
-			    87,				&bmGifFile },
-	{ "EPS  Encapsulated Postscript", "eps1File",
-			    1,				&bmEpsFile },
-	{ "JPG  Independent JPEG Group (.jpg)", "jpg1File",
-			    1,				&bmJpgFile },
-	{ "JPEG  Independent JPEG Group (.jpeg)", "jpeg1File",
-			    1,				&bmJpegFile },
-	{ "XBM  X/11 Bitmap File", "xbm11File",
-			    11,				&bmXbmFile },
-	{ "PNG  Portable Network Graphics", "png1File",
-			    0,				&bmPngFile },
-	{ "ICO  Microsoft 3.x Windows Icon", "ico3File",
-			    40,				&bmIcoFile },
 	{ "PCX  Zsoft PC Paintbrush file", "pcx1File",
 			    0,				&bmPcxFile },
 	{ "XPM  X11 Pixmap File", "xpm1File",
@@ -270,6 +325,16 @@ BitmapFileFormat	bmFileFormats[]=
 			    3,				&bmPbmFile },
 	{ "PPM  Pixmap (Raw)", "pgm6File",
 			    6,				&bmPbmFile },
+	{ "WBMP  Wireless Bitmap", "wbmp0File",
+			    0,				&bmWbmpFile },
+	{ "WMF  Windows Meta File", "wmfFile",
+			    0,				&bmWmfFile },
+	{ "RTF  Rich Text Format (\\pngblip)", "rtfPngFile",
+			    0,				&bmRtfFile },
+	{ "RTF  Rich Text Format (\\jpegblip)", "rtfJpegFile",
+			    1,				&bmRtfFile },
+	{ "RTF  Rich Text Format (\\wmetafile)", "rtfWmfFile",
+			    2,				&bmRtfFile },
 	/*
 	{ "SGI Image File", "sgiFile",
 			    0,				&bmSgiFile },
@@ -279,3 +344,48 @@ BitmapFileFormat	bmFileFormats[]=
 const int bmNumberOfFileFormats= sizeof(bmFileFormats)/sizeof(BitmapFileFormat);
 const int bmNumberOfFileTypes= sizeof(bmFileTypes)/sizeof(BitmapFileType *);
 
+
+static int bmTestWrite(		const BitmapDescription *	bd,
+				const char *			ext,
+				const BitmapFileFormat *	bff )
+    {
+    if  ( ext && strcmp( bff->bffFileType->bftFileExtension, ext ) )
+	{ return -1;	}
+    if  ( ! bff->bffFileType->bftWrite )
+	{ return -1;	}
+    if  ( (*bff->bffFileType->bftCanWrite) ( bd, bff->bffPrivate ) )
+	{ return -1;	}
+
+    return 0;
+    }
+
+int bmSuggestFormat(	const char *			filename,
+			int				suggestedFormat,
+			const BitmapDescription *	bd )
+    {
+    const char *		ext;
+    int				format= suggestedFormat;
+
+    ext= appFileExtensionOfName( filename );
+
+    if  ( ! ext						||
+	  format < 0					||
+	  bmTestWrite( bd, ext, bmFileFormats+ format ) )
+	{
+	int			i;
+	BitmapFileFormat *	bff;
+
+	bff= bmFileFormats;
+	for ( i= 0; i < bmNumberOfFileFormats; bff++, i++ )
+	    {
+	    if  ( ! bmTestWrite( bd, ext, bff ) )
+		{ format= i; break; }
+	    }
+
+	if  ( i == bmNumberOfFileFormats )
+	    { SLLDEB(filename,i,bmNumberOfFileFormats);	}
+	}
+
+    return format;
+    }
+
