@@ -27,195 +27,170 @@ extern unsigned char	Bmc7Masks[8];
 extern int bmReadTiffFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmWriteTiffFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmCanWriteTiffFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadBmpFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
+
+extern int bmReadWbmpFile(	const char *		filename,
+				unsigned char **	pBuffer,
+				BitmapDescription *	bd,
+				int *			pPrivateFormat );
 
 extern int bmReadIcoFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmCanWriteBmpFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
+
+extern int bmCanWriteWbmpFile( const BitmapDescription *	bd,
+				int			privateFormat );
 
 extern int bmCanWriteIcoFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmWriteBmpFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription * bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
+
+extern int bmWriteWbmpFile(	const char *		filename,
+				const unsigned char *	buffer,
+				const BitmapDescription * bd,
+				int			privateFormat );
 
 extern int bmWriteIcoFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription * bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadXwdFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmWriteXwdFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription * bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadSgiFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmWriteSgiFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription * bd,
-				int			privateFormat,
-				double			compressionFactor );
-
-extern int bmCanWriteGifFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadGifFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
-extern int bmWriteGifFile(	const char *		filename,
-				const unsigned char *	buffer,
-				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+extern int bmCanWriteEpsFile(	const BitmapDescription *	bd,
+				int			privateFormat );
 
-extern int bmCanWriteEpsFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+extern int bmCanWritePdfFile(	const BitmapDescription *	bd,
+				int			privateFormat );
 
 extern int bmWriteEpsFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
+
+extern int bmWritePdfFile(	const char *		filename,
+				const unsigned char *	buffer,
+				const BitmapDescription *	bd,
+				int			privateFormat );
+
 extern int bmReadJpegFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
-
-extern int bmCanWriteJpegFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
-
-extern int bmWriteJpegFile(	const char *		filename,
-				const unsigned char *	buffer,
-				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmReadXbmFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmCanWriteXbmFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmWriteXbmFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
+
+extern int bmCanWriteWmfFile( const BitmapDescription *	bd,
+				int			privateFormat );
+
+extern int bmWriteWmfFile(	const char *		filename,
+				const unsigned char *	buffer,
+				const BitmapDescription *	bd,
+				int			privateFormat );
 
 extern int bmReadPngFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
-
-extern int bmCanWritePngFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmWritePngFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadPcxFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmCanWritePcxFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmWritePcxFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadXpmFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmCanWriteXpmFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmWriteXpmFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmReadPbmFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
-				int *			pPrivateFormat,
-				double *		pCompressionFactor );
+				int *			pPrivateFormat );
 
 extern int bmCanWritePbmFile( const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmWritePbmFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription *	bd,
-				int			privateFormat,
-				double			compressionFactor );
+				int			privateFormat );
 
 extern int bmFillBlock(	unsigned char *		buffer,
 			int			x0,
@@ -225,24 +200,9 @@ extern int bmFillBlock(	unsigned char *		buffer,
 			int			bytesPerRow );
 
 extern unsigned char *	bmBackgroundBuffer(	int	bufferLength,
-						int	colorEncoding	);
-
-extern int bmDrawLine(	unsigned char *		buffer,
-			int			x0,
-			int			y0,
-			int			x1,
-			int			y1,
-			int			wide,
-			int			bytesPerRow );
-
-extern int bmDrawBox(	unsigned char *		buffer,
-			int			x0,
-			int			y0,
-			int			x1,
-			int			y1,
-			int			width,
-			int			bytesPerRow );
-
+						int	colorEncoding );
+extern unsigned char *	bmForegroundBuffer(	int	bufferLength,
+						int	colorEncoding );
 
 extern int bmMakeGrayPalette(	const BitmapDescription *	bd,
 				int *				pColorCount,
@@ -261,19 +221,16 @@ extern int bmInflateTo8bitGray(
 			int				pixelsWide );
 
 extern int bmMakeMonochrome(		BitmapDescription *	bd,
-					RGB8Color *		palette,
 					unsigned char *		buffer );
 
-extern int bmSetColorAllocatorForPaletteImage(
-					ColorAllocator *		ca,
+extern int bmPlanarToChunky(		unsigned char *			to,
+					const unsigned char *		from,
 					const BitmapDescription *	bd );
 
-extern int bmSetColorAllocatorForBWImage(
-					ColorAllocator *		ca,
-					const BitmapDescription *	bd );
-
-extern int bmSetColorAllocatorForWBImage(
-					ColorAllocator *		ca,
-					const BitmapDescription *	bd );
+extern int bmPaletteColor(	BitmapDescription *		bd,
+				int				r,
+				int				g,
+				int				b,
+				int				a );
 
 #   endif /*  BM_INTERN_H	*/
