@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
@@ -82,8 +82,6 @@ static int write_JPEG_file (	BmJpegOutputDestination *	bjod );
 
 /**/
 
-static void my_error_exit (j_common_ptr cinfo);
-
 /************************************************************************/
 /*									*/
 /*  Read  a JPEG File.							*/
@@ -157,7 +155,8 @@ static void bmJpegSkipInputData(	j_decompress_ptr	cinfo,
 /*									*/
 /************************************************************************/
 
-static int bmJpegGetReadParameters( struct jpeg_decompress_struct * cinfo )
+static int bmJpegGetReadParameters(
+				struct jpeg_decompress_struct *	cinfo )
     {
     BmJpegInputSource *		bjis= (BmJpegInputSource *)cinfo->src;
 
@@ -167,42 +166,76 @@ static int bmJpegGetReadParameters( struct jpeg_decompress_struct * cinfo )
 
     switch( cinfo->density_unit )
 	{
-	case	0:
+	case 0:
 	    /* LDEB(cinfo->density_unit); */
-	    if  ( cinfo->X_density < 5 )
+	    if  ( cinfo->X_density == 1 && cinfo->Y_density == 1 )
 		{
-		bjis->bjisBd.bdUnit= BMunPOINT;
-		if  ( cinfo->X_density < 1 )
-		    { cinfo->X_density= 1;	}
-		if  ( cinfo->Y_density < 1 )
-		    { cinfo->Y_density= 1;	}
+		int		xd;
+		int		yd;
+
+		if  ( cinfo->image_width >= cinfo->image_height )
+		    {
+		    xd= ( 100* cinfo->image_width  )/ 16;
+		    yd= ( 100* cinfo->image_height )/ 12;
+		    }
+		else{
+		    xd= ( 100* cinfo->image_width  )/ 12;
+		    yd= ( 100* cinfo->image_height )/ 16;
+		    }
+
+		if  ( xd > yd )	{ yd= xd;	}
+		else		{ xd= yd;	}
+
+		if  ( xd > 80 )
+		    {
+		    xd= 100* ( ( xd+ 20 )/ 100 );
+		    yd= 100* ( ( yd+ 20 )/ 100 );
+
+		    bjis->bjisBd.bdUnit= BMunM;
+		    bjis->bjisBd.bdXResolution= xd;
+		    bjis->bjisBd.bdYResolution= yd;
+		    break;
+		    }
+		}
+
+	    if  ( cinfo->X_density < 5 || cinfo->Y_density < 5 )
+		{
+	      pixel_density:
+		bjis->bjisBd.bdUnit= BMunPIXEL;
+		bjis->bjisBd.bdXResolution= 1;
+		bjis->bjisBd.bdYResolution= 1;
+		}
+	    else{
+		bjis->bjisBd.bdUnit= BMunINCH;
 		bjis->bjisBd.bdXResolution= cinfo->X_density;
 		bjis->bjisBd.bdYResolution= cinfo->Y_density;
 		}
-	    else{
-		if  ( cinfo->X_density < 30 )
-		    {
-		    bjis->bjisBd.bdUnit= BMunCM;
-		    bjis->bjisBd.bdXResolution= cinfo->X_density;
-		    bjis->bjisBd.bdYResolution= cinfo->Y_density;
-		    }
-		else{
-		    bjis->bjisBd.bdUnit= BMunINCH;
-		    bjis->bjisBd.bdXResolution= cinfo->X_density;
-		    bjis->bjisBd.bdYResolution= cinfo->Y_density;
-		    }
-		}
 	    break;
-	case	1:
+
+	case 1:
+	    if  (  cinfo->X_density < 5 || cinfo->Y_density < 5 )
+		{
+		LLLDEB(cinfo->density_unit,cinfo->X_density,cinfo->Y_density);
+		goto pixel_density;
+		}
+
 	    bjis->bjisBd.bdUnit= BMunINCH;
 	    bjis->bjisBd.bdXResolution= cinfo->X_density;
 	    bjis->bjisBd.bdYResolution= cinfo->Y_density;
 	    break;
-	case	2:
-	    bjis->bjisBd.bdUnit= BMunCM;
-	    bjis->bjisBd.bdXResolution= cinfo->X_density;
-	    bjis->bjisBd.bdYResolution= cinfo->Y_density;
+
+	case 2:
+	    if  (  cinfo->X_density < 2 || cinfo->Y_density < 2 )
+		{
+		LLLDEB(cinfo->density_unit,cinfo->X_density,cinfo->Y_density);
+		goto pixel_density;
+		}
+
+	    bjis->bjisBd.bdUnit= BMunM;
+	    bjis->bjisBd.bdXResolution= 100* cinfo->X_density;
+	    bjis->bjisBd.bdYResolution= 100* cinfo->Y_density;
 	    break;
+
 	default:
 	    LDEB(cinfo->density_unit);
 	    return -1;
@@ -234,15 +267,20 @@ static int bmJpegGetReadParameters( struct jpeg_decompress_struct * cinfo )
 	    break;
 
 	case JCS_UNKNOWN:
+	    LLDEB(JCS_UNKNOWN,cinfo->out_color_space); return -1;
 	case JCS_YCbCr:
+	    LLDEB(JCS_YCbCr,cinfo->out_color_space); return -1;
 	case JCS_YCCK:
+	    LLDEB(JCS_YCCK,cinfo->out_color_space); return -1;
 	case JCS_CMYK:
-	    LDEB(cinfo->out_color_space);
-	    return -1;
-
+	    /*  Illegal Inverted CMYK file from photoshop.	*/
+	    LLDEB(JCS_CMYK,cinfo->out_color_space);
+	    bjis->bjisBd.bdColorEncoding= BMcoRGB;
+	    bjis->bjisBd.bdSamplesPerPixel= 3;
+	    bjis->bjisBd.bdBitsPerPixel= 3* bjis->bjisBd.bdBitsPerSample;
+	    break;
 	default:
-	    LDEB(cinfo->out_color_space);
-	    return -1;
+	    LDEB(cinfo->out_color_space); return -1;
 	}
 
     bjis->bjisBd.bdBytesPerRow= (	bjis->bjisBd.bdPixelsWide*
@@ -265,11 +303,11 @@ static int bmJpegGetReadParameters( struct jpeg_decompress_struct * cinfo )
 
 /************************************************************************/
 /*									*/
-/*  Store one row of pixels.						*/
+/*  Remember one row of pixels.						*/
 /*									*/
 /************************************************************************/
 
-static int bmJpegStoreScanline( JSAMPARRAY			pixel_data,
+static int bmJpegRememberScanline( JSAMPARRAY			pixel_data,
 				struct jpeg_decompress_struct * cinfo	)
     {
     BmJpegInputSource *		bjis= (BmJpegInputSource *)cinfo->src;
@@ -322,10 +360,48 @@ static int bmJpegStoreScanline( JSAMPARRAY			pixel_data,
 		}
 	    break;
 
+	case JCS_CMYK:
+	    /************************************************************/
+	    /*  Illegal Inverted CMYK file from photoshop.		*/
+	    /*  Very much approximate: only to get the image on screen.	*/
+	    /*  Ted tries to emit the image to PostScript as jpeg	*/
+	    /*  anyway. (Though jfif requires YCbCr)			*/
+	    /************************************************************/
+	    switch( bjis->bjisBd.bdBitsPerSample )
+		{
+		case 8:
+		    to= bjis->bjisBitmapBuffer+
+			bjis->bjisBd.bdBytesPerRow*
+					    (bjis->bjisRowsReceived++);
+
+		    ptr0 = pixel_data[0];
+
+		    for ( col = 0; col < cinfo->image_width; col++ )
+			{
+			int	c= 255- GETJSAMPLE(*(ptr0++));
+			int	m= 255- GETJSAMPLE(*(ptr0++));
+			int	y= 255- GETJSAMPLE(*(ptr0++));
+			int	k= 255- GETJSAMPLE(*(ptr0++));
+
+			c= c- (c*k)/255+ k;
+			m= m- (m*k)/255+ k;
+			y= y- (y*k)/255+ k;
+
+			*(to++)= 255- c; /* red */
+			*(to++)= 255- m; /* green */
+			*(to++)= 255- y; /* blue */
+			}
+		    break;
+
+		default:
+		    LDEB(bjis->bjisBd.bdBitsPerSample); 
+		    return -1;
+		}
+	    break;
+
 	case JCS_UNKNOWN:
 	case JCS_YCbCr:
 	case JCS_YCCK:
-	case JCS_CMYK:
 	    LDEB(cinfo->out_color_space);
 	    return -1;
 	    break;
@@ -382,8 +458,7 @@ int bmJpegReadJfif(	BitmapDescription *	bd,
 int bmReadJpegFile(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
-			int *			pPrivateFormat,
-			double *		pCompressionFactor )
+			int *			pPrivateFormat )
     {
     SimpleInputStream *		sis;
 
@@ -430,7 +505,12 @@ static int read_JPEG_file(	BmJpegInputSource * bjis )
 
     (void) jpeg_read_header( &cinfo, TRUE );
 
-    bmJpegGetReadParameters( &cinfo );
+    if  ( bmJpegGetReadParameters( &cinfo ) )
+	{
+	LDEB(1);
+	jpeg_destroy_decompress( &cinfo );
+	return 0;
+	}
 
     jpeg_start_decompress( &cinfo );
 
@@ -442,7 +522,7 @@ static int read_JPEG_file(	BmJpegInputSource * bjis )
 	{
 	(void)jpeg_read_scanlines( &cinfo, buffer, 1 );
 
-	bmJpegStoreScanline( buffer, &cinfo );
+	bmJpegRememberScanline( buffer, &cinfo );
 	}
 
     (void)jpeg_finish_decompress( &cinfo );
@@ -459,8 +539,7 @@ static int read_JPEG_file(	BmJpegInputSource * bjis )
 /************************************************************************/
 
 int bmCanWriteJpegFile(	const BitmapDescription *	bd,
-			int				PrivateFormat,
-			double				CompressionFactor )
+			int				privateFormat )
     {
     if  ( bd->bdBitsPerSample != 8 )
 	{ /* LDEB(bd->bdBitsPerSample); */ return -1;	}
@@ -502,18 +581,20 @@ static int bmJpegFillCompressor(	struct jpeg_compress_struct *	cinfo,
     cinfo->image_width= bd->bdPixelsWide;
     cinfo->image_height= bd->bdPixelsHigh;
 
-    cinfo->input_components= bd->bdSamplesPerPixel;
-
     switch( bd->bdColorEncoding )
 	{
 	case BMcoBLACKWHITE:
 	case BMcoWHITEBLACK:
+	    cinfo->input_components= 1;
 	    cinfo->in_color_space= JCS_GRAYSCALE;
 	    break;
+
 	case BMcoRGB:
 	case BMcoRGB8PALETTE:
+	    cinfo->input_components= 3;
 	    cinfo->in_color_space= JCS_RGB;
 	    break;
+
 	default:
 	    LDEB(bd->bdColorEncoding);
 	    return -1;
@@ -528,16 +609,22 @@ static int bmJpegFillCompressor(	struct jpeg_compress_struct *	cinfo,
 	    cinfo->X_density= bd->bdXResolution;
 	    cinfo->Y_density= bd->bdYResolution;
 	    break;
-	case BMunCM:
+
+	case BMunM:
 	    cinfo->density_unit= 2;
-	    cinfo->X_density= bd->bdXResolution;
-	    cinfo->Y_density= bd->bdYResolution;
+	    cinfo->X_density= ( bd->bdXResolution+ 50 )/ 100;
+	    cinfo->Y_density= ( bd->bdYResolution+ 50 )/ 100;
 	    break;
+
 	case BMunPOINT:
 	    cinfo->density_unit= 1;
 	    cinfo->X_density= 72* bd->bdXResolution;
 	    cinfo->Y_density= 72* bd->bdYResolution;
 	    break;
+
+	case BMunPIXEL:
+	    break;
+
 	default:
 	    LDEB(bd->bdUnit);
 	    /************************************/
@@ -573,8 +660,16 @@ static int bmJpegCompressRow(	struct jpeg_compress_struct *	cinfo,
 	{
 	case BMcoBLACKWHITE:
 	    to= bjod->bjodScratchBuffer;
-	    for ( i= 0; i < bd->bdBytesPerRow; to++, from++, i++ )
-		{ *to= 255- *from; }
+
+	    if  ( bd->bdHasAlpha )
+		{
+		for ( i= 0; i < bd->bdPixelsWide; to++, from += 2, i++ )
+		    { *to= 255- *from; }
+		}
+	    else{
+		for ( i= 0; i < bd->bdPixelsWide; to++, from++, i++ )
+		    { *to= 255- *from; }
+		}
 
 	    row_pointer[0]= (JSAMPROW)bjod->bjodScratchBuffer;
 	    res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
@@ -583,16 +678,65 @@ static int bmJpegCompressRow(	struct jpeg_compress_struct *	cinfo,
 	case BMcoRGB8PALETTE:
 	    bmExpandRGB8Palette( bjod->bjodScratchBuffer, from,
 				    bd->bdPixelsWide, bd->bdBitsPerPixel,
-				    bd->bdRGB8Palette, 0 );
+				    bd->bdRGB8Palette, bd->bdHasAlpha );
 
 	    row_pointer[0]= (JSAMPROW)bjod->bjodScratchBuffer;
 	    res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
 	    break;
 
-	default:
-	    row_pointer[0]= (JSAMPROW)from;
-	    res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
+	case BMcoWHITEBLACK:
+	    if  ( bd->bdHasAlpha )
+		{
+		to= bjod->bjodScratchBuffer;
+
+		for ( i= 0; i < bd->bdPixelsWide; i++ )
+		    {
+		    *(to++)= *(from++);
+		    from++;
+		    }
+
+		row_pointer[0]= (JSAMPROW)bjod->bjodScratchBuffer;
+		res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
+		}
+	    else{
+		row_pointer[0]= (JSAMPROW)from;
+		res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
+		}
 	    break;
+
+	case BMcoRGB:
+	    if  ( bd->bdHasAlpha )
+		{
+		to= bjod->bjodScratchBuffer;
+
+		for ( i= 0; i < bd->bdPixelsWide; i++ )
+		    {
+		    if  ( from[3] >= 128 )
+			{
+			*(to++)= *(from++);
+			*(to++)= *(from++);
+			*(to++)= *(from++);
+			from++;
+			}
+		    else{
+			*(to++)= 255;
+			*(to++)= 255;
+			*(to++)= 255;
+			from += 4;
+			}
+		    }
+
+		row_pointer[0]= (JSAMPROW)bjod->bjodScratchBuffer;
+		res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
+		}
+	    else{
+		row_pointer[0]= (JSAMPROW)from;
+		res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
+		}
+	    break;
+
+	default:
+	    LDEB(bd->bdColorEncoding); return -1;
 	}
 
     if  ( res != 1 )
@@ -650,6 +794,7 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 			const unsigned char *		buffer,
 			SimpleOutputStream *		sos )
     {
+    int				rval= 0;
     BmJpegOutputDestination	bjod;
 
     bjod.bjodBd= bd;
@@ -668,7 +813,7 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 	    if  ( ! bjod.bjodScratchBuffer )
 		{
 		LXDEB(bd->bdBytesPerRow,bjod.bjodScratchBuffer);
-		return -1;
+		rval= -1; goto ready;
 		}
 	    break;
 
@@ -677,7 +822,19 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 	    if  ( ! bjod.bjodScratchBuffer )
 		{
 		LXDEB(bd->bdBytesPerRow,bjod.bjodScratchBuffer);
-		return -1;
+		rval= -1; goto ready;
+		}
+	    break;
+
+	case BMcoRGB:
+	    if  ( bd->bdHasAlpha )
+		{
+		bjod.bjodScratchBuffer= malloc( 3* bd->bdPixelsWide );
+		if  ( ! bjod.bjodScratchBuffer )
+		    {
+		    LXDEB(bd->bdBytesPerRow,bjod.bjodScratchBuffer);
+		    rval= -1; goto ready;
+		    }
 		}
 	    break;
 
@@ -686,26 +843,20 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 	}
 
     if  ( write_JPEG_file( &bjod ) )
-	{
-	LDEB(1);
+	{ LDEB(1); rval= -1; goto ready; }
 
-	if  ( bjod.bjodScratchBuffer )
-	    { free( bjod.bjodScratchBuffer );	}
-
-	return -1;
-	}
+  ready:
 
     if  ( bjod.bjodScratchBuffer )
 	{ free( bjod.bjodScratchBuffer );	}
 
-    return 0;
+    return rval;
     }
 
 int bmWriteJpegFile(	const char *			filename,
 			const unsigned char *		buffer,
 			const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor )
+			int				privateFormat )
     {
     SimpleOutputStream *	sos;
 
