@@ -15,27 +15,65 @@ typedef struct DilateJob
     {
     BitmapDescription		djBdOut;
     unsigned char *		djBufOut;
-    unsigned char		djInvertMaskSel;
+				    /************************************/
+				    /*  Output destination.		*/
+				    /************************************/
+
     const BitmapDescription *	djBdSel;
     const unsigned char *	djBufSel;
+    unsigned char		djInvertMaskSel;
+				    /************************************/
+				    /* Structuring element.		*/
+				    /************************************/
     int				djRowSelOrig;
     int				djColSelOrig;
+				    /************************************/
+				    /* Origin of structuring element.	*/
+				    /************************************/
+
+    const BitmapDescription *	djBdIn;
+    const unsigned char *	djBufIn;
+    unsigned char		djInvertMaskIn;
+				    /************************************/
+				    /* Input image.			*/
+				    /************************************/
     } DilateJob;
 
-static void bmMorphoDilatePixel(	void *			voiddj,
-					int			rowIn,
-					int			colIn )
+/************************************************************************/
+/*									*/
+/*  Draw the input buffer for a foreground pixel in the structuring	*/
+/*  element.								*/
+/*									*/
+/*  As the input image is supposed to be much bigger than the		*/
+/*  structuring element, it is more efficient to repeatedly draw the	*/
+/*  image than repeatedly draw the element: The tight loops that copy	*/
+/*  the rows of the image are more efficient than the coordinating code	*/
+/*  around them.							*/
+/*									*/
+/*  NOTE: The efficiency assumption could be wrong for a big black	*/
+/*  structuring element.						*/
+/*									*/
+/************************************************************************/
+
+static int bmMorphoDilateElementPixel(	void *			voiddj,
+					int			rowSel,
+					int			colSel )
     
     {
     DilateJob *	dj= (DilateJob *)voiddj;
 
     bmDraw1BitImage( &(dj->djBdOut), dj->djBufOut,
-					dj->djBdSel, dj->djBufSel,
-					dj->djInvertMaskSel,
-					rowIn- dj->djRowSelOrig,
-					colIn- dj->djColSelOrig );
+					dj->djBdIn, dj->djBufIn,
+					dj->djInvertMaskIn,
+					rowSel- dj->djRowSelOrig,
+					colSel- dj->djColSelOrig );
 
-    return;
+    /* test code:
+    bmCopyArea( colIn- dj->djColSelOrig, rowIn- dj->djRowSelOrig,
+		    dj->djBufOut, dj->djBufIn, &(dj->djBdOut), dj->djBdIn );
+    */
+
+    return 0;
     }
 
 /************************************************************************/
@@ -91,19 +129,30 @@ static int bmMorphoCheckInputFormats(	const BitmapDescription *	bdIn,
 
 static void bmMorphoStartDilateJob(
 				DilateJob *			dj,
+				const BitmapDescription *	bdIn,
+				const unsigned char *		bufIn,
 				const BitmapDescription *	bdSel,
 				const unsigned char *		bufSel,
-				int				rowOrig,
-				int				colOrig )
+				int				rowSelOrig,
+				int				colSelOrig )
     {
     bmInitDescription( &(dj->djBdOut) );
 
     dj->djBufOut= (unsigned char *)0;
-    dj->djInvertMaskSel= 0x00;
+    dj->djBdIn= bdIn;
+    dj->djBufIn= bufIn;
     dj->djBdSel= bdSel;
     dj->djBufSel= bufSel;
-    dj->djRowSelOrig= rowOrig;
-    dj->djColSelOrig= colOrig;
+    dj->djRowSelOrig= rowSelOrig;
+    dj->djColSelOrig= colSelOrig;
+
+    if  ( bdIn->bdColorEncoding == BMcoWHITEBLACK )
+	{ dj->djInvertMaskIn= 0xff;	}
+    else{ dj->djInvertMaskIn= 0x00;	}
+
+    if  ( bdSel->bdColorEncoding == BMcoWHITEBLACK )
+	{ dj->djInvertMaskSel= 0xff;	}
+    else{ dj->djInvertMaskSel= 0x00;	}
 
     return;
     }
@@ -118,42 +167,70 @@ static void bmMorphoStartDilateJob(
 /*  this means that the colors are swapped.				*/
 /*									*/
 /*  1)  Allocate output resources.					*/
-/*  2)  Determine the mask to ignore the trailing bits in the input	*/
-/*	image.								*/
-/*  3)  Dilate the individual rows.					*/
+/*  2)  Draw input image for all pixels in the structuring element.	*/
+/*  3)  Return output.							*/
+/*  4)  Cleanup.							*/
 /*									*/
 /************************************************************************/
 
 static int bmMorphoDilateLow(	DilateJob *			dj,
+				int				reverse,
 				BitmapDescription *		bdOut,
-				const BitmapDescription *	bdIn,
-				unsigned char **		pBufOut,
-				const unsigned char *		bufIn,
-				unsigned char			invertMaskIn )
+				unsigned char **		pBufOut )
     {
     int			rval= 0;
 
     /*  1  */
-    if  ( bmCopyDescription( &(dj->djBdOut), bdIn ) )
+    if  ( bmCopyDescription( &(dj->djBdOut), dj->djBdIn ) )
 	{ LDEB(1); rval= -1; goto ready;	}
 
-    dj->djBufOut= malloc( dj->djBdOut.bdBufferLength+ 1 );
+    /*
+    dj->djBdOut.bdPixelsWide += dj->djBdSel->bdPixelsWide- 1;
+    dj->djBdOut.bdPixelsHigh += dj->djBdSel->bdPixelsHigh- 1;
+    */
+
+    bmCalculateSizes( &(dj->djBdOut) );
+
+    if  ( reverse )
+	{
+	dj->djBufOut= bmForegroundBuffer( dj->djBdOut.bdBufferLength,
+						dj->djBdOut.bdColorEncoding );
+	}
+    else{
+	dj->djBufOut= bmBackgroundBuffer( dj->djBdOut.bdBufferLength,
+						dj->djBdOut.bdColorEncoding );
+	}
     if  ( ! dj->djBufOut )
 	{
 	LXDEB(dj->djBdOut.bdBufferLength,dj->djBufOut);
 	rval= -1; goto ready;
 	}
 
-#   if	PARANOIA
-    bdSelPara= bdSel;
-    bdOutPara= &dj->djBdOut;
+#   if 0
+    {
+    const char *	name= "/tmp/background.png";
+    int			format;
+
+    format=  bmSuggestFormat( name, -1, &(dj->djBdOut) );
+    bmWrite( name, dj->djBufOut, &(dj->djBdOut), format, 1.0 );
+    }
 #   endif
 
-    memset( dj->djBufOut, 0, dj->djBdOut.bdBufferLength+ 1 );
+    /*  2  */
+    bmForAll1Pixels( dj->djBdSel, dj->djBufSel,
+	    dj->djInvertMaskSel, (void *)dj, bmMorphoDilateElementPixel );
 
-    bmForAll1Pixels( bdIn, bufIn, invertMaskIn,
-					(void *)dj, bmMorphoDilatePixel );
+#   if 0
+    {
+    const char *	name= "/tmp/dilated.png";
+    int			format;
 
+    format=  bmSuggestFormat( name, -1, &(dj->djBdOut) );
+    bmWrite( name, dj->djBufOut, &(dj->djBdOut), format, 1.0 );
+    }
+#   endif
+
+    /*  3  */
     if  ( bmCopyDescription( bdOut, &(dj->djBdOut) ) )
 	{ LDEB(1); rval= -1; goto ready;	}
 
@@ -163,6 +240,7 @@ static int bmMorphoDilateLow(	DilateJob *			dj,
 
   ready:
 
+    /*  4  */
     if  ( dj->djBufOut )
 	{ free( dj->djBufOut );	}
 
@@ -183,18 +261,19 @@ int bmMorphoDilate(		BitmapDescription *		bdOut,
 				const unsigned char *		bufIn,
 				const BitmapDescription *	bdSel,
 				const unsigned char *		bufSel,
-				int				rowOrig,
-				int				colOrig )
+				int				rowSelOrig,
+				int				selColOrig )
     {
-    const unsigned char	invertMaskIn= 0x00;
     DilateJob		dj;
+    int			reverse= 0;
 
     if  ( bmMorphoCheckInputFormats( bdIn, bdSel ) )
 	{ LDEB(1); return -1;	}
 
-    bmMorphoStartDilateJob( &dj, bdSel, bufSel, rowOrig, colOrig );
+    bmMorphoStartDilateJob( &dj, bdIn, bufIn, bdSel, bufSel,
+						rowSelOrig, selColOrig );
 
-    return bmMorphoDilateLow( &dj, bdOut, bdIn, pBufOut, bufIn, invertMaskIn );
+    return bmMorphoDilateLow( &dj, reverse, bdOut, pBufOut );
     }
 
 /************************************************************************/
@@ -204,6 +283,14 @@ int bmMorphoDilate(		BitmapDescription *		bdOut,
 /*  Use the fact that the erosion is the complement of the dilation of	*/
 /*  the complement of the input with the mirrored structuring element.	*/
 /*									*/
+/*  1)  Initialization and verification of the input parameters.	*/
+/*  2)  Mirror structuring element.					*/
+/*  3)  Set dilate parameters. Realize that these are dilate parameters	*/
+/*	and that we want to erode.					*/
+/*  4)  To get the complement of the input image, swap the invert mask	*/
+/*	for the input image.						*/
+/*  5)  Invert the output image.					*/
+/*									*/
 /************************************************************************/
 
 int bmMorphoErode(		BitmapDescription *		bdOut,
@@ -212,37 +299,37 @@ int bmMorphoErode(		BitmapDescription *		bdOut,
 				const unsigned char *		bufIn,
 				const BitmapDescription *	bdSel,
 				const unsigned char *		bufSel,
-				int				rowOrig,
-				int				colOrig )
+				int				rowSelOrig,
+				int				selColOrig )
     {
     int			rval= 0;
-    const unsigned char	invertMaskIn= 0xff;
     DilateJob		dj;
 
     unsigned char *	bufRot= (unsigned char *)0;
     BitmapDescription	bdRot;
 
-    int			i;
-    unsigned char *	to;
+    int			reverse= 1;
 
+    /*  1  */
     bmInitDescription( &bdRot );
 
     if  ( bmMorphoCheckInputFormats( bdIn, bdSel ) )
 	{ LDEB(1); rval= -1; goto ready;	}
 
-    if  ( bmUpsideDown( &bdRot, bdSel, &bufRot, bufSel, 180 ) )
+    /*  2  */
+    if  ( bmRotate180( &bdRot, bdSel, &bufRot, bufSel, 180 ) )
 	{ LDEB(180); rval= -1; goto ready;	}
 
-    bmMorphoStartDilateJob( &dj, &bdRot, bufRot,
-				bdSel->bdPixelsHigh- rowOrig- 1,
-				bdSel->bdPixelsWide- colOrig- 1 );
+    /*  3  */
+    bmMorphoStartDilateJob( &dj, bdIn, bufIn, &bdRot, bufRot,
+						    rowSelOrig, selColOrig );
 
-    if  ( bmMorphoDilateLow( &dj, bdOut, bdIn, pBufOut, bufIn, invertMaskIn ) )
-	{ XDEB(invertMaskIn); rval= -1; goto ready;	}
+    /*  4,5  */
+    dj.djInvertMaskIn= ~dj.djInvertMaskIn;
+    reverse= 1;
 
-    to= *pBufOut;
-    for ( i= 0; i < bdOut->bdBufferLength; i++, to++ )
-	{ to[0]= ~to[0]; }
+    if  ( bmMorphoDilateLow( &dj, reverse, bdOut, pBufOut ) )
+	{ XDEB(dj.djInvertMaskIn); rval= -1; goto ready;	}
 
   ready:
 
@@ -267,35 +354,8 @@ int bmMorphoSetSimpleSe(	BitmapDescription *		bdOut,
     const int		wide= size;
     const int		high= size;
 
-    int			bytesPerRow;
-    int			bufferLength;
-
     unsigned char *	bufOut;
 
-    int			row;
-    unsigned char	last;
-
-    bytesPerRow= ( wide+ 7 )/ 8;
-    bufferLength= high* bytesPerRow;
-
-    bufOut= malloc( bufferLength );
-    if  ( ! bufOut )
-	{ LXDEB(bufferLength,bufOut); return -1;	}
-
-    last= 0xff;
-    if  ( wide % 8 )
-	{ last= ( 0xff << ( 8- wide % 8 ) ) & 0xff;	}
-
-    for ( row= 0; row < high; row++ )
-	{
-	unsigned char *	to= bufOut+ row* bytesPerRow;
-	int		byte;
-
-	for ( byte= 0; byte < bytesPerRow- 1; to++, byte++ )
-	    { *to= 0xff; }
-	*to= last;
-	}
-
     bmInitDescription( bdOut );
 
     bdOut->bdColorEncoding= BMcoWHITEBLACK;
@@ -307,10 +367,13 @@ int bmMorphoSetSimpleSe(	BitmapDescription *		bdOut,
     bdOut->bdXResolution= 1;
     bdOut->bdYResolution= 1;
     bdOut->bdUnit= BMunPIXEL;
-    bdOut->bdColorEncoding= BMcoWHITEBLACK;
 
-    bdOut->bdBytesPerRow= bytesPerRow;
-    bdOut->bdBufferLength= bufferLength;
+    bmCalculateSizes( bdOut );
+
+    bufOut= bmBackgroundBuffer(
+			    bdOut->bdBufferLength, bdOut->bdColorEncoding );
+    if  ( ! bufOut )
+	{ LXDEB(bdOut->bdBufferLength,bufOut); return -1;	}
 
     *pBufOut= bufOut;
     return 0;
@@ -336,8 +399,8 @@ int bmMorphoDilateSimple(	BitmapDescription *		bdOut,
     BitmapDescription	bdSel;
     unsigned char *	bufSel= (unsigned char *)0;
 
-    const int		rowOrig= 1;
-    const int		colOrig= 1;
+    const int		rowSelOrig= 1;
+    const int		selColOrig= 1;
 
     bmInitDescription( &bdSel );
 
@@ -345,8 +408,8 @@ int bmMorphoDilateSimple(	BitmapDescription *		bdOut,
 	{ LDEB(1); rval= -1; goto ready;	}
 
     if  ( bmMorphoDilate( bdOut, bdIn, pBufOut, bufIn,
-					&bdSel, bufSel, rowOrig, colOrig ) )
-	{ LLDEB(rowOrig,colOrig); rval= -1; goto ready;	}
+				    &bdSel, bufSel, rowSelOrig, selColOrig ) )
+	{ LLDEB(rowSelOrig,selColOrig); rval= -1; goto ready;	}
 
   ready:
 
@@ -371,11 +434,9 @@ int bmMorphoErodeSimple(	BitmapDescription *		bdOut,
     BitmapDescription	bdSel;
     unsigned char *	bufSel= (unsigned char *)0;
 
-    const int		rowOrig= 1;
-    const int		colOrig= 1;
-
-    int			i;
-    unsigned char *	to;
+    const int		rowSelOrig= 1;
+    const int		selColOrig= 1;
+    int			reverse= 1;
 
     bmInitDescription( &bdSel );
 
@@ -385,15 +446,16 @@ int bmMorphoErodeSimple(	BitmapDescription *		bdOut,
     if  ( bmMorphoCheckInputFormats( bdIn, &bdSel ) )
 	{ LDEB(1); rval= -1; goto ready;	}
 
-    bmMorphoStartDilateJob( &dj, &bdSel, bufSel, rowOrig, colOrig );
+    bmMorphoStartDilateJob( &dj, bdIn, bufIn, &bdSel, bufSel,
+						    rowSelOrig, selColOrig );
 
-    if  ( bmMorphoDilateLow( &dj, bdOut, bdIn, pBufOut, bufIn, invertMaskIn ) )
+    /*  4  */
+    dj.djInvertMaskIn= ~dj.djInvertMaskIn;
+    reverse= 1;
+
+    if  ( bmMorphoDilateLow( &dj, reverse, bdOut, pBufOut ) )
 	{ XDEB(invertMaskIn); rval= -1; goto ready;	}
 
-    to= *pBufOut;
-    for ( i= 0; i < bdOut->bdBufferLength; i++, to++ )
-	{ to[0]= ~to[0]; }
-
   ready:
 
     if  ( bufSel )
@@ -419,10 +481,7 @@ int bmMorphoLineElement(	BitmapDescription *		bdOut,
 				int				x1,
 				int				y1 )
     {
-    int			bytesPerRow;
-    int			bufferLength;
-
-    unsigned char *	bufOut;
+    unsigned char *	bufOut= (unsigned char *)0;
 
     if  ( x0 < 0 || x0 >= wide )
 	{ LLDEB(x0,wide); return -1;	}
@@ -434,13 +493,6 @@ int bmMorphoLineElement(	BitmapDescription *		bdOut,
     if  ( y1 < 0 || y1 >= high )
 	{ LLDEB(y1,high); return -1;	}
 
-    bytesPerRow= ( wide+ 7 )/ 8;
-    bufferLength= high* bytesPerRow;
-
-    bufOut= malloc( bufferLength );
-    if  ( ! bufOut )
-	{ LXDEB(bufferLength,bufOut); return -1;	}
-
     bmInitDescription( bdOut );
 
     bdOut->bdColorEncoding= BMcoWHITEBLACK;
@@ -452,12 +504,13 @@ int bmMorphoLineElement(	BitmapDescription *		bdOut,
     bdOut->bdXResolution= 1;
     bdOut->bdYResolution= 1;
     bdOut->bdUnit= BMunPIXEL;
-    bdOut->bdColorEncoding= BMcoWHITEBLACK;
 
-    bdOut->bdBytesPerRow= bytesPerRow;
-    bdOut->bdBufferLength= bufferLength;
+    bmCalculateSizes( bdOut );
 
-    memset( bufOut, 0x00, bufferLength );
+    bufOut= bmBackgroundBuffer(
+			    bdOut->bdBufferLength, bdOut->bdColorEncoding );
+    if  ( ! bufOut )
+	{ LXDEB(bdOut->bdBufferLength,bufOut); return -1;	}
 
     bmDrawLine( bufOut, bdOut, x0, y0, x1, y1, 1 );
 
