@@ -109,11 +109,11 @@ static int bmReadPcxRow(	unsigned char *	to,
 /************************************************************************/
 /*  Read a PCX file.							*/
 /************************************************************************/
+
 int bmReadPcxFile(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
-			int *			pPrivateFormat,
-			double *		pCompressionFactor	)
+			int *			pPrivateFormat )
     {
     FILE *		f;
     PCXHEADER		pch;
@@ -184,7 +184,7 @@ int bmReadPcxFile(	const char *		filename,
 		    LLDEB(pch.BitsPixelPlane,pch.NumberOfPlanes);
 		    SDEB(filename); fclose( f ); return -1;
 		}
-	    break;
+
 	case 4:
 	    switch( pch.NumberOfPlanes )
 		{
@@ -193,7 +193,7 @@ int bmReadPcxFile(	const char *		filename,
 		    LLDEB(pch.BitsPixelPlane,pch.NumberOfPlanes);
 		    SDEB(filename); fclose( f ); return -1;
 		}
-	    break;
+
 	case 8:
 	    switch( pch.NumberOfPlanes )
 		{
@@ -287,11 +287,9 @@ int bmReadPcxFile(	const char *		filename,
 int bmWritePcxFile(	const char *			filename,
 			const unsigned char *		buffer,
 			const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor )
+			int				privateFormat )
     { LDEB(-1); return -1; }
 
 int bmCanWritePcxFile( const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor )
+			int				privateFormat )
     { return -1; }
