@@ -68,6 +68,13 @@ static int bpPngiToBitmap(	const png_info *		pngi,
 	    break;
 
 	case PNG_COLOR_TYPE_RGB_ALPHA:
+	    bd->bdHasAlpha= 1;
+	    bd->bdColorEncoding= BMcoRGB;
+	    bd->bdBitsPerSample= pngi->bit_depth;
+	    bd->bdSamplesPerPixel= pngi->channels;
+	    bd->bdBitsPerPixel= pngi->pixel_depth;
+	    break;
+
 	case PNG_COLOR_TYPE_GRAY_ALPHA:
 	    bd->bdHasAlpha= 1;
 	    LDEB(pngi->color_type); return -1;
@@ -148,10 +155,9 @@ static int bmPngReadContents(	png_info *		pngi,
 
     png_read_end( png, pngi );
 
-    if  ( bd->bdColorEncoding == BMcoRGB8PALETTE		&&
-	  ! bd->bdHasAlpha					&&
-	  ! bmMakeMonochrome( bd, bd->bdRGB8Palette, buffer )	)
-	{ bd->bdRGB8Palette= (RGB8Color *)0;	}
+    if  ( bd->bdColorEncoding == BMcoRGB8PALETTE	&&
+	  ! bd->bdHasAlpha				)
+	{ bmMakeMonochrome( bd, buffer );	}
 
     *pBuffer= buffer; return 0;
     }
@@ -159,8 +165,7 @@ static int bmPngReadContents(	png_info *		pngi,
 int bmReadPngFile(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
-			int *			pPrivateFormat,
-			double *		pCompressionFactor )
+			int *			pPrivateFormat )
     {
     SimpleInputStream *	sis;
 
@@ -233,14 +238,14 @@ int bmPngReadPng(	BitmapDescription *	bd,
 
     if  ( bpPngiToBitmap( pngip, bd ) )
 	{
-	LLDEB(bd->bdColorCount,bd->bdRGB8Palette); 
+	LLLDEB(pngip->color_type,bd->bdColorCount,bd->bdRGB8Palette); 
 	png_destroy_read_struct( &pngp, &pngip, (png_infop *)0 );
 	return -1;
 	}
 
     if  ( bmPngReadContents( pngip, pngp, bd, &buffer ) )
 	{
-	LLDEB(bd->bdBufferLength,buffer);
+	LDEB(bd->bdBufferLength);
 	png_destroy_read_struct( &pngp, &pngip, (png_infop *)0 );
 	return -1;
 	}
@@ -260,9 +265,14 @@ int bmPngReadPng(	BitmapDescription *	bd,
 /************************************************************************/
 
 int bmCanWritePngFile( const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor )
-    { return 0; }
+			int				privateFormat )
+    {
+    if  ( bd->bdColorEncoding == BMcoRGB8PALETTE	&&
+	  bd->bdBitsPerPixel > 8			)
+	{ return -1;	}
+
+    return 0;
+    }
 
 /************************************************************************/
 /*									*/
@@ -270,7 +280,8 @@ int bmCanWritePngFile( const BitmapDescription *	bd,
 /*									*/
 /************************************************************************/
 
-static int bpPngiFromBitmap(	png_info *			pngi,
+static int bpPngiFromBitmap(	png_structp			png,
+				png_info *			pngi,
 				const BitmapDescription *       bd )
     {
     png_info_init( pngi );
@@ -319,13 +330,17 @@ static int bpPngiFromBitmap(	png_info *			pngi,
 	case BMcoBLACKWHITE:
 	case BMcoWHITEBLACK:
 	    pngi->bit_depth= bd->bdBitsPerPixel;
-	    pngi->color_type= PNG_COLOR_TYPE_GRAY;
+	    if  ( bd->bdHasAlpha )
+		{ pngi->color_type= PNG_COLOR_TYPE_GRAY_ALPHA;	}
+	    else{ pngi->color_type= PNG_COLOR_TYPE_GRAY;	}
 	    pngi->sig_bit.gray= bd->bdBitsPerSample;
 	    break;
 
 	case BMcoRGB:
 	    pngi->bit_depth= bd->bdBitsPerSample;
-	    pngi->color_type= PNG_COLOR_TYPE_RGB;
+	    if  ( bd->bdHasAlpha )
+		{ pngi->color_type= PNG_COLOR_TYPE_RGB_ALPHA;	}
+	    else{ pngi->color_type= PNG_COLOR_TYPE_RGB;		}
 	    pngi->sig_bit.red= bd->bdBitsPerSample;
 	    pngi->sig_bit.green= bd->bdBitsPerSample;
 	    pngi->sig_bit.blue= bd->bdBitsPerSample;
@@ -451,8 +466,7 @@ static void bmPngWriteContents(	png_structp			png,
 int bmWritePngFile(	const char *			filename,
 			const unsigned char *		buffer,
 			const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor )
+			int				privateFormat )
     {
     SimpleOutputStream *	sos;
 
@@ -527,13 +541,17 @@ int bmPngWritePng(		const BitmapDescription *	bd,
     png_init_io( pngp, (FILE *)0 );
     png_set_write_fn( pngp, (void *)sos, bmPngWriteBytes, bmPngFlushBytes );
 
-    if  ( bpPngiFromBitmap( pngip, bd ) )
+    if  ( bpPngiFromBitmap( pngp, pngip, bd ) )
 	{
 	LDEB(bd->bdColorEncoding);
 	png_destroy_write_struct( &pngp, &pngip );
 	return -1;
 	}
 
+    /*
+    png_write_info( pngp, pngip );
+    */
+
     bmPngWriteContents( pngp, pngip, buffer, bd );
 
     png_destroy_write_struct( &pngp, &pngip );
