@@ -1,6 +1,7 @@
 #   include	"bitmapConfig.h"
 
 #   include	"bmintern.h"
+#   include	"bmRender.h"
 #   include	<string.h>
 #   include	<appDebugon.h>
 
@@ -63,8 +64,19 @@ typedef struct CutNode
 #   define	CN_GREEN	2
 #   define	CN_BLUE		3
 
+typedef struct ColorReducer
+    {
+    HashBucket **		crHashTable;
+    HistogramEntry *		crHistogram;
+    RGB8Color *			crPalette;
+    CutNode *			crCutNodes;
+    int				crColorsCounted;
+    } ColorReducer;
+
 /************************************************************************/
+/*									*/
 /*  Color hash varia.							*/
+/*									*/
 /************************************************************************/
 
 #define HASH_SIZE 6553
@@ -123,6 +135,81 @@ static int bmInsertHash(	HashBucket **	hashTable,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Utility functions: Manage a color reducer.				*/
+/*									*/
+/************************************************************************/
+
+static void bmInitColorReducer(	ColorReducer *	cr )
+    {
+    cr->crHashTable= (HashBucket **)0;
+    cr->crHistogram= (HistogramEntry *)0;
+    cr->crPalette= (RGB8Color *)0;
+    cr->crCutNodes= (CutNode *)0;
+
+    return;
+    }
+
+static void bmCleanColorReducer( ColorReducer *	cr )
+    {
+    if  ( cr->crPalette )
+	{ free( cr->crPalette );		}
+    if  ( cr->crHistogram )
+	{ free( cr->crHistogram );		}
+    if  ( cr->crCutNodes )
+	{ free( cr->crCutNodes );		}
+    if  ( cr->crHashTable )
+	{ bmFreeColorHash( cr->crHashTable );	}
+
+    return;
+    }
+
+static int bmAllocateReducerPalette(	ColorReducer *	cr,
+					int		count )
+    {
+    RGB8Color *	fresh;
+
+    fresh= (RGB8Color *)realloc( cr->crPalette, count* sizeof(RGB8Color) );
+    if  ( ! fresh )
+	{ LLDEB(count,fresh); return -1;	}
+
+    cr->crPalette= fresh;
+    memset( cr->crPalette, 0, count* sizeof(RGB8Color) );
+
+    return 0;
+    }
+
+static int bmAllocateCutNodes(		ColorReducer *	cr,
+					int		colorCount )
+    {
+    CutNode *	fresh;
+
+    fresh= (CutNode *)realloc( cr->crCutNodes, 2* colorCount* sizeof(CutNode) );
+    if  ( ! fresh )
+	{ LLDEB(colorCount,fresh); return -1;	} 
+
+    cr->crCutNodes= fresh;
+    memset( cr->crCutNodes, 0, 2* colorCount* sizeof(CutNode) );
+
+    return 0;
+    }
+
+static int bmAllocateHistogram(		ColorReducer *	cr,
+					int		colorCount )
+    {
+    HistogramEntry *	fresh;
+
+    fresh= (HistogramEntry *)realloc( cr->crHistogram,
+					colorCount* sizeof(HistogramEntry) );
+    if  ( ! fresh )
+	{ LLDEB(colorCount,fresh); return -1; }
+
+    cr->crHistogram= fresh;
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Determine median.							*/
@@ -523,34 +610,27 @@ static void bmSplitBox(	HistogramEntry *	histogram,
 /*									*/
 /************************************************************************/
 
-static int bmFindColors(	HistogramEntry *	histogram,
-				RGB8Color *		palette,
-				CutNode *		cutNodes,
+static int bmFindCuts(		ColorBox *		boxes,
+				CutNode *		nodes,
+				HistogramEntry *	histo,
 				int			colorCount,
 				int			pixelCount,
 				int			maxcolors )
     {
-    ColorBox *			boxes;
-    const ColorBox *		cb;
     int				i;
-    int				j;
     int				boxCount;
     int				nodeCount= 0;
 
-    boxes= (ColorBox *)malloc( maxcolors* sizeof(ColorBox) );
-    if  ( ! boxes )
-	{ LLDEB(maxcolors,boxes); return -1; }
-
     boxes[0].cbHistogramOffset= 0;
     boxes[0].cbColorCount= colorCount;
     boxes[0].cbPixelCount= pixelCount;
     boxes[0].cbCutNumber= 0;
     boxCount= 1;
 
-    cutNodes[0].cnComponent= CN_LEAF;
-    cutNodes[0].cnLeftMaximum= 0;	/*  Irrelevant !	*/
-    cutNodes[0].cnLeft= 0;		/*  Both to this color.	*/
-    cutNodes[0].cnRight= 0;		/*  Both to this color.	*/
+    nodes[0].cnComponent= CN_LEAF;
+    nodes[0].cnLeftMaximum= 0;		/*  Irrelevant !	*/
+    nodes[0].cnLeft= 0;		/*  Both to this color.	*/
+    nodes[0].cnRight= 0;		/*  Both to this color.	*/
     nodeCount= 1;
 
     while( boxCount < maxcolors )
@@ -576,30 +656,31 @@ static int bmFindColors(	HistogramEntry *	histogram,
 		{ biggestBox= i;	}
 	    }
 
-	bmDelimitColorBox( boxes+ biggestBox, histogram );
+	bmDelimitColorBox( boxes+ biggestBox, histo );
 
 	cbSplit= boxes[biggestBox];
-	bmSplitBox( histogram, &cbSplit, boxes+ biggestBox, boxes+ boxCount,
-						&leftMaximum, &cutComponent );
+	bmSplitBox( histo,
+				&cbSplit, boxes+ biggestBox, boxes+ boxCount,
+				&leftMaximum, &cutComponent );
 
 	/****************/
 	/*  Cut Nodes	*/
 	/****************/
 	i= cbSplit.cbCutNumber;
-	cutNodes[i].cnComponent= cutComponent;
-	cutNodes[i].cnLeftMaximum= leftMaximum;
-	cutNodes[i].cnLeft= nodeCount;
-	cutNodes[i].cnRight= nodeCount+ 1;
+	nodes[i].cnComponent= cutComponent;
+	nodes[i].cnLeftMaximum= leftMaximum;
+	nodes[i].cnLeft= nodeCount;
+	nodes[i].cnRight= nodeCount+ 1;
 
-	cutNodes[nodeCount].cnComponent= CN_LEAF;
-	cutNodes[nodeCount].cnLeftMaximum= 0;	/*  Irrelevant !	*/
-	cutNodes[nodeCount].cnLeft= biggestBox;	/*  Both to this color.	*/
-	cutNodes[nodeCount].cnRight= biggestBox;/*  Both to this color.	*/
+	nodes[nodeCount].cnComponent= CN_LEAF;
+	nodes[nodeCount].cnLeftMaximum= 0;
+	nodes[nodeCount].cnLeft= biggestBox;
+	nodes[nodeCount].cnRight= biggestBox;
 
-	cutNodes[nodeCount+1].cnComponent= CN_LEAF;
-	cutNodes[nodeCount+1].cnLeftMaximum= 0;	/*  Irrelevant !	*/
-	cutNodes[nodeCount+1].cnLeft= boxCount;	/*  Both to this color.	*/
-	cutNodes[nodeCount+1].cnRight= boxCount;/*  Both to this color.	*/
+	nodes[nodeCount+1].cnComponent= CN_LEAF;
+	nodes[nodeCount+1].cnLeftMaximum= 0;
+	nodes[nodeCount+1].cnLeft= boxCount;
+	nodes[nodeCount+1].cnRight= boxCount;
 
 	/****************/
 	/*  Boxes	*/
@@ -610,6 +691,27 @@ static int bmFindColors(	HistogramEntry *	histogram,
 	nodeCount += 2; boxCount++;
 	}
 
+    return boxCount;
+    }
+
+static int bmFindColors(	ColorReducer *		cr,
+				int			colorCount,
+				int			pixelCount,
+				int			maxcolors )
+    {
+    ColorBox *			boxes;
+    const ColorBox *		cb;
+    int				i;
+    int				j;
+    int				boxCount;
+
+    boxes= (ColorBox *)malloc( maxcolors* sizeof(ColorBox) );
+    if  ( ! boxes )
+	{ LLDEB(maxcolors,boxes); return -1; }
+
+    boxCount= bmFindCuts( boxes, cr->crCutNodes, cr->crHistogram,
+					colorCount, pixelCount, maxcolors );
+
     cb= boxes;
     for ( i= 0; i < boxCount; cb++, i++ )
 	{
@@ -618,7 +720,7 @@ static int bmFindColors(	HistogramEntry *	histogram,
 	long			sB= 0;
 	long			N= 0;
 
-	HistogramEntry *	he= histogram+ cb->cbHistogramOffset;
+	HistogramEntry *	he= cr->crHistogram+ cb->cbHistogramOffset;
 
 	for ( j= 0; j < boxes[i].cbColorCount; j++, he++ )
 	    {
@@ -628,9 +730,9 @@ static int bmFindColors(	HistogramEntry *	histogram,
 	    N  += he->heCount;
 	    }
 
-	palette[i].rgb8Red= sR/ N;
-	palette[i].rgb8Green= sG/ N;
-	palette[i].rgb8Blue= sB/ N;
+	cr->crPalette[i].rgb8Red= sR/ N;
+	cr->crPalette[i].rgb8Green= sG/ N;
+	cr->crPalette[i].rgb8Blue= sB/ N;
 
 #	if LOG_PALETTE
 	appDebug( "BOX %3d: "
@@ -640,14 +742,14 @@ static int bmFindColors(	HistogramEntry *	histogram,
 			"%4dC %6ldP\n",
 
 			i,
-			cb->cbRMin, palette[i].rgb8Red, cb->cbRMax,
-			cb->cbGMin, palette[i].rgb8Green, cb->cbGMax,
-			cb->cbBMin, palette[i].rgb8Blue, cb->cbBMax,
+			cb->cbRMin, cr->crPalette[i].rgb8Red, cb->cbRMax,
+			cb->cbGMin, cr->crPalette[i].rgb8Green, cb->cbGMax,
+			cb->cbBMin, cr->crPalette[i].rgb8Blue, cb->cbBMax,
 			boxes[i].cbColorCount, N );
 #	endif
 	}
 
-    free( (char *)boxes );
+    free( boxes );
 
     return boxCount;
     }
@@ -658,134 +760,100 @@ static int bmFindColors(	HistogramEntry *	histogram,
 /*									*/
 /************************************************************************/
 
-static void bm24to8or4Cut(	int			pixelsHigh,
-				int			pixelsWide,
-				int			inBytesPerRow,
-				int			outBytesPerRow,
-				const unsigned char *	bufIn,
-				unsigned char *		bufOut,
-				CutNode *		cutNodes,
-				int			bitsPerPixel )
+static int bmReduceAllocateColorCut(	AllocatorColor *	ac,
+					ColorAllocator *	ca,
+					unsigned int		r,
+					unsigned int		g,
+					unsigned int		b )
     {
-    int				row;
-    int				col;
-    int				i;
+    ColorReducer *	cr= (ColorReducer *)ca->caSystemPrivate;
+    const CutNode *	cn= cr->crCutNodes;
+    int			i;
 
-    unsigned char		r;
-    unsigned char		g;
-    unsigned char		b;
-
-    const unsigned char *	from;
-    unsigned char *		to;
-
-    for ( row= 0; row < pixelsHigh; row++ )
+    i= 0;
+    for (;;)
 	{
-	from= bufIn+ row* inBytesPerRow;
-	to= bufOut+ row* outBytesPerRow;
-
-	for ( col= 0; col < pixelsWide; col++ )
+	switch( cn[i].cnComponent )
 	    {
-	    r= *(from++); g= *(from++); b= *(from++);
-	    i= 0;
-	    for (;;)
-		{
-		switch( cutNodes[i].cnComponent )
-		    {
-		    case CN_RED:
-			if  ( r <= cutNodes[i].cnLeftMaximum )
-			    { i= cutNodes[i].cnLeft;	continue; }
-			else{ i= cutNodes[i].cnRight;	continue; }
-		    case CN_GREEN:
-			if  ( g <= cutNodes[i].cnLeftMaximum )
-			    { i= cutNodes[i].cnLeft;	continue; }
-			else{ i= cutNodes[i].cnRight;	continue; }
-		    case CN_BLUE:
-			if  ( b <= cutNodes[i].cnLeftMaximum )
-			    { i= cutNodes[i].cnLeft;	continue; }
-			else{ i= cutNodes[i].cnRight;	continue; }
-		    case CN_LEAF:
-			break;
-		    default:
-			LDEB(cutNodes[i].cnComponent); return;
-		    }
-		break;
-		}
+	    case CN_RED:
+		if  ( r <= cn[i].cnLeftMaximum )
+		    { i= cn[i].cnLeft;	}
+		else{ i= cn[i].cnRight;	}
+		continue;
 
-	    switch( bitsPerPixel )
-		{
-		case 8:
-		    *(to++)= cutNodes[i].cnLeft; break;
-		case 4:
-		    if  ( col % 2 )
-			{ *(to++) |=  cutNodes[i].cnLeft;	 break; }
-		    else{ *(to  )=  ( cutNodes[i].cnLeft << 4 ); break; }
-		default:
-		    LDEB(bitsPerPixel); return;
-		}
+	    case CN_GREEN:
+		if  ( g <= cn[i].cnLeftMaximum )
+		    { i= cn[i].cnLeft;	}
+		else{ i= cn[i].cnRight;	}
+		continue;
+
+	    case CN_BLUE:
+		if  ( b <= cn[i].cnLeftMaximum )
+		    { i= cn[i].cnLeft;	}
+		else{ i= cn[i].cnRight;	}
+		continue;
+
+	    case CN_LEAF:
+		i= cn[i].cnLeft;
+
+		ac->acRed= 257* cr->crPalette[i].rgb8Red;
+		ac->acGreen= 257* cr->crPalette[i].rgb8Green;
+		ac->acBlue= 257* cr->crPalette[i].rgb8Blue;
+		ac->acColorNumber= i;
+		return 0;
+
+	    default:
+		LDEB(cn[i].cnComponent); return -1;
 	    }
 	}
     }
 
-static void bm24to8or4Hash(	int			pixelsHigh,
-				int			pixelsWide,
-				int			inBytesPerRow,
-				int			outBytesPerRow,
-				const unsigned char *	bufIn,
-				unsigned char *		bufOut,
-				HashBucket **		hashTable,
-				int			bitsPerPixel )
+static int bmReduceAllocateColorHash(	AllocatorColor *	ac,
+					ColorAllocator *	ca,
+					unsigned int		r,
+					unsigned int		g,
+					unsigned int		b )
     {
-    int				row;
-    int				col;
+    ColorReducer *	cr= (ColorReducer *)ca->caSystemPrivate;
+    int			hash;
+    HashBucket *	hashBucket;
+    int			i;
 
-    unsigned char		r;
-    unsigned char		g;
-    unsigned char		b;
-
-    const unsigned char *	from;
-    unsigned char *		to;
-
-    for ( row= 0; row < pixelsHigh; row++ )
+    hash= ppm_hash( r, g, b );
+    hashBucket= cr->crHashTable[hash];
+    while( hashBucket )
 	{
-	from= bufIn+ row* inBytesPerRow;
-	to= bufOut+ row* outBytesPerRow;
+	if  ( hashBucket->hbRed == r	&&
+	      hashBucket->hbGreen == g	&&
+	      hashBucket->hbBlue == b	)
+	    { break;	}
 
-	for ( col= 0; col < pixelsWide; col++ )
-	    {
-	    int			hash;
-	    HashBucket *	hashBucket;
-
-	    r= *(from++); g= *(from++); b= *(from++);
-
-	    hash= ppm_hash( r, g, b );
-	    hashBucket= hashTable[hash];
-	    while( hashBucket )
-		{
-		if  ( hashBucket->hbRed == r	&&
-		      hashBucket->hbGreen == g	&&
-		      hashBucket->hbBlue == b	)
-		    { break;	}
-
-		hashBucket= hashBucket->hbNext;
-		}
-
-	    if  ( ! hashBucket )
-		{ XDEB(hashBucket); r= 0;	}
-	    else{ r= hashBucket->hbNumber;	}
-
-	    switch( bitsPerPixel )
-		{
-		case 8:
-		    *(to++)= r; break;
-		case 4:
-		    if  ( col % 2 )
-			{ *(to++) |=  r;        break; }
-		    else{ *(to  )=  ( r << 4 ); break; }
-		default:
-		    LDEB(bitsPerPixel); return;
-		}
-	    }
+	hashBucket= hashBucket->hbNext;
 	}
+
+    if  ( ! hashBucket )
+	{ XDEB(hashBucket); i= 0;	}
+    else{ i= hashBucket->hbNumber;	}
+
+    ac->acRed= 257* cr->crPalette[i].rgb8Red;
+    ac->acGreen= 257* cr->crPalette[i].rgb8Green;
+    ac->acBlue= 257* cr->crPalette[i].rgb8Blue;
+    ac->acColorNumber= i;
+
+    return 0;
+    }
+
+static void bmReduceCleanupAllocator(	ColorAllocator *	ca )
+    {
+    ColorReducer *	cr= (ColorReducer *)ca->caSystemPrivate;
+
+    if  ( cr )
+	{
+	bmCleanColorReducer( cr );
+	free( cr );
+	}
+
+    return;
     }
 
 /************************************************************************/
@@ -797,14 +865,15 @@ static void bm24to8or4Hash(	int			pixelsHigh,
 /*									*/
 /************************************************************************/
 
-static int bmHashColors24(	HashBucket ***		pHashTable,
+static int bmHashColorsRGB8(	HashBucket ***		pHashTable,
 				int *			pColorCount,
 				int			pixelsHigh,
 				int			pixelsWide,
 				int			bytesPerRow,
+				int			hasAlpha,
 				const unsigned char *	bufIn,
 				int			maxcolors,
-				unsigned int		mask		)
+				unsigned int		mask )
     {
     unsigned int		r;
     unsigned int		g;
@@ -837,6 +906,9 @@ static int bmHashColors24(	HashBucket ***		pHashTable,
 	    r= *(from++) & mask; g= *(from++) & mask; b= *(from++) & mask;
 	    hash= ppm_hash( r, g, b );
 
+	    if  ( hasAlpha )
+		{ from++;	}
+
 	    if  ( hash < 0 || hash >= HASH_SIZE )
 		{ LLDEB(hash,HASH_SIZE); return -1; }
 
@@ -872,38 +944,30 @@ static int bmHashColors24(	HashBucket ***		pHashTable,
 /*									*/
 /************************************************************************/
 
-static int bmHashToHistogram(	HistogramEntry **	pHistogram,
-				HashBucket **		hashTable,
+static int bmHashToHistogram(	ColorReducer *		cr,
 				int			colorCount )
     {
     HashBucket *		hashBucket;
-    HistogramEntry *		histogram;
 
     int				row;
     int				i;
 
-    /*  3  */
-    histogram= (HistogramEntry *)malloc( colorCount* sizeof(HistogramEntry) );
-    if  ( ! histogram )
-	{ LLDEB(colorCount,histogram); return -1; }
-
     row= 0;
     for ( i= 0; i < HASH_SIZE; i++ )
 	{
-	hashBucket= hashTable[i];
+	hashBucket= cr->crHashTable[i];
 
 	while( hashBucket )
 	    {
-	    histogram[row++]= hashBucket->hbHistogramEntry;
+	    cr->crHistogram[row++]= hashBucket->hbHistogramEntry;
 	    hashBucket= hashBucket->hbNext;
 	    }
 	}
 
-    *pHistogram= histogram; return 0;
+    return 0;
     }
 
-static void bmHashToPalette(	RGB8Color *		palette,
-				HashBucket **		hashTable )
+static void bmHashToPalette(	ColorReducer *		cr )
     {
     HashBucket *		hashBucket;
 
@@ -911,11 +975,11 @@ static void bmHashToPalette(	RGB8Color *		palette,
 
     for ( i= 0; i < HASH_SIZE; i++ )
 	{
-	hashBucket= hashTable[i];
+	hashBucket= cr->crHashTable[i];
 
 	while( hashBucket )
 	    {
-	    palette[hashBucket->hbNumber]=
+	    cr->crPalette[hashBucket->hbNumber]=
 					hashBucket->hbHistogramEntry.heColor;
 
 	    hashBucket= hashBucket->hbNext;
@@ -947,83 +1011,100 @@ static void bmHashToPalette(	RGB8Color *		palette,
 int bmColorReduce(	BitmapDescription *		bdOut,
 			const BitmapDescription *	bdIn,
 			unsigned char **		pBufOut,
-			const unsigned char *		bufIn,
+			const unsigned char *		bufferIn,
 			int				maxcolors )
     {
+    int				rval= 0;
+
     int				row;
     int				i;
 
-    unsigned char		mask;
+    unsigned char		mask= 0xff;
 
-    HashBucket **		hashTable= (HashBucket **)0;
-    HistogramEntry *		histogram= (HistogramEntry *)0;
-    RGB8Color *			palette= (RGB8Color *)0;
-    CutNode *			cutNodes= (CutNode *)0;
+    ColorAllocator		ca;
+    ColorReducer *		cr;
 
     int				colorCount= 0;
     int				colorsFound;
 
     int				bitsPerPixel;
 
+    int				bitmapUnit= 0;
+    int				swapBitmapBytes= 0;
+    int				swapBitmapBits= 0;
+    const int			dither= 0;
+
+    bmInitColorAllocator( &ca );
+
+    cr= malloc( sizeof( ColorReducer ) );
+    if  ( ! cr )
+	{ XDEB(cr); rval= -1; goto ready;	}
+
+    bmInitColorReducer( cr );
+
+    ca.caSystemPrivate= (void *)cr;
+    ca.caAllocationType= CA_ALLOCATOR;
+    ca.caSystemCleanup= bmReduceCleanupAllocator;
+
     if  ( maxcolors > 256 )
-	{ LDEB(maxcolors); return -1;	}
+	{ LDEB(maxcolors); rval= -1; goto ready;	}
 
     if  ( maxcolors <= 16 )
 	{ bitsPerPixel= 4;	}
     else{ bitsPerPixel= 8;	}
 
     /*  0  */
-    palette= (RGB8Color *)malloc( maxcolors* sizeof(RGB8Color) );
-    if  ( ! palette )
-	{ LLDEB(maxcolors,palette); goto failure;	}
-
-    memset( palette, 0, maxcolors* sizeof(RGB8Color) );
+    if  ( bmAllocateReducerPalette( cr, maxcolors ) )
+	{ LDEB(maxcolors); rval= -1; goto ready;	}
 
     /*  1, 2  */
     switch( bdIn->bdBitsPerPixel )
 	{
 	case 24:
+	case 32:
 	    for ( i= 0; i < 8; i++ )
 		{
 		mask= ~( 0xff >> ( 8- i ) );
 
-		row= bmHashColors24( &hashTable, &colorCount,
+		row= bmHashColorsRGB8( &(cr->crHashTable), &colorCount,
 				bdIn->bdPixelsHigh, bdIn->bdPixelsWide,
-				bdIn->bdBytesPerRow, bufIn, 32768, mask );
+				bdIn->bdBytesPerRow, bdIn->bdHasAlpha,
+				bufferIn, 32768, mask );
 		if  ( row < 0 )
-		    { LDEB(row); return -1;	}
+		    { LDEB(row); rval= -1; goto ready;	}
 		if  ( row == 0 )
-		    { break;	}
+		    { break;				}
 		}
 	    break;
+
 	default:
-	    LDEB(bdIn->bdBitsPerPixel); return -1;
+	    LDEB(bdIn->bdBitsPerPixel); rval= -1; goto ready;
 	}
 
     /*  3  */
     if  ( mask == 0xff && colorCount <= maxcolors )
 	{
-	bmHashToPalette( palette, hashTable );
+	bmHashToPalette( cr );
 
 	colorsFound= colorCount;
 	}
     else{
 	/*  4  */
-	if  ( bmHashToHistogram( &histogram, hashTable, colorCount ) )
-	    { LDEB(1); goto failure;	}
+	if  ( bmAllocateHistogram( cr, colorCount ) )
+	    { LDEB(colorCount); rval= -1; goto ready;	}
 
-	cutNodes= (CutNode *)malloc( 2* maxcolors* sizeof(CutNode) );
-	if  ( ! cutNodes )
-	    { LLDEB(maxcolors,cutNodes); goto failure;	} 
+	if  ( bmHashToHistogram( cr, colorCount ) )
+	    { LDEB(colorCount); rval= -1; goto ready;	}
 
-	memset( cutNodes, 0, 2* maxcolors* sizeof(CutNode) );
+	if  ( bmAllocateCutNodes( cr, maxcolors ) )
+	    { LDEB(maxcolors); rval= -1; goto ready;	}
 
 	/*  5  */
-	colorsFound= bmFindColors( histogram, palette, cutNodes, colorCount,
-			    bdIn->bdPixelsHigh* bdIn->bdPixelsWide, maxcolors );
+	colorsFound= bmFindColors( cr, colorCount,
+			bdIn->bdPixelsHigh* bdIn->bdPixelsWide, maxcolors );
 
 	if  ( colorsFound < 1 )
-	    { LLDEB(maxcolors,colorsFound); goto failure; }
+	    { LLDEB(maxcolors,colorsFound); rval= -1; goto ready; }
 	}
 
     /*  6  */
@@ -1045,62 +1126,147 @@ int bmColorReduce(	BitmapDescription *		bdOut,
 
     bdOut->bdHasAlpha= 0;
 
-    bdOut->bdRGB8Palette= palette;
-
     *pBufOut= malloc( bdOut->bdBufferLength );
     if  ( ! *pBufOut )
-	{ LLDEB( bdOut->bdBufferLength,*pBufOut); goto failure; }
+	{ LLDEB( bdOut->bdBufferLength,*pBufOut); rval= -1; goto ready; }
 
     /*  7  */
-    switch( bdIn->bdBitsPerPixel )
-	{
-	case 24:
-	    switch( bitsPerPixel )
-		{
-		case 8:
-		case 4:
-		    if  ( cutNodes )
-			{
-			bm24to8or4Cut( bdIn->bdPixelsHigh, bdIn->bdPixelsWide,
-			    bdIn->bdBytesPerRow, bdOut->bdBytesPerRow,
-			    bufIn, *pBufOut, cutNodes, bitsPerPixel );
-			}
-		    else{
-			bm24to8or4Hash( bdIn->bdPixelsHigh, bdIn->bdPixelsWide,
-			    bdIn->bdBytesPerRow, bdOut->bdBytesPerRow,
-			    bufIn, *pBufOut, hashTable, bitsPerPixel );
-			}
-		    break;
+    if  ( cr->crCutNodes )
+	{ ca.caSystemAllocator= bmReduceAllocateColorCut;	}
+    else{ ca.caSystemAllocator= bmReduceAllocateColorHash;	}
 
-		default:
-		    LLDEB(bdIn->bdBitsPerPixel,bdOut->bdBitsPerPixel);
-		    goto failure;
-		}
-	    break;
-	default:
-	    LLDEB(bdIn->bdBitsPerPixel,bdOut->bdBitsPerPixel);
-	    goto failure;
-	}
+    if  ( bmFillImage( &ca, bitmapUnit, swapBitmapBytes, swapBitmapBits,
+				dither, *pBufOut, bufferIn, bdOut,
+				bdIn, (const DocumentRectangle *)0 ) )
+	{ LDEB(1); rval= -1; goto ready;	}
 
-    if  ( histogram )
-	{ free( histogram );	}
-    if  ( cutNodes )
-	{ free( cutNodes );	}
-    if  ( hashTable )
-	{ bmFreeColorHash( hashTable );	}
+    bdOut->bdRGB8Palette= cr->crPalette; /*  steal */
+    cr->crPalette= (RGB8Color *)0;
 
-    return 0;
+  ready:
 
-  failure:
-    if  ( histogram )
-	{ free( histogram );	}
-    if  ( cutNodes )
-	{ free( cutNodes );	}
-    if  ( palette )
-	{ free( palette );	}
-    if  ( hashTable )
-	{ bmFreeColorHash( hashTable );	}
+    bmCleanColorAllocator( &ca ); /* also frees cr */
 
-    return -1;
+    return rval;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Make a color allocator to be used with a certain palette.		*/
+/*									*/
+/************************************************************************/
+
+int bmSetColorAllocatorForPaletteImage(	ColorAllocator *		ca,
+					const BitmapDescription *	bd )
+    {
+    int				rval= 0;
+
+    int				i;
+    ColorReducer *		cr= (ColorReducer *)0;
+    ColorBox *			boxes= (ColorBox *)0;
+    int *			map= (int *)0;
+    int				boxCount;
+
+    if  ( bd->bdColorEncoding != BMcoRGB8PALETTE )
+	{ LLDEB(bd->bdColorEncoding,BMcoRGB8PALETTE); rval= -1; goto ready; }
+
+    cr= malloc( sizeof( ColorReducer ) );
+    if  ( ! cr )
+	{ XDEB(cr); rval= -1; goto ready;	}
+
+    bmInitColorReducer( cr );
+
+    ca->caSystemPrivate= (void *)cr;
+    ca->caAllocationType= CA_ALLOCATOR;
+    ca->caSystemCleanup= bmReduceCleanupAllocator;
+
+    if  ( bmAllocateReducerPalette( cr, bd->bdColorCount ) )
+	{ LDEB(bd->bdColorCount); rval= -1; goto ready;	}
+
+    if  ( bmAllocateCutNodes( cr, bd->bdColorCount ) )
+	{ LDEB(bd->bdColorCount); rval= -1; goto ready;	}
+
+    if  ( bmAllocateHistogram( cr, bd->bdColorCount ) )
+	{ LDEB(bd->bdColorCount); rval= -1; goto ready;	}
+
+    boxes= (ColorBox *)malloc( bd->bdColorCount* sizeof(ColorBox) );
+    if  ( ! boxes )
+	{ LLDEB(bd->bdColorCount,boxes); return -1; }
+
+    map= (int *)malloc( bd->bdColorCount* sizeof(int) );
+    if  ( ! map )
+	{ LLDEB(bd->bdColorCount,map); return -1; }
+
+    for ( i= 0; i < bd->bdColorCount; i++ )
+	{
+	cr->crPalette[i]= bd->bdRGB8Palette[i];
+
+	cr->crHistogram[i].heColor= bd->bdRGB8Palette[i];
+	cr->crHistogram[i].heCount= 1;
+
+	map[i]= -1;
+	}
+
+    boxCount= bmFindCuts( boxes, cr->crCutNodes, cr->crHistogram,
+							    bd->bdColorCount,
+							    bd->bdColorCount,
+							    bd->bdColorCount );
+
+    if  ( boxCount != bd->bdColorCount )
+	{ LLDEB(boxCount,bd->bdColorCount); rval= -1; goto ready;	}
+
+    ca->caSystemAllocator= bmReduceAllocateColorCut;
+
+    for ( i= 0; i < bd->bdColorCount; i++ )
+	{
+	AllocatorColor	ac;
+
+	bmReduceAllocateColorCut( &ac, ca,
+				    bd->bdRGB8Palette[i].rgb8Red,
+				    bd->bdRGB8Palette[i].rgb8Green,
+				    bd->bdRGB8Palette[i].rgb8Blue );
+
+	if  ( /*ac.acColorNumber < 0 ||*/ ac.acColorNumber >= bd->bdColorCount )
+	    { LLDEB(i,ac.acColorNumber);	}
+
+	map[ac.acColorNumber]= i;
+	}
+
+    for ( i= 0; i < 2* bd->bdColorCount; i++ )
+	{
+	if  ( cr->crCutNodes[i].cnComponent != CN_LEAF )
+	    { continue;	}
+
+	cr->crCutNodes[i].cnLeft=map[cr->crCutNodes[i].cnLeft];
+	}
+
+#   if 0
+    for ( i= 0; i < bd->bdColorCount; i++ )
+	{
+	AllocatorColor	ac;
+
+	bmReduceAllocateColorCut( &ac, ca,
+				    bd->bdRGB8Palette[i].rgb8Red,
+				    bd->bdRGB8Palette[i].rgb8Green,
+				    bd->bdRGB8Palette[i].rgb8Blue );
+
+	if  ( ac.acColorNumber != i )
+	    {
+	    LLDEB(i,ac.acColorNumber);
+	    LDEB(bd->bdRGB8Palette[i].rgb8Red);
+	    LDEB(bd->bdRGB8Palette[i].rgb8Green);
+	    LDEB(bd->bdRGB8Palette[i].rgb8Blue);
+	    }
+	}
+#   endif
+
+  ready:
+
+    if  ( boxes )
+	{ free( boxes );	}
+    if  ( map )
+	{ free( map );	}
+
+    return rval;
     }
 
