@@ -31,8 +31,7 @@ typedef struct SgiImageHeader
 int bmReadSgiFile(	const char *		filename,
 			unsigned char **	pBuf,
 			BitmapDescription *	bd,
-			int *			pPrivateFormat,
-			double *		pCompressionFactor	)
+			int *			pPrivateFormat )
     {
     SimpleInputStream *		sis;
 
@@ -51,6 +50,5 @@ int bmReadSgiFile(	const char *		filename,
 int bmWriteSgiFile(	const char *			filename,
 			const unsigned char *		buffer,
 			const BitmapDescription *	bd,
-			int				privateFormat,
-			double				compressionFactor )
+			int				privateFormat )
     { LDEB(-1); return -1; }
