@@ -3,6 +3,7 @@
 #   include	<bmGrayHisto.h>
 
 #   include	"bmintern.h"
+#   include	"bmRender.h"
 #   include	<string.h>
 #   include	<appDebugon.h>
 
@@ -170,7 +171,8 @@ int bmThreshold(	BitmapDescription *		bdOut,
 
     /*  7  */
     if  ( bmFillImage( &ca, bitmapUnit, swapBitmapBytes, swapBitmapBits,
-				dither, buffer, bufferIn, &bd, bdIn ) )
+					dither, buffer, bufferIn, &bd,
+					bdIn, (const DocumentRectangle *)0 ) )
 	{ LDEB(1); rval= -1; goto ready;	}
 
     *pBufOut= buffer;
