@@ -6,11 +6,17 @@
 /*									*/
 /************************************************************************/
 
+#   ifndef APP_DEBUG_H
+#   define APP_DEBUG_H
+
 #   ifdef	APP_DEB
 #   undef	APP_DEB
 #   endif
 
-/*  # include "../mecheck/mecheck.h"  */
+# ifdef __cplusplus
+extern "C"
+    {
+# endif
 
 #ifdef	__GNUC__
 extern int appDebug(	const char *	format, ... )
@@ -19,6 +25,7 @@ extern int appDebug(	const char *	format, ... )
 extern int appDebug(	const char *	format, ... );
 #endif
 
+extern const int appDebugZero;
 extern int appDebugSetFile(	const char *	filename,
 				const char *	mode );
 
@@ -29,8 +36,10 @@ extern int appDebugSetFile(	const char *	filename,
 #	define	APP_DEB(x)
 # endif
 
-# define DEBS(sa)	((sa)?"\"":""), ((sa)?(sa):"0x0"), ((sa)?"\"":"")
+# define DEB0(sa)	((sa)?"\"":""), ((sa)?(sa):"0x0"), ((sa)?"\"":"")
+# define DEBS(sa)	(((sa)+appDebugZero)?"\"":""), (((sa)+appDebugZero)?(sa):"0x0"), (((sa)+appDebugZero)?"\"":"")
 # define DEBX(lo)	((unsigned long)(lo))
+# define DEBP(pt)	((void *)(pt))
 # define DEBO(lo)	((unsigned long)(lo))
 # define DEBL(lo)	((long)(lo))
 # define DEBD(do)	((double)(do))
@@ -38,6 +47,7 @@ extern int appDebugSetFile(	const char *	filename,
 
 # define DFMS	" %s=%s%s%s"
 # define DFMX	" %s=0x%lx"
+# define DFMP	" %s=%p"
 # define DFMO	" %s=0%lo"
 # define DFML	" %s=%ld"
 # define DFMD	" %s=%g"
@@ -62,6 +72,10 @@ extern int appDebugSetFile(	const char *	filename,
 			    DFMX				\
 			    "\n", __FILE__, __LINE__,		\
 			    #l1, DEBL(l1)			))
+# define PDEB(p1) APP_DEB(appDebug( "%s(%3d)"			\
+			    DFMP				\
+			    "\n", __FILE__, __LINE__,		\
+			    #p1, DEBP(p1)			))
 
 # define FDEB(d1) APP_DEB(appDebug( "%s(%3d)"			\
 			    DFMD				\
@@ -80,6 +94,31 @@ extern int appDebugSetFile(	const char *	filename,
 			    #c1, DEBC(c1),			\
 			    #l2, DEBL(l2)			))
 
+# define CSDEB(c1,s2) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMC				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #c1, DEBC(c1),			\
+			    #s2, DEBS(s2)			))
+
+# define CLSDEB(c1,l2,s3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMC				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #c1, DEBC(c1),			\
+			    #l2, DEBL(l2),			\
+			    #s3, DEBS(s3)			))
+
+# define CLLDEB(c1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMC				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #c1, DEBC(c1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3)			))
+
 # define CCDEB(c1,c2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMC				\
 			    DFMC				\
@@ -96,6 +135,33 @@ extern int appDebugSetFile(	const char *	filename,
 			    #c2, DEBC(c2),			\
 			    #c3, DEBC(c3)			))
 
+# define LCXDEB(l1,c2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMC				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #c2, DEBC(c2),			\
+			    #l3, DEBX(l3)			))
+
+# define LCLDEB(l1,c2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMC				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #c2, DEBC(c2),			\
+			    #l3, DEBL(l3)			))
+
+# define LLCDEB(l1,l2,c3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFML				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #c3, DEBC(c3)			))
+
 # define SSDEB(s1,s2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -131,6 +197,13 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l1, DEBL(l1),			\
 			    #l2, DEBL(l2)			))
 
+# define PPDEB(p1,p2) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMP				\
+			    DFMP				\
+			    "\n", __FILE__, __LINE__,		\
+			    #p1, DEBP(p1),			\
+			    #p2, DEBP(p2)			))
+
 # define XLDEB(l1,l2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMX				\
 			    DFML				\
@@ -144,6 +217,12 @@ extern int appDebugSetFile(	const char *	filename,
 			    "\n", __FILE__, __LINE__,		\
 			    #l1, DEBL(l1),			\
 			    #l2, DEBL(l2)			))
+# define LPDEB(l1,p2) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMP				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #p2, DEBP(p2)			))
 
 # define SLDEB(s1,l2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
@@ -179,6 +258,12 @@ extern int appDebugSetFile(	const char *	filename,
 			    "\n", __FILE__, __LINE__,		\
 			    #s1, DEBS(s1),			\
 			    #l2, DEBL(l2)			))
+# define SPDEB(s1,p2) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMS				\
+			    DFMP				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #p2, DEBP(p2)			))
 
 # define SXSDEB(s1,l2,s3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
@@ -198,6 +283,28 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #s3, DEBS(s3)			))
 
+# define SLSSDEB(s1,l2,s3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFMS				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #s3, DEBS(s3),			\
+			    #s4, DEBS(s4)			))
+
+# define LLSSDEB(l1,l2,s3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFMS				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #s3, DEBS(s3),			\
+			    #s4, DEBS(s4)			))
+
 # define XSDEB(l1,s2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMX				\
 			    DFMS				\
@@ -212,6 +319,24 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l1, DEBL(l1),			\
 			    #d2, DEBD(d2)			))
 
+# define LFLDEB(l1,d2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMD				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #l3, DEBL(l3)			))
+
+# define LFCDEB(l1,d2,c3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMD				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #c3, DEBC(c3)			))
+
 # define FFDEB(d1,d2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMD				\
 			    DFMD				\
@@ -244,6 +369,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #l3, DEBL(l3)			))
 
+# define SXLDEB(s1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMS				\
+			    DFMX				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBL(l3)			))
+
 # define SLXDEB(s1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFML				\
@@ -320,6 +454,24 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s2, DEBS(s2),			\
 			    #l3, DEBL(l3)			))
 
+# define LSCDEB(l1,s2,c3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMS				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #s2, DEBS(s2),			\
+			    #c3, DEBC(c3)			))
+
+# define XSLDEB(l1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMX				\
+			    DFMS				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #s2, DEBS(s2),			\
+			    #l3, DEBL(l3)			))
+
 # define LSXDEB(l1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFMS				\
@@ -371,13 +523,70 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define SXLLDEB(s1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFMX				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4)			))
+
+# define SLLLLDEB(s1,l2,l3,l4,l5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4),			\
+			    #l5, DEBL(l5)			))
+
+# define SLLXDEB(s1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFML				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBX(l4)			))
+
 # define LLLSDEB(l1,l2,l3,s4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
 			    DFML				\
 			    DFMS				\
 			    "\n", __FILE__, __LINE__,		\
-			    #l1, DEBS(l1),			\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #s4, DEBS(s4)			))
+
+# define LLFFDEB(l1,l2,d3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4)			))
+
+# define SLLSDEB(s1,l2,l3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
 			    #l2, DEBL(l2),			\
 			    #l3, DEBL(l3),			\
 			    #s4, DEBS(s4)			))
@@ -391,6 +600,32 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #l3, DEBL(l3)			))
 
+# define XXXDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBX(l3)			))
+
+# define XLLDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMX				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3)			))
+# define PLLDEB(p1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMP				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #p1, DEBP(p1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3)			))
+
 # define XLXDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMX				\
 			    DFML				\
@@ -411,6 +646,85 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define LLLXDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBX(l4)			))
+
+# define LLLLXDEB(l1,l2,l3,l4,l5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4),			\
+			    #l5, DEBX(l5)			))
+
+# define XXLLDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMX				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4)			))
+
+# define LLXXDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFMX				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBX(l4)			))
+
+# define FFLLDEB(d1,d2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMD				\
+			    DFMD				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #d1, DEBD(d1),			\
+			    #d2, DEBD(d2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4)			))
+
+# define XXXXDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBX(l4)			))
+
+# define XXXLDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBL(l4)			))
+
 # define LLLLLDEB(l1,l2,l3,l4,l5) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
@@ -424,6 +738,34 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l4, DEBL(l4),			\
 			    #l5, DEBL(l5)			))
 
+# define XXXLLDEB(l1,l2,l3,l4,l5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBL(l4),			\
+			    #l5, DEBL(l5)			))
+
+# define LLLLLLDEB(l1,l2,l3,l4,l5,l6) APP_DEB(appDebug( "%s(%3d)" \
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4),			\
+			    #l5, DEBL(l5),			\
+			    #l6, DEBL(l6)			))
+
 # define LLLFDEB(l1,l2,l3,d4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
@@ -471,6 +813,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #d2, DEBD(d2),			\
 			    #d3, DEBD(d3)			))
 
+# define LFLLDEB(l1,d2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMD				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4)			))
+
 # define LLFFFDEB(l1,l2,d3,d4,d5) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
@@ -526,6 +879,24 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #s3, DEBS(s3)			))
 
+# define XLSDEB(l1,l2,s3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMX				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBL(l2),			\
+			    #s3, DEBS(s3)			))
+
+# define LXSDEB(l1,l2,s3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMX				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBX(l2),			\
+			    #s3, DEBS(s3)			))
+
 # define SSXDEB(s1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -534,6 +905,14 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s1, DEBS(s1),			\
 			    #s2, DEBS(s2),			\
 			    #l3, DEBX(l3)			))
+# define SSPDEB(s1,s2,p3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMS				\
+			    DFMS				\
+			    DFMP				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #s2, DEBS(s2),			\
+			    #p3, DEBP(p3)			))
 
 # define SSLDEB(s1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
@@ -544,6 +923,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s2, DEBS(s2),			\
 			    #l3, DEBL(l3)			))
 
+# define SSLSDEB(s1,s2,l3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFMS				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #s2, DEBS(s2),			\
+			    #l3, DEBL(l3),			\
+			    #s4, DEBS(s4)			))
+
 # define SSFDEB(s1,s2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -639,6 +1029,30 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s3, DEBS(s3),			\
 			    #s4, DEBS(s4)			))
 
+# define LSSLDEB(l1,s2,s3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMS				\
+			    DFMS				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #s2, DEBS(s2),			\
+			    #s3, DEBS(s3),			\
+			    #l4, DEBL(l4)			))
+
+# define LSSLLDEB(l1,s2,s3,l4,l5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMS				\
+			    DFMS				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #s2, DEBS(s2),			\
+			    #s3, DEBS(s3),			\
+			    #l4, DEBL(l4),			\
+			    #l5, DEBL(l5)			))
+
 # define LSSDEB(l1,s2,s3) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFMS				\
@@ -677,3 +1091,8 @@ extern int appDebugSetFile(	const char *	filename,
 			    #d3, DEBD(d3),			\
 			    #d4, DEBD(d4)			))
 
+# ifdef __cplusplus
+    }
+# endif
+
+#   endif /* APP_DEBUG_H */
