@@ -1,6 +1,8 @@
 /****************************************************************/
+/*								*/
 /*  Utility functions for scanning documents for typographical	*/
 /*  errors.							*/
+/*								*/
 /****************************************************************/
 
 #   include	"indConfig.h"
@@ -8,7 +10,7 @@
 #   include	<charnames.h>
 
 #   include	"indlocal.h"
-#   include	<utilFontEncoding.h>
+#   include	<utilCharacterKinds.h>
 #   include	<appDebugon.h>
 
 /****************************************************************/
@@ -83,13 +85,14 @@ void indCleanSpellGuessContext(	SpellGuessContext *	sgc )
     return;
     }
 
-/****************************************************************/
-/*  Start a new possibility.					*/
-/****************************************************************/
+/************************************************************************/
+/*									*/
+/*  Start a new possibility.						*/
+/*									*/
+/************************************************************************/
 
 int	indNewPossibility(	SpellScanJob *		ssj,
-				int			position,
-				int			firstCharacter	)
+				int			position )
     {
     PossibleWord *	next= ssj->ssjPossibleWords;
     PossibleWord *	pw= (PossibleWord *)malloc(sizeof(PossibleWord));
@@ -97,13 +100,12 @@ int	indNewPossibility(	SpellScanJob *		ssj,
     if  ( ! pw )
 	{ XDEB(pw); return -1;	}
 
-    pw->pwStartPosition= position;
-    pw->pwInsertionPoint= 1;
+    pw->pwStartAt= position;
+    pw->pwInsertionPoint= 0;
     pw->pwRejectedAt= -1;
     pw->pwAcceptedAt= -1;
     pw->pwNext= next;
-    pw->pwForm[0]= firstCharacter;
-    pw->pwForm[1]= '\0';
+    pw->pwForm[0]= '\0';
 
     ssj->ssjPossibleWords= pw;
 
@@ -115,7 +117,7 @@ int	indNewPossibility(	SpellScanJob *		ssj,
 /****************************************************************/
 
 void indAddCharacterToPossibilities(	SpellScanJob *	ssj,
-					int		c	)
+					int		c )
     {
     PossibleWord *	pw= ssj->ssjPossibleWords;
 
@@ -125,7 +127,7 @@ void indAddCharacterToPossibilities(	SpellScanJob *	ssj,
 	    {
 	    if  ( pw->pwInsertionPoint >= FORM_MAX )
 		{
-		pw->pwRejectedAt= pw->pwStartPosition+ pw->pwInsertionPoint;
+		pw->pwRejectedAt= pw->pwStartAt+ pw->pwInsertionPoint;
 		}
 	    else{
 		pw->pwForm[pw->pwInsertionPoint++]= c;
@@ -149,7 +151,7 @@ void indAddCharacterToPossibilities(	SpellScanJob *	ssj,
 /************************************************************************/
 
 /*  1  */
-static int indCheckWord(	const unsigned char *	word,
+static int indCheckWord(	const char *		word,
 				SpellCheckContext *	scc,
 				int			asPrefix )
     {
@@ -162,19 +164,18 @@ static int indCheckWord(	const unsigned char *	word,
     if  ( ! asPrefix && scc->sccForgotInd )
 	{
 	/*  a  */
-	if  ( indGet( &accepted, scc->sccForgotInd, word ) >= 0	&&
-	      accepted						)
+	if  ( indGetUtf8( &accepted, scc->sccForgotInd, word ) >= 0	&&
+	      accepted							)
 	    { return -1;	}
 	}
 
     /*  b  */
-    if  ( ! indGetWord( &ignoredHow, scc->sccStaticInd, word, asPrefix,
-				    scc->sccCharKinds, scc->sccCharShifts ) )
+    if  ( ! indGetWord( &ignoredHow, scc->sccStaticInd, word, asPrefix ) )
 	{ return 0;	}
 
     /*  c  */
     if  ( scc->sccLearntInd &&
-	  indGet( &accepted, scc->sccLearntInd, word ) >= 0	&&
+	  indGetUtf8( &accepted, scc->sccLearntInd, word ) >= 0	&&
 	  ( accepted || asPrefix )				)
 	{ return 0; }
 
@@ -185,9 +186,7 @@ static int indCheckWord(	const unsigned char *	word,
 int indCountPossibilities(	SpellScanJob *		ssj,
 				SpellCheckContext *	scc,
 				int			position,
-				int			nextPosition,
-				int			rejectPrefices,
-				int			nextCharacter )
+				int			rejectPrefices )
     {
     PossibleWord *	pw= ssj->ssjPossibleWords;
     int			count= 0;
@@ -201,7 +200,7 @@ int indCountPossibilities(	SpellScanJob *		ssj,
 
 	    rejectedAsWord= indCheckWord( pw->pwForm, scc, 0 );
 
-	    /* SLDEB(pw->pwForm,rejected); */
+	    /* LSLDEB(pw->pwStartAt,pw->pwForm,rejectedAsWord); */
 
 	    if  ( rejectedAsWord )
 		{
@@ -217,18 +216,12 @@ int indCountPossibilities(	SpellScanJob *		ssj,
 
 	    if  ( ! rejectedAsPrefix )
 		{
-		pw->pwForm[pw->pwInsertionPoint  ]= nextCharacter;
-		pw->pwForm[pw->pwInsertionPoint+1]= '\0';
 		rejectedAsPrefix= indCheckWord( pw->pwForm, scc, 1 );
 		/* SLDEB(pw->pwForm,rejected); */
 		pw->pwForm[pw->pwInsertionPoint  ]= '\0';
 
 		if  ( rejectedAsPrefix )
-		    {
-		    if  ( rejectedAsWord )
-			{ pw->pwRejectedAt= position;		}
-		    else{ pw->pwRejectedAt= nextPosition;	}
-		    }
+		    { pw->pwRejectedAt= position;	}
 		}
 
 	    if  ( ! rejectedAsWord || ! rejectedAsPrefix )
@@ -241,9 +234,11 @@ int indCountPossibilities(	SpellScanJob *		ssj,
     return count;
     }
 
-/****************************************************************/
-/*  Make a list of the possibilities.				*/
-/****************************************************************/
+/************************************************************************/
+/*									*/
+/*  Make a list of the possibilities.					*/
+/*									*/
+/************************************************************************/
 
 void	indLogPossibilities(	SpellScanJob *	ssj	)
     {
@@ -251,8 +246,9 @@ void	indLogPossibilities(	SpellScanJob *	ssj	)
 
     while( pw )
 	{
-	appDebug( "\"%s\": Rejected %d Accepted %d\r\n",
-			pw->pwForm, pw->pwRejectedAt, pw->pwAcceptedAt );
+	appDebug( "\"%s\": Start %d Rejected %d Accepted %d\r\n",
+			pw->pwForm,
+			pw->pwStartAt, pw->pwRejectedAt, pw->pwAcceptedAt );
 
 	pw= pw->pwNext;
 	}
@@ -260,9 +256,16 @@ void	indLogPossibilities(	SpellScanJob *	ssj	)
     return;
     }
 
-/****************************************************************/
-/*  Cleanup routine: Remove rejected possibilities.		*/
-/****************************************************************/
+/************************************************************************/
+/*									*/
+/*  Cleanup routine: Remove rejected possibilities.			*/
+/*									*/
+/*  1)  Scan the list opto the first possibility that has not yet been	*/
+/*	rejected.							*/
+/*  2)  Also remove rejected alternatives from the rest of the list.	*/
+/*									*/
+/************************************************************************/
+
 void indRejectPossibilities(	int *			pAcceptedPos,
 				int			acceptedPos,
 				SpellScanJob *		ssj	)
@@ -271,6 +274,7 @@ void indRejectPossibilities(	int *			pAcceptedPos,
     PossibleWord *	rval= pw;
     PossibleWord *	next;
 
+    /*  1  */
     while( rval						&&
 	   rval->pwRejectedAt != -1			&&
 	   rval->pwRejectedAt >= rval->pwAcceptedAt	)
@@ -285,6 +289,7 @@ void indRejectPossibilities(	int *			pAcceptedPos,
 	rval= next;
 	}
 
+    /*  2  */
     if  ( rval )
 	{
 	pw= rval;
@@ -292,7 +297,7 @@ void indRejectPossibilities(	int *			pAcceptedPos,
 	    {
 	    next= pw->pwNext;
 
-	    while( next					&&
+	    while( next						&&
 		   next->pwRejectedAt != -1			&&
 		   next->pwRejectedAt >= next->pwAcceptedAt	)
 		{
@@ -342,10 +347,10 @@ PossibleWord * indMaximalPossibility(	SpellScanJob *	ssj	)
 	{
 	if  ( pw->pwAcceptedAt == -1 )
 	    {
-	    if  ( pw->pwStartPosition < rval->pwStartPosition )
+	    if  ( pw->pwStartAt < rval->pwStartAt )
 		{ rval= pw; }
 	    else{
-		if  ( pw->pwStartPosition == rval->pwStartPosition	&&
+		if  ( pw->pwStartAt == rval->pwStartAt	&&
 		      pw->pwRejectedAt    >  rval->pwRejectedAt   	)
 		    { rval= pw; }
 		}
@@ -356,61 +361,3 @@ PossibleWord * indMaximalPossibility(	SpellScanJob *	ssj	)
 
     return rval;
     }
-
-/************************************************************************/
-/*									*/
-/*  Initialise character classification.				*/
-/*									*/
-/************************************************************************/
-
-void indSpellIso1CharacterKinds(	SpellCheckContext *	scc )
-    {
-    int		i;
-
-    memset( scc->sccCharKinds, 0, 256 );
-    for ( i= 0; i < 256; i++ )
-	{ scc->sccCharShifts[i]= i; }
-
-    utilSetLatin1CharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
-
-    return;
-    }
-
-void indSpellIso2CharacterKinds(	SpellCheckContext *	scc )
-    {
-    int		i;
-
-    memset( scc->sccCharKinds, 0, 256 );
-    for ( i= 0; i < 256; i++ )
-	{ scc->sccCharShifts[i]= i; }
-
-    utilSetLatin2CharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
-
-    return;
-    }
-
-void indSpellIso7CharacterKinds(	SpellCheckContext *	scc )
-    {
-    int		i;
-
-    memset( scc->sccCharKinds, 0, 256 );
-    for ( i= 0; i < 256; i++ )
-	{ scc->sccCharShifts[i]= i; }
-
-    utilSetLatin2CharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
-
-    return;
-    }
-
-void indSpellKoi8rCharacterKinds(	SpellCheckContext *	scc )
-    {
-    int		i;
-
-    memset( scc->sccCharKinds, 0, 256 );
-    for ( i= 0; i < 256; i++ )
-	{ scc->sccCharShifts[i]= i; }
-
-    utilSetKoi8rCharacterKinds( scc->sccCharKinds, scc->sccCharShifts );
-
-    return;
-    }
