@@ -11,27 +11,30 @@
 #   include	<appDebugon.h>
 
 /************************************************************************/
+/*									*/
 /*  Move a word from forgotten to learnt, or the other way round.	*/
+/*									*/
 /************************************************************************/
-int	indMoveWord(	void *			fromInd,
-			void *			toInd,
-			const unsigned char *	word )
+
+int	indMoveWord(	void *		fromInd,
+			void *		toInd,
+			const char *	word )
     {
     int				rval= 0;
     int				accepted;
 
     if  ( fromInd					&&
-	  indGet( &accepted, fromInd, word ) >= 0	&&
+	  indGetUtf8( &accepted, fromInd, word ) >= 0	&&
 	  accepted					)
 	{
 	if  ( indForget( fromInd, word ) )
 	    { SDEB((char *)word); rval= -1;	}
 	}
 
-    if  ( indGet( &accepted, toInd, word ) < 0	||
+    if  ( indGetUtf8( &accepted, toInd, word ) < 0	||
 	  ! accepted					)
 	{
-	if  ( indPut( toInd, word ) < 0 )
+	if  ( indPutUtf8( toInd, word ) < 0 )
 	    { SDEB((char *)word); rval= -1;	}
 	}
 
@@ -51,7 +54,7 @@ int indReadPrivateDictionary(	FILE *		f,
     void *		learntInd= *pLearntInd;
     void *		forgotInd= *pForgotInd;
     
-    unsigned char	buf[SPELBUFL+ 1];
+    char		buf[SPELBUFL+ 1];
 
     int			rval= 0;
 
@@ -102,7 +105,7 @@ int indReadPrivateDictionary(	FILE *		f,
 int indLearnWord(	FILE *			f,
 			void *			learntInd,
 			void *			forgotInd,
-			const unsigned char *	word	)
+			const char *		word )
     {
     fprintf( f, "%s %s\n", "L", word );
 
@@ -115,7 +118,7 @@ int indLearnWord(	FILE *			f,
 int indForgetWord(	FILE *			f,
 			void *			learntInd,
 			void *			forgotInd,
-			const unsigned char *	word	)
+			const char *		word )
     {
     fprintf( f, "%s %s\n", "F", word );
 
